@@ -12,7 +12,7 @@ set_help_text load_icons \
 }
 
 set_tabcompletion_proc load_icons [namespace code tab_load_icons]
-proc tab_load_icons { args } {
+proc tab_load_icons {args} {
 	set num [llength $args]
 	if {$num == 2} {
 		set r1 [glob -nocomplain -tails -type d -directory $::env(OPENMSX_USER_DATA)/skins *]
@@ -31,7 +31,7 @@ variable current_fade_delay_active
 variable current_fade_delay_non_active
 variable fade_id
 
-proc trace_icon_status { name1 name2 op } {
+proc trace_icon_status {name1 name2 op} {
 	variable last_change
 	global $name1
 	set icon [string trimleft $name1 ":"]
@@ -39,7 +39,7 @@ proc trace_icon_status { name1 name2 op } {
 	redraw_osd_icons $icon
 }
 
-proc redraw_osd_icons { icon } {
+proc redraw_osd_icons {icon} {
 	variable last_change
 	variable current_fade_delay_active
 	variable current_fade_delay_non_active
@@ -47,9 +47,9 @@ proc redraw_osd_icons { icon } {
 	global $icon
 
 	# handle 'unset' variables  (when current msx machine got deleted)
-	if [catch {set value [set $icon]}] { set value false }
+	if {[catch {set value [set $icon]}]} {set value false}
 
-	if $value {
+	if {$value} {
 		set widget  osd_icons.${icon}_on
 		set widget2 osd_icons.${icon}_off
 		set fade_delay $current_fade_delay_active($icon)
@@ -64,14 +64,14 @@ proc redraw_osd_icons { icon } {
 		# no fading yet
 		osd configure $widget -fadeCurrent 1 -fadeTarget 1
 	} else {
-		set diff [expr [openmsx_info realtime] - $last_change($icon)]
+		set diff [expr {[openmsx_info realtime] - $last_change($icon)}]
 		if {$diff < $fade_delay} {
 			# no fading yet
 			osd configure $widget -fadeCurrent 1 -fadeTarget 1
 			catch {
 				after cancel $fade_id($icon)
 			}
-			set fade_id($icon) [after realtime [expr $fade_delay - $diff] "load_icons::redraw_osd_icons $icon"]
+			set fade_id($icon) [after realtime [expr {$fade_delay - $diff}] "load_icons::redraw_osd_icons $icon"]
 		} else {
 			# fading out
 			osd configure $widget -fadeTarget 0
@@ -79,14 +79,14 @@ proc redraw_osd_icons { icon } {
 	}
 }
 
-proc load_icons {{set_name "-show"} { position_param "default" }} {
+proc load_icons {{set_name "-show"} {position_param "default"}} {
 	variable icon_list
 	variable current_osd_leds_set
 	variable current_osd_leds_pos
 	variable current_fade_delay_active
 	variable current_fade_delay_non_active
 
-	if {$set_name == "-show"} {
+	if {$set_name eq "-show"} {
 		# Show list of available skins
 		set user_skins   \
 		    [glob -tails -types d -directory $::env(OPENMSX_USER_DATA)/skins   *]
@@ -106,11 +106,7 @@ proc load_icons {{set_name "-show"} { position_param "default" }} {
 	}
 
 	# Check position
-	if {($position_param != "top") &&
-	    ($position_param != "bottom") &&
-	    ($position_param != "left") &&
-	    ($position_param != "right") &&
-	    ($position_param != "default")} {
+	if {$position_param ni [list "top" "bottom" "left" "right" "default"]} {
 		error "Invalid position: $position_param"
 	}
 
@@ -130,56 +126,56 @@ proc load_icons {{set_name "-show"} { position_param "default" }} {
 	# but allow to override these values by the skin script
 	set icons $icon_list  ;# the 'none' skin needs this
 	set script $skin_set_dir/skin.tcl
-	if [file isfile $script] { source $script }
+	if {[file isfile $script]} {source $script}
 
-	set invscale [expr 1.0 / $scale]
-	set xbase    [expr $xbase    * $invscale]
-	set ybase    [expr $ybase    * $invscale]
-	set xwidth   [expr $xwidth   * $invscale]
-	set yheight  [expr $yheight  * $invscale]
-	set xspacing [expr $xspacing * $invscale]
-	set yspacing [expr $yspacing * $invscale]
+	set invscale [expr {1.0 / $scale}]
+	set xbase    [expr {$xbase    * $invscale}]
+	set ybase    [expr {$ybase    * $invscale}]
+	set xwidth   [expr {$xwidth   * $invscale}]
+	set yheight  [expr {$yheight  * $invscale}]
+	set xspacing [expr {$xspacing * $invscale}]
+	set yspacing [expr {$yspacing * $invscale}]
 
 	# change according to <position> parameter
-	if {$position == "default"} {
+	if {$position eq "default"} {
 		# script didn't set a default, so we choose a "default default"
 		set position "bottom"
 	}
-	if { $position == "left" } {
+	if {$position eq "left"} {
 		set horizontal 0
-	} elseif { $position == "right" } {
+	} elseif {$position eq "right"} {
 		set horizontal 0
-	        set xbase [expr 320 - $xwidth]
-	} elseif { $position == "bottom" } {
-	        set ybase [expr 240 - $yheight]
+	        set xbase [expr {320 - $xwidth}]
+	} elseif {$position eq "bottom"} {
+	        set ybase [expr {240 - $yheight}]
 	}
-	set vertical [expr !$horizontal]
+	set vertical [expr {!$horizontal}]
 
-	proc __try_dirs { skin_set_dir file fallback } {
+	proc __try_dirs {skin_set_dir file fallback} {
 		# don't touch already resolved pathnames
-		if {[file normalize $file] == $file} { return $file }
+		if {[file normalize $file] eq $file} {return $file}
 		# first look in specified skin-set directory
 		set f1 [file normalize $skin_set_dir/$file]
-		if [file isfile $f1] { return $f1 }
+		if {[file isfile $f1]} {return $f1}
 		# look for the falback image in the skin directory
 		set f2 [file normalize $skin_set_dir/$fallback]
-		if [file isfile $f2] { return $f2 }
+		if {[file isfile $f2]} {return $f2}
 		# if it's not there look in the root skin directory
 		# (system or user directory)
 		set f3 [file normalize [data_file "skins/$file"]]
-		if [file isfile $f3] { return $f3 }
+		if {[file isfile $f3]} {return $f3}
 		# still not found, look for the fallback image in system and
 		# user root skin dir
 		set f4 [file normalize [data_file "skins/$fallback"]]
-		if [file isfile $f4] { return $f4 }
+		if {[file isfile $f4]} {return $f4}
 		return ""
 	}
 	# Calculate default parameter values ...
-	for { set i 0 } { $i < [llength $icons] } { incr i } {
+	for {set i 0} {$i < [llength $icons]} {incr i} {
 		set icon [lindex $icons $i]
 
-		set xcoord($icon) [expr $i * $xspacing * $horizontal]
-		set ycoord($icon) [expr $i * $yspacing * $vertical]
+		set xcoord($icon) [expr {$i * $xspacing * $horizontal}]
+		set ycoord($icon) [expr {$i * $yspacing * $vertical}]
 
 		set fade_delay_active($icon)     $fade_delay
 		set fade_delay_non_active($icon) $fade_delay
@@ -215,7 +211,7 @@ proc load_icons {{set_name "-show"} { position_param "default" }} {
 	}
 
 	# ... but allow to override these calculated values (again) by the skin script
-	if [file isfile $script] { source $script }
+	if {[file isfile $script]} {source $script}
 
 	osd configure osd_icons -x $xbase -y $ybase
 
@@ -237,7 +233,7 @@ proc load_icons {{set_name "-show"} { position_param "default" }} {
 	# Also try to load "frame.png"
 	osd destroy osd_frame
 	set framefile "$skin_set_dir/frame.png"
-	if [file isfile $framefile] {
+	if {[file isfile $framefile]} {
 		osd create rectangle osd_frame -z 0 -x 0 -y 0 -w 320 -h 240 \
 		                               -scaled true -image $framefile
 	}
@@ -266,8 +262,8 @@ proc trace_osd_icon_vars {name1 name2 op} {
 
 	# avoid executing load_icons multiple times
 	# (because of the assignments to the settings in that proc)
-	if {($::osd_leds_set == $current_osd_leds_set) &&
-	    ($::osd_leds_pos == $current_osd_leds_pos)} {
+	if {($::osd_leds_set eq $current_osd_leds_set) &&
+	    ($::osd_leds_pos eq $current_osd_leds_pos)} {
 		return
 	}
 	load_icons $::osd_leds_set $::osd_leds_pos
@@ -286,7 +282,7 @@ proc machine_switch_osd_icons {} {
 
 # Available icons. Icons are also drawn in this order (by default)
 set icon_list [list "led_power" "led_caps" "led_kana" "led_pause" "led_turbo" "led_FDD" \
-                  "pause" "throttle" "mute" "breaked"]
+                    "pause" "throttle" "mute" "breaked"]
 
 # create OSD widgets
 osd create rectangle osd_icons -scaled true -alpha 0 -z 1
