@@ -50,7 +50,22 @@ register "normal" [namespace code enter_normal_mode] \
 
 ## Define tas mode
 
+variable old_inputdelay
+variable old_reversebar_fadeout_time
+
 proc enter_tas_mode {} {
+
+	variable old_inputdelay
+	variable old_reversebar_fadeout_time
+
+	# in tas mode, set inputdelay to 0
+	set old_inputdelay $::inputdelay
+	set ::inputdelay 0
+
+	# in tas mode, do not fadeout reverse bar
+	set old_reversebar_fadeout_time $::reversebar_fadeout_time
+	set ::reversebar_fadeout_time 0
+
 	if {![osd exists framecount]} {
 		toggle_frame_counter
 	}
@@ -69,13 +84,21 @@ proc enter_tas_mode {} {
 	bind_default F8 tas::save_replay_slot
 
 	# Set up frame advance/reverse
-	bind_default END -repeat advance_frame
-	bind_default SCROLLOCK -repeat reverse_frame
-
-	puts "WARNING: TAS mode is still very experimental and will almost certainly change next release!"
+	bind_default END -repeat next_frame
+	bind_default SCROLLOCK -repeat prev_frame
 }
 
 proc leave_tas_mode {} {
+
+	variable old_inputdelay
+	variable old_reversebar_fadeout_time
+
+	# restore inputdelay
+	set ::inputdelay $old_inputdelay
+
+	# restore reversebar_fadeout_time 
+	set ::reversebar_fadeout_time $old_reversebar_fadeout_time
+
 	if {[osd exists lag_counter]} {
 		toggle_lag_counter
 	}
