@@ -1,7 +1,6 @@
-// $Id: Autofire.cc 12627 2012-06-14 20:14:52Z m9710797 $
-
 #include "Autofire.hh"
 #include "IntegerSetting.hh"
+#include "memory.hh"
 #include <algorithm>
 #include <cassert>
 
@@ -13,8 +12,8 @@ Autofire::Autofire(CommandController& commandController,
                    unsigned newMinInts, unsigned newMaxInts, string_ref name)
 	: min_ints(std::max(newMinInts, 1u))
 	, max_ints(std::max(newMaxInts, min_ints + 1))
-	, speedSetting(new IntegerSetting(commandController, name,
-	               "controls the speed of this autofire circuit", 0, 0, 100))
+	, speedSetting(make_unique<IntegerSetting>(commandController, name,
+		"controls the speed of this autofire circuit", 0, 0, 100))
 	, clock(EmuTime::zero)
 {
 	setClock();
@@ -28,7 +27,7 @@ Autofire::~Autofire()
 
 void Autofire::setClock()
 {
-	int speed = speedSetting->getValue();
+	int speed = speedSetting->getInt();
 	clock.setFreq(
 	    (2 * 50 * 60) / (max_ints - (speed * (max_ints - min_ints)) / 100));
 }
@@ -42,7 +41,7 @@ void Autofire::update(const Setting& setting)
 
 bool Autofire::getSignal(EmuTime::param time)
 {
-	return speedSetting->getValue() == 0 ?
+	return speedSetting->getInt() == 0 ?
 		false : clock.getTicksTill(time) & 1;
 }
 
