@@ -1,5 +1,3 @@
-// $Id: CLIOption.cc 11609 2010-07-22 21:46:22Z m9710797 $
-
 #include "CLIOption.hh"
 #include "MSXException.hh"
 
@@ -15,7 +13,7 @@ string CLIOption::getArgument(const string& option, deque<string>& cmdLine) cons
 	if (cmdLine.empty()) {
 		throw FatalError("Missing argument for option \"" + option + '\"');
 	}
-	string argument = cmdLine.front();
+	string argument = std::move(cmdLine.front());
 	cmdLine.pop_front();
 	return argument;
 }
