@@ -1,5 +1,3 @@
-// $Id: CliExtension.cc 12628 2012-06-14 20:15:37Z m9710797 $
-
 #include "CliExtension.hh"
 #include "CommandLineParser.hh"
 #include "MSXMotherBoard.hh"
@@ -13,10 +11,10 @@ namespace openmsx {
 CliExtension::CliExtension(CommandLineParser& cmdLineParser_)
 	: cmdLineParser(cmdLineParser_)
 {
-	cmdLineParser.registerOption("-ext", *this, 7);
+	cmdLineParser.registerOption("-ext", *this, CommandLineParser::PHASE_EXT);
 }
 
-bool CliExtension::parseOption(const string& option, std::deque<string>& cmdLine)
+void CliExtension::parseOption(const string& option, std::deque<string>& cmdLine)
 {
 	try {
 		string extensionName = getArgument(option, cmdLine);
@@ -26,7 +24,6 @@ bool CliExtension::parseOption(const string& option, std::deque<string>& cmdLine
 	} catch (MSXException& e) {
 		throw FatalError(e.getMessage());
 	}
-	return true;
 }
 
 string_ref CliExtension::optionHelp() const
