@@ -1,8 +1,7 @@
-// $Id: CommandLineParser.cc 12814 2012-08-13 20:22:45Z m9710797 $
-
 #include "CommandLineParser.hh"
 #include "CLIOption.hh"
 #include "GlobalCommandController.hh"
+#include "Interpreter.hh"
 #include "SettingsConfig.hh"
 #include "File.hh"
 #include "FileContext.hh"
@@ -25,12 +24,15 @@
 #include "XMLLoader.hh"
 #include "XMLElement.hh"
 #include "XMLException.hh"
-#include "HostCPU.hh"
 #include "StringOp.hh"
 #include "utf8_checked.hh"
+#include "xrange.hh"
 #include "GLUtil.hh"
 #include "Reactor.hh"
+#include "RomInfo.hh"
 #include "StringMap.hh"
+#include "memory.hh"
+#include "stl.hh"
 #include "build-info.hh"
 #include "components.hh"
 
@@ -45,8 +47,6 @@
 using std::cout;
 using std::endl;
 using std::deque;
-using std::map;
-using std::set;
 using std::string;
 using std::vector;
 
@@ -60,7 +60,7 @@ class HelpOption : public CLIOption
 {
 public:
 	explicit HelpOption(CommandLineParser& parser);
-	virtual bool parseOption(const string& option, deque<string>& cmdLine);
+	virtual void parseOption(const string& option, deque<string>& cmdLine);
 	virtual string_ref optionHelp() const;
 private:
 	CommandLineParser& parser;
@@ -70,7 +70,7 @@ class VersionOption : public CLIOption
 {
 public:
 	explicit VersionOption(CommandLineParser& parser);
-	virtual bool parseOption(const string& option, deque<string>& cmdLine);
+	virtual void parseOption(const string& option, deque<string>& cmdLine);
 	virtual string_ref optionHelp() const;
 private:
 	CommandLineParser& parser;
@@ -80,7 +80,7 @@ class ControlOption : public CLIOption
 {
 public:
 	explicit ControlOption(CommandLineParser& parser);
-	virtual bool parseOption(const string& option, deque<string>& cmdLine);
+	virtual void parseOption(const string& option, deque<string>& cmdLine);
 	virtual string_ref optionHelp() const;
 private:
 	CommandLineParser& parser;
@@ -90,7 +90,7 @@ class ScriptOption : public CLIOption, public CLIFileType
 {
 public:
 	const CommandLineParser::Scripts& getScripts() const;
-	virtual bool parseOption(const string& option, deque<string>& cmdLine);
+	virtual void parseOption(const string& option, deque<string>& cmdLine);
 	virtual string_ref optionHelp() const;
 	virtual void parseFileType(const std::string& filename,
                                    std::deque<std::string>& cmdLine);
@@ -104,7 +104,7 @@ class MachineOption : public CLIOption
 {
 public:
 	explicit MachineOption(CommandLineParser& parser);
-	virtual bool parseOption(const string& option, deque<string>& cmdLine);
+	virtual void parseOption(const string& option, deque<string>& cmdLine);
 	virtual string_ref optionHelp() const;
 private:
 	CommandLineParser& parser;
@@ -114,34 +114,15 @@ class SettingOption : public CLIOption
 {
 public:
 	explicit SettingOption(CommandLineParser& parser);
-	virtual bool parseOption(const string& option, deque<string>& cmdLine);
+	virtual void parseOption(const string& option, deque<string>& cmdLine);
 	virtual string_ref optionHelp() const;
 private:
 	CommandLineParser& parser;
 };
 
-class NoMMXOption : public CLIOption
-{
-public:
-	virtual bool parseOption(const string& option, deque<string>& cmdLine);
-	virtual string_ref optionHelp() const;
-};
-
-class NoSSEOption : public CLIOption {
-public:
-	virtual bool parseOption(const string& option, deque<string>& cmdLine);
-	virtual string_ref optionHelp() const;
-};
-
-class NoSSE2Option : public CLIOption {
-public:
-	virtual bool parseOption(const string& option, deque<string>& cmdLine);
-	virtual string_ref optionHelp() const;
-};
-
 class NoPBOOption : public CLIOption {
 public:
-	virtual bool parseOption(const string& option, deque<string>& cmdLine);
+	virtual void parseOption(const string& option, deque<string>& cmdLine);
 	virtual string_ref optionHelp() const;
 };
 
@@ -149,7 +130,17 @@ class TestConfigOption : public CLIOption
 {
 public:
 	explicit TestConfigOption(CommandLineParser& parser);
-	virtual bool parseOption(const string& option, deque<string>& cmdLine);
+	virtual void parseOption(const string& option, deque<string>& cmdLine);
+	virtual string_ref optionHelp() const;
+private:
+	CommandLineParser& parser;
+};
+
+class BashOption : public CLIOption
+{
+public:
+	explicit BashOption(CommandLineParser& parser);
+	virtual void parseOption(const string& option, deque<string>& cmdLine);
 	virtual string_ref optionHelp() const;
 private:
 	CommandLineParser& parser;
@@ -158,57 +149,57 @@ private:
 
 // class CommandLineParser
 
+typedef LessTupleElement<0> CmpOptions;
+typedef CmpTupleElement<0, StringOp::caseless> CmpFileTypes;
+
 CommandLineParser::CommandLineParser(Reactor& reactor_)
 	: reactor(reactor_)
-	, settingsConfig(reactor.getGlobalCommandController().getSettingsConfig())
-	, output(reactor.getCliComm())
-	, helpOption(new HelpOption(*this))
-	, versionOption(new VersionOption(*this))
-	, controlOption(new ControlOption(*this))
-	, scriptOption(new ScriptOption())
-	, machineOption(new MachineOption(*this))
-	, settingOption(new SettingOption(*this))
-	, noMMXOption(new NoMMXOption())
-	, noSSEOption(new NoSSEOption())
-	, noSSE2Option(new NoSSE2Option())
-	, noPBOOption(new NoPBOOption())
-	, testConfigOption(new TestConfigOption(*this))
-	, msxRomCLI(new MSXRomCLI(*this))
-	, cliExtension(new CliExtension(*this))
-	, replayCLI(new ReplayCLI(*this))
-	, saveStateCLI(new SaveStateCLI(*this))
-	, cassettePlayerCLI(new CassettePlayerCLI(*this))
+	, helpOption(make_unique<HelpOption>(*this))
+	, versionOption(make_unique<VersionOption>(*this))
+	, controlOption(make_unique<ControlOption>(*this))
+	, scriptOption(make_unique<ScriptOption>())
+	, machineOption(make_unique<MachineOption>(*this))
+	, settingOption(make_unique<SettingOption>(*this))
+	, noPBOOption(make_unique<NoPBOOption>())
+	, testConfigOption(make_unique<TestConfigOption>(*this))
+	, bashOption(make_unique<BashOption>(*this))
+	, msxRomCLI(make_unique<MSXRomCLI>(*this))
+	, cliExtension(make_unique<CliExtension>(*this))
+	, replayCLI(make_unique<ReplayCLI>(*this))
+	, saveStateCLI(make_unique<SaveStateCLI>(*this))
+	, cassettePlayerCLI(make_unique<CassettePlayerCLI>(*this))
 #if COMPONENT_LASERDISC
-	, laserdiscPlayerCLI(new LaserdiscPlayerCLI(*this))
+	, laserdiscPlayerCLI(make_unique<LaserdiscPlayerCLI>(*this))
 #endif
-	, diskImageCLI(new DiskImageCLI(*this))
-	, hdImageCLI(new HDImageCLI(*this))
-	, cdImageCLI(new CDImageCLI(*this))
+	, diskImageCLI(make_unique<DiskImageCLI>(*this))
+	, hdImageCLI(make_unique<HDImageCLI>(*this))
+	, cdImageCLI(make_unique<CDImageCLI>(*this))
 	, parseStatus(UNPARSED)
 {
 	haveConfig = false;
 	haveSettings = false;
 
-	registerOption("-machine",    *machineOption, 4);
-	registerOption("-setting",    *settingOption, 2);
-	registerOption("-h",          *helpOption, 1, 1);
-	registerOption("--help",      *helpOption, 1, 1);
-	registerOption("-v",          *versionOption, 1, 1);
-	registerOption("--version",   *versionOption, 1, 1);
-	registerOption("-control",    *controlOption, 1, 1);
-	registerOption("-script",     *scriptOption, 1, 1);
-	registerFileClass("Tcl script", *scriptOption);
-	#if ASM_X86
-	registerOption("-nommx",      *noMMXOption, 1, 1);
-	registerOption("-nosse",      *noSSEOption, 1, 1);
-	registerOption("-nosse2",     *noSSE2Option, 1, 1);
-	#endif
-	#if COMPONENT_GL
-	registerOption("-nopbo",      *noPBOOption, 1, 1);
-	#endif
-	registerOption("-testconfig", *testConfigOption, 1, 1);
+	registerOption("-h",          *helpOption,    PHASE_BEFORE_INIT, 1);
+	registerOption("--help",      *helpOption,    PHASE_BEFORE_INIT, 1);
+	registerOption("-v",          *versionOption, PHASE_BEFORE_INIT, 1);
+	registerOption("--version",   *versionOption, PHASE_BEFORE_INIT, 1);
+	registerOption("-bash",       *bashOption,    PHASE_BEFORE_INIT, 1);
 
-	registerFileTypes();
+	registerOption("-setting",    *settingOption, PHASE_BEFORE_SETTINGS);
+	registerOption("-control",    *controlOption, PHASE_BEFORE_SETTINGS, 1);
+	registerOption("-script",     *scriptOption,  PHASE_BEFORE_SETTINGS, 1); // correct phase?
+	#if COMPONENT_GL
+	registerOption("-nopbo",      *noPBOOption,   PHASE_BEFORE_SETTINGS, 1);
+	#endif
+	registerOption("-testconfig", *testConfigOption, PHASE_BEFORE_SETTINGS, 1);
+
+	registerOption("-machine",    *machineOption, PHASE_BEFORE_MACHINE);
+
+	registerFileType("tcl", *scriptOption);
+
+	// At this point all options and file-types must be registered
+	sort(options.begin(), options.end(), CmpOptions());
+	sort(fileTypes.begin(), fileTypes.end(), CmpFileTypes());
 }
 
 CommandLineParser::~CommandLineParser()
@@ -216,69 +207,34 @@ CommandLineParser::~CommandLineParser()
 }
 
 void CommandLineParser::registerOption(
-	const char* str, CLIOption& cliOption, unsigned prio, unsigned length)
+	const char* str, CLIOption& cliOption, ParsePhase phase, unsigned length)
 {
 	OptionData temp;
 	temp.option = &cliOption;
-	temp.prio = prio;
+	temp.phase = phase;
 	temp.length = length;
-	optionMap[str] = temp;
+	options.push_back(std::make_pair(str, temp));
 }
 
-void CommandLineParser::registerFileClass(
-	const char* str, CLIFileType& cliFileType)
+void CommandLineParser::registerFileType(
+	string_ref extensions, CLIFileType& cliFileType)
 {
-	fileClassMap[str] = &cliFileType;
-}
-
-void CommandLineParser::registerFileTypes()
-{
-	if (const XMLElement* config =
-			settingsConfig.getXMLElement().findChild("FileTypes")) {
-		for (FileClassMap::const_iterator i = fileClassMap.begin();
-		     i != fileClassMap.end(); ++i) {
-			XMLElement::Children extensions;
-			config->getChildren(i->first, extensions);
-			for (XMLElement::Children::const_iterator j = extensions.begin();
-			     j != extensions.end(); ++j) {
-				fileTypeMap[(*j)->getData()] = i->second;
-			}
-		}
-	} else {
-		map<string, string> fileExtMap;
-		fileExtMap["rom"] = "romimage";
-		fileExtMap["ri"]  = "romimage";
-		fileExtMap["dsk"] = "diskimage";
-		fileExtMap["dmk"] = "diskimage";
-		fileExtMap["di1"] = "diskimage";
-		fileExtMap["di2"] = "diskimage";
-		fileExtMap["xsa"] = "diskimage";
-		fileExtMap["wav"] = "cassetteimage";
-		fileExtMap["cas"] = "cassetteimage";
-		fileExtMap["ogv"] = "laserdiscimage";
-		fileExtMap["omr"] = "openMSX replay";
-		fileExtMap["oms"] = "openMSX savestate";
-		fileExtMap["tcl"] = "Tcl script";
-		for (map<string, string>::const_iterator j = fileExtMap.begin();
-		     j != fileExtMap.end(); ++j) {
-			FileClassMap::const_iterator i = fileClassMap.find(j->second);
-			if (i != fileClassMap.end()) {
-				fileTypeMap[j->first] = i->second;
-			}
-		}
+	for (auto& ext: StringOp::split(extensions, ',')) {
+		fileTypes.push_back(std::make_pair(ext, &cliFileType));
 	}
 }
 
 bool CommandLineParser::parseOption(
-	const string& arg, deque<string>& cmdLine, unsigned priority)
+	const string& arg, deque<string>& cmdLine, ParsePhase phase)
 {
-	map<string, OptionData>::const_iterator it1 = optionMap.find(arg);
-	if (it1 != optionMap.end()) {
+	auto it = lower_bound(options.begin(), options.end(), arg,
+	                      CmpOptions());
+	if ((it != options.end()) && (it->first == arg)) {
 		// parse option
-		if (it1->second.prio <= priority) {
+		if (it->second.phase <= phase) {
 			try {
-				return it1->second.option->parseOption(
-					arg, cmdLine);
+				it->second.option->parseOption(arg, cmdLine);
+				return true;
 			} catch (MSXException& e) {
 				throw FatalError(e.getMessage());
 			}
@@ -297,8 +253,7 @@ bool CommandLineParser::parseFileName(const string& arg, deque<string>& cmdLine)
 	bool processed = parseFileNameInner(arg, arg, cmdLine);
 	if (!processed) {
 		try {
-			UserFileContext context;
-			File file(context.resolve(arg));
+			File file(UserFileContext().resolve(arg));
 			string originalName = file.getOriginalName();
 			processed = parseFileNameInner(originalName, arg, cmdLine);
 		} catch (FileException&) {
@@ -311,20 +266,24 @@ bool CommandLineParser::parseFileName(const string& arg, deque<string>& cmdLine)
 bool CommandLineParser::parseFileNameInner(const string& name, const string& originalPath, deque<string>& cmdLine)
 {
 	string_ref extension = FileOperations::getExtension(name);
-	if (!extension.empty()) {
-		// there is an extension
-		FileTypeMap::const_iterator it = fileTypeMap.find(extension.str());
-		if (it != fileTypeMap.end()) {
-			try {
-				// parse filetype
-				it->second->parseFileType(originalPath, cmdLine);
-				return true; // file processed
-			} catch (MSXException& e) {
-				throw FatalError(e.getMessage());
-			}
-		}
+	if (extension.empty()) {
+		return false; // no extension
+	}
+
+	auto it = lower_bound(fileTypes.begin(), fileTypes.end(), extension,
+	                      CmpFileTypes());
+	StringOp::casecmp cmp;
+	if ((it == fileTypes.end()) || !cmp(it->first, extension)) {
+		return false; // unknown extension
+	}
+
+	try {
+		// parse filetype
+		it->second->parseFileType(originalPath, cmdLine);
+		return true; // file processed
+	} catch (MSXException& e) {
+		throw FatalError(e.getMessage());
 	}
-	return false; // unknown
 }
 
 void CommandLineParser::parse(int argc, char** argv)
@@ -333,21 +292,29 @@ void CommandLineParser::parse(int argc, char** argv)
 
 	deque<string> cmdLine;
 	deque<string> backupCmdLine;
-	for (int i = 1; i < argc; i++) {
+	for (auto i : xrange(1, argc)) {
 		cmdLine.push_back(FileOperations::getConventionalPath(argv[i]));
 	}
 
-	for (int priority = 1; (priority <= 8) && (parseStatus != EXIT); ++priority) {
-		switch (priority) {
-		case 2: // after ControlOption has been parsed
+	for (ParsePhase phase = PHASE_BEFORE_INIT;
+	     (phase <= PHASE_LAST) && (parseStatus != EXIT);
+	     phase = static_cast<ParsePhase>(phase + 1)) {
+		switch (phase) {
+		case PHASE_INIT:
+			reactor.init();
+			reactor.getGlobalCommandController().getInterpreter().init(argv[0]);
+			break;
+		case PHASE_LOAD_SETTINGS:
+			// after -control and -setting has been parsed
 			if (parseStatus != CONTROL) {
 				// if there already is a XML-StdioConnection, we
 				// can't also show plain messages on stdout
-				GlobalCliComm& cliComm = reactor.getGlobalCliComm();
+				auto& cliComm = reactor.getGlobalCliComm();
 				cliComm.addListener(new StdioMessages());
 			}
-		case 3:
 			if (!haveSettings) {
+				auto& settingsConfig =
+					reactor.getGlobalCommandController().getSettingsConfig();
 				// Load default settings file in case the user
 				// didn't specify one.
 				SystemFileContext context;
@@ -355,7 +322,7 @@ void CommandLineParser::parse(int argc, char** argv)
 				try {
 					settingsConfig.loadSetting(context, filename);
 				} catch (XMLException& e) {
-					output.printWarning(
+					reactor.getCliComm().printWarning(
 						"Loading of settings failed: " +
 						e.getMessage() + "\n"
 						"Reverting to default settings.");
@@ -372,21 +339,20 @@ void CommandLineParser::parse(int argc, char** argv)
 				settingsConfig.setSaveFilename(context, filename);
 			}
 			break;
-		case 5: {
+		case PHASE_LOAD_MACHINE: {
 			if (!haveConfig) {
 				// load default config file in case the user didn't specify one
-				const string& machine =
-					reactor.getMachineSetting().getValueString();
+				const auto& machine =
+					reactor.getMachineSetting().getString();
 				try {
 					reactor.switchMachine(machine);
 				} catch (MSXException& e) {
-					output.printInfo(
-						"Failed to initialize default machine: " + e.getMessage()
-						);
+					reactor.getCliComm().printInfo(
+						"Failed to initialize default machine: " + e.getMessage());
 					// Default machine is broken; fall back to C-BIOS config.
-					const string& fallbackMachine =
-						reactor.getMachineSetting().getRestoreValueString();
-					output.printInfo("Using fallback machine: " + fallbackMachine);
+					const auto& fallbackMachine =
+						reactor.getMachineSetting().getRestoreValue();
+					reactor.getCliComm().printInfo("Using fallback machine: " + fallbackMachine);
 					try {
 						reactor.switchMachine(fallbackMachine);
 					} catch (MSXException& e2) {
@@ -401,21 +367,20 @@ void CommandLineParser::parse(int argc, char** argv)
 		default:
 			// iterate over all arguments
 			while (!cmdLine.empty()) {
-				string arg = cmdLine.front();
+				string arg = std::move(cmdLine.front());
 				cmdLine.pop_front();
 				// first try options
-				if (!parseOption(arg, cmdLine, priority)) {
+				if (!parseOption(arg, cmdLine, phase)) {
 					// next try the registered filetypes (xml)
-					if ((priority < 7) ||
+					if ((phase != PHASE_LAST) ||
 					    !parseFileName(arg, cmdLine)) {
 						// no option or known file
 						backupCmdLine.push_back(arg);
-						map<string, OptionData>::const_iterator it1 =
-							optionMap.find(arg);
-						if (it1 != optionMap.end()) {
-							for (unsigned i = 0; i < it1->second.length - 1; ++i) {
+						auto it = lower_bound(options.begin(), options.end(), arg, CmpOptions());
+						if ((it != options.end()) && (it->first == arg)) {
+							for (unsigned i = 0; i < it->second.length - 1; ++i) {
 								if (!cmdLine.empty()) {
-									backupCmdLine.push_back(cmdLine.front());
+									backupCmdLine.push_back(std::move(cmdLine.front()));
 									cmdLine.pop_front();
 								}
 							}
@@ -471,27 +436,27 @@ ControlOption::ControlOption(CommandLineParser& parser_)
 {
 }
 
-bool ControlOption::parseOption(const string& option, deque<string>& cmdLine)
+void ControlOption::parseOption(const string& option, deque<string>& cmdLine)
 {
-	string fullType = getArgument(option, cmdLine);
+	const auto& fullType = getArgument(option, cmdLine);
 	string_ref type, arguments;
 	StringOp::splitOnFirst(fullType, ':', type, arguments);
 
-	CommandController& controller = parser.getGlobalCommandController();
-	EventDistributor& distributor = parser.reactor.getEventDistributor();
-	GlobalCliComm& cliComm        = parser.reactor.getGlobalCliComm();
-	std::auto_ptr<CliListener> connection;
+	auto& controller  = parser.getGlobalCommandController();
+	auto& distributor = parser.reactor.getEventDistributor();
+	auto& cliComm     = parser.reactor.getGlobalCliComm();
+	std::unique_ptr<CliListener> connection;
 	if (type == "stdio") {
-		connection.reset(new StdioConnection(
-			controller, distributor));
+		connection = make_unique<StdioConnection>(
+			controller, distributor);
 #ifdef _WIN32
 	} else if (type == "pipe") {
 		OSVERSIONINFO info;
 		info.dwOSVersionInfoSize = sizeof(info);
 		GetVersionEx(&info);
 		if (info.dwPlatformId == VER_PLATFORM_WIN32_NT) {
-			connection.reset(new PipeConnection(
-				controller, distributor, arguments));
+			connection = make_unique<PipeConnection>(
+				controller, distributor, arguments);
 		} else {
 			throw FatalError("Pipes are not supported on this "
 			                 "version of Windows");
@@ -503,7 +468,6 @@ bool ControlOption::parseOption(const string& option, deque<string>& cmdLine)
 	cliComm.addListener(connection.release());
 
 	parser.parseStatus = CommandLineParser::CONTROL;
-	return true;
 }
 
 string_ref ControlOption::optionHelp() const
@@ -519,10 +483,9 @@ const CommandLineParser::Scripts& ScriptOption::getScripts() const
 	return scripts;
 }
 
-bool ScriptOption::parseOption(const string& option, deque<string>& cmdLine)
+void ScriptOption::parseOption(const string& option, deque<string>& cmdLine)
 {
 	parseFileType(getArgument(option, cmdLine), cmdLine);
-	return true;
 }
 
 string_ref ScriptOption::optionHelp() const
@@ -543,25 +506,23 @@ string_ref ScriptOption::fileTypeHelp() const
 
 // Help option
 
-static string formatSet(const set<string>& inputSet, string::size_type columns)
+static string formatSet(const vector<string_ref>& inputSet, string::size_type columns)
 {
 	StringOp::Builder outString;
 	string::size_type totalLength = 0; // ignore the starting spaces for now
-	for (set<string>::const_iterator it = inputSet.begin();
-	     it != inputSet.end(); ++it) {
-		string temp = *it;
+	for (auto& temp : inputSet) {
 		if (totalLength == 0) {
 			// first element ?
 			outString << "    " << temp;
-			totalLength = temp.length();
+			totalLength = temp.size();
 		} else {
 			outString << ", ";
-			if ((totalLength + temp.length()) > columns) {
+			if ((totalLength + temp.size()) > columns) {
 				outString << "\n    " << temp;
-				totalLength = temp.length();
+				totalLength = temp.size();
 			} else {
 				outString << temp;
-				totalLength += 2 + temp.length();
+				totalLength += 2 + temp.size();
 			}
 		}
 	}
@@ -577,7 +538,7 @@ static string formatHelptext(string_ref helpText,
 	string outText;
 	string_ref::size_type index = 0;
 	while (helpText.substr(index).size() > maxLength) {
-		string_ref::size_type pos = helpText.substr(index, maxLength).rfind(' ');
+		auto pos = helpText.substr(index, maxLength).rfind(' ');
 		if (pos == string_ref::npos) {
 			pos = helpText.substr(maxLength).find(' ');
 			if (pos == string_ref::npos) {
@@ -593,17 +554,16 @@ static string formatHelptext(string_ref helpText,
 	return outText;
 }
 
-static void printItemMap(const StringMap<set<string> >& itemMap)
+static void printItemMap(const StringMap<vector<string_ref>>& itemMap)
 {
-	set<string> printSet;
-	for (StringMap<set<string> >::const_iterator it = itemMap.begin();
-	     it != itemMap.end(); ++it) {
-		printSet.insert(formatSet(it->second, 15) + ' ' +
-		                formatHelptext(it->first(), 50, 20));
+	vector<string> printSet;
+	for (auto& p : itemMap) {
+		printSet.push_back(formatSet(p.second, 15) + ' ' +
+		                   formatHelptext(p.first(), 50, 20));
 	}
-	for (set<string>::const_iterator it = printSet.begin();
-	     it != printSet.end(); ++it) {
-		cout << *it << endl;
+	sort(printSet.begin(), printSet.end());
+	for (auto& s : printSet) {
+		cout << s << endl;
 	}
 }
 
@@ -612,40 +572,37 @@ HelpOption::HelpOption(CommandLineParser& parser_)
 {
 }
 
-bool HelpOption::parseOption(const string& /*option*/,
+void HelpOption::parseOption(const string& /*option*/,
                              deque<string>& /*cmdLine*/)
 {
-	if (!parser.haveSettings) {
-		return false; // not parsed yet, load settings first
-	}
-	cout << Version::FULL_VERSION << endl;
-	cout << string(Version::FULL_VERSION.length(), '=') << endl;
+	const auto& fullVersion = Version::full();
+	cout << fullVersion << endl;
+	cout << string(fullVersion.size(), '=') << endl;
 	cout << endl;
 	cout << "usage: openmsx [arguments]" << endl;
 	cout << "  an argument is either an option or a filename" << endl;
 	cout << endl;
 	cout << "  this is the list of supported options:" << endl;
 
-	StringMap<set<string> > optionMap;
-	for (map<string, CommandLineParser::OptionData>::const_iterator it =
-	        parser.optionMap.begin(); it != parser.optionMap.end(); ++it) {
-		optionMap[it->second.option->optionHelp()].insert(it->first);
+	StringMap<vector<string_ref>> optionMap;
+	for (auto& p : parser.options) {
+		const auto& helpText = p.second.option->optionHelp();
+		if (!helpText.empty()) {
+			optionMap[helpText].push_back(p.first);
+		}
 	}
 	printItemMap(optionMap);
 
 	cout << endl;
 	cout << "  this is the list of supported file types:" << endl;
 
-	StringMap<set<string> > extMap;
-	for (CommandLineParser::FileTypeMap::const_iterator it =
-	         parser.fileTypeMap.begin();
-	     it != parser.fileTypeMap.end(); ++it) {
-		extMap[it->second->fileTypeHelp()].insert(it->first);
+	StringMap<vector<string_ref>> extMap;
+	for (auto& p : parser.fileTypes) {
+		extMap[p.second->fileTypeHelp()].push_back(p.first);
 	}
 	printItemMap(extMap);
 
 	parser.parseStatus = CommandLineParser::EXIT;
-	return true;
 }
 
 string_ref HelpOption::optionHelp() const
@@ -661,14 +618,13 @@ VersionOption::VersionOption(CommandLineParser& parser_)
 {
 }
 
-bool VersionOption::parseOption(const string& /*option*/,
+void VersionOption::parseOption(const string& /*option*/,
                                 deque<string>& /*cmdLine*/)
 {
-	cout << Version::FULL_VERSION << endl;
+	cout << Version::full() << endl;
 	cout << "flavour: " << BUILD_FLAVOUR << endl;
 	cout << "components: " << BUILD_COMPONENTS << endl;
 	parser.parseStatus = CommandLineParser::EXIT;
-	return true;
 }
 
 string_ref VersionOption::optionHelp() const
@@ -684,19 +640,17 @@ MachineOption::MachineOption(CommandLineParser& parser_)
 {
 }
 
-bool MachineOption::parseOption(const string& option, deque<string>& cmdLine)
+void MachineOption::parseOption(const string& option, deque<string>& cmdLine)
 {
 	if (parser.haveConfig) {
 		throw FatalError("Only one machine option allowed");
 	}
-	string machine(getArgument(option, cmdLine));
 	try {
-		parser.reactor.switchMachine(machine);
+		parser.reactor.switchMachine(getArgument(option, cmdLine));
 	} catch (MSXException& e) {
 		throw FatalError(e.getMessage());
 	}
 	parser.haveConfig = true;
-	return true;
 }
 string_ref MachineOption::optionHelp() const
 {
@@ -711,22 +665,21 @@ SettingOption::SettingOption(CommandLineParser& parser_)
 {
 }
 
-bool SettingOption::parseOption(const string& option, deque<string>& cmdLine)
+void SettingOption::parseOption(const string& option, deque<string>& cmdLine)
 {
 	if (parser.haveSettings) {
 		throw FatalError("Only one setting option allowed");
 	}
 	try {
-		CurrentDirFileContext context;
-		parser.settingsConfig.loadSetting(
-			context, getArgument(option, cmdLine));
+		auto& settingsConfig = parser.reactor.getGlobalCommandController().getSettingsConfig();
+		settingsConfig.loadSetting(
+			CurrentDirFileContext(), getArgument(option, cmdLine));
 		parser.haveSettings = true;
 	} catch (FileException& e) {
 		throw FatalError(e.getMessage());
 	} catch (ConfigException& e) {
 		throw FatalError(e.getMessage());
 	}
-	return true;
 }
 
 string_ref SettingOption::optionHelp() const
@@ -735,64 +688,15 @@ string_ref SettingOption::optionHelp() const
 }
 
 
-// class NoMMXOption
-
-bool NoMMXOption::parseOption(const string& /*option*/,
-                              deque<string>& /*cmdLine*/)
-{
-	cout << "Disabling MMX" << endl;
-	HostCPU::forceDisableMMX();
-	return true;
-}
-
-string_ref NoMMXOption::optionHelp() const
-{
-	return "Disables usage of MMX, SSE and SSE2 (for debugging)";
-}
-
-
-// class NoSSEOption
-
-bool NoSSEOption::parseOption(const string& /*option*/,
-                              deque<string>& /*cmdLine*/)
-{
-	cout << "Disabling SSE" << endl;
-	HostCPU::forceDisableSSE();
-	return true;
-}
-
-string_ref NoSSEOption::optionHelp() const
-{
-	return "Disables usage of SSE and SSE2 (for debugging)";
-}
-
-
-// class NoSSE2Option
-
-bool NoSSE2Option::parseOption(const string& /*option*/,
-                               deque<string>& /*cmdLine*/)
-{
-	cout << "Disabling SSE2" << endl;
-	HostCPU::forceDisableSSE2();
-	return true;
-}
-
-string_ref NoSSE2Option::optionHelp() const
-{
-	return "Disables usage of SSE2 (for debugging)";
-}
-
-
 // class NoPBOOption
 
-bool NoPBOOption::parseOption(const string& /*option*/,
+void NoPBOOption::parseOption(const string& /*option*/,
                               deque<string>& /*cmdLine*/)
 {
 	#if COMPONENT_GL
 	cout << "Disabling PBO" << endl;
 	PixelBuffers::enabled = false;
 	#endif
-	return true;
 }
 
 string_ref NoPBOOption::optionHelp() const
@@ -808,11 +712,10 @@ TestConfigOption::TestConfigOption(CommandLineParser& parser_)
 {
 }
 
-bool TestConfigOption::parseOption(const string& /*option*/,
+void TestConfigOption::parseOption(const string& /*option*/,
                                    deque<string>& /*cmdLine*/)
 {
 	parser.parseStatus = CommandLineParser::TEST;
-	return true;
 }
 
 string_ref TestConfigOption::optionHelp() const
@@ -820,4 +723,42 @@ string_ref TestConfigOption::optionHelp() const
 	return "Test if the specified config works and exit";
 }
 
+// class BashOption
+
+BashOption::BashOption(CommandLineParser& parser_)
+	: parser(parser_)
+{
+}
+
+void BashOption::parseOption(const string& /*option*/,
+                             deque<string>& cmdLine)
+{
+	string last = cmdLine.empty() ? "" : cmdLine.front();
+	cmdLine.clear(); // eat all remaining parameters
+
+	if (last == "-machine") {
+		for (auto& s : Reactor::getHwConfigs("machines")) {
+			cout << s << '\n';
+		}
+	} else if (last == "-ext") {
+		for (auto& s : Reactor::getHwConfigs("extensions")) {
+			cout << s << '\n';
+		}
+	} else if (last == "-romtype") {
+		for (auto& s : RomInfo::getAllRomTypes()) {
+			cout << s << '\n';
+		}
+	} else {
+		for (auto& p : parser.options) {
+			cout << p.first << '\n';
+		}
+	}
+	parser.parseStatus = CommandLineParser::EXIT;
+}
+
+string_ref BashOption::optionHelp() const
+{
+	return ""; // don't include this option in --help
+}
+
 } // namespace openmsx
