@@ -1,5 +1,3 @@
-// $Id: Connector.cc 12630 2012-06-14 20:17:01Z m9710797 $
-
 #include "Connector.hh"
 #include "Pluggable.hh"
 #include "PluggingController.hh"
@@ -9,10 +7,10 @@
 namespace openmsx {
 
 Connector::Connector(PluggingController& pluggingController_,
-                     string_ref name_, std::auto_ptr<Pluggable> dummy_)
+                     string_ref name_, std::unique_ptr<Pluggable> dummy_)
 	: pluggingController(pluggingController_)
-	, name(name_.data(), name_.size())
-	, dummy(dummy_)
+	, name(name_.str())
+	, dummy(std::move(dummy_))
 {
 	plugged = dummy.get();
 	pluggingController.registerConnector(*this);
