@@ -1,10 +1,9 @@
-// $Id: DebugDevice.cc 12622 2012-06-14 20:11:33Z m9710797 $
-
 #include "DebugDevice.hh"
 #include "Clock.hh"
 #include "FileOperations.hh"
 #include "FilenameSetting.hh"
 #include "serialize.hh"
+#include "memory.hh"
 #include <iostream>
 #include <iomanip>
 
@@ -15,11 +14,11 @@ namespace openmsx {
 DebugDevice::DebugDevice(const DeviceConfig& config)
 	: MSXDevice(config)
 {
-	string_ref outputFile = config.getChildData("filename", "stdout");
-	fileNameSetting.reset(new FilenameSetting(
+	const auto& outputFile = config.getChildData("filename", "stdout");
+	fileNameSetting = make_unique<FilenameSetting>(
 		getCommandController(), "debugoutput",
-		"name of the file the debugdevice outputs to", outputFile.str()));
-	openOutput(fileNameSetting->getValueString());
+		"name of the file the debugdevice outputs to", outputFile);
+	openOutput(fileNameSetting->getString());
 	reset(EmuTime::dummy());
 }
 
@@ -35,7 +34,7 @@ void DebugDevice::reset(EmuTime::param /*time*/)
 
 void DebugDevice::writeIO(word port, byte value, EmuTime::param time)
 {
-	string newName = fileNameSetting->getValueString();
+	const auto& newName = fileNameSetting->getString();
 	if (newName != fileNameString) {
 		openOutput(newName);
 	}
