@@ -1,13 +1,11 @@
-// $Id: EmuDuration.cc 8041 2008-07-07 12:43:20Z m9710797 $
-
 #include "EmuDuration.hh"
 #include "serialize.hh"
 #include <limits>
 
 namespace openmsx {
 
-const EmuDuration EmuDuration::zero(uint64(0));
-const EmuDuration EmuDuration::infinity(std::numeric_limits<uint64>::max());
+const EmuDuration EmuDuration::zero(uint64_t(0));
+const EmuDuration EmuDuration::infinity(std::numeric_limits<uint64_t>::max());
 
 template<typename Archive>
 void EmuDuration::serialize(Archive& ar, unsigned /*version*/)
