@@ -1,5 +1,3 @@
-// $Id: EmuTime.cc 8352 2008-11-12 18:39:08Z m9710797 $
-
 #include "EmuTime.hh"
 #include "serialize.hh"
 #include <limits>
@@ -7,8 +5,8 @@
 
 namespace openmsx {
 
-const EmuTime EmuTime::zero(uint64(0));
-const EmuTime EmuTime::infinity(std::numeric_limits<uint64>::max());
+const EmuTime EmuTime::zero(uint64_t(0));
+const EmuTime EmuTime::infinity(std::numeric_limits<uint64_t>::max());
 
 std::ostream& operator<<(std::ostream& os, EmuTime::param et)
 {
