@@ -1,5 +1,3 @@
-// $Id: FirmwareSwitch.cc 12528 2012-05-17 17:36:10Z m9710797 $
-
 #include "FirmwareSwitch.hh"
 #include "BooleanSetting.hh"
 #include "CliComm.hh"
@@ -7,19 +5,20 @@
 #include "FileContext.hh"
 #include "File.hh"
 #include "FileException.hh"
+#include "memory.hh"
 
 using std::string;
 
 namespace openmsx {
 
-static const std::string filename = "firmwareswitch";
+static const char* const filename = "firmwareswitch";
 
 FirmwareSwitch::FirmwareSwitch(const DeviceConfig& config_)
 	: config(config_)
-	, setting(new BooleanSetting(config.getCommandController(),
-	          "firmwareswitch",
-	          "This setting controls the firmware switch",
-	          false, Setting::DONT_SAVE))
+	, setting(make_unique<BooleanSetting>(
+		config.getCommandController(), "firmwareswitch",
+		"This setting controls the firmware switch",
+		false, Setting::DONT_SAVE))
 {
 	// load firmware switch setting from persistent data
 	try {
@@ -27,7 +26,7 @@ FirmwareSwitch::FirmwareSwitch(const DeviceConfig& config_)
 		          File::LOAD_PERSISTENT);
 		byte bytebuf;
 		file.read(&bytebuf, 1);
-		setting->changeValue(bytebuf != 0);
+		setting->setBoolean(bytebuf != 0);
 	} catch (FileException& e) {
 		config.getCliComm().printWarning(
 			"Couldn't load firmwareswitch status: " + e.getMessage());
@@ -40,7 +39,7 @@ FirmwareSwitch::~FirmwareSwitch()
 	try {
 		File file(config.getFileContext().resolveCreate(filename),
 		          File::SAVE_PERSISTENT);
-		byte bytebuf = setting->getValue() ? 0xFF : 0x00;
+		byte bytebuf = setting->getBoolean() ? 0xFF : 0x00;
 		file.write(&bytebuf, 1);
 	} catch (FileException& e) {
 		config.getCliComm().printWarning(
@@ -50,7 +49,7 @@ FirmwareSwitch::~FirmwareSwitch()
 
 bool FirmwareSwitch::getStatus() const
 {
-	return setting->getValue();
+	return setting->getBoolean();
 }
 
 } // namespace openmsx
