@@ -1,12 +1,10 @@
-// $Id: LedStatus.cc 12508 2012-05-12 18:24:39Z m9710797 $
-
 #include "LedStatus.hh"
 #include "Reactor.hh"
 #include "AlarmEvent.hh"
 #include "MSXCliComm.hh"
-#include "BooleanSetting.hh"
 #include "ReadOnlySetting.hh"
 #include "Timer.hh"
+#include "memory.hh"
 
 namespace openmsx {
 
@@ -23,17 +21,16 @@ LedStatus::LedStatus(
 		CommandController& commandController,
 		MSXCliComm& msxCliComm_)
 	: msxCliComm(msxCliComm_)
-	, alarm(new AlarmEvent(eventDistributor, *this, OPENMSX_THROTTLE_LED_EVENT))
+	, alarm(make_unique<AlarmEvent>(
+		eventDistributor, *this, OPENMSX_THROTTLE_LED_EVENT))
 {
 	lastTime = Timer::getTime();
 	for (int i = 0; i < NUM_LEDS; ++i) {
 		ledValue[i] = false;
 		std::string name = getLedName(static_cast<Led>(i));
-		ledStatus[i].reset(new ReadOnlySetting<BooleanSetting>(
-			commandController,
-			"led_" + name,
-			"Current status for LED: " + name,
-			ledValue[i]));
+		ledStatus[i] = make_unique<ReadOnlySetting>(
+			commandController, "led_" + name,
+			"Current status for LED: " + name, "off");
 	}
 }
 
@@ -50,8 +47,8 @@ void LedStatus::setLed(Led led, bool status)
 	// the LEDs as a VU meter while playing samples). Without throttling
 	// all these events overload the host CPU. That's why we limit it to
 	// 100 events per second.
-	unsigned long long now = Timer::getTime();
-	unsigned long long diff = now - lastTime;
+	auto now = Timer::getTime();
+	auto diff = now - lastTime;
 	if (diff > 10000) { // 1/100 s
 		// handle now
 		lastTime = now;
@@ -66,20 +63,19 @@ void LedStatus::setLed(Led led, bool status)
 
 void LedStatus::handleEvent(Led led)
 {
-	ledStatus[led]->setReadOnlyValue(ledValue[led]);
-
 	static const std::string ON  = "on";
 	static const std::string OFF = "off";
-	msxCliComm.update(
-		CliComm::LED, getLedName(led),
-		ledValue[led] ? ON : OFF);
+	const std::string& str = ledValue[led] ? ON : OFF;
+
+	ledStatus[led]->setReadOnlyValue(str);
+	msxCliComm.update(CliComm::LED, getLedName(led), str);
 }
 
-int LedStatus::signalEvent(const shared_ptr<const Event>& /*event*/)
+int LedStatus::signalEvent(const std::shared_ptr<const Event>& /*event*/)
 {
 	// Runs in main thread.
 	for (int i = 0; i < NUM_LEDS; ++i) {
-		if (ledValue[i] != ledStatus[i]->getValue()) {
+		if (ledValue[i] != ledStatus[i]->getValue().getBoolean()) {
 			handleEvent(static_cast<Led>(i));
 		}
 	}
