@@ -1,5 +1,3 @@
-// $Id: MSXDeviceSwitch.cc 12528 2012-05-17 17:36:10Z m9710797 $
-
 #include "MSXDeviceSwitch.hh"
 #include "MSXSwitchedDevice.hh"
 #include "MSXCPUInterface.hh"
@@ -14,7 +12,7 @@ MSXDeviceSwitch::MSXDeviceSwitch(const DeviceConfig& config)
 	: MSXDevice(config)
 {
 	for (int i = 0; i < 256; ++i) {
-		devices[i] = NULL;
+		devices[i] = nullptr;
 	}
 	count = 0;
 	selected = 0;
@@ -24,7 +22,7 @@ MSXDeviceSwitch::~MSXDeviceSwitch()
 {
 	for (int i = 0; i < 256; ++i) {
 		// all devices must be unregistered
-		assert(devices[i] == NULL);
+		assert(!devices[i]);
 	}
 	assert(count == 0);
 }
@@ -55,8 +53,8 @@ void MSXDeviceSwitch::unregisterDevice(byte id)
 			getCPUInterface().unregister_IO_In (port, this);
 		}
 	}
-	assert(devices[id] != NULL);
-	devices[id] = NULL;
+	assert(devices[id]);
+	devices[id] = nullptr;
 }
 
 bool MSXDeviceSwitch::hasRegisteredDevices() const
