@@ -1,5 +1,3 @@
-// $Id: MSXMotherBoard.cc 12824 2012-08-20 20:08:30Z m9710797 $
-
 #include "MSXMotherBoard.hh"
 #include "Reactor.hh"
 #include "MSXDevice.hh"
@@ -34,6 +32,7 @@
 #include "DeviceFactory.hh"
 #include "BooleanSetting.hh"
 #include "GlobalSettings.hh"
+#include "VideoSourceSetting.hh"
 #include "Command.hh"
 #include "CommandException.hh"
 #include "RecordedCommand.hh"
@@ -47,15 +46,17 @@
 #include "serialize_stl.hh"
 #include "ScopedAssign.hh"
 #include "unreachable.hh"
-#include "ref.hh"
+#include "memory.hh"
 #include <cassert>
+#include <functional>
 #include <vector>
+#include <set>
 #include <iostream>
 
 using std::set;
 using std::string;
 using std::vector;
-using std::auto_ptr;
+using std::unique_ptr;
 
 namespace openmsx {
 
@@ -97,12 +98,12 @@ public:
 	bool isTurboR() const;
 	string loadMachine(MSXMotherBoard& self, const string& machine);
 
-	typedef std::vector<HardwareConfig*> Extensions;
+	typedef std::vector<std::unique_ptr<HardwareConfig>> Extensions;
 	const Extensions& getExtensions() const;
 	HardwareConfig* findExtension(string_ref extensionName);
 	string loadExtension(MSXMotherBoard& self, const string& extensionName);
 	string insertExtension(const std::string& name,
-	                       std::auto_ptr<HardwareConfig> extension);
+	                       unique_ptr<HardwareConfig> extension);
 	void removeExtension(const HardwareConfig& extension);
 
 	CliComm& getMSXCliComm();
@@ -125,6 +126,7 @@ public:
 	LedStatus& getLedStatus();
 	ReverseManager& getReverseManager();
 	Reactor& getReactor();
+	VideoSourceSetting& getVideoSource();
 	CommandController& getCommandController();
 	InfoCommand& getMachineInfoCommand();
 	EmuTime::param getCurrentTime();
@@ -154,14 +156,12 @@ private:
 	string machineID;
 	string machineName;
 
-	typedef vector<MSXDevice*> Devices;
-	Devices availableDevices; // no ownership
+	vector<MSXDevice*> availableDevices; // no ownership
 
-	typedef StringMap<MSXMotherBoard::SharedStuff> SharedStuffMap;
-	SharedStuffMap sharedStuffMap;
-	StringMap<set<string> > userNames;
+	StringMap<MSXMotherBoard::SharedStuff> sharedStuffMap;
+	StringMap<set<string>> userNames;
 
-	auto_ptr<MSXMapperIO> mapperIO;
+	unique_ptr<MSXMapperIO> mapperIO;
 	unsigned mapperIOCounter;
 
 	// These two should normally be the same, only during savestate loading
@@ -169,45 +169,46 @@ private:
 	// This is important when an exception happens during loading of
 	// machineConfig2 (otherwise machineConfig2 gets deleted twice).
 	// See also HardwareConfig::serialize() and setMachineConfig()
-	auto_ptr<HardwareConfig> machineConfig2;
+	unique_ptr<HardwareConfig> machineConfig2;
 	HardwareConfig* machineConfig;
 
 	Extensions extensions;
 
-	// order of auto_ptr's is important!
-	auto_ptr<AddRemoveUpdate> addRemoveUpdate;
-	auto_ptr<MSXCliComm> msxCliComm;
-	auto_ptr<MSXEventDistributor> msxEventDistributor;
-	auto_ptr<StateChangeDistributor> stateChangeDistributor;
-	auto_ptr<MSXCommandController> msxCommandController;
-	auto_ptr<Scheduler> scheduler;
-	auto_ptr<EventDelay> eventDelay;
-	auto_ptr<RealTime> realTime;
-	auto_ptr<Debugger> debugger;
-	auto_ptr<MSXMixer> msxMixer;
-	auto_ptr<PluggingController> pluggingController;
-	auto_ptr<MSXCPU> msxCpu;
-	auto_ptr<MSXCPUInterface> msxCpuInterface;
-	auto_ptr<PanasonicMemory> panasonicMemory;
-	auto_ptr<MSXDeviceSwitch> deviceSwitch;
-	auto_ptr<CassettePortInterface> cassettePort;
-	auto_ptr<JoystickPortIf> joystickPort[2];
-	auto_ptr<JoyPortDebuggable> joyPortDebuggable;
-	auto_ptr<RenShaTurbo> renShaTurbo;
-	auto_ptr<LedStatus> ledStatus;
+	// order of unique_ptr's is important!
+	unique_ptr<AddRemoveUpdate> addRemoveUpdate;
+	unique_ptr<MSXCliComm> msxCliComm;
+	unique_ptr<MSXEventDistributor> msxEventDistributor;
+	unique_ptr<StateChangeDistributor> stateChangeDistributor;
+	unique_ptr<MSXCommandController> msxCommandController;
+	unique_ptr<Scheduler> scheduler;
+	unique_ptr<EventDelay> eventDelay;
+	unique_ptr<RealTime> realTime;
+	unique_ptr<Debugger> debugger;
+	unique_ptr<MSXMixer> msxMixer;
+	unique_ptr<PluggingController> pluggingController;
+	unique_ptr<MSXCPU> msxCpu;
+	unique_ptr<MSXCPUInterface> msxCpuInterface;
+	unique_ptr<PanasonicMemory> panasonicMemory;
+	unique_ptr<MSXDeviceSwitch> deviceSwitch;
+	unique_ptr<CassettePortInterface> cassettePort;
+	unique_ptr<JoystickPortIf> joystickPort[2];
+	unique_ptr<JoyPortDebuggable> joyPortDebuggable;
+	unique_ptr<RenShaTurbo> renShaTurbo;
+	unique_ptr<LedStatus> ledStatus;
+	unique_ptr<VideoSourceSetting> videoSourceSetting;
 
-	auto_ptr<CartridgeSlotManager> slotManager;
-	auto_ptr<ReverseManager> reverseManager;
-	auto_ptr<ResetCmd>     resetCommand;
-	auto_ptr<LoadMachineCmd> loadMachineCommand;
-	auto_ptr<ListExtCmd>   listExtCommand;
-	auto_ptr<ExtCmd>       extCommand;
-	auto_ptr<RemoveExtCmd> removeExtCommand;
-	auto_ptr<MachineNameInfo> machineNameInfo;
-	auto_ptr<DeviceInfo>   deviceInfo;
+	unique_ptr<CartridgeSlotManager> slotManager;
+	unique_ptr<ReverseManager> reverseManager;
+	unique_ptr<ResetCmd>     resetCommand;
+	unique_ptr<LoadMachineCmd> loadMachineCommand;
+	unique_ptr<ListExtCmd>   listExtCommand;
+	unique_ptr<ExtCmd>       extCommand;
+	unique_ptr<RemoveExtCmd> removeExtCommand;
+	unique_ptr<MachineNameInfo> machineNameInfo;
+	unique_ptr<DeviceInfo>   deviceInfo;
 	friend class DeviceInfo;
 
-	auto_ptr<FastForwardHelper> fastForwardHelper;
+	unique_ptr<FastForwardHelper> fastForwardHelper;
 
 	BooleanSetting& powerSetting;
 
@@ -332,49 +333,61 @@ MSXMotherBoard::Impl::Impl(
 	: reactor(reactor_)
 	, machineID(StringOp::Builder() << "machine" << ++machineIDCounter)
 	, mapperIOCounter(0)
-	, machineConfig(NULL)
-	, msxCliComm(new MSXCliComm(self, reactor.getGlobalCliComm()))
-	, msxEventDistributor(new MSXEventDistributor())
-	, stateChangeDistributor(new StateChangeDistributor())
-	, msxCommandController(new MSXCommandController(
+	, machineConfig(nullptr)
+	, msxCliComm(make_unique<MSXCliComm>(self, reactor.getGlobalCliComm()))
+	, msxEventDistributor(make_unique<MSXEventDistributor>())
+	, stateChangeDistributor(make_unique<StateChangeDistributor>())
+	, msxCommandController(make_unique<MSXCommandController>(
 		reactor.getGlobalCommandController(), reactor,
 		self, *msxEventDistributor, machineID))
-	, scheduler(new Scheduler())
-	, msxMixer(new MSXMixer(
+	, scheduler(make_unique<Scheduler>())
+	, msxMixer(make_unique<MSXMixer>(
 		reactor.getMixer(), *scheduler, *msxCommandController,
 		reactor.getGlobalSettings()))
-	, fastForwardHelper(new FastForwardHelper(*this))
+	, fastForwardHelper(make_unique<FastForwardHelper>(*this))
 	, powerSetting(reactor.getGlobalSettings().getPowerSetting())
 	, powered(false)
 	, active(false)
 	, fastForwarding(false)
 {
+#if UNIQUE_PTR_BUG
+	self.pimpl2.reset(this);
+	self.pimpl = self.pimpl2.get();
+#else
 	self.pimpl.reset(this);
+#endif
 
-	slotManager.reset(new CartridgeSlotManager(self));
-	reverseManager.reset(new ReverseManager(self));
-	resetCommand.reset(new ResetCmd(*this));
-	loadMachineCommand.reset(new LoadMachineCmd(self));
-	listExtCommand.reset(new ListExtCmd(*this));
-	extCommand.reset(new ExtCmd(self));
-	removeExtCommand.reset(new RemoveExtCmd(*this));
-	machineNameInfo.reset(new MachineNameInfo(*this));
-	deviceInfo.reset(new DeviceInfo(*this));
+	slotManager = make_unique<CartridgeSlotManager>(self);
+	reverseManager = make_unique<ReverseManager>(self);
+	resetCommand = make_unique<ResetCmd>(*this);
+	loadMachineCommand = make_unique<LoadMachineCmd>(self);
+	listExtCommand = make_unique<ListExtCmd>(*this);
+	extCommand = make_unique<ExtCmd>(self);
+	removeExtCommand = make_unique<RemoveExtCmd>(*this);
+	machineNameInfo = make_unique<MachineNameInfo>(*this);
+	deviceInfo = make_unique<DeviceInfo>(*this);
+	debugger = make_unique<Debugger>(self);
 
 	msxMixer->mute(); // powered down
+
+	// Do this before machine-specific settings are created, otherwise
+	// a setting-info clicomm message is send with a machine id that hasn't
+	// been announced yet over clicomm.
+	addRemoveUpdate = make_unique<AddRemoveUpdate>(*this);
+
 	// TODO: Initialization of this field cannot be done much earlier because
 	//       EventDelay creates a setting, calling getMSXCliComm()
 	//       on MSXMotherBoard, so "pimpl" has to be set up already.
-	eventDelay.reset(new EventDelay(
+	eventDelay = make_unique<EventDelay>(
 		*scheduler, *msxCommandController,
 		reactor.getEventDistributor(), *msxEventDistributor,
-		*reverseManager));
-	realTime.reset(new RealTime(
-		self, reactor.getGlobalSettings(), *eventDelay));
-	debugger.reset(new Debugger(self));
-	powerSetting.attach(*this);
+		*reverseManager);
+	videoSourceSetting = make_unique<VideoSourceSetting>(
+		*msxCommandController);
+	realTime = make_unique<RealTime>(
+		self, reactor.getGlobalSettings(), *eventDelay);
 
-	addRemoveUpdate.reset(new AddRemoveUpdate(*this));
+	powerSetting.attach(*this);
 }
 
 MSXMotherBoard::Impl::~Impl()
@@ -385,7 +398,7 @@ MSXMotherBoard::Impl::~Impl()
 	assert(mapperIOCounter == 0);
 	assert(availableDevices.empty());
 	assert(extensions.empty());
-	assert(!machineConfig2.get());
+	assert(!machineConfig2);
 	assert(!getMachineConfig());
 }
 
@@ -403,7 +416,7 @@ void MSXMotherBoard::Impl::deleteMachine()
 	}
 
 	machineConfig2.reset();
-	machineConfig = NULL;
+	machineConfig = nullptr;
 }
 
 const string& MSXMotherBoard::Impl::getMachineID()
@@ -428,23 +441,23 @@ void MSXMotherBoard::Impl::setMachineConfig(
 	machineConfig = machineConfig_;
 
 	// make sure the CPU gets instantiated from the main thread
-	assert(!msxCpu.get());
-	msxCpu.reset(new MSXCPU(self));
-	msxCpuInterface.reset(new MSXCPUInterface(self));
+	assert(!msxCpu);
+	msxCpu = make_unique<MSXCPU>(self);
+	msxCpuInterface = make_unique<MSXCPUInterface>(self);
 }
 
 bool MSXMotherBoard::Impl::isTurboR() const
 {
 	const HardwareConfig* config = getMachineConfig();
 	assert(config);
-	return config->getConfig().getChild("devices").findChild("S1990") != NULL;
+	return config->getConfig().getChild("devices").findChild("S1990") != nullptr;
 }
 
 string MSXMotherBoard::Impl::loadMachine(MSXMotherBoard& self, const string& machine)
 {
 	assert(machineName.empty());
 	assert(extensions.empty());
-	assert(!machineConfig2.get());
+	assert(!machineConfig2);
 	assert(!getMachineConfig());
 
 	try {
@@ -464,7 +477,7 @@ string MSXMotherBoard::Impl::loadMachine(MSXMotherBoard& self, const string& mac
 		throw MSXException("Error in \"" + machine + "\" machine: " +
 		                   e.getMessage());
 	}
-	if (powerSetting.getValue()) {
+	if (powerSetting.getBoolean()) {
 		powerUp();
 	}
 	machineName = machine;
@@ -473,7 +486,7 @@ string MSXMotherBoard::Impl::loadMachine(MSXMotherBoard& self, const string& mac
 
 string MSXMotherBoard::Impl::loadExtension(MSXMotherBoard& self, const string& name)
 {
-	auto_ptr<HardwareConfig> extension;
+	unique_ptr<HardwareConfig> extension;
 	try {
 		extension = HardwareConfig::createExtensionConfig(self, name);
 	} catch (FileException& e) {
@@ -483,11 +496,11 @@ string MSXMotherBoard::Impl::loadExtension(MSXMotherBoard& self, const string& n
 		throw MSXException(
 			"Error in \"" + name + "\" extension: " + e.getMessage());
 	}
-	return insertExtension(name, extension);
+	return insertExtension(name, std::move(extension));
 }
 
 string MSXMotherBoard::Impl::insertExtension(
-	const string& name, auto_ptr<HardwareConfig> extension)
+	const string& name, unique_ptr<HardwareConfig> extension)
 {
 	try {
 		extension->parseSlots();
@@ -497,20 +510,17 @@ string MSXMotherBoard::Impl::insertExtension(
 			"Error in \"" + name + "\" extension: " + e.getMessage());
 	}
 	string result = extension->getName();
-	extensions.push_back(extension.release());
+	extensions.push_back(std::move(extension));
 	getMSXCliComm().update(CliComm::EXTENSION, result, "add");
 	return result;
 }
 
 HardwareConfig* MSXMotherBoard::Impl::findExtension(string_ref extensionName)
 {
-	for (Extensions::const_iterator it = extensions.begin();
-	     it != extensions.end(); ++it) {
-		if ((*it)->getName() == extensionName) {
-			return *it;
-		}
-	}
-	return NULL;
+	auto it = std::find_if(extensions.begin(), extensions.end(),
+		[&](Extensions::value_type& v) {
+			return v->getName() == extensionName; });
+	return (it != extensions.end()) ? it->get() : nullptr;
 }
 
 const MSXMotherBoard::Impl::Extensions& MSXMotherBoard::Impl::getExtensions() const
@@ -521,11 +531,11 @@ const MSXMotherBoard::Impl::Extensions& MSXMotherBoard::Impl::getExtensions() co
 void MSXMotherBoard::Impl::removeExtension(const HardwareConfig& extension)
 {
 	extension.testRemove();
-	Extensions::iterator it =
-		find(extensions.begin(), extensions.end(), &extension);
+	auto it = std::find_if(extensions.begin(), extensions.end(),
+		[&](Extensions::value_type& v) {
+			return v.get() == &extension; });
 	assert(it != extensions.end());
 	getMSXCliComm().update(CliComm::EXTENSION, extension.getName(), "remove");
-	delete &extension;
 	extensions.erase(it);
 }
 
@@ -577,8 +587,8 @@ MSXMixer& MSXMotherBoard::Impl::getMSXMixer()
 PluggingController& MSXMotherBoard::Impl::getPluggingController(MSXMotherBoard& self)
 {
 	assert(getMachineConfig()); // needed for PluggableFactory::createAll()
-	if (!pluggingController.get()) {
-		pluggingController.reset(new PluggingController(self));
+	if (!pluggingController) {
+		pluggingController = make_unique<PluggingController>(self);
 	}
 	return *pluggingController;
 }
@@ -598,15 +608,15 @@ MSXCPUInterface& MSXMotherBoard::Impl::getCPUInterface()
 
 PanasonicMemory& MSXMotherBoard::Impl::getPanasonicMemory(MSXMotherBoard& self)
 {
-	if (!panasonicMemory.get()) {
-		panasonicMemory.reset(new PanasonicMemory(self));
+	if (!panasonicMemory) {
+		panasonicMemory = make_unique<PanasonicMemory>(self);
 	}
 	return *panasonicMemory;
 }
 
 MSXDeviceSwitch& MSXMotherBoard::Impl::getDeviceSwitch()
 {
-	if (!deviceSwitch.get()) {
+	if (!deviceSwitch) {
 		deviceSwitch = DeviceFactory::createDeviceSwitch(*getMachineConfig());
 	}
 	return *deviceSwitch;
@@ -614,12 +624,12 @@ MSXDeviceSwitch& MSXMotherBoard::Impl::getDeviceSwitch()
 
 CassettePortInterface& MSXMotherBoard::Impl::getCassettePort()
 {
-	if (!cassettePort.get()) {
+	if (!cassettePort) {
 		assert(getMachineConfig());
 		if (getMachineConfig()->getConfig().findChild("CassettePort")) {
-			cassettePort.reset(new CassettePort(*getMachineConfig()));
+			cassettePort = make_unique<CassettePort>(*getMachineConfig());
 		} else {
-			cassettePort.reset(new DummyCassettePort());
+			cassettePort = make_unique<DummyCassettePort>();
 		}
 	}
 	return *cassettePort;
@@ -629,7 +639,7 @@ JoystickPortIf& MSXMotherBoard::Impl::getJoystickPort(
 	unsigned port, MSXMotherBoard& self)
 {
 	assert(port < 2);
-	if (!joystickPort[0].get()) {
+	if (!joystickPort[0]) {
 		assert(getMachineConfig());
 		// some MSX machines only have 1 instead of 2 joystick ports
 		string_ref ports = getMachineConfig()->getConfig().getChildData(
@@ -642,41 +652,41 @@ JoystickPortIf& MSXMotherBoard::Impl::getJoystickPort(
 		}
 		PluggingController& ctrl = getPluggingController(self);
 		if ((ports == "AB") || (ports == "A")) {
-			joystickPort[0].reset(new JoystickPort(
-				ctrl, "joyporta", "MSX Joystick port A"));
+			joystickPort[0] = make_unique<JoystickPort>(
+				ctrl, "joyporta", "MSX Joystick port A");
 		} else {
-			joystickPort[0].reset(new DummyJoystickPort());
+			joystickPort[0] = make_unique<DummyJoystickPort>();
 		}
 		if ((ports == "AB") || (ports == "B")) {
-			joystickPort[1].reset(new JoystickPort(
-				ctrl, "joyportb", "MSX Joystick port B"));
+			joystickPort[1] = make_unique<JoystickPort>(
+				ctrl, "joyportb", "MSX Joystick port B");
 		} else {
-			joystickPort[1].reset(new DummyJoystickPort());
+			joystickPort[1] = make_unique<DummyJoystickPort>();
 		}
-		joyPortDebuggable.reset(new JoyPortDebuggable(self));
+		joyPortDebuggable = make_unique<JoyPortDebuggable>(self);
 	}
 	return *joystickPort[port];
 }
 
 RenShaTurbo& MSXMotherBoard::Impl::getRenShaTurbo()
 {
-	if (!renShaTurbo.get()) {
+	if (!renShaTurbo) {
 		assert(getMachineConfig());
-		renShaTurbo.reset(new RenShaTurbo(
+		renShaTurbo = make_unique<RenShaTurbo>(
 			*msxCommandController,
-			getMachineConfig()->getConfig()));
+			getMachineConfig()->getConfig());
 	}
 	return *renShaTurbo;
 }
 
 LedStatus& MSXMotherBoard::Impl::getLedStatus()
 {
-	if (!ledStatus.get()) {
+	if (!ledStatus) {
 		getMSXCliComm(); // force init, to be on the safe side
-		ledStatus.reset(new LedStatus(
+		ledStatus = make_unique<LedStatus>(
 			reactor.getEventDistributor(),
 			*msxCommandController,
-			*msxCliComm));
+			*msxCliComm);
 	}
 	return *ledStatus;
 }
@@ -691,6 +701,11 @@ Reactor& MSXMotherBoard::Impl::getReactor()
 	return reactor;
 }
 
+VideoSourceSetting& MSXMotherBoard::Impl::getVideoSource()
+{
+	return *videoSourceSetting;
+}
+
 CommandController& MSXMotherBoard::Impl::getCommandController()
 {
 	return *msxCommandController;
@@ -759,8 +774,8 @@ void MSXMotherBoard::Impl::addDevice(MSXDevice& device)
 
 void MSXMotherBoard::Impl::removeDevice(MSXDevice& device)
 {
-	Devices::iterator it = find(availableDevices.begin(),
-	                            availableDevices.end(), &device);
+	auto it = find(availableDevices.begin(), availableDevices.end(),
+	               &device);
 	assert(it != availableDevices.end());
 	availableDevices.erase(it);
 }
@@ -772,23 +787,21 @@ void MSXMotherBoard::Impl::doReset()
 
 	EmuTime::param time = getCurrentTime();
 	getCPUInterface().reset();
-	for (Devices::iterator it = availableDevices.begin();
-	     it != availableDevices.end(); ++it) {
-		(*it)->reset(time);
+	for (auto& d : availableDevices) {
+		d->reset(time);
 	}
 	getCPU().doReset(time);
 	// let everyone know we're booting, note that the fact that this is
 	// done after the reset call to the devices is arbitrary here
 	reactor.getEventDistributor().distributeEvent(
-		new SimpleEvent(OPENMSX_BOOT_EVENT));
+		std::make_shared<SimpleEvent>(OPENMSX_BOOT_EVENT));
 }
 
 byte MSXMotherBoard::Impl::readIRQVector()
 {
 	byte result = 0xff;
-	for (Devices::iterator it = availableDevices.begin();
-	     it != availableDevices.end(); ++it) {
-		result &= (*it)->readIRQVector();
+	for (auto& d : availableDevices) {
+		result &= d->readIRQVector();
 	}
 	return result;
 }
@@ -803,24 +816,23 @@ void MSXMotherBoard::Impl::powerUp()
 	//       there is not really a point in keeping it.
 	// TODO: assert disabled see note in Reactor::run() where this method
 	//       is called
-	//assert(powerSetting.getValue() == powered);
-	powerSetting.changeValue(true);
+	//assert(powerSetting.getBoolean() == powered);
+	powerSetting.setBoolean(true);
 	// TODO: We could make the power LED a device, so we don't have to handle
 	//       it separately here.
 	getLedStatus().setLed(LedStatus::POWER, true);
 
 	EmuTime::param time = getCurrentTime();
 	getCPUInterface().reset();
-	for (Devices::iterator it = availableDevices.begin();
-	     it != availableDevices.end(); ++it) {
-		(*it)->powerUp(time);
+	for (auto& d : availableDevices) {
+		d->powerUp(time);
 	}
 	getCPU().doReset(time);
 	msxMixer->unmute();
 	// let everyone know we're booting, note that the fact that this is
 	// done after the reset call to the devices is arbitrary here
 	reactor.getEventDistributor().distributeEvent(
-		new SimpleEvent(OPENMSX_BOOT_EVENT));
+		std::make_shared<SimpleEvent>(OPENMSX_BOOT_EVENT));
 }
 
 void MSXMotherBoard::Impl::powerDown()
@@ -831,25 +843,23 @@ void MSXMotherBoard::Impl::powerDown()
 	// TODO: This assertion fails in 1 case: when quitting with a running MSX.
 	//       How do we want the Reactor to shutdown: immediately or after
 	//       handling all pending commands/events/updates?
-	//assert(powerSetting.getValue() == powered);
-	powerSetting.changeValue(false);
+	//assert(powerSetting.getBoolean() == powered);
+	powerSetting.setBoolean(false);
 	getLedStatus().setLed(LedStatus::POWER, false);
 
 	msxMixer->mute();
 
 	EmuTime::param time = getCurrentTime();
-	for (Devices::iterator it = availableDevices.begin();
-	     it != availableDevices.end(); ++it) {
-		(*it)->powerDown(time);
+	for (auto& d : availableDevices) {
+		d->powerDown(time);
 	}
 }
 
 void MSXMotherBoard::Impl::activate(bool active_)
 {
 	active = active_;
-	MSXEventDistributor::EventPtr event = active
-		? MSXEventDistributor::EventPtr(new SimpleEvent(OPENMSX_MACHINE_ACTIVATED))
-		: MSXEventDistributor::EventPtr(new SimpleEvent(OPENMSX_MACHINE_DEACTIVATED));
+	auto event = std::make_shared<SimpleEvent>(
+		active ? OPENMSX_MACHINE_ACTIVATED : OPENMSX_MACHINE_DEACTIVATED);
 	msxEventDistributor->distributeEvent(event, scheduler->getCurrentTime());
 	if (active) {
 		realTime->resync();
@@ -880,7 +890,7 @@ void MSXMotherBoard::Impl::exitCPULoopSync()
 void MSXMotherBoard::Impl::update(const Setting& setting)
 {
 	if (&setting == &powerSetting) {
-		if (powerSetting.getValue()) {
+		if (powerSetting.getBoolean()) {
 			powerUp();
 		} else {
 			powerDown();
@@ -892,13 +902,12 @@ void MSXMotherBoard::Impl::update(const Setting& setting)
 
 MSXDevice* MSXMotherBoard::Impl::findDevice(string_ref name)
 {
-	for (Devices::iterator it = availableDevices.begin();
-	     it != availableDevices.end(); ++it) {
-		if ((*it)->getName() == name) {
-			return *it;
+	for (auto& d : availableDevices) {
+		if (d->getName() == name) {
+			return d;
 		}
 	}
-	return NULL;
+	return nullptr;
 }
 
 MSXMotherBoard::SharedStuff& MSXMotherBoard::Impl::getSharedStuff(
@@ -928,7 +937,7 @@ MSXMapperIO* MSXMotherBoard::Impl::createMapperIO()
 
 void MSXMotherBoard::Impl::destroyMapperIO()
 {
-	assert(mapperIO.get());
+	assert(mapperIO);
 	assert(mapperIOCounter);
 	--mapperIOCounter;
 	if (mapperIOCounter == 0) {
@@ -948,7 +957,7 @@ void MSXMotherBoard::Impl::destroyMapperIO()
 
 string MSXMotherBoard::Impl::getUserName(const string& hwName)
 {
-	set<string>& s = userNames[hwName];
+	auto& s = userNames[hwName];
 	unsigned n = 0;
 	string userName;
 	do {
@@ -961,7 +970,7 @@ string MSXMotherBoard::Impl::getUserName(const string& hwName)
 void MSXMotherBoard::Impl::freeUserName(const string& hwName,
                                       const string& userName)
 {
-	set<string>& s = userNames[hwName];
+	auto& s = userNames[hwName];
 	assert(s.find(userName) != s.end());
 	s.erase(userName);
 }
@@ -1055,8 +1064,7 @@ string LoadMachineCmd::help(const vector<string>& /*tokens*/) const
 
 void LoadMachineCmd::tabCompletion(vector<string>& tokens) const
 {
-	set<string> machines;
-	Reactor::getHwConfigs("machines", machines);
+	auto machines = Reactor::getHwConfigs("machines");
 	completeString(tokens, machines);
 }
 
@@ -1071,10 +1079,8 @@ ListExtCmd::ListExtCmd(MSXMotherBoard::Impl& motherBoard_)
 void ListExtCmd::execute(const vector<TclObject>& /*tokens*/,
                          TclObject& result)
 {
-	const MSXMotherBoard::Impl::Extensions& extensions = motherBoard.getExtensions();
-	for (MSXMotherBoard::Impl::Extensions::const_iterator it = extensions.begin();
-	     it != extensions.end(); ++it) {
-		result.addListElement((*it)->getName());
+	for (auto& e : motherBoard.getExtensions()) {
+		result.addListElement(e->getName());
 	}
 }
 
@@ -1113,8 +1119,7 @@ string ExtCmd::help(const vector<string>& /*tokens*/) const
 
 void ExtCmd::tabCompletion(vector<string>& tokens) const
 {
-	set<string> extensions;
-	Reactor::getHwConfigs("extensions", extensions);
+	auto extensions = Reactor::getHwConfigs("extensions");
 	completeString(tokens, extensions);
 }
 
@@ -1155,11 +1160,9 @@ string RemoveExtCmd::help(const vector<string>& /*tokens*/) const
 void RemoveExtCmd::tabCompletion(vector<string>& tokens) const
 {
 	if (tokens.size() == 2) {
-		set<string> names;
-		for (MSXMotherBoard::Impl::Extensions::const_iterator it =
-		         motherBoard.getExtensions().begin();
-		     it != motherBoard.getExtensions().end(); ++it) {
-			names.insert((*it)->getName());
+		vector<string_ref> names;
+		for (auto& e : motherBoard.getExtensions()) {
+			names.push_back(e->getName());
 		}
 		completeString(tokens, names);
 	}
@@ -1199,10 +1202,8 @@ void DeviceInfo::execute(const vector<TclObject>& tokens,
 {
 	switch (tokens.size()) {
 	case 2:
-		for (MSXMotherBoard::Impl::Devices::const_iterator it =
-		        motherBoard.availableDevices.begin();
-		     it != motherBoard.availableDevices.end(); ++it) {
-			result.addListElement((*it)->getName());
+		for (auto& d : motherBoard.availableDevices) {
+			result.addListElement(d->getName());
 		}
 		break;
 	case 3: {
@@ -1229,11 +1230,9 @@ string DeviceInfo::help(const vector<string>& /*tokens*/) const
 void DeviceInfo::tabCompletion(vector<string>& tokens) const
 {
 	if (tokens.size() == 3) {
-		set<string> names;
-		for (MSXMotherBoard::Impl::Devices::const_iterator it =
-		         motherBoard.availableDevices.begin();
-		     it != motherBoard.availableDevices.end(); ++it) {
-			names.insert((*it)->getName());
+		vector<string> names;
+		for (auto& d : motherBoard.availableDevices) {
+			names.push_back(d->getName());
 		}
 		completeString(tokens, names);
 	}
@@ -1300,13 +1299,11 @@ void MSXMotherBoard::Impl::serialize(MSXMotherBoard& self, Archive& ar, unsigned
 	}
 
 	ar.serialize("name", machineName);
-	ar.serializeWithID("config", machineConfig2, ref(self));
+	ar.serializeWithID("config", machineConfig2, std::ref(self));
 	assert(getMachineConfig() == machineConfig2.get());
-	ar.serializeWithID("extensions", extensions, ref(self));
+	ar.serializeWithID("extensions", extensions, std::ref(self));
 
-	if (mapperIO.get()) {
-		ar.serialize("mapperIO", *mapperIO);
-	}
+	if (mapperIO) ar.serialize("mapperIO", *mapperIO);
 
 	MSXDeviceSwitch& devSwitch = getDeviceSwitch();
 	if (devSwitch.hasRegisteredDevices()) {
@@ -1318,23 +1315,23 @@ void MSXMotherBoard::Impl::serialize(MSXMotherBoard& self, Archive& ar, unsigned
 	}
 	ar.serialize("cpuInterface", getCPUInterface());
 
-	if (CassettePort* port = dynamic_cast<CassettePort*>(&getCassettePort())) {
+	if (auto port = dynamic_cast<CassettePort*>(&getCassettePort())) {
 		ar.serialize("cassetteport", *port);
 	}
 	if (ar.versionAtLeast(version, 4)) {
-		if (JoystickPort* port = dynamic_cast<JoystickPort*>(
-		                                      joystickPort[0].get())) {
+		if (auto port = dynamic_cast<JoystickPort*>(
+				joystickPort[0].get())) {
 			ar.serialize("joystickportA", *port);
 		}
-		if (JoystickPort* port = dynamic_cast<JoystickPort*>(
-		                                      joystickPort[1].get())) {
+		if (auto port = dynamic_cast<JoystickPort*>(
+				joystickPort[1].get())) {
 			ar.serialize("joystickportB", *port);
 		}
 	}
 
 	if (ar.isLoader()) {
 		powered = true; // must come before changing power setting
-		powerSetting.changeValue(true);
+		powerSetting.setBoolean(true);
 		getLedStatus().setLed(LedStatus::POWER, true);
 		msxMixer->unmute();
 	}
@@ -1373,6 +1370,10 @@ void MSXMotherBoard::exitCPULoopAsync()
 {
 	pimpl->exitCPULoopAsync();
 }
+void MSXMotherBoard::exitCPULoopSync()
+{
+	pimpl->exitCPULoopSync();
+}
 void MSXMotherBoard::pause()
 {
 	pimpl->pause();
@@ -1426,9 +1427,9 @@ string MSXMotherBoard::loadExtension(const string& extensionName)
 	return pimpl->loadExtension(*this, extensionName);
 }
 string MSXMotherBoard::insertExtension(
-	const string& name, auto_ptr<HardwareConfig> extension)
+	const string& name, unique_ptr<HardwareConfig> extension)
 {
-	return pimpl->insertExtension(name, extension);
+	return pimpl->insertExtension(name, std::move(extension));
 }
 void MSXMotherBoard::removeExtension(const HardwareConfig& extension)
 {
@@ -1515,6 +1516,10 @@ Reactor& MSXMotherBoard::getReactor()
 {
 	return pimpl->getReactor();
 }
+VideoSourceSetting& MSXMotherBoard::getVideoSource()
+{
+	return pimpl->getVideoSource();
+}
 CommandController& MSXMotherBoard::getCommandController()
 {
 	return pimpl->getCommandController();
