@@ -1,5 +1,3 @@
-// $Id: MSXPPI.cc 12622 2012-06-14 20:11:33Z m9710797 $
-
 #include "MSXPPI.hh"
 #include "I8255.hh"
 #include "Keyboard.hh"
@@ -12,13 +10,14 @@
 #include "RenShaTurbo.hh"
 #include "serialize.hh"
 #include "unreachable.hh"
+#include "memory.hh"
 #include <string>
 
 namespace openmsx {
 
 // MSXDevice
 
-static Keyboard* createKeyboard(const DeviceConfig& config)
+static std::unique_ptr<Keyboard> createKeyboard(const DeviceConfig& config)
 {
 	bool keyGhosting = config.getChildDataAsBool("key_ghosting", true);
 	bool keyGhostingSGCprotected =
@@ -29,23 +28,24 @@ static Keyboard* createKeyboard(const DeviceConfig& config)
 	bool codeKanaLocks = config.getChildDataAsBool("code_kana_locks", false);
 	bool graphLocks = config.getChildDataAsBool("graph_locks", false);
 	MSXMotherBoard& motherBoard = config.getMotherBoard();
-	return new Keyboard(motherBoard,
-	                    motherBoard.getScheduler(),
-	                    motherBoard.getCommandController(),
-	                    motherBoard.getReactor().getEventDistributor(),
-	                    motherBoard.getMSXEventDistributor(),
-	                    motherBoard.getStateChangeDistributor(),
-	                    keyboardType, hasKeypad, hasYesNoKeys,
-	                    keyGhosting, keyGhostingSGCprotected,
-	                    codeKanaLocks, graphLocks);
+	return make_unique<Keyboard>(
+		motherBoard,
+		motherBoard.getScheduler(),
+		motherBoard.getCommandController(),
+		motherBoard.getReactor().getEventDistributor(),
+		motherBoard.getMSXEventDistributor(),
+		motherBoard.getStateChangeDistributor(),
+		keyboardType, hasKeypad, hasYesNoKeys,
+		keyGhosting, keyGhostingSGCprotected,
+		codeKanaLocks, graphLocks);
 }
 
 MSXPPI::MSXPPI(const DeviceConfig& config)
 	: MSXDevice(config)
 	, cassettePort(getMotherBoard().getCassettePort())
 	, renshaTurbo(getMotherBoard().getRenShaTurbo())
-	, i8255(new I8255(*this, getCurrentTime(), getCliComm()))
-	, click(new KeyClick(config))
+	, i8255(make_unique<I8255>(*this, getCurrentTime(), getCliComm()))
+	, click(make_unique<KeyClick>(config))
 	, keyboard(createKeyboard(config))
 	, prevBits(15)
 	, selectedRow(0)
