@@ -1,17 +1,17 @@
-// $Id: MSXRTC.cc 12528 2012-05-17 17:36:10Z m9710797 $
-
 #include "MSXRTC.hh"
 #include "SRAM.hh"
 #include "RP5C01.hh"
 #include "serialize.hh"
 #include "unreachable.hh"
+#include "memory.hh"
 
 namespace openmsx {
 
 MSXRTC::MSXRTC(const DeviceConfig& config)
 	: MSXDevice(config)
-	, sram(new SRAM(getName() + " SRAM", 4 * 13, config))
-	, rp5c01(new RP5C01(getCommandController(), *sram, getCurrentTime()))
+	, sram(make_unique<SRAM>(getName() + " SRAM", 4 * 13, config))
+	, rp5c01(make_unique<RP5C01>(
+		getCommandController(), *sram, getCurrentTime(), getName()))
 {
 	reset(getCurrentTime());
 }
@@ -65,4 +65,3 @@ INSTANTIATE_SERIALIZE_METHODS(MSXRTC);
 REGISTER_MSXDEVICE(MSXRTC, "RTC");
 
 } // namespace openmsx
-
