@@ -1,5 +1,3 @@
-// $Id: MSXS1990.cc 12528 2012-05-17 17:36:10Z m9710797 $
-
 #include "MSXS1990.hh"
 #include "MSXCPU.hh"
 #include "MSXMotherBoard.hh"
@@ -8,6 +6,7 @@
 #include "SimpleDebuggable.hh"
 #include "serialize.hh"
 #include "unreachable.hh"
+#include "memory.hh"
 
 namespace openmsx {
 
@@ -24,8 +23,8 @@ private:
 
 MSXS1990::MSXS1990(const DeviceConfig& config)
 	: MSXDevice(config)
-	, firmwareSwitch(new FirmwareSwitch(config))
-	, debuggable(new S1990Debuggable(getMotherBoard(), *this))
+	, firmwareSwitch(make_unique<FirmwareSwitch>(config))
+	, debuggable(make_unique<S1990Debuggable>(getMotherBoard(), *this))
 {
 	reset(EmuTime::dummy());
 }
