@@ -1,5 +1,3 @@
-// $Id: Pluggable.cc 11609 2010-07-22 21:46:22Z m9710797 $
-
 #include "Pluggable.hh"
 #include "PlugException.hh"
 #include "Connector.hh"
@@ -12,7 +10,7 @@ namespace openmsx {
 
 Pluggable::Pluggable()
 {
-	setConnector(NULL);
+	setConnector(nullptr);
 }
 
 Pluggable::~Pluggable()
@@ -44,7 +42,7 @@ void Pluggable::unplug(EmuTime::param time)
 	} catch (MSXException&) {
 		UNREACHABLE;
 	}
-	setConnector(NULL);
+	setConnector(nullptr);
 }
 
 Connector* Pluggable::getConnector() const
