@@ -1,5 +1,3 @@
-// $Id: PluggableFactory.cc 12527 2012-05-17 17:34:11Z m9710797 $
-
 #include "PluggableFactory.hh"
 #include "PluggingController.hh"
 #include "MSXMotherBoard.hh"
@@ -16,6 +14,7 @@
 #include "MidiOutLogger.hh"
 #include "Mouse.hh"
 #include "Trackball.hh"
+#include "Touchpad.hh"
 #include "PrinterPortLogger.hh"
 #include "PrinterPortSimpl.hh"
 #include "Printer.hh"
@@ -28,10 +27,11 @@
 #if defined(__APPLE__)
 #include "MidiOutCoreMIDI.hh"
 #endif
+#include "memory.hh"
 
 namespace openmsx {
 
-using std::auto_ptr;
+using std::unique_ptr;
 
 void PluggableFactory::createAll(PluggingController& controller,
                                  MSXMotherBoard& motherBoard)
@@ -48,64 +48,67 @@ void PluggableFactory::createAll(PluggingController& controller,
 	// TODO: Support hot-plugging of input devices:
 	// - additional key joysticks can be created by the user
 	// - real joysticks and mice can be hotplugged (USB)
-	controller.registerPluggable(auto_ptr<Pluggable>(new ArkanoidPad(
-		msxEventDistributor, stateChangeDistributor)));
-	controller.registerPluggable(auto_ptr<Pluggable>(new Mouse(
-		msxEventDistributor, stateChangeDistributor)));
-	controller.registerPluggable(auto_ptr<Pluggable>(new Trackball(
-		msxEventDistributor, stateChangeDistributor)));
-	controller.registerPluggable(auto_ptr<Pluggable>(new JoyTap(
-		controller, "joytap")));
-	controller.registerPluggable(auto_ptr<Pluggable>(new NinjaTap(
-		controller, "ninjatap")));
-	controller.registerPluggable(auto_ptr<Pluggable>(new KeyJoystick(
+	controller.registerPluggable(make_unique<ArkanoidPad>(
+		msxEventDistributor, stateChangeDistributor));
+	controller.registerPluggable(make_unique<Mouse>(
+		msxEventDistributor, stateChangeDistributor));
+	controller.registerPluggable(make_unique<Trackball>(
+		msxEventDistributor, stateChangeDistributor));
+	controller.registerPluggable(make_unique<Touchpad>(
+		msxEventDistributor, stateChangeDistributor,
+		commandController));
+	controller.registerPluggable(make_unique<JoyTap>(
+		controller, "joytap"));
+	controller.registerPluggable(make_unique<NinjaTap>(
+		controller, "ninjatap"));
+	controller.registerPluggable(make_unique<KeyJoystick>(
 		commandController, msxEventDistributor,
-		stateChangeDistributor, "keyjoystick1")));
-	controller.registerPluggable(auto_ptr<Pluggable>(new KeyJoystick(
+		stateChangeDistributor, "keyjoystick1"));
+	controller.registerPluggable(make_unique<KeyJoystick>(
 		commandController, msxEventDistributor,
-		stateChangeDistributor, "keyjoystick2")));
+		stateChangeDistributor, "keyjoystick2"));
 	Joystick::registerAll(msxEventDistributor, stateChangeDistributor,
-	                      controller);
+	                      commandController, controller);
 	JoyMega::registerAll(msxEventDistributor, stateChangeDistributor,
 	                      controller);
 
 	// Dongles
-	controller.registerPluggable(auto_ptr<Pluggable>(new SETetrisDongle()));
-	controller.registerPluggable(auto_ptr<Pluggable>(new MagicKey()));
+	controller.registerPluggable(make_unique<SETetrisDongle>());
+	controller.registerPluggable(make_unique<MagicKey>());
 
 	// Logging:
-	controller.registerPluggable(auto_ptr<Pluggable>(new PrinterPortLogger(
-		commandController)));
-	controller.registerPluggable(auto_ptr<Pluggable>(new MidiOutLogger(
-		commandController)));
+	controller.registerPluggable(make_unique<PrinterPortLogger>(
+		commandController));
+	controller.registerPluggable(make_unique<MidiOutLogger>(
+		commandController));
 
 	// Serial communication:
-	controller.registerPluggable(auto_ptr<Pluggable>(new RS232Tester(
-		eventDistributor, scheduler, commandController)));
+	controller.registerPluggable(make_unique<RS232Tester>(
+		eventDistributor, scheduler, commandController));
 
 	// Sampled audio:
-	controller.registerPluggable(auto_ptr<Pluggable>(new PrinterPortSimpl(
-		*motherBoard.getMachineConfig())));
-	controller.registerPluggable(auto_ptr<Pluggable>(new WavAudioInput(
-		commandController)));
+	controller.registerPluggable(make_unique<PrinterPortSimpl>(
+		*motherBoard.getMachineConfig()));
+	controller.registerPluggable(make_unique<WavAudioInput>(
+		commandController));
 
 	// MIDI:
-	controller.registerPluggable(auto_ptr<Pluggable>(new MidiInReader(
-		eventDistributor, scheduler, commandController)));
+	controller.registerPluggable(make_unique<MidiInReader>(
+		eventDistributor, scheduler, commandController));
 #if defined(_WIN32)
 	MidiInWindows::registerAll(eventDistributor, scheduler, controller);
 	MidiOutWindows::registerAll(controller);
 #endif
 #if defined(__APPLE__)
-	controller.registerPluggable(auto_ptr<Pluggable>(new MidiOutCoreMIDIVirtual()));
+	controller.registerPluggable(make_unique<MidiOutCoreMIDIVirtual>());
 	MidiOutCoreMIDI::registerAll(controller);
 #endif
 
 	// Printers
-	controller.registerPluggable(auto_ptr<Pluggable>(new ImagePrinterMSX(
-		motherBoard)));
-	controller.registerPluggable(auto_ptr<Pluggable>(new ImagePrinterEpson(
-		motherBoard)));
+	controller.registerPluggable(make_unique<ImagePrinterMSX>(
+		motherBoard));
+	controller.registerPluggable(make_unique<ImagePrinterEpson>(
+		motherBoard));
 }
 
 } // namespace openmsx
