@@ -1,5 +1,3 @@
-// $Id: PluggingController.cc 12805 2012-08-13 19:26:18Z m9710797 $
-
 #include "PluggingController.hh"
 #include "PlugException.hh"
 #include "RecordedCommand.hh"
@@ -12,11 +10,12 @@
 #include "MSXMotherBoard.hh"
 #include "CliComm.hh"
 #include "StringOp.hh"
+#include "memory.hh"
 #include "openmsx.hh"
 #include <cassert>
 #include <iostream>
+#include <set>
 
-using std::set;
 using std::string;
 using std::vector;
 
@@ -91,37 +90,34 @@ private:
 };
 
 
-PluggingController::PluggingController(MSXMotherBoard& motherBoard)
-	: plugCmd  (new PlugCmd  (motherBoard.getCommandController(),
-	                          motherBoard.getStateChangeDistributor(),
-	                          motherBoard.getScheduler(), *this))
-	, unplugCmd(new UnplugCmd(motherBoard.getCommandController(),
-	                          motherBoard.getStateChangeDistributor(),
-	                          motherBoard.getScheduler(), *this))
-	, pluggableInfo(new PluggableInfo(
-		motherBoard.getMachineInfoCommand(), *this))
-	, connectorInfo(new ConnectorInfo(
-		motherBoard.getMachineInfoCommand(), *this))
-	, connectionClassInfo(new ConnectionClassInfo(
-		motherBoard.getMachineInfoCommand(), *this))
-	, cliComm(motherBoard.getMSXCliComm())
+PluggingController::PluggingController(MSXMotherBoard& motherBoard_)
+	: plugCmd(make_unique<PlugCmd>(
+		motherBoard_.getCommandController(),
+		motherBoard_.getStateChangeDistributor(),
+		motherBoard_.getScheduler(), *this))
+	, unplugCmd(make_unique<UnplugCmd>(
+		motherBoard_.getCommandController(),
+		motherBoard_.getStateChangeDistributor(),
+		motherBoard_.getScheduler(), *this))
+	, pluggableInfo(make_unique<PluggableInfo>(
+		motherBoard_.getMachineInfoCommand(), *this))
+	, connectorInfo(make_unique<ConnectorInfo>(
+		motherBoard_.getMachineInfoCommand(), *this))
+	, connectionClassInfo(make_unique<ConnectionClassInfo>(
+		motherBoard_.getMachineInfoCommand(), *this))
+	, motherBoard(motherBoard_)
 {
 	PluggableFactory::createAll(*this, motherBoard);
 }
 
 PluggingController::~PluggingController()
 {
-	for (Pluggables::iterator it = pluggables.begin();
-	     it != pluggables.end(); ++it) {
-		delete *it;
-	}
 #ifndef NDEBUG
 	// This is similar to an assert: it should never print anything,
 	// but if it does, it helps to catch an error.
-	for (Connectors::const_iterator it = connectors.begin();
-	     it != connectors.end(); ++it) {
+	for (auto& c : connectors) {
 		std::cerr << "ERROR: Connector still registered at shutdown: "
-		          << (*it)->getName() << std::endl;
+		          << c->getName() << std::endl;
 	}
 #endif
 }
@@ -129,23 +125,22 @@ PluggingController::~PluggingController()
 void PluggingController::registerConnector(Connector& connector)
 {
 	connectors.push_back(&connector);
-	cliComm.update(CliComm::CONNECTOR, connector.getName(), "add");
+	getCliComm().update(CliComm::CONNECTOR, connector.getName(), "add");
 }
 
 void PluggingController::unregisterConnector(Connector& connector)
 {
-	Connectors::iterator it = find(connectors.begin(), connectors.end(),
-	                               &connector);
+	auto it = find(connectors.begin(), connectors.end(), &connector);
 	assert(it != connectors.end());
 	connectors.erase(it);
 
-	cliComm.update(CliComm::CONNECTOR, connector.getName(), "remove");
+	getCliComm().update(CliComm::CONNECTOR, connector.getName(), "remove");
 }
 
 
-void PluggingController::registerPluggable(std::auto_ptr<Pluggable> pluggable)
+void PluggingController::registerPluggable(std::unique_ptr<Pluggable> pluggable)
 {
-	pluggables.push_back(pluggable.release());
+	pluggables.push_back(std::move(pluggable));
 }
 
 
@@ -167,23 +162,21 @@ string PlugCmd::execute(const vector<string>& tokens, EmuTime::param time)
 	StringOp::Builder result;
 	switch (tokens.size()) {
 	case 1: {
-		for (PluggingController::Connectors::const_iterator it =
-		                       pluggingController.connectors.begin();
-		     it != pluggingController.connectors.end(); ++it) {
-			result << (*it)->getName() << ": "
-			       << (*it)->getPlugged().getName() << '\n';
+		for (auto& c : pluggingController.connectors) {
+			result << c->getName() << ": "
+			       << c->getPlugged().getName() << '\n';
 		}
 		break;
 	}
 	case 2: {
-		Connector& connector = pluggingController.getConnector(tokens[1]);
+		auto& connector = pluggingController.getConnector(tokens[1]);
 		result << connector.getName() << ": "
 		       << connector.getPlugged().getName();
 		break;
 	}
 	case 3: {
-		Connector& connector = pluggingController.getConnector(tokens[1]);
-		Pluggable& pluggable = pluggingController.getPluggable(tokens[2]);
+		auto& connector = pluggingController.getConnector(tokens[1]);
+		auto& pluggable = pluggingController.getPluggable(tokens[2]);
 		if (&connector.getPlugged() == &pluggable) {
 			// already plugged, don't unplug/replug
 			break;
@@ -195,7 +188,7 @@ string PlugCmd::execute(const vector<string>& tokens, EmuTime::param time)
 		connector.unplug(time);
 		try {
 			connector.plug(pluggable, time);
-			pluggingController.cliComm.update(
+			pluggingController.getCliComm().update(
 				CliComm::PLUG, tokens[1], tokens[2]);
 		} catch (PlugException& e) {
 			throw CommandException("plug: plug failed: " + e.getMessage());
@@ -218,24 +211,19 @@ void PlugCmd::tabCompletion(vector<string>& tokens) const
 {
 	if (tokens.size() == 2) {
 		// complete connector
-		set<string> connectors;
-		for (PluggingController::Connectors::const_iterator it =
-		                       pluggingController.connectors.begin();
-		     it != pluggingController.connectors.end(); ++it) {
-			connectors.insert((*it)->getName());
+		vector<string_ref> connectors;
+		for (auto& c : pluggingController.connectors) {
+			connectors.push_back(c->getName());
 		}
 		completeString(tokens, connectors);
 	} else if (tokens.size() == 3) {
 		// complete pluggable
-		set<string> pluggables;
-		Connector* connector = pluggingController.findConnector(tokens[1]);
+		vector<string_ref> pluggables;
+		auto* connector = pluggingController.findConnector(tokens[1]);
 		string_ref className = connector ? connector->getClass() : "";
-		for (PluggingController::Pluggables::const_iterator it =
-		                        pluggingController.pluggables.begin();
-		     it != pluggingController.pluggables.end(); ++it) {
-			Pluggable* pluggable = *it;
-			if (pluggable->getClass() == className) {
-				pluggables.insert(pluggable->getName());
+		for (auto& p : pluggingController.pluggables) {
+			if (p->getClass() == className) {
+				pluggables.push_back(p->getName());
 			}
 		}
 		completeString(tokens, pluggables);
@@ -265,9 +253,9 @@ string UnplugCmd::execute(const vector<string>& tokens, EmuTime::param time)
 	if (tokens.size() != 2) {
 		throw SyntaxError();
 	}
-	Connector& connector = pluggingController.getConnector(tokens[1]);
+	auto& connector = pluggingController.getConnector(tokens[1]);
 	connector.unplug(time);
-	pluggingController.cliComm.update(CliComm::UNPLUG, tokens[1], "");
+	pluggingController.getCliComm().update(CliComm::UNPLUG, tokens[1], "");
 	return "";
 }
 
@@ -281,12 +269,9 @@ void UnplugCmd::tabCompletion(vector<string>& tokens) const
 {
 	if (tokens.size() == 2) {
 		// complete connector
-		set<string> connectors;
-		for (PluggingController::Connectors::const_iterator it =
-		                       pluggingController.connectors.begin();
-		     it != pluggingController.connectors.end();
-		     ++it) {
-			connectors.insert((*it)->getName());
+		vector<string_ref> connectors;
+		for (auto& c : pluggingController.connectors) {
+			connectors.push_back(c->getName());
 		}
 		completeString(tokens, connectors);
 	}
@@ -294,47 +279,48 @@ void UnplugCmd::tabCompletion(vector<string>& tokens) const
 
 Connector* PluggingController::findConnector(string_ref name) const
 {
-	for (Connectors::const_iterator it = connectors.begin();
-	     it != connectors.end(); ++it) {
-		if ((*it)->getName() == name) {
-			return *it;
+	for (auto& c : connectors) {
+		if (c->getName() == name) {
+			return c;
 		}
 	}
-	return NULL;
+	return nullptr;
 }
 
 Connector& PluggingController::getConnector(string_ref name) const
 {
-	Connector* result = findConnector(name);
-	if (!result) {
-		throw CommandException("No such connector: " + name);
+	if (auto* result = findConnector(name)) {
+		return *result;
 	}
-	return *result;
+	throw CommandException("No such connector: " + name);
 }
 
 Pluggable* PluggingController::findPluggable(string_ref name) const
 {
-	for (Pluggables::const_iterator it = pluggables.begin();
-	     it != pluggables.end(); ++it) {
-		if ((*it)->getName() == name) {
-			return *it;
+	for (auto& p : pluggables) {
+		if (p->getName() == name) {
+			return p.get();
 		}
 	}
-	return NULL;
+	return nullptr;
 }
 
 Pluggable& PluggingController::getPluggable(string_ref name) const
 {
-	Pluggable* result = findPluggable(name);
-	if (!result) {
-		throw CommandException("No such pluggable: " + name);
+	if (auto* result = findPluggable(name)) {
+		return *result;
 	}
-	return *result;
+	throw CommandException("No such pluggable: " + name);
 }
 
 CliComm& PluggingController::getCliComm()
 {
-	return cliComm;
+	return motherBoard.getMSXCliComm();
+}
+
+EmuTime::param PluggingController::getCurrentTime() const
+{
+	return motherBoard.getCurrentTime();
 }
 
 
@@ -352,14 +338,12 @@ void PluggableInfo::execute(const vector<TclObject>& tokens,
 {
 	switch (tokens.size()) {
 	case 2:
-		for (PluggingController::Pluggables::const_iterator it =
-			 pluggingController.pluggables.begin();
-		     it != pluggingController.pluggables.end(); ++it) {
-			result.addListElement((*it)->getName());
+		for (auto& p : pluggingController.pluggables) {
+			result.addListElement(p->getName());
 		}
 		break;
 	case 3: {
-		const Pluggable& pluggable = pluggingController.getPluggable(
+		auto& pluggable = pluggingController.getPluggable(
 				tokens[2].getString());
 		result.setString(pluggable.getDescription());
 		break;
@@ -378,11 +362,9 @@ string PluggableInfo::help(const vector<string>& /*tokens*/) const
 void PluggableInfo::tabCompletion(vector<string>& tokens) const
 {
 	if (tokens.size() == 3) {
-		set<string> pluggables;
-		for (PluggingController::Pluggables::const_iterator it =
-			 pluggingController.pluggables.begin();
-		     it != pluggingController.pluggables.end(); ++it) {
-			pluggables.insert((*it)->getName());
+		vector<string_ref> pluggables;
+		for (auto& p : pluggingController.pluggables) {
+			pluggables.push_back(p->getName());
 		}
 		completeString(tokens, pluggables);
 	}
@@ -402,14 +384,12 @@ void ConnectorInfo::execute(const vector<TclObject>& tokens,
 {
 	switch (tokens.size()) {
 	case 2:
-		for (PluggingController::Connectors::const_iterator it =
-			 pluggingController.connectors.begin();
-		     it != pluggingController.connectors.end(); ++it) {
-			result.addListElement((*it)->getName());
+		for (auto& c : pluggingController.connectors) {
+			result.addListElement(c->getName());
 		}
 		break;
 	case 3: {
-		const Connector& connector = pluggingController.getConnector(tokens[2].getString());
+		auto& connector = pluggingController.getConnector(tokens[2].getString());
 		result.setString(connector.getDescription());
 		break;
 	}
@@ -426,11 +406,9 @@ string ConnectorInfo::help(const vector<string>& /*tokens*/) const
 void ConnectorInfo::tabCompletion(vector<string>& tokens) const
 {
 	if (tokens.size() == 3) {
-		set<string> connectors;
-		for (PluggingController::Connectors::const_iterator it =
-		                      pluggingController.connectors.begin();
-		     it != pluggingController.connectors.end(); ++it) {
-			connectors.insert((*it)->getName());
+		vector<string_ref> connectors;
+		for (auto& c : pluggingController.connectors) {
+			connectors.push_back(c->getName());
 		}
 		completeString(tokens, connectors);
 	}
@@ -451,29 +429,23 @@ void ConnectionClassInfo::execute(const vector<TclObject>& tokens,
 {
 	switch (tokens.size()) {
 	case 2: {
-		set<string_ref> classes;
-		for (PluggingController::Connectors::const_iterator it =
-			 pluggingController.connectors.begin();
-		     it != pluggingController.connectors.end(); ++it) {
-			classes.insert((*it)->getClass());
+		std::set<string_ref> classes; // filter duplicates
+		for (auto& c : pluggingController.connectors) {
+			classes.insert(c->getClass());
 		}
-		for (PluggingController::Pluggables::const_iterator it =
-			 pluggingController.pluggables.begin();
-		     it != pluggingController.pluggables.end(); ++it) {
-			classes.insert((*it)->getClass());
+		for (auto& p : pluggingController.pluggables) {
+			classes.insert(p->getClass());
 		}
-		result.addListElements(classes.begin(), classes.end());
+		result.addListElements(classes);
 		break;
 	}
 	case 3: {
-		string_ref arg = tokens[2].getString();
-		if (const Connector* connector =
-		    pluggingController.findConnector(arg)) {
+		const auto& arg = tokens[2].getString();
+		if (auto* connector = pluggingController.findConnector(arg)) {
 			result.setString(connector->getClass());
 			break;
 		}
-		if (const Pluggable* pluggable =
-		    pluggingController.findPluggable(arg)) {
+		if (auto* pluggable = pluggingController.findPluggable(arg)) {
 			result.setString(pluggable->getClass());
 			break;
 		}
@@ -493,16 +465,12 @@ string ConnectionClassInfo::help(const vector<string>& /*tokens*/) const
 void ConnectionClassInfo::tabCompletion(vector<string>& tokens) const
 {
 	if (tokens.size() == 3) {
-		set<string> names;
-		for (PluggingController::Connectors::const_iterator it =
-			               pluggingController.connectors.begin();
-		     it != pluggingController.connectors.end(); ++it) {
-			names.insert((*it)->getName());
+		vector<string_ref> names;
+		for (auto& c : pluggingController.connectors) {
+			names.push_back(c->getName());
 		}
-		for (PluggingController::Pluggables::const_iterator it =
-			               pluggingController.pluggables.begin();
-		     it != pluggingController.pluggables.end(); ++it) {
-			names.insert((*it)->getName());
+		for (auto& p : pluggingController.pluggables) {
+			names.push_back(p->getName());
 		}
 		completeString(tokens, names);
 	}
