@@ -1,5 +1,3 @@
-// $Id: Printer.cc 12631 2012-06-14 20:18:24Z m9710797 $
-
 #include "Printer.hh"
 #include "PNG.hh"
 #include "FileOperations.hh"
@@ -10,6 +8,7 @@
 #include "Math.hh"
 #include "MemBuffer.hh"
 #include "serialize.hh"
+#include "memory.hh"
 #include "vla.hh"
 #include <algorithm>
 #include <vector>
@@ -94,7 +93,7 @@ RawPrinter::RawPrinter()
 {
 	Properties* properties = propGetGlobalProperties();
 	hFile = CreateFile(properties->ports.Lpt.portName, GENERIC_WRITE,
-	                   0, NULL, OPEN_EXISTING, FILE_FLAG_WRITE_THROUGH, NULL);
+	                   0, nullptr, OPEN_EXISTING, FILE_FLAG_WRITE_THROUGH, nullptr);
 	if (hFile == INVALID_HANDLE_VALUE) {
 		throw MSXException();
 	}
@@ -108,7 +107,7 @@ void RawPrinter::~RawPrinter()
 void RawPrinter::write(byte value)
 {
 	unsigned dwWritten;
-	WriteFile(hFile, &value, 1, &dwWritten, NULL);
+	WriteFile(hFile, &value, 1, &dwWritten, nullptr);
 }
 
 void RawPrinter::forceFormFeed()
@@ -123,10 +122,9 @@ ImagePrinter::ImagePrinter(MSXMotherBoard& motherBoard_, bool graphicsHiLo_)
 	: motherBoard(motherBoard_)
 	, graphicsHiLo(graphicsHiLo_)
 {
-	MSXMotherBoard::SharedStuff& info =
-		motherBoard.getSharedStuff("print-resolution");
+	auto& info = motherBoard.getSharedStuff("print-resolution");
 	if (info.counter == 0) {
-		assert(info.stuff == NULL);
+		assert(!info.stuff);
 		info.stuff = new IntegerSetting(
 		    motherBoard.getCommandController(), "print-resolution",
 		    "resolution of the output image of emulated dot matrix printer in DPI",
@@ -171,15 +169,14 @@ ImagePrinter::~ImagePrinter()
 {
 	flushEmulatedPrinter();
 
-	MSXMotherBoard::SharedStuff& info =
-		motherBoard.getSharedStuff("print-resolution");
+	auto& info = motherBoard.getSharedStuff("print-resolution");
 	assert(info.counter);
 	assert(dpiSetting);
 	assert(dpiSetting == info.stuff);
 	--info.counter;
 	if (info.counter == 0) {
 		delete dpiSetting;
-		info.stuff = NULL;
+		info.stuff = nullptr;
 	}
 }
 
@@ -276,10 +273,10 @@ void ImagePrinter::seekPrinterHeadRelative(double offset)
 
 void ImagePrinter::ensurePrintPage()
 {
-	if (!paper.get()) {
+	if (!paper) {
 		// A4 paper format (210mm x 297mm) at 300dpi
 		// TODO make this configurable
-		double dpi = dpiSetting->getValue();
+		int dpi = dpiSetting->getInt();
 		unsigned paperSizeX = unsigned((210 / 25.4) * dpi);
 		unsigned paperSizeY = unsigned((297 / 25.4) * dpi);
 
@@ -288,14 +285,14 @@ void ImagePrinter::ensurePrintPage()
 		pixelSizeX = double(paperSizeX) / dotsX;
 		pixelSizeY = double(paperSizeY) / dotsY;
 
-		paper.reset(new Paper(paperSizeX, paperSizeY,
-		                      pixelSizeX, pixelSizeY));
+		paper = make_unique<Paper>(paperSizeX, paperSizeY,
+		                           pixelSizeX, pixelSizeY);
 	}
 }
 
 void ImagePrinter::flushEmulatedPrinter()
 {
-	if (paper.get()) {
+	if (paper) {
 		if (printAreaBottom > printAreaTop) {
 			try {
 				string filename = paper->save();
@@ -717,7 +714,8 @@ void ImagePrinterMSX::resetSettings()
 	fontWidth    = 8;
 	eightBit     = -1;
 
-	memcpy(fontInfo.rom, MSXFont, sizeof(fontInfo.rom));
+	// note: this only overwrites 9/12 of the fontInfo.rom array.
+	memcpy(fontInfo.rom, MSXFont, sizeof(MSXFont));
 	fontInfo.charWidth = 9;
 	fontInfo.pixelDelta = 1.0;
 	fontInfo.useRam = false;
@@ -1262,7 +1260,7 @@ void ImagePrinterEpson::resetSettings()
 	fontWidth    = 6;
 	eightBit     = -1;
 
-	memcpy(fontInfo.rom, EpsonFontRom, sizeof(fontInfo.rom));
+	memcpy(fontInfo.rom, EpsonFontRom, sizeof(EpsonFontRom));
 	fontInfo.charWidth = 12;
 	fontInfo.pixelDelta = 0.5;
 	fontInfo.useRam = false;
