@@ -1,18 +1,17 @@
-// $Id: PrinterPortLogger.cc 12631 2012-06-14 20:18:24Z m9710797 $
-
 #include "PrinterPortLogger.hh"
 #include "PlugException.hh"
 #include "FileException.hh"
 #include "File.hh"
 #include "FilenameSetting.hh"
 #include "serialize.hh"
+#include "memory.hh"
 #include <cassert>
 
 namespace openmsx {
 
 PrinterPortLogger::PrinterPortLogger(CommandController& commandController)
-	: logFilenameSetting(new FilenameSetting(commandController,
-		"printerlogfilename",
+	: logFilenameSetting(make_unique<FilenameSetting>(
+		commandController, "printerlogfilename",
 		"filename of the file where the printer output is logged to",
 		"printer.log"))
 	, toPrint(0) // Initialize to avoid a static analysis (cppcheck) warning.
@@ -36,7 +35,7 @@ bool PrinterPortLogger::getStatus(EmuTime::param /*time*/)
 
 void PrinterPortLogger::setStrobe(bool strobe, EmuTime::param /*time*/)
 {
-	if (file.get() && !strobe && prevStrobe) {
+	if (file && !strobe && prevStrobe) {
 		// falling edge
 		file->write(&toPrint, 1);
 		file->flush(); // optimize when it turns out flushing
@@ -54,8 +53,8 @@ void PrinterPortLogger::plugHelper(
 		Connector& /*connector*/, EmuTime::param /*time*/)
 {
 	try {
-		file.reset(new File(logFilenameSetting->getValue(),
-		                    File::TRUNCATE));
+		file = make_unique<File>(logFilenameSetting->getString(),
+		                         File::TRUNCATE);
 	} catch (FileException& e) {
 		throw PlugException("Couldn't plug printer logger: " +
 		                    e.getMessage());
