@@ -1,12 +1,11 @@
-// $Id: PrinterPortSimpl.cc 12631 2012-06-14 20:18:24Z m9710797 $
-
 #include "PrinterPortSimpl.hh"
 #include "DACSound8U.hh"
 #include "DeviceConfig.hh"
 #include "XMLElement.hh"
 #include "serialize.hh"
+#include "memory.hh"
 
-using std::auto_ptr;
+using std::unique_ptr;
 
 namespace openmsx {
 
@@ -15,6 +14,10 @@ PrinterPortSimpl::PrinterPortSimpl(const HardwareConfig& hwConf_)
 {
 }
 
+PrinterPortSimpl::~PrinterPortSimpl()
+{
+}
+
 bool PrinterPortSimpl::getStatus(EmuTime::param /*time*/)
 {
 	return true; // TODO check
@@ -33,17 +36,16 @@ void PrinterPortSimpl::writeData(byte data, EmuTime::param time)
 static XMLElement createXML()
 {
 	XMLElement xml("simpl");
-	auto_ptr<XMLElement> sound(new XMLElement("sound"));
-	sound->addChild(auto_ptr<XMLElement>(new XMLElement("volume", "12000")));
-	xml.addChild(sound);
+	xml.addChild(XMLElement("sound"))
+	       .addChild(XMLElement("volume", "12000"));
 	return xml;
 }
 
 void PrinterPortSimpl::createDAC()
 {
 	static XMLElement xml = createXML();
-	dac.reset(new DACSound8U("simpl", getDescription(),
-	                         DeviceConfig(hwConf, xml)));
+	dac = make_unique<DACSound8U>("simpl", getDescription(),
+	                              DeviceConfig(hwConf, xml));
 }
 
 void PrinterPortSimpl::plugHelper(Connector& /*connector*/, EmuTime::param /*time*/)
