@@ -1,9 +1,8 @@
-// $Id: RP5C01.cc 12091 2011-04-10 16:30:17Z mthuurne $
-
 #include "RP5C01.hh"
 #include "EnumSetting.hh"
 #include "SRAM.hh"
 #include "serialize.hh"
+#include "memory.hh"
 #include <cassert>
 #include <ctime>
 
@@ -40,20 +39,23 @@ static const nibble mask[4][13] = {
 	{ 0xf, 0xf, 0xf, 0xf, 0xf, 0xf, 0xf, 0xf, 0xf, 0xf, 0xf, 0xf, 0xf}
 };
 
-static EnumSetting<RP5C01::RTCMode>* createModeSetting(
-	CommandController& commandController)
+static std::unique_ptr<EnumSetting<RP5C01::RTCMode>> createModeSetting(
+	CommandController& commandController, const std::string& name)
 {
 	EnumSetting<RP5C01::RTCMode>::Map modeMap;
-	modeMap["EmuTime"]  = RP5C01::EMUTIME;
-	modeMap["RealTime"] = RP5C01::REALTIME;
-	return new EnumSetting<RP5C01::RTCMode>(commandController,
-		"rtcmode", "Real Time Clock mode", RP5C01::EMUTIME, modeMap);
+	modeMap.push_back(std::make_pair("EmuTime",  RP5C01::EMUTIME));
+	modeMap.push_back(std::make_pair("RealTime", RP5C01::REALTIME));
+	return make_unique<EnumSetting<RP5C01::RTCMode>>(
+		commandController,
+		((name == "Real time clock") ? "rtcmode" // bw-compat
+		                             : (name + " mode")),
+		"Real Time Clock mode", RP5C01::EMUTIME, modeMap);
 }
 
 RP5C01::RP5C01(CommandController& commandController, SRAM& regs_,
-               EmuTime::param time)
+               EmuTime::param time, const std::string& name)
 	: regs(regs_)
-	, modeSetting(createModeSetting(commandController))
+	, modeSetting(createModeSetting(commandController, name))
 	, reference(time)
 {
 	initializeTime();
@@ -127,7 +129,7 @@ void RP5C01::writePort(nibble port, nibble value, EmuTime::param time)
 
 void RP5C01::initializeTime()
 {
-	time_t t = time(NULL);
+	time_t t = time(nullptr);
 	struct tm *tm = localtime(&t);
 	fraction = 0;			// fractions of a second
 	seconds  = tm->tm_sec;		// 0-59
@@ -194,7 +196,7 @@ static int daysInMonth(int month, unsigned leapYear)
 
 void RP5C01::updateTimeRegs(EmuTime::param time)
 {
-	if (modeSetting->getValue() == EMUTIME) {
+	if (modeSetting->getEnum() == EMUTIME) {
 		// sync with EmuTime, perfect emulation
 		unsigned elapsed = unsigned(reference.getTicksTill(time));
 		reference.advance(time);
