@@ -1,5 +1,3 @@
-// $Id: Reactor.cc 12805 2012-08-13 19:26:18Z m9710797 $
-
 #include "Reactor.hh"
 #include "CommandLineParser.hh"
 #include "EventDistributor.hh"
@@ -43,13 +41,14 @@
 #include "checked_cast.hh"
 #include "statp.hh"
 #include "unreachable.hh"
+#include "memory.hh"
+#include "build-info.hh"
 #include <cassert>
-#include <memory>
 #include <iostream>
 
 using std::string;
 using std::vector;
-using std::set;
+using std::make_shared;
 
 namespace openmsx {
 
@@ -155,6 +154,7 @@ class PollEventGenerator : private Alarm
 public:
 	explicit PollEventGenerator(EventDistributor& eventDistributor);
 	virtual ~PollEventGenerator();
+	void pollNow();
 private:
 	virtual bool alarm();
 	EventDistributor& eventDistributor;
@@ -180,69 +180,100 @@ public:
 	                     TclObject& result) const;
 	virtual string help(const vector<string>& tokens) const;
 private:
-	const unsigned long long reference;
+	const uint64_t reference;
 };
 
 
 Reactor::Reactor()
 	: mbSem(1)
-	, eventDistributor(new EventDistributor(*this))
-	, globalCliComm(new GlobalCliComm())
-	, globalCommandController(new GlobalCommandController(
-		*eventDistributor, *globalCliComm, *this))
-	, globalSettings(new GlobalSettings(*globalCommandController))
-	, inputEventGenerator(new InputEventGenerator(
-		*globalCommandController, *eventDistributor))
-	, mixer(new Mixer(*this, *globalCommandController))
-	, diskFactory(new DiskFactory(*this))
-	, diskManipulator(new DiskManipulator(*globalCommandController, *this))
-	, virtualDrive(new DiskChanger("virtual_drive", *globalCommandController,
-	                               *diskFactory, *diskManipulator, true))
-	, filePool(new FilePool(*globalCommandController, *eventDistributor))
-	, pauseSetting(getGlobalSettings().getPauseSetting())
-	, pauseOnLostFocusSetting(getGlobalSettings().getPauseOnLostFocusSetting())
-	, userSettings(new UserSettings(*globalCommandController))
-	, softwareDatabase(new RomDatabase(*globalCommandController, *globalCliComm))
-	, afterCommand(new AfterCommand(*this, *eventDistributor,
-	                                *globalCommandController))
-	, quitCommand(new QuitCommand(*globalCommandController, *eventDistributor))
-	, messageCommand(new MessageCommand(*globalCommandController))
-	, machineCommand(new MachineCommand(*globalCommandController, *this))
-	, testMachineCommand(new TestMachineCommand(*globalCommandController, *this))
-	, createMachineCommand(new CreateMachineCommand(*globalCommandController, *this))
-	, deleteMachineCommand(new DeleteMachineCommand(*globalCommandController, *this))
-	, listMachinesCommand(new ListMachinesCommand(*globalCommandController, *this))
-	, activateMachineCommand(new ActivateMachineCommand(*globalCommandController, *this))
-	, storeMachineCommand(new StoreMachineCommand(*globalCommandController, *this))
-	, restoreMachineCommand(new RestoreMachineCommand(*globalCommandController, *this))
-	, aviRecordCommand(new AviRecorder(*this))
-	, extensionInfo(new ConfigInfo(getOpenMSXInfoCommand(), "extensions"))
-	, machineInfo  (new ConfigInfo(getOpenMSXInfoCommand(), "machines"))
-	, realTimeInfo(new RealTimeInfo(getOpenMSXInfoCommand()))
-	, tclCallbackMessages(new TclCallbackMessages(*globalCliComm,
-	                                              *globalCommandController))
+	, activeBoard(nullptr)
 	, blockedCounter(0)
 	, paused(false)
 	, running(true)
+	, isInit(false)
 {
+#if UNIQUE_PTR_BUG
+	display = nullptr;
+#endif
+}
+
+void Reactor::init()
+{
+	eventDistributor = make_unique<EventDistributor>(*this);
+	globalCliComm = make_unique<GlobalCliComm>();
+	globalCommandController = make_unique<GlobalCommandController>(
+		*eventDistributor, *globalCliComm, *this);
+	globalSettings = make_unique<GlobalSettings>(
+		*globalCommandController);
+	inputEventGenerator = make_unique<InputEventGenerator>(
+		*globalCommandController, *eventDistributor);
+	mixer = make_unique<Mixer>(
+		*this, *globalCommandController);
+	diskFactory = make_unique<DiskFactory>(
+		*this);
+	diskManipulator = make_unique<DiskManipulator>(
+		*globalCommandController, *this);
+	virtualDrive = make_unique<DiskChanger>(
+		"virtual_drive", *globalCommandController,
+		*diskFactory, *diskManipulator, true);
+	filePool = make_unique<FilePool>(
+		*globalCommandController, *eventDistributor);
+	userSettings = make_unique<UserSettings>(
+		*globalCommandController);
+	softwareDatabase = make_unique<RomDatabase>(
+		*globalCommandController, *globalCliComm);
+	afterCommand = make_unique<AfterCommand>(
+		*this, *eventDistributor, *globalCommandController);
+	quitCommand = make_unique<QuitCommand>(
+		*globalCommandController, *eventDistributor);
+	messageCommand = make_unique<MessageCommand>(
+		*globalCommandController);
+	machineCommand = make_unique<MachineCommand>(
+		*globalCommandController, *this);
+	testMachineCommand = make_unique<TestMachineCommand>(
+		*globalCommandController, *this);
+	createMachineCommand = make_unique<CreateMachineCommand>(
+		*globalCommandController, *this);
+	deleteMachineCommand = make_unique<DeleteMachineCommand>(
+		*globalCommandController, *this);
+	listMachinesCommand = make_unique<ListMachinesCommand>(
+		*globalCommandController, *this);
+	activateMachineCommand = make_unique<ActivateMachineCommand>(
+		*globalCommandController, *this);
+	storeMachineCommand = make_unique<StoreMachineCommand>(
+		*globalCommandController, *this);
+	restoreMachineCommand = make_unique<RestoreMachineCommand>(
+		*globalCommandController, *this);
+	aviRecordCommand = make_unique<AviRecorder>(*this);
+	extensionInfo = make_unique<ConfigInfo>(
+		getOpenMSXInfoCommand(), "extensions");
+	machineInfo   = make_unique<ConfigInfo>(
+		getOpenMSXInfoCommand(), "machines");
+	realTimeInfo = make_unique<RealTimeInfo>(
+		getOpenMSXInfoCommand());
+	tclCallbackMessages = make_unique<TclCallbackMessages>(
+		*globalCliComm, *globalCommandController);
+
 	createMachineSetting();
 
-	pauseSetting.attach(*this);
+	getGlobalSettings().getPauseSetting().attach(*this);
 
 	eventDistributor->registerEventListener(OPENMSX_QUIT_EVENT, *this);
 	eventDistributor->registerEventListener(OPENMSX_FOCUS_EVENT, *this);
 	eventDistributor->registerEventListener(OPENMSX_DELETE_BOARDS, *this);
+	isInit = true;
 }
 
 Reactor::~Reactor()
 {
+	if (!isInit) return;
 	deleteBoard(activeBoard);
 
 	eventDistributor->unregisterEventListener(OPENMSX_QUIT_EVENT, *this);
 	eventDistributor->unregisterEventListener(OPENMSX_FOCUS_EVENT, *this);
 	eventDistributor->unregisterEventListener(OPENMSX_DELETE_BOARDS, *this);
 
-	pauseSetting.detach(*this);
+	getGlobalSettings().getPauseSetting().detach(*this);
 }
 
 EventDistributor& Reactor::getEventDistributor()
@@ -272,7 +303,7 @@ InputEventGenerator& Reactor::getInputEventGenerator()
 
 Display& Reactor::getDisplay()
 {
-	assert(display.get());
+	assert(display);
 	return *display;
 }
 
@@ -321,69 +352,74 @@ InfoCommand& Reactor::getOpenMSXInfoCommand()
 	return globalCommandController->getOpenMSXInfoCommand();
 }
 
-void Reactor::getHwConfigs(const string& type, set<string>& result)
+vector<string> Reactor::getHwConfigs(string_ref type)
 {
-	SystemFileContext context;
-	vector<string> paths = context.getPaths();
-	for (vector<string>::const_iterator it = paths.begin();
-	     it != paths.end(); ++it) {
-		string path = FileOperations::join(*it, type);
+	vector<string> result;
+	for (auto& p : SystemFileContext().getPaths()) {
+		const auto& path = FileOperations::join(p, type);
 		ReadDir configsDir(path);
-		while (dirent* d = configsDir.getEntry()) {
-			string name = d->d_name;
-			string dir = FileOperations::join(path, name);
-			string config = FileOperations::join(dir, "hardwareconfig.xml");
-			if (FileOperations::isDirectory(dir) &&
-			    FileOperations::isRegularFile(config)) {
-				result.insert(name);
+		while (auto* entry = configsDir.getEntry()) {
+			string_ref name = entry->d_name;
+			const auto& fullname = FileOperations::join(path, name);
+			if (name.ends_with(".xml") &&
+			    FileOperations::isRegularFile(fullname)) {
+				name.remove_suffix(4);
+				result.push_back(name.str());
+			} else if (FileOperations::isDirectory(fullname)) {
+				const auto& config = FileOperations::join(
+					fullname, "hardwareconfig.xml");
+				if (FileOperations::isRegularFile(config)) {
+					result.push_back(name.str());
+				}
 			}
 		}
 	}
+	// remove duplicates
+	sort(result.begin(), result.end());
+	result.erase(unique(result.begin(), result.end()), result.end());
+	return result;
 }
 
 void Reactor::createMachineSetting()
 {
 	EnumSetting<int>::Map machines; // int's are unique dummy values
 	int count = 1;
-	set<string> names;
-	getHwConfigs("machines", names);
-	for (set<string>::const_iterator it = names.begin();
-	     it != names.end(); ++it) {
-		machines[*it] = count++;
+	for (auto& name : getHwConfigs("machines")) {
+		machines.push_back(std::make_pair(name, count++));
 	}
-	machines["C-BIOS_MSX2+"] = 0; // default machine
+	machines.push_back(std::make_pair("C-BIOS_MSX2+", 0)); // default machine
 
-	machineSetting.reset(new EnumSetting<int>(
+	machineSetting = make_unique<EnumSetting<int>>(
 		*globalCommandController, "default_machine",
 		"default machine (takes effect next time openMSX is started)",
-		0, machines));
+		0, machines);
 }
 
 MSXMotherBoard* Reactor::getMotherBoard() const
 {
 	assert(Thread::isMainThread());
-	return activeBoard.get();
+	return activeBoard;
 }
 
 string Reactor::getMachineID() const
 {
-	return activeBoard.get() ? activeBoard->getMachineID() : "";
+	return activeBoard ? activeBoard->getMachineID() : "";
 }
 
-void Reactor::getMachineIDs(set<string>& result) const
+vector<string_ref> Reactor::getMachineIDs() const
 {
-	for (Reactor::Boards::const_iterator it = boards.begin();
-	     it != boards.end(); ++it) {
-		result.insert((*it)->getMachineID());
+	vector<string_ref> result;
+	for (auto& b : boards) {
+		result.push_back(b->getMachineID());
 	}
+	return result;
 }
 
-Reactor::Board Reactor::getMachine(const string& machineID) const
+MSXMotherBoard& Reactor::getMachine(const string& machineID) const
 {
-	for (Boards::const_iterator it = boards.begin();
-	     it != boards.end(); ++it) {
-		if ((*it)->getMachineID() == machineID) {
-			return *it;
+	for (auto& b : boards) {
+		if (b->getMachineID() == machineID) {
+			return *b;
 		}
 	}
 	throw CommandException("No machine with ID: " + machineID);
@@ -391,25 +427,26 @@ Reactor::Board Reactor::getMachine(const string& machineID) const
 
 Reactor::Board Reactor::createEmptyMotherBoard()
 {
-	return Board(new MSXMotherBoard(*this));
+	return make_unique<MSXMotherBoard>(*this);
 }
 
-void Reactor::replaceBoard(MSXMotherBoard& oldBoard_, const Board& newBoard)
+void Reactor::replaceBoard(MSXMotherBoard& oldBoard_, Board newBoard_)
 {
 	assert(Thread::isMainThread());
 
 	// Add new board.
-	boards.push_back(newBoard);
+	auto* newBoard = newBoard_.get();
+	boards.push_back(move(newBoard_));
 
 	// Lookup old board (it must be present).
-	Boards::iterator it = boards.begin();
+	auto it = boards.begin();
 	while (it->get() != &oldBoard_) {
 		++it;
 		assert(it != boards.end());
 	}
 
 	// If the old board was the active board, then activate the new board
-	if (*it == activeBoard) {
+	if (it->get() == activeBoard) {
 		switchBoard(newBoard);
 	}
 
@@ -422,8 +459,13 @@ void Reactor::replaceBoard(MSXMotherBoard& oldBoard_, const Board& newBoard)
 
 void Reactor::switchMachine(const string& machine)
 {
-	if (!display.get()) {
-		display.reset(new Display(*this));
+	if (!display) {
+#if UNIQUE_PTR_BUG
+		display2 = make_unique<Display>(*this);
+		display = display2.get();
+#else
+		display = make_unique<Display>(*this);
+#endif
 		// TODO: Currently it is not possible to move this call into the
 		//       constructor of Display because the call to createVideoSystem()
 		//       indirectly calls Reactor.getDisplay().
@@ -437,23 +479,28 @@ void Reactor::switchMachine(const string& machine)
 	assert(Thread::isMainThread());
 	// Note: loadMachine can throw an exception and in that case the
 	//       motherboard must be considered as not created at all.
-	Board newBoard = createEmptyMotherBoard();
+	auto newBoard_ = createEmptyMotherBoard();
+	auto* newBoard = newBoard_.get();
 	newBoard->loadMachine(machine);
-	boards.push_back(newBoard);
+	boards.push_back(move(newBoard_));
 
-	Board oldBoard = activeBoard;
+	auto* oldBoard = activeBoard;
 	switchBoard(newBoard);
 	deleteBoard(oldBoard);
 }
 
-void Reactor::switchBoard(const Board& newBoard)
+void Reactor::switchBoard(MSXMotherBoard* newBoard)
 {
 	assert(Thread::isMainThread());
-	assert(!newBoard.get() ||
-	       (find(boards.begin(), boards.end(), newBoard) != boards.end()));
-	assert(!activeBoard.get() ||
-	       (find(boards.begin(), boards.end(), activeBoard) != boards.end()));
-	if (activeBoard.get()) {
+	assert(!newBoard ||
+	       (find_if(boards.begin(), boards.end(),
+	               [&](Boards::value_type& b) { return b.get() == newBoard; })
+	        != boards.end()));
+	assert(!activeBoard ||
+	       (find_if(boards.begin(), boards.end(),
+	                [&](Boards::value_type& b) { return b.get() == activeBoard; })
+	        != boards.end()));
+	if (activeBoard) {
 		activeBoard->activate(false);
 	}
 	{
@@ -464,14 +511,14 @@ void Reactor::switchBoard(const Board& newBoard)
 		activeBoard = newBoard;
 	}
 	eventDistributor->distributeEvent(
-		new SimpleEvent(OPENMSX_MACHINE_LOADED_EVENT));
+		make_shared<SimpleEvent>(OPENMSX_MACHINE_LOADED_EVENT));
 	globalCliComm->update(CliComm::HARDWARE, getMachineID(), "select");
-	if (activeBoard.get()) {
+	if (activeBoard) {
 		activeBoard->activate(true);
 	}
 }
 
-void Reactor::deleteBoard(Board board)
+void Reactor::deleteBoard(MSXMotherBoard* board)
 {
 	// Note: pass 'board' by-value to keep the parameter from changing
 	// after the call to switchBoard(). switchBoard() changes the
@@ -479,67 +526,69 @@ void Reactor::deleteBoard(Board board)
 	// if it were passed by reference to this method (AFAICS this only
 	// happens in ~Reactor()).
 	assert(Thread::isMainThread());
-	if (!board.get()) return;
+	if (!board) return;
 
 	if (board == activeBoard) {
 		// delete active board -> there is no active board anymore
-		switchBoard(Reactor::Board(NULL));
+		switchBoard(nullptr);
 	}
-	Boards::iterator it = find(boards.begin(), boards.end(), board);
+	auto it = find_if(boards.begin(), boards.end(),
+	                  [&](Boards::value_type& b) { return b.get() == board; });
 	assert(it != boards.end());
+	auto board_ = move(*it);
 	boards.erase(it);
 	// Don't immediately delete old boards because it's possible this
 	// routine is called via a code path that goes through the old
 	// board. Instead remember this board and delete it at a safe moment
 	// in time.
-	garbageBoards.push_back(board);
+	garbageBoards.push_back(move(board_));
 	eventDistributor->distributeEvent(
-		new SimpleEvent(OPENMSX_DELETE_BOARDS));
+		make_shared<SimpleEvent>(OPENMSX_DELETE_BOARDS));
 }
 
 void Reactor::enterMainLoop()
 {
 	// Note: this method can get called from different threads
-	ScopedLock lock;
-	if (!Thread::isMainThread()) {
+	if (Thread::isMainThread()) {
 		// Don't take lock in main thread to avoid recursive locking.
-		lock.take(mbSem);
-	}
-	if (activeBoard.get()) {
-		activeBoard->exitCPULoopAsync();
+		if (activeBoard) {
+			activeBoard->exitCPULoopSync();
+		}
+	} else {
+		ScopedLock lock(mbSem);
+		if (activeBoard) {
+			activeBoard->exitCPULoopAsync();
+		}
 	}
 }
 
+void Reactor::pollNow()
+{
+	pollEventGenerator->pollNow();
+}
+
 void Reactor::run(CommandLineParser& parser)
 {
-	GlobalCommandController& commandController = *globalCommandController;
-
-	PRT_DEBUG("Reactor::run Trying to execute init.tcl...");
+	auto& commandController = *globalCommandController;
 
 	// execute init.tcl
 	try {
-		PreferSystemFileContext context;
-		commandController.source(context.resolve("init.tcl"));
+		commandController.source(
+			PreferSystemFileContext().resolve("init.tcl"));
 	} catch (FileException&) {
 		// no init.tcl, ignore
 	}
 
-	PRT_DEBUG("Reactor::run Executing startup scripts...");
 	// execute startup scripts
-	const CommandLineParser::Scripts& scripts = parser.getStartupScripts();
-	for (CommandLineParser::Scripts::const_iterator it = scripts.begin();
-	     it != scripts.end(); ++it) {
-		PRT_DEBUG("Reactor::run Executing startup script..." << *it);
+	for (auto& s : parser.getStartupScripts()) {
 		try {
-			UserFileContext context;
-			commandController.source(context.resolve(*it));
+			commandController.source(UserFileContext().resolve(s));
 		} catch (FileException& e) {
 			throw FatalError("Couldn't execute script: " +
 			                 e.getMessage());
 		}
 	}
 
-	PRT_DEBUG("Reactor::run Powering up active board...");
 	// Run
 	if (parser.getParseStatus() == CommandLineParser::RUN) {
 		// don't use Tcl to power up the machine, we cannot pass
@@ -548,20 +597,18 @@ void Reactor::run(CommandLineParser& parser)
 		// between devices so ADVRAM can check the error condition
 		// in its constructor
 		//commandController.executeCommand("set power on");
-		if (activeBoard.get()) {
+		if (activeBoard) {
 			activeBoard->powerUp();
 		}
 	}
 
-	PRT_DEBUG("Reactor::run Instantiate poll event generator...");
-	PollEventGenerator pollEventGenerator(*eventDistributor);
+	pollEventGenerator = make_unique<PollEventGenerator>(*eventDistributor);
 
 	while (running) {
 		eventDistributor->deliverEvents();
 		assert(garbageBoards.empty());
-		MSXMotherBoard* motherboard = activeBoard.get();
-		bool blocked = (blockedCounter > 0) || !motherboard;
-		if (!blocked) blocked = !motherboard->execute();
+		bool blocked = (blockedCounter > 0) || !activeBoard;
+		if (!blocked) blocked = !activeBoard->execute();
 		if (blocked) {
 			// At first sight a better alternative is to use the
 			// SDL_WaitEvent() function. Though when inspecting
@@ -610,8 +657,9 @@ void Reactor::unblock()
 // Observer<Setting>
 void Reactor::update(const Setting& setting)
 {
+	auto& pauseSetting = getGlobalSettings().getPauseSetting();
 	if (&setting == &pauseSetting) {
-		if (pauseSetting.getValue()) {
+		if (pauseSetting.getBoolean()) {
 			pause();
 		} else {
 			unpause();
@@ -620,24 +668,46 @@ void Reactor::update(const Setting& setting)
 }
 
 // EventListener
-int Reactor::signalEvent(const shared_ptr<const Event>& event)
+int Reactor::signalEvent(const std::shared_ptr<const Event>& event)
 {
-	EventType type = event->getType();
+	auto type = event->getType();
 	if (type == OPENMSX_QUIT_EVENT) {
 		enterMainLoop();
 		running = false;
 	} else if (type == OPENMSX_FOCUS_EVENT) {
-		if (!pauseOnLostFocusSetting.getValue()) return 0;
-		const FocusEvent& focusEvent = checked_cast<const FocusEvent&>(*event);
+#if PLATFORM_ANDROID
+		// Android SDL port sends a (un)focus event when an app is put in background
+		// by the OS for whatever reason (like an incoming phone call) and all screen
+		// resources are taken away from the app.
+		// In such case the app is supposed to behave as a good citizen
+		// and minize its resource usage and related battery drain.
+		// The SDL Android port already takes care of halting the Java
+		// part of the sound processing. The Display class makes sure that it wont try
+		// to render anything to the (temporary missing) graphics resources but the
+		// main emulation should also be temporary stopped, in order to minimize CPU usage
+		auto& focusEvent = checked_cast<const FocusEvent&>(*event);
+		if (focusEvent.getGain()) {
+			unblock();
+		} else {
+			block();
+		}
+#else
+		// On other platforms, the user may specify if openMSX should be
+		// halted on loss of focus.
+		if (!getGlobalSettings().getPauseOnLostFocusSetting().getBoolean()) {
+			return 0;
+		}
+		auto& focusEvent = checked_cast<const FocusEvent&>(*event);
 		if (focusEvent.getGain()) {
 			// gained focus
-			if (!pauseSetting.getValue()) {
+			if (!getGlobalSettings().getPauseSetting().getBoolean()) {
 				unpause();
 			}
 		} else {
 			// lost focus
 			pause();
 		}
+#endif
 	} else if (type == OPENMSX_DELETE_BOARDS) {
 		assert(!garbageBoards.empty());
 		garbageBoards.erase(garbageBoards.begin());
@@ -659,7 +729,7 @@ QuitCommand::QuitCommand(CommandController& commandController,
 
 string QuitCommand::execute(const vector<string>& /*tokens*/)
 {
-	distributor.distributeEvent(new QuitEvent());
+	distributor.distributeEvent(make_shared<QuitEvent>());
 	return "";
 }
 
@@ -703,9 +773,7 @@ string MachineCommand::help(const vector<string>& /*tokens*/) const
 
 void MachineCommand::tabCompletion(vector<string>& tokens) const
 {
-	set<string> machines;
-	Reactor::getHwConfigs("machines", machines);
-	completeString(tokens, machines);
+	completeString(tokens, Reactor::getHwConfigs("machines"));
 }
 
 
@@ -741,9 +809,7 @@ string TestMachineCommand::help(const vector<string>& /*tokens*/) const
 
 void TestMachineCommand::tabCompletion(vector<string>& tokens) const
 {
-	set<string> machines;
-	Reactor::getHwConfigs("machines", machines);
-	completeString(tokens, machines);
+	completeString(tokens, Reactor::getHwConfigs("machines"));
 }
 
 
@@ -761,9 +827,10 @@ string CreateMachineCommand::execute(const vector<string>& tokens)
 	if (tokens.size() != 1) {
 		throw SyntaxError();
 	}
-	Reactor::Board newBoard = reactor.createEmptyMotherBoard();
-	reactor.boards.push_back(newBoard);
-	return newBoard->getMachineID();
+	auto newBoard = reactor.createEmptyMotherBoard();
+	auto result = newBoard->getMachineID();
+	reactor.boards.push_back(move(newBoard));
+	return result;
 }
 
 string CreateMachineCommand::help(const vector<string>& /*tokens*/) const
@@ -793,7 +860,7 @@ string DeleteMachineCommand::execute(const vector<string>& tokens)
 	if (tokens.size() != 2) {
 		throw SyntaxError();
 	}
-	reactor.deleteBoard(reactor.getMachine(tokens[1]));
+	reactor.deleteBoard(&reactor.getMachine(tokens[1]));
 	return "";
 }
 
@@ -804,9 +871,7 @@ string DeleteMachineCommand::help(const vector<string>& /*tokens*/) const
 
 void DeleteMachineCommand::tabCompletion(vector<string>& tokens) const
 {
-	set<string> ids;
-	reactor.getMachineIDs(ids);
-	completeString(tokens, ids);
+	completeString(tokens, reactor.getMachineIDs());
 }
 
 
@@ -822,9 +887,7 @@ ListMachinesCommand::ListMachinesCommand(
 void ListMachinesCommand::execute(const vector<TclObject>& /*tokens*/,
                                   TclObject& result)
 {
-	set<string> ids;
-	reactor.getMachineIDs(ids);
-	result.addListElements(ids.begin(), ids.end());
+	result.addListElements(reactor.getMachineIDs());
 }
 
 string ListMachinesCommand::help(const vector<string>& /*tokens*/) const
@@ -848,7 +911,7 @@ string ActivateMachineCommand::execute(const vector<string>& tokens)
 	case 1:
 		return reactor.getMachineID();
 	case 2: {
-		reactor.switchBoard(reactor.getMachine(tokens[1]));
+		reactor.switchBoard(&reactor.getMachine(tokens[1]));
 		return reactor.getMachineID();
 	}
 	default:
@@ -865,9 +928,7 @@ string ActivateMachineCommand::help(const vector<string>& /*tokens*/) const
 
 void ActivateMachineCommand::tabCompletion(vector<string>& tokens) const
 {
-	set<string> ids;
-	reactor.getMachineIDs(ids);
-	completeString(tokens, ids);
+	completeString(tokens, reactor.getMachineIDs());
 }
 
 
@@ -901,10 +962,10 @@ string StoreMachineCommand::execute(const vector<string>& tokens)
 		throw SyntaxError();
 	}
 
-	Reactor::Board board = reactor.getMachine(machineID);
+	auto& board = reactor.getMachine(machineID);
 
 	XmlOutputArchive out(filename);
-	out.serialize("machine", *board);
+	out.serialize("machine", board);
 	return filename;
 }
 
@@ -920,9 +981,7 @@ string StoreMachineCommand::help(const vector<string>& /*tokens*/) const
 
 void StoreMachineCommand::tabCompletion(vector<string>& tokens) const
 {
-	set<string> ids;
-	reactor.getMachineIDs(ids);
-	completeString(tokens, ids);
+	completeString(tokens, reactor.getMachineIDs());
 }
 
 
@@ -937,7 +996,7 @@ RestoreMachineCommand::RestoreMachineCommand(
 
 string RestoreMachineCommand::execute(const vector<string>& tokens)
 {
-	Reactor::Board newBoard = reactor.createEmptyMotherBoard();
+	auto newBoard = reactor.createEmptyMotherBoard();
 
 	string filename;
 	string machineID;
@@ -987,8 +1046,9 @@ string RestoreMachineCommand::execute(const vector<string>& tokens)
 	// now we want the MSX to see the actual host keyboard state.
 	newBoard->getStateChangeDistributor().stopReplay(newBoard->getCurrentTime());
 
-	reactor.boards.push_back(newBoard);
-	return newBoard->getMachineID();
+	auto result = newBoard->getMachineID();
+	reactor.boards.push_back(move(newBoard));
+	return result;
 }
 
 string RestoreMachineCommand::help(const vector<string>& /*tokens*/) const
@@ -1002,10 +1062,8 @@ string RestoreMachineCommand::help(const vector<string>& /*tokens*/) const
 
 void RestoreMachineCommand::tabCompletion(vector<string>& tokens) const
 {
-	set<string> defaults;
-	// TODO: put the default files in defaults (state files in user's savestates dir)
-	UserFileContext context;
-	completeFileName(tokens, context, defaults);
+	// TODO: add the default files (state files in user's savestates dir)
+	completeFileName(tokens, UserFileContext());
 }
 
 
@@ -1014,7 +1072,7 @@ void RestoreMachineCommand::tabCompletion(vector<string>& tokens) const
 PollEventGenerator::PollEventGenerator(EventDistributor& eventDistributor_)
 	: eventDistributor(eventDistributor_)
 {
-	schedule(20 * 1000); // 50 times per second
+	pollNow();
 }
 
 PollEventGenerator::~PollEventGenerator()
@@ -1022,9 +1080,19 @@ PollEventGenerator::~PollEventGenerator()
 	prepareDelete();
 }
 
+void PollEventGenerator::pollNow()
+{
+	PollEventGenerator::alarm();
+	// The MSX (when not paused) will call this method at 50/60Hz rate.
+	// Though in case the MSX is paused (or emulated very slowly), we
+	// still want to be responsive to host events.
+	schedule(25 * 1000); // 40 times per second (preferably less than 50)
+}
+
 bool PollEventGenerator::alarm()
 {
-	eventDistributor.distributeEvent(new SimpleEvent(OPENMSX_POLL_EVENT));
+	eventDistributor.distributeEvent(
+		make_shared<SimpleEvent>(OPENMSX_POLL_EVENT));
 	return true; // reschedule
 }
 
@@ -1044,27 +1112,17 @@ void ConfigInfo::execute(const vector<TclObject>& tokens,
 	// TODO make meta info available through this info topic
 	switch (tokens.size()) {
 	case 2: {
-		set<string> configs;
-		Reactor::getHwConfigs(configName, configs);
-		result.addListElements(configs.begin(), configs.end());
+		result.addListElements(Reactor::getHwConfigs(configName));
 		break;
 	}
 	case 3: {
 		try {
-			string filename = SystemFileContext().resolve(
-				FileOperations::join(
-					configName, tokens[2].getString(),
-					"hardwareconfig.xml"));
-			std::auto_ptr<XMLElement> config =
-				HardwareConfig::loadConfig(filename);
-			if (XMLElement* info = config->findChild("info")) {
-				const XMLElement::Children& children =
-					info->getChildren();
-				for (XMLElement::Children::const_iterator it =
-					children.begin();
-				     it != children.end(); ++it) {
-					result.addListElement((*it)->getName());
-					result.addListElement((*it)->getData());
+			auto config = HardwareConfig::loadConfig(
+				configName, tokens[2].getString());
+			if (auto* info = config.findChild("info")) {
+				for (auto& i : info->getChildren()) {
+					result.addListElement(i.getName());
+					result.addListElement(i.getData());
 				}
 			}
 		} catch (MSXException& e) {
@@ -1086,9 +1144,7 @@ string ConfigInfo::help(const vector<string>& /*tokens*/) const
 
 void ConfigInfo::tabCompletion(vector<string>& tokens) const
 {
-	set<string> configs;
-	Reactor::getHwConfigs(configName, configs);
-	completeString(tokens, configs);
+	completeString(tokens, Reactor::getHwConfigs(configName));
 }
 
 
@@ -1103,7 +1159,7 @@ RealTimeInfo::RealTimeInfo(InfoCommand& openMSXInfoCommand)
 void RealTimeInfo::execute(const vector<TclObject>& /*tokens*/,
                            TclObject& result) const
 {
-	unsigned long long delta = Timer::getTime() - reference;
+	auto delta = Timer::getTime() - reference;
 	result.setDouble(delta / 1000000.0);
 }
 
