@@ -1,8 +1,7 @@
-// $Id: RenShaTurbo.cc 8352 2008-11-12 18:39:08Z m9710797 $
-
 #include "RenShaTurbo.hh"
 #include "XMLElement.hh"
 #include "Autofire.hh"
+#include "memory.hh"
 
 namespace openmsx {
 
@@ -12,8 +11,8 @@ RenShaTurbo::RenShaTurbo(CommandController& commandController,
 	if (const XMLElement* config = machineConfig.findChild("RenShaTurbo")) {
 		int min_ints = config->getChildDataAsInt("min_ints", 47);
 		int max_ints = config->getChildDataAsInt("max_ints", 221);
-		autofire.reset(new Autofire(commandController, min_ints,
-		                            max_ints, "renshaturbo"));
+		autofire = make_unique<Autofire>(
+			commandController, min_ints, max_ints, "renshaturbo");
 	}
 }
 
@@ -23,7 +22,7 @@ RenShaTurbo::~RenShaTurbo()
 
 bool RenShaTurbo::getSignal(EmuTime::param time)
 {
-	return autofire.get() ? autofire->getSignal(time) : false;
+	return autofire ? autofire->getSignal(time) : false;
 }
 
 } // namespace openmsx
