@@ -1,5 +1,3 @@
-// $Id: ReplayCLI.cc 12628 2012-06-14 20:15:37Z m9710797 $
-
 #include "ReplayCLI.hh"
 #include "CommandLineParser.hh"
 #include "GlobalCommandController.hh"
@@ -11,17 +9,16 @@ using std::string;
 
 namespace openmsx {
 
-ReplayCLI::ReplayCLI(CommandLineParser& commandLineParser)
-	: commandController(commandLineParser.getGlobalCommandController())
+ReplayCLI::ReplayCLI(CommandLineParser& parser_)
+	: parser(parser_)
 {
-	commandLineParser.registerOption("-replay", *this);
-	commandLineParser.registerFileClass("openMSX replay", *this);
+	parser.registerOption("-replay", *this);
+	parser.registerFileType("omr", *this);
 }
 
-bool ReplayCLI::parseOption(const string& option, deque<string>& cmdLine)
+void ReplayCLI::parseOption(const string& option, deque<string>& cmdLine)
 {
 	parseFileType(getArgument(option, cmdLine), cmdLine);
-	return true;
 }
 
 string_ref ReplayCLI::optionHelp() const
@@ -32,7 +29,7 @@ string_ref ReplayCLI::optionHelp() const
 void ReplayCLI::parseFileType(const string& filename,
                                       deque<string>& /*cmdLine*/)
 {
-	TclObject command(commandController.getInterpreter());
+	TclObject command(parser.getGlobalCommandController().getInterpreter());
 	command.addListElement("reverse");
 	command.addListElement("loadreplay");
 	command.addListElement("-viewonly");
