@@ -1,5 +1,3 @@
-// $Id: ReverseManager.cc 12805 2012-08-13 19:26:18Z m9710797 $
-
 #include "ReverseManager.hh"
 #include "MSXMotherBoard.hh"
 #include "EventDistributor.hh"
@@ -8,6 +6,7 @@
 #include "Debugger.hh"
 #include "EventDelay.hh"
 #include "MSXMixer.hh"
+#include "MSXCommandController.hh"
 #include "XMLException.hh"
 #include "XMLElement.hh"
 #include "TclObject.hh"
@@ -22,12 +21,15 @@
 #include "serialize.hh"
 #include "serialize_stl.hh"
 #include "checked_cast.hh"
-#include "ref.hh"
+#include "memory.hh"
+#include "xrange.hh"
+#include <functional>
 #include <cassert>
 
 using std::string;
 using std::vector;
-using std::set;
+using std::shared_ptr;
+using std::move;
 
 namespace openmsx {
 
@@ -40,7 +42,7 @@ static const unsigned MAX_NOF_SNAPSHOTS = 10;
 // Min distance between snapshots in replay (in seconds)
 static const EmuDuration MIN_PARTITION_LENGTH = EmuDuration(60.0);
 
-static const string REPLAY_DIR = "replays";
+static const char* const REPLAY_DIR = "replays";
 
 // A replay is a struct that contains a vector of motherboards and an MSX event
 // log. Those combined are a replay, because you can replay the events from an
@@ -49,8 +51,6 @@ static const string REPLAY_DIR = "replays";
 // merely in-between snapshots, so it is quicker to jump to a later time in the
 // event log.
 
-typedef std::vector<Reactor::Board> MotherBoards;
-
 struct Replay
 {
 	Replay(Reactor& reactor_)
@@ -59,7 +59,7 @@ struct Replay
 	Reactor& reactor;
 
 	ReverseManager::Events* events;
-	MotherBoards motherBoards;
+	std::vector<Reactor::Board> motherBoards;
 	EmuTime currentTime;
 	// this is the amount of times the reverse goto command was used, which
 	// is interesting for the TAS community (see tasvideos.org). It's an
@@ -71,11 +71,11 @@ struct Replay
 	void serialize(Archive& ar, unsigned version)
 	{
 		if (ar.versionAtLeast(version, 2)) {
-			ar.serializeWithID("snapshots", motherBoards, ref(reactor));
+			ar.serializeWithID("snapshots", motherBoards, std::ref(reactor));
 		} else {
 			Reactor::Board newBoard = reactor.createEmptyMotherBoard();
 			ar.serialize("snapshot", *newBoard);
-			motherBoards.push_back(newBoard);
+			motherBoards.push_back(move(newBoard));
 		}
 
 		ar.serialize("events", *events);
@@ -130,10 +130,22 @@ ReverseManager::ReverseChunk::ReverseChunk()
 {
 }
 
-ReverseManager::ReverseChunk::~ReverseChunk()
+ReverseManager::ReverseChunk::ReverseChunk(ReverseChunk&& rhs)
+	: time      (move(rhs.time))
+	, savestate (move(rhs.savestate))
+	, eventCount(move(rhs.eventCount))
 {
 }
 
+ReverseManager::ReverseChunk& ReverseManager::ReverseChunk::operator=(
+	ReverseChunk&& rhs)
+{
+	time       = move(rhs.time);
+	savestate  = move(rhs.savestate);
+	eventCount = move(rhs.eventCount);
+	return *this;
+}
+
 
 class EndLogEvent : public StateChange
 {
@@ -162,9 +174,10 @@ ReverseManager::ReverseManager(MSXMotherBoard& motherBoard_)
 	: Schedulable(motherBoard_.getScheduler())
 	, motherBoard(motherBoard_)
 	, eventDistributor(motherBoard.getReactor().getEventDistributor())
-	, reverseCmd(new ReverseCmd(*this, motherBoard.getCommandController()))
-	, keyboard(0)
-	, eventDelay(0)
+	, reverseCmd(make_unique<ReverseCmd>(
+		*this, motherBoard.getCommandController()))
+	, keyboard(nullptr)
+	, eventDelay(nullptr)
 	, replayIndex(0)
 	, collecting(false)
 	, pendingTakeSnapshot(false)
@@ -238,7 +251,7 @@ void ReverseManager::stop()
 EmuTime::param ReverseManager::getEndTime(const ReverseHistory& history) const
 {
 	if (!history.events.empty()) {
-		if (const EndLogEvent* ev = dynamic_cast<const EndLogEvent*>(
+		if (auto* ev = dynamic_cast<const EndLogEvent*>(
 				history.events.back().get())) {
 			// last log element is EndLogEvent, use that
 			return ev->getTime();
@@ -275,9 +288,8 @@ void ReverseManager::status(TclObject& result) const
 
 	result.addListElement("snapshots");
 	TclObject snapshots;
-	for (Chunks::const_iterator it = history.chunks.begin();
-	     it != history.chunks.end(); ++it) {
-		EmuTime time = it->second.time;
+	for (auto& p : history.chunks) {
+		EmuTime time = p.second.time;
 		snapshots.addListElement((time - EmuTime::zero).toDouble());
 	}
 	result.addListElement(snapshots);
@@ -288,16 +300,15 @@ void ReverseManager::debugInfo(TclObject& result) const
 	// TODO this is useful during development, but for the end user this
 	// information means nothing. We should remove this later.
 	StringOp::Builder res;
-	unsigned totalSize = 0;
-	for (Chunks::const_iterator it = history.chunks.begin();
-	     it != history.chunks.end(); ++it) {
-		const ReverseChunk& chunk = it->second;
-		res << it->first << ' '
+	size_t totalSize = 0;
+	for (auto& p : history.chunks) {
+		auto& chunk = p.second;
+		res << p.first << ' '
 		    << (chunk.time - EmuTime::zero).toDouble() << ' '
 		    << ((chunk.time - EmuTime::zero).toDouble() / (getCurrentTime() - EmuTime::zero).toDouble()) * 100 << '%'
-		    << " (" << chunk.savestate->size() << ')'
+		    << " (" << chunk.savestate.size() << ')'
 		    << " (next event index: " << chunk.eventCount << ")\n";
-		totalSize += chunk.savestate->size();
+		totalSize += chunk.savestate.size();
 	}
 	res << "total size: " << totalSize << '\n';
 	result.setString(string(res));
@@ -307,7 +318,7 @@ static void parseGoTo(const vector<TclObject>& tokens, bool& novideo, double& ti
 {
 	novideo = false;
 	bool hasTime = false;
-	for (unsigned i = 2; i < tokens.size(); ++i) {
+	for (auto i : xrange(size_t(2), tokens.size())) {
 		if (tokens[i].getString() == "-novideo") {
 			novideo = true;
 		} else {
@@ -365,7 +376,7 @@ void ReverseManager::goTo(
 	EmuTime::param target, bool novideo, ReverseHistory& history,
 	bool sameTimeLine)
 {
-	MSXMixer& mixer = motherBoard.getMSXMixer();
+	auto& mixer = motherBoard.getMSXMixer();
 	try {
 		// The call to MSXMotherBoard::fastForward() below may take
 		// some time to execute. The DirectX sound driver has a problem
@@ -379,7 +390,7 @@ void ReverseManager::goTo(
 		// -- Locate destination snapshot --
 		// We can't go back further in the past than the first snapshot.
 		assert(!history.chunks.empty());
-		Chunks::iterator it = history.chunks.begin();
+		auto it = history.chunks.begin();
 		EmuTime firstTime = it->second.time;
 		EmuTime targetTime = std::max(target, firstTime);
 		// Also don't go further into the future than 'end time'.
@@ -419,10 +430,10 @@ void ReverseManager::goTo(
 		// THOUGH only when we're currently in the same time-line
 		//   e.g. OK for a 'reverse goto' command, but not for a
 		//   'reverse loadreplay' command.
-		Reactor& reactor = motherBoard.getReactor();
+		auto& reactor = motherBoard.getReactor();
 		EmuTime currentTime = getCurrentTime();
 		MSXMotherBoard* newBoard;
-		Reactor::Board newBoard_; // either NULL or the same as newBoard
+		Reactor::Board newBoard_; // either nullptr or the same as newBoard
 		if (sameTimeLine &&
 		    (currentTime <= preTarget) &&
 		    ((snapshotTime <= currentTime) ||
@@ -433,8 +444,8 @@ void ReverseManager::goTo(
 			// -- restore old snapshot --
 			newBoard_ = reactor.createEmptyMotherBoard();
 			newBoard = newBoard_.get();
-			MemInputArchive in(it->second.savestate->data(),
-					   it->second.savestate->size());
+			MemInputArchive in(it->second.savestate.data(),
+					   it->second.savestate.size());
 			in.serialize("machine", *newBoard);
 
 			if (eventDelay) {
@@ -447,14 +458,14 @@ void ReverseManager::goTo(
 			// terminate replay log with EndLogEvent (if not there already)
 			if (history.events.empty() ||
 			    !dynamic_cast<const EndLogEvent*>(history.events.back().get())) {
-				history.events.push_back(shared_ptr<StateChange>(
-					new EndLogEvent(currentTime)));
+				history.events.push_back(
+					std::make_shared<EndLogEvent>(currentTime));
 			}
 
 			// Transfer history to the new ReverseManager.
 			// Also we should stop collecting in this ReverseManager,
 			// and start collecting in the new one.
-			ReverseManager& newManager = newBoard->getReverseManager();
+			auto& newManager = newBoard->getReverseManager();
 			newManager.transferHistory(history, it->second.eventCount);
 
 			// transfer (or copy) state from old to new machine
@@ -472,8 +483,10 @@ void ReverseManager::goTo(
 		// switch to the new MSXMotherBoard
 		//  Note: this deletes the current MSXMotherBoard and
 		//  ReverseManager. So we can't access those objects anymore.
-		if (newBoard_.get()) {
-			reactor.replaceBoard(motherBoard, newBoard_);
+		bool unmute = true;
+		if (newBoard_) {
+			unmute = false;
+			reactor.replaceBoard(motherBoard, move(newBoard_));
 		}
 
 		// Fast forward to actual target time with board activated.
@@ -482,7 +495,7 @@ void ReverseManager::goTo(
 
 		// In case we didn't actually create a new board, don't leave
 		// the (old) board muted.
-		if (!newBoard_.get()) {
+		if (unmute) {
 			mixer.unmute();
 		}
 
@@ -498,14 +511,12 @@ void ReverseManager::goTo(
 void ReverseManager::transferState(MSXMotherBoard& newBoard)
 {
 	// Transfer viewonly mode
-	const StateChangeDistributor& oldDistributor =
-		motherBoard.getStateChangeDistributor();
-	StateChangeDistributor& newDistributor =
-		newBoard.getStateChangeDistributor();
+	const auto& oldDistributor = motherBoard.getStateChangeDistributor();
+	      auto& newDistributor = newBoard   .getStateChangeDistributor();
 	newDistributor.setViewOnlyMode(oldDistributor.isViewOnlyMode());
 
 	// transfer keyboard state
-	ReverseManager& newManager = newBoard.getReverseManager();
+	auto& newManager = newBoard.getReverseManager();
 	if (newManager.keyboard && keyboard) {
 		newManager.keyboard->transferHostKeyMatrix(*keyboard);
 	}
@@ -515,11 +526,15 @@ void ReverseManager::transferState(MSXMotherBoard& newBoard)
 
 	// copy rerecord count
 	newManager.reRecordCount = reRecordCount;
+
+	// transfer settings
+	const auto& oldController = motherBoard.getMSXCommandController();
+	newBoard.getMSXCommandController().transferSettings(oldController);
 }
 
 void ReverseManager::saveReplay(const vector<TclObject>& tokens, TclObject& result)
 {
-	const Chunks& chunks = history.chunks;
+	const auto& chunks = history.chunks;
 	if (chunks.empty()) {
 		throw CommandException("No recording...");
 	}
@@ -538,7 +553,7 @@ void ReverseManager::saveReplay(const vector<TclObject>& tokens, TclObject& resu
 	filename = FileOperations::parseCommandFileArgument(
 		filename, REPLAY_DIR, "openmsx", ".omr");
 
-	Reactor& reactor = motherBoard.getReactor();
+	auto& reactor = motherBoard.getReactor();
 	Replay replay(reactor);
 	replay.reRecordCount = reRecordCount;
 
@@ -547,21 +562,21 @@ void ReverseManager::saveReplay(const vector<TclObject>& tokens, TclObject& resu
 	replay.currentTime = getCurrentTime();
 
 	// restore first snapshot to be able to serialize it to a file
-	Reactor::Board initialBoard = reactor.createEmptyMotherBoard();
-	MemInputArchive in(chunks.begin()->second.savestate->data(),
-	                   chunks.begin()->second.savestate->size());
+	auto initialBoard = reactor.createEmptyMotherBoard();
+	MemInputArchive in(chunks.begin()->second.savestate.data(),
+	                   chunks.begin()->second.savestate.size());
 	in.serialize("machine", *initialBoard);
-	replay.motherBoards.push_back(initialBoard);
+	replay.motherBoards.push_back(move(initialBoard));
 
 	// determine which extra snapshots to put in the replay
-	const EmuTime& startTime = chunks.begin()->second.time;
-	const EmuTime& endTime   = chunks.rbegin()->second.time;
+	const auto& startTime = chunks.begin()->second.time;
+	const auto& endTime   = chunks.rbegin()->second.time;
 	EmuDuration totalLength = endTime - startTime;
 	EmuDuration partitionLength = totalLength.divRoundUp(MAX_NOF_SNAPSHOTS);
 	partitionLength = std::max(MIN_PARTITION_LENGTH, partitionLength);
 	EmuTime nextPartitionEnd = startTime + partitionLength;
-	Chunks::const_iterator it = chunks.begin();
-	Chunks::const_iterator lastAddedIt = chunks.begin(); // already added
+	auto it = chunks.begin();
+	auto lastAddedIt = chunks.begin(); // already added
 	while (it != chunks.end()) {
 		++it;
 		if (it == chunks.end() || (it->second.time > nextPartitionEnd)) {
@@ -570,10 +585,10 @@ void ReverseManager::saveReplay(const vector<TclObject>& tokens, TclObject& resu
 			if (it != lastAddedIt) {
 				// this is a new one, add it to the list of snapshots
 				Reactor::Board board = reactor.createEmptyMotherBoard();
-				MemInputArchive in(it->second.savestate->data(),
-				                   it->second.savestate->size());
+				MemInputArchive in(it->second.savestate.data(),
+				                   it->second.savestate.size());
 				in.serialize("machine", *board);
-				replay.motherBoards.push_back(board);
+				replay.motherBoards.push_back(move(board));
 				lastAddedIt = it;
 			}
 			++it;
@@ -589,8 +604,8 @@ void ReverseManager::saveReplay(const vector<TclObject>& tokens, TclObject& resu
 		!dynamic_cast<EndLogEvent*>(history.events.back().get());
 	if (addSentinel) {
 		/// make sure the replay log ends with a EndLogEvent
-		history.events.push_back(shared_ptr<StateChange>(
-			new EndLogEvent(getCurrentTime())));
+		history.events.push_back(std::make_shared<EndLogEvent>(
+			getCurrentTime()));
 	}
 	try {
 		XmlOutputArchive out(filename);
@@ -619,10 +634,10 @@ void ReverseManager::loadReplay(const vector<TclObject>& tokens, TclObject& resu
 	if (tokens.size() < 3) throw SyntaxError();
 
 	vector<string> arguments;
-	const TclObject* whereArg = NULL;
+	const TclObject* whereArg = nullptr;
 	bool enableViewOnly = false;
 
-	for (unsigned i = 2; i < tokens.size(); ++i) {
+	for (size_t i = 2; i < tokens.size(); ++i) {
 		string_ref token = tokens[i].getString();
 		if (token == "-viewonly") {
 			enableViewOnly = true;
@@ -658,7 +673,7 @@ void ReverseManager::loadReplay(const vector<TclObject>& tokens, TclObject& resu
 	}}}
 
 	// restore replay
-	Reactor& reactor = motherBoard.getReactor();
+	auto& reactor = motherBoard.getReactor();
 	Replay replay(reactor);
 	Events events;
 	replay.events = &events;
@@ -672,8 +687,8 @@ void ReverseManager::loadReplay(const vector<TclObject>& tokens, TclObject& resu
 	}
 
 	// get destination time index
-	EmuTime destination = EmuTime::zero;
-	string_ref where = (whereArg == NULL) ? "begin" : whereArg->getString();
+	auto destination = EmuTime::zero;
+	string_ref where = whereArg ? whereArg->getString() : "begin";
 	if (where == "begin") {
 		destination = EmuTime::zero;
 	} else if (where == "end") {
@@ -690,8 +705,8 @@ void ReverseManager::loadReplay(const vector<TclObject>& tokens, TclObject& resu
 	motherBoard.getStateChangeDistributor().setViewOnlyMode(enableViewOnly);
 
 	assert(!replay.motherBoards.empty());
-	ReverseManager& newReverseManager = replay.motherBoards[0]->getReverseManager();
-	ReverseHistory& newHistory = newReverseManager.history;
+	auto& newReverseManager = replay.motherBoards[0]->getReverseManager();
+	auto& newHistory = newReverseManager.history;
 
 	if (newReverseManager.reRecordCount == 0) {
 		// serialize Replay version >= 4
@@ -703,17 +718,16 @@ void ReverseManager::loadReplay(const vector<TclObject>& tokens, TclObject& resu
 
 	// Restore event log
 	swap(newHistory.events, events);
-	Events& newEvents = newHistory.events;
+	auto& newEvents = newHistory.events;
 
 	// Restore snapshots
 	unsigned replayIndex = 0;
-	for (MotherBoards::const_iterator it = replay.motherBoards.begin();
-	     it != replay.motherBoards.end(); ++it) {
+	for (auto& m : replay.motherBoards) {
 		ReverseChunk newChunk;
-		newChunk.time = (*it)->getCurrentTime();
+		newChunk.time = m->getCurrentTime();
 
 		MemOutputArchive out;
-		out.serialize("machine", **it);
+		out.serialize("machine", *m);
 		newChunk.savestate = out.releaseBuffer();
 
 		// update replayIndex
@@ -724,7 +738,8 @@ void ReverseManager::loadReplay(const vector<TclObject>& tokens, TclObject& resu
 		}
 		newChunk.eventCount = replayIndex;
 
-		newHistory.chunks[newHistory.getNextSeqNum(newChunk.time)] = newChunk;
+		newHistory.chunks[newHistory.getNextSeqNum(newChunk.time)] =
+			move(newChunk);
 	}
 
 	// Note: untill this point we didn't make any changes to the current
@@ -789,10 +804,10 @@ void ReverseManager::executeUntil(EmuTime::param /*time*/, int userData)
 		//     should not be *exactly* equally far apart in time.
 		pendingTakeSnapshot = true;
 		eventDistributor.distributeEvent(
-			new SimpleEvent(OPENMSX_TAKE_REVERSE_SNAPSHOT));
+			std::make_shared<SimpleEvent>(OPENMSX_TAKE_REVERSE_SNAPSHOT));
 		break;
 	case INPUT_EVENT:
-		shared_ptr<StateChange> event = history.events[replayIndex];
+		auto event = history.events[replayIndex];
 		try {
 			// deliver current event at current time
 			motherBoard.getStateChangeDistributor().distributeReplay(event);
@@ -829,7 +844,7 @@ unsigned ReverseManager::ReverseHistory::getNextSeqNum(EmuTime::param time) cons
 	if (chunks.empty()) {
 		return 1;
 	}
-	const EmuTime& startTime = chunks.begin()->second.time;
+	const auto& startTime = chunks.begin()->second.time;
 	double duration = (time - startTime).toDouble();
 	return unsigned(duration / SNAPSHOT_PERIOD + 0.5) + 1;
 }
@@ -872,11 +887,7 @@ void ReverseManager::signalStateChange(const shared_ptr<StateChange>& event)
 		// this is an event we just replayed
 		assert(event == history.events[replayIndex]);
 		if (dynamic_cast<EndLogEvent*>(event.get())) {
-			motherBoard.getStateChangeDistributor().stopReplay(
-				event->getTime());
-			// this is needed to prevent a reRecordCount increase
-			// due to this sentinel ending the replay
-			reRecordCount--;
+			signalStopReplay(event->getTime());
 		} else {
 			// ignore all other events
 		}
@@ -888,6 +899,14 @@ void ReverseManager::signalStateChange(const shared_ptr<StateChange>& event)
 	}
 }
 
+void ReverseManager::signalStopReplay(EmuTime::param time)
+{
+	motherBoard.getStateChangeDistributor().stopReplay(time);
+	// this is needed to prevent a reRecordCount increase
+	// due to this action ending the replay
+	reRecordCount--;
+}
+
 void ReverseManager::stopReplay(EmuTime::param time)
 {
 	if (isReplaying()) {
@@ -896,7 +915,7 @@ void ReverseManager::stopReplay(EmuTime::param time)
 		Events& events = history.events;
 		events.erase(events.begin() + replayIndex, events.end());
 		// search snapshots that are newer than 'time' and erase them
-		Chunks::iterator it = history.chunks.begin();
+		auto it = history.chunks.begin();
 		while ((it != history.chunks.end()) &&
 		       (it->second.time <= time)) {
 			++it;
@@ -972,7 +991,7 @@ void ReverseCmd::execute(const vector<TclObject>& tokens, TclObject& result)
 	} else if (subcommand == "loadreplay") {
 		return manager.loadReplay(tokens, result);
 	} else if (subcommand == "viewonlymode") {
-		StateChangeDistributor& distributor = manager.motherBoard.getStateChangeDistributor();
+		auto& distributor = manager.motherBoard.getStateChangeDistributor();
 		switch (tokens.size()) {
 		case 2:
 			result.setString(distributor.isViewOnlyMode() ? "true" : "false");
@@ -983,6 +1002,10 @@ void ReverseCmd::execute(const vector<TclObject>& tokens, TclObject& result)
 		default:
 			throw SyntaxError();
 		}
+	} else if (subcommand == "truncatereplay") {
+		if (manager.isReplaying()) {
+			manager.signalStopReplay(manager.getCurrentTime());
+		}
 	} else {
 		throw CommandException("Invalid subcommand: " + subcommand);
 	}
@@ -996,6 +1019,7 @@ string ReverseCmd::help(const vector<string>& /*tokens*/) const
 	       "goback <n>          go back <n> seconds in time\n"
 	       "goto <time>         go to an absolute moment in time\n"
 	       "viewonlymode <bool> switch viewonly mode on or off\n"
+	       "truncatereplay      stop replaying and remove all 'future' data\n"
 	       "savereplay [<name>] save the first snapshot and all replay data as a 'replay' (with optional name)\n"
 	       "loadreplay [-goto <begin|end|savetime|<n>>] [-viewonly] <name>   load a replay (snapshot and replay data) with given name and start replaying\n";
 }
@@ -1003,29 +1027,23 @@ string ReverseCmd::help(const vector<string>& /*tokens*/) const
 void ReverseCmd::tabCompletion(vector<string>& tokens) const
 {
 	if (tokens.size() == 2) {
-		set<string> subCommands;
-		subCommands.insert("start");
-		subCommands.insert("stop");
-		subCommands.insert("status");
-		subCommands.insert("goback");
-		subCommands.insert("goto");
-		subCommands.insert("savereplay");
-		subCommands.insert("loadreplay");
-		subCommands.insert("viewonlymode");
+		static const char* const subCommands[] = {
+			"start", "stop", "status", "goback", "goto",
+			"savereplay", "loadreplay", "viewonlymode",
+			"truncatereplay",
+		};
 		completeString(tokens, subCommands);
 	} else if ((tokens.size() == 3) || (tokens[1] == "loadreplay")) {
 		if (tokens[1] == "loadreplay" || tokens[1] == "savereplay") {
-			std::set<string> cmds;
+			std::vector<const char*> cmds;
 			if (tokens[1] == "loadreplay") {
-				const char* const str[2] = { "-goto", "-viewonly" };
-				cmds = std::set<string>(str, str + 2);
+				cmds.push_back("-goto");
+				cmds.push_back("-viewonly");
 			}
 			UserDataFileContext context(REPLAY_DIR);
 			completeFileName(tokens, context, cmds);
 		} else if (tokens[1] == "viewonlymode") {
-			set<string> options;
-			options.insert("true");
-			options.insert("false");
+			static const char* const options[] = { "true", "false" };
 			completeString(tokens, options);
 		}
 	}
