@@ -1,5 +1,3 @@
-// $Id: Schedulable.cc 12425 2012-02-29 20:26:04Z m9710797 $
-
 #include "Schedulable.hh"
 #include "Scheduler.hh"
 #include "serialize.hh"
@@ -39,7 +37,7 @@ void Schedulable::removeSyncPoints()
 	scheduler.removeSyncPoints(*this);
 }
 
-bool Schedulable::pendingSyncPoint(int userData)
+bool Schedulable::pendingSyncPoint(int userData) const
 {
 	return scheduler.pendingSyncPoint(*this, userData);
 }
@@ -59,14 +57,13 @@ void Schedulable::serialize(Archive& ar, unsigned /*version*/)
 {
 	Scheduler::SyncPoints syncPoints;
 	if (!ar.isLoader()) {
-		scheduler.getSyncPoints(syncPoints, *this);
+		syncPoints = scheduler.getSyncPoints(*this);
 	}
 	ar.serialize("syncPoints", syncPoints);
 	if (ar.isLoader()) {
 		removeSyncPoints();
-		for (Scheduler::SyncPoints::const_iterator it = syncPoints.begin();
-		     it != syncPoints.end(); ++it) {
-			setSyncPoint(it->getTime(), it->getUserData());
+		for (auto& s : syncPoints) {
+			setSyncPoint(s.getTime(), s.getUserData());
 		}
 	}
 }
