@@ -1,5 +1,3 @@
-// $Id: Scheduler.cc 12425 2012-02-29 20:26:04Z m9710797 $
-
 #include "Scheduler.hh"
 #include "Schedulable.hh"
 #include "Thread.hh"
@@ -11,22 +9,6 @@
 
 namespace openmsx {
 
-// TODO move to utils
-template<typename InputIterator, typename OutputIterator, typename Predicate>
-OutputIterator copy_if(InputIterator begin, InputIterator end,
-                       OutputIterator out, Predicate p)
-{
-	while (begin != end) {
-		if (p(*begin)) {
-			*out = *begin;
-			++out;
-		}
-		++begin;
-	}
-	return out;
-}
-
-
 struct LessSyncPoint {
 	bool operator()(EmuTime::param time,
 	                const SynchronizationPoint& sp) const;
@@ -54,24 +36,29 @@ bool LessSyncPoint::operator()(
 
 
 struct FindSchedulable {
-	explicit FindSchedulable(const Schedulable& schedulable);
-	bool operator()(const SynchronizationPoint& sp) const;
+	explicit FindSchedulable(const Schedulable& schedulable_)
+		: schedulable(schedulable_) {}
+	bool operator()(const SynchronizationPoint& sp) const {
+		return sp.getDevice() == &schedulable;
+	}
 	const Schedulable& schedulable;
 };
-FindSchedulable::FindSchedulable(const Schedulable& schedulable_)
-	: schedulable(schedulable_)
-{
-}
 
-bool FindSchedulable::operator()(const SynchronizationPoint& sp) const
-{
-	return sp.getDevice() == &schedulable;
-}
+struct EqualSchedulable {
+	EqualSchedulable(const Schedulable& schedulable_, int userdata_)
+		: schedulable(schedulable_), userdata(userdata_) {}
+	bool operator()(const SynchronizationPoint& sp) const {
+		return (sp.getDevice() == &schedulable) &&
+		       (sp.getUserData() == userdata);
+	}
+	const Schedulable& schedulable;
+	int userdata;
+};
 
 
 Scheduler::Scheduler()
 	: scheduleTime(EmuTime::zero)
-	, cpu(0)
+	, cpu(nullptr)
 	, scheduleInProgress(false)
 {
 }
@@ -79,10 +66,9 @@ Scheduler::Scheduler()
 Scheduler::~Scheduler()
 {
 	assert(!cpu);
-	SyncPoints copy(syncPoints);
-	for (SyncPoints::const_iterator it = copy.begin();
-	     it != copy.end(); ++it) {
-		it->getDevice()->schedulerDeleted();
+	auto copy = syncPoints;
+	for (auto& s : copy) {
+		s.getDevice()->schedulerDeleted();
 	}
 
 	assert(syncPoints.empty());
@@ -94,9 +80,8 @@ void Scheduler::setSyncPoint(EmuTime::param time, Schedulable& device, int userD
 	assert(time >= scheduleTime);
 
 	// Push sync point into queue.
-	SyncPoints::iterator it =
-		std::upper_bound(syncPoints.begin(), syncPoints.end(), time,
-		            LessSyncPoint());
+	auto it = std::upper_bound(syncPoints.begin(), syncPoints.end(), time,
+	                           LessSyncPoint());
 	syncPoints.insert(it, SynchronizationPoint(time, &device, userData));
 
 	if (!scheduleInProgress && cpu) {
@@ -107,24 +92,25 @@ void Scheduler::setSyncPoint(EmuTime::param time, Schedulable& device, int userD
 	}
 }
 
-void Scheduler::getSyncPoints(SyncPoints& result, const Schedulable& device) const
+Scheduler::SyncPoints Scheduler::getSyncPoints(const Schedulable& device) const
 {
-	openmsx::copy_if(syncPoints.begin(), syncPoints.end(), back_inserter(result),
-	                 FindSchedulable(device));
+	SyncPoints result;
+	copy_if(syncPoints.begin(), syncPoints.end(), back_inserter(result),
+	        FindSchedulable(device));
+	return result;
 }
 
 bool Scheduler::removeSyncPoint(Schedulable& device, int userData)
 {
 	assert(Thread::isMainThread());
-	for (SyncPoints::iterator it = syncPoints.begin();
-	     it != syncPoints.end(); ++it) {
-		if (((*it).getDevice() == &device) &&
-		    ((*it).getUserData() == userData)) {
-			syncPoints.erase(it);
-			return true;
-		}
+	auto it = find_if(syncPoints.begin(), syncPoints.end(),
+	                  EqualSchedulable(device, userData));
+	if (it != syncPoints.end()) {
+		syncPoints.erase(it);
+		return true;
+	} else {
+		return false;
 	}
-	return false; // nothing removed
 }
 
 void Scheduler::removeSyncPoints(Schedulable& device)
@@ -135,17 +121,11 @@ void Scheduler::removeSyncPoints(Schedulable& device)
 	                 syncPoints.end());
 }
 
-bool Scheduler::pendingSyncPoint(Schedulable& device, int userData)
+bool Scheduler::pendingSyncPoint(const Schedulable& device, int userData) const
 {
 	assert(Thread::isMainThread());
-	for (SyncPoints::iterator it = syncPoints.begin();
-	     it != syncPoints.end(); ++it) {
-		if ((it->getDevice() == &device) &&
-		    (it->getUserData() == userData)) {
-			return true;
-		}
-	}
-	return false;
+	return find_if(syncPoints.begin(), syncPoints.end(),
+	               EqualSchedulable(device, userData)) != syncPoints.end();
 }
 
 EmuTime::param Scheduler::getCurrentTime() const
@@ -160,11 +140,8 @@ void Scheduler::scheduleHelper(EmuTime::param limit)
 	scheduleInProgress = true;
 	while (true) {
 		// Get next sync point.
-		const SynchronizationPoint sp =
-			  syncPoints.empty()
-			? SynchronizationPoint(EmuTime::infinity, NULL, 0)
-			: syncPoints.front();
-		EmuTime::param time = sp.getTime();
+		const auto& sp = syncPoints.front();
+		EmuTime time = sp.getTime();
 		if (time > limit) {
 			break;
 		}
@@ -172,11 +149,12 @@ void Scheduler::scheduleHelper(EmuTime::param limit)
 		assert(scheduleTime <= time);
 		scheduleTime = time;
 
-		syncPoints.erase(syncPoints.begin());
-
 		Schedulable* device = sp.getDevice();
 		assert(device);
 		int userData = sp.getUserData();
+
+		syncPoints.erase(syncPoints.begin());
+
 		device->executeUntil(time, userData);
 	}
 	scheduleInProgress = false;
