@@ -1,17 +1,17 @@
-// $Id: ThrottleManager.cc 8194 2008-09-02 18:17:55Z m9710797 $
-
 #include "ThrottleManager.hh"
 #include "BooleanSetting.hh"
+#include "memory.hh"
 
 namespace openmsx {
 
 // class ThrottleManager:
 
 ThrottleManager::ThrottleManager(CommandController& commandController)
-	: throttleSetting(new BooleanSetting(commandController, "throttle",
+	: throttleSetting(make_unique<BooleanSetting>(
+		commandController, "throttle",
 		"controls speed throttling", true, Setting::DONT_SAVE))
-	, fullSpeedLoadingSetting(new BooleanSetting(commandController,
-		"fullspeedwhenloading",
+	, fullSpeedLoadingSetting(make_unique<BooleanSetting>(
+		commandController, "fullspeedwhenloading",
 		"sets openMSX to full speed when the MSX is loading", false))
 	, loading(0), throttle(true)
 {
@@ -27,8 +27,8 @@ ThrottleManager::~ThrottleManager()
 
 void ThrottleManager::updateStatus()
 {
-	bool newThrottle = throttleSetting->getValue() &&
-	                   (!loading || !fullSpeedLoadingSetting->getValue());
+	bool newThrottle = throttleSetting->getBoolean() &&
+	                   (!loading || !fullSpeedLoadingSetting->getBoolean());
 	if (throttle != newThrottle) {
 		throttle = newThrottle;
 		notify();
