@@ -1,21 +1,25 @@
-// $Id: CasImage.cc 12309 2011-11-11 18:14:12Z manuelbi $
-
 #include "CasImage.hh"
 #include "File.hh"
 #include "Filename.hh"
 #include "CliComm.hh"
 #include "Clock.hh"
 #include "MSXException.hh"
+#include "xrange.hh"
 #include <cstring> // for memcmp
 
 namespace openmsx {
 
 // output settings
 
-// 4.6 times normal speed (5520 baud, higher doesn't work anymore, possibly
-// because the BIOS code (and Z80) is too slow for that)
-static const unsigned FACTOR = 46;
-static const unsigned OUTPUT_FREQUENCY = 2400 * FACTOR / 10;
+// a higher baudrate doesn't work anymore, but it is unclear why, because 4600
+// baud should work (known from Speedsave 4000 and Turbo 5000 programs).
+// 3765 still works on a Toshiba HX-10 and Philips NMS 8250, but not on a
+// Panasonic FS-A1WSX, on which 3763 is the max. National CF-2000 has 3762 as
+// the max. Let's take 3760 then as a safe value.
+// UPDATE: that seems to break RUN"CAS:" type of programs. 3744 seems to work
+// for those as well (we don't understand why yet)
+static const unsigned BAUDRATE = 3744;
+static const unsigned OUTPUT_FREQUENCY = 4 * BAUDRATE; // 4 samples per bit
 // We oversample the audio signal for better sound quality (especially in
 // combination with the hq resampler). Without oversampling the audio output
 // could contain portions like this:
@@ -72,13 +76,14 @@ void CasImage::fillBuffer(unsigned pos, int** bufs, unsigned num) const
 {
 	size_t nbSamples = output.size();
 	if ((pos / AUDIO_OVERSAMPLE) < nbSamples) {
-		for (unsigned i = 0; i < num; ++i, ++pos) {
+		for (auto i : xrange(num)) {
 			bufs[0][i] = ((pos / AUDIO_OVERSAMPLE) < nbSamples)
 			           ? output[pos / AUDIO_OVERSAMPLE] * 256
 			           : 0;
+			++pos;
 		}
 	} else {
-		bufs[0] = 0;
+		bufs[0] = nullptr;
 	}
 }
 
@@ -117,7 +122,7 @@ void CasImage::writeByte(byte b)
 	// one start bit
 	write0();
 	// eight data bits
-	for (int i = 0; i < 8; ++i) {
+	for (auto i : xrange(8)) {
 		if (b & (1 << i)) {
 			write1();
 		} else {
@@ -130,7 +135,7 @@ void CasImage::writeByte(byte b)
 }
 
 // write data until a header is detected
-bool CasImage::writeData(const byte* buf, const unsigned size, unsigned& pos)
+bool CasImage::writeData(const byte* buf, size_t size, size_t& pos)
 {
 	bool eof = false;
 	while ((pos + 8) <= size) {
@@ -152,14 +157,14 @@ bool CasImage::writeData(const byte* buf, const unsigned size, unsigned& pos)
 void CasImage::convert(const Filename& filename, FilePool& filePool, CliComm& cliComm)
 {
 	File file(filename);
-	unsigned size;
+	size_t size;
 	const byte* buf = file.mmap(size);
 
 	// search for a header in the .cas file
 	bool issueWarning = false;
 	bool headerFound = false;
 	bool firstFile = true;
-	unsigned pos = 0;
+	size_t pos = 0;
 	while ((pos + 8) <= size) {
 		if (!memcmp(&buf[pos], CAS_HEADER, 8)) {
 			// it probably works fine if a long header is used for every
