@@ -1,5 +1,3 @@
-// $Id: CassettePlayer.cc 12651 2012-06-19 20:32:20Z manuelbi $
-
 // TODO:
 // - improve consistency when a reset occurs: tape is removed when you were
 //   recording, but it is not removed when you were playing
@@ -51,13 +49,13 @@
 #include "StringOp.hh"
 #include "serialize.hh"
 #include "unreachable.hh"
+#include "memory.hh"
 #include <algorithm>
 #include <cassert>
 
-using std::auto_ptr;
+using std::unique_ptr;
 using std::string;
 using std::vector;
-using std::set;
 
 namespace openmsx {
 
@@ -88,9 +86,8 @@ private:
 static XMLElement createXML()
 {
 	XMLElement xml("cassetteplayer");
-	auto_ptr<XMLElement> sound(new XMLElement("sound"));
-	sound->addChild(auto_ptr<XMLElement>(new XMLElement("volume", "5000")));
-	xml.addChild(sound);
+	xml.addChild(XMLElement("sound"))
+	        .addChild(XMLElement("volume", "5000"));
 	return xml;
 }
 
@@ -101,12 +98,13 @@ CassettePlayer::CassettePlayer(const HardwareConfig& hwConf)
 	, prevSyncTime(EmuTime::zero)
 	, audioPos(0)
 	, motherBoard(hwConf.getMotherBoard())
-	, tapeCommand(new TapeCommand(motherBoard.getCommandController(),
-	                              motherBoard.getStateChangeDistributor(),
-	                              motherBoard.getScheduler(), *this))
-	, loadingIndicator(new LoadingIndicator(
+	, tapeCommand(make_unique<TapeCommand>(
+		motherBoard.getCommandController(),
+		motherBoard.getStateChangeDistributor(),
+		motherBoard.getScheduler(), *this))
+	, loadingIndicator(make_unique<LoadingIndicator>(
 		motherBoard.getReactor().getGlobalSettings().getThrottleManager()))
-	, autoRunSetting(new BooleanSetting(
+	, autoRunSetting(make_unique<BooleanSetting>(
 		motherBoard.getCommandController(),
 		"autoruncassettes", "automatically try to run cassettes", true))
 	, sampcnt(0)
@@ -140,38 +138,67 @@ CassettePlayer::~CassettePlayer()
 
 void CassettePlayer::autoRun()
 {
-	if (!playImage.get()) return;
+	if (!playImage) return;
 
 	// try to automatically run the tape, if that's set
 	CassetteImage::FileType type = playImage->getFirstFileType();
-	if (!autoRunSetting->getValue() || type == CassetteImage::UNKNOWN) {
+	if (!autoRunSetting->getBoolean() || type == CassetteImage::UNKNOWN) {
 		return;
 	}
-	string loadingInstruction;
+	string instr1, instr2;
 	switch (type) {
 		case CassetteImage::ASCII:
-			loadingInstruction = "RUN\\\"CAS:\\\"";
+			instr1 = "RUN\\\"CAS:\\\"";
 			break;
 		case CassetteImage::BINARY:
-			loadingInstruction = "BLOAD\\\"CAS:\\\",R";
+			instr1 = "BLOAD\\\"CAS:\\\",R";
 			break;
 		case CassetteImage::BASIC:
-			loadingInstruction = "CLOAD\\\\rRUN";
+			// Note that CLOAD:RUN won't work: BASIC ignores stuff
+			// after the CLOAD command. That's why it's split in two.
+			instr1 = "CLOAD";
+			instr2 = "RUN";
 			break;
 		default:
 			UNREACHABLE; // Shouldn't be possible
 	}
-	string var = "::auto_run_cas_counter";
 	string command =
-		"if ![info exists " + var + "] { set " + var + " 0 }\n"
-		"incr " + var + "\n"
-		"after time 2 \"if $" + var + "==\\$" + var + " { "
-		"type " + loadingInstruction + "\\\\r }\"";
+		"namespace eval ::openmsx {\n"
+		"  variable auto_run_bp\n"
+
+		"  proc auto_run_cb {args} {\n"
+		"    variable auto_run_bp\n"
+		"    debug remove_bp $auto_run_bp\n"
+		"    unset auto_run_bp\n"
+
+		// Without the 0.1s delay here, the type command gets messed up
+		// on MSX1 machines for some reason (starting to type too early?)
+		"    after time 0.1 \"type [lindex $args 0]\\\\r\"\n"
+
+		"    set next [lrange $args 1 end]\n"
+		"    if {[llength $next] == 0} return\n"
+
+		// H_READ isn't called after CLOAD, but H_MAIN is. However, it's
+		// also called right after H_READ, so we wait a little before
+		// setting up the breakpoint.
+		"    set cmd1 \"openmsx::auto_run_cb $next\"\n"
+		"    set cmd2 \"set openmsx::auto_run_bp \\[debug set_bp 0xFF0C 1 \\\"$cmd1\\\"\\]\"\n" // H_MAIN
+		"    after time 0.2 $cmd2\n"
+		"  }\n"
+
+		"  if {[info exists auto_run_bp]} {debug remove_bp $auto_run_bp\n}\n"
+		"  set auto_run_bp [debug set_bp 0xFF07 1 {\n" // H_READ
+		"    openmsx::auto_run_cb " + instr1 + " " + instr2 + "\n"
+		"  }]\n"
+
+		// re-trigger hook(s), needed when already booted in BASIC
+		"  type \'\\r\n"
+		"}";
 	try {
 		motherBoard.getCommandController().executeCommand(command);
 	} catch (CommandException& e) {
 		motherBoard.getMSXCliComm().printWarning(
-			"Error executing loading instruction for AutoRun: " +
+			"Error executing loading instruction using command \"" + command + "\" for AutoRun: " +
 			e.getMessage() + "\n Please report a bug.");
 	}
 }
@@ -209,7 +236,7 @@ double CassettePlayer::getTapePos(EmuTime::param time)
 
 double CassettePlayer::getTapeLength(EmuTime::param time)
 {
-	if (playImage.get()) {
+	if (playImage) {
 		return (playImage->getEndTime() - EmuTime::zero).toDouble();
 	} else if (getState() == RECORD) {
 		return getTapePos(time);
@@ -222,8 +249,8 @@ void CassettePlayer::checkInvariants() const
 {
 	switch (getState()) {
 	case STOP:
-		assert(!recordImage.get());
-		if (playImage.get()) {
+		assert(!recordImage);
+		if (playImage) {
 			// we're at end-of tape
 			assert(!getImageName().empty());
 		} else {
@@ -232,13 +259,13 @@ void CassettePlayer::checkInvariants() const
 		break;
 	case PLAY:
 		assert(!getImageName().empty());
-		assert(!recordImage.get());
-		assert(playImage.get());
+		assert(!recordImage);
+		assert(playImage);
 		break;
 	case RECORD:
 		assert(!getImageName().empty());
-		assert(recordImage.get());
-		assert(!playImage.get());
+		assert(recordImage);
+		assert(!playImage);
 		break;
 	default:
 		UNREACHABLE;
@@ -260,9 +287,10 @@ void CassettePlayer::setState(State newState, const Filename& newImage,
 	assert(!((oldState == RECORD) && (newState == PLAY)));
 
 	// stuff for leaving the old state
-	if (oldState == RECORD) {
+	//  'recordImage==nullptr' can happen in case of loadstate.
+	if ((oldState == RECORD) && recordImage) {
 		flushOutput();
-		bool empty = recordImage.get()->isEmpty();
+		bool empty = recordImage->isEmpty();
 		recordImage.reset();
 		if (empty) {
 			// delete the created WAV file, as it is useless
@@ -320,12 +348,13 @@ void CassettePlayer::insertTape(const Filename& filename)
 		FilePool& filePool = motherBoard.getReactor().getFilePool();
 		try {
 			// first try WAV
-			playImage.reset(new WavImage(filename, filePool));
+			playImage = make_unique<WavImage>(filename, filePool);
 		} catch (MSXException& e) {
 			try {
 				// if that fails use CAS
-				playImage.reset(new CasImage(
-					filename, filePool, motherBoard.getMSXCliComm()));
+				playImage = make_unique<CasImage>(
+					filename, filePool,
+					motherBoard.getMSXCliComm());
 			} catch (MSXException& e2) {
 				throw MSXException(
 					"Failed to insert WAV image: \"" +
@@ -345,8 +374,7 @@ void CassettePlayer::insertTape(const Filename& filename)
 	}
 
 	// possibly recreate resampler
-	unsigned inputRate = playImage.get() ? playImage->getFrequency()
-	                                     : 44100;
+	unsigned inputRate = playImage ? playImage->getFrequency() : 44100;
 	if (inputRate != getInputRate()) {
 		setInputRate(inputRate);
 		createResampler();
@@ -385,7 +413,7 @@ void CassettePlayer::rewind(EmuTime::param time)
 void CassettePlayer::recordTape(const Filename& filename, EmuTime::param time)
 {
 	removeTape(time); // flush (possible) previous recording
-	recordImage.reset(new Wav8Writer(filename, 1, RECORD_FREQ));
+	recordImage = make_unique<Wav8Writer>(filename, 1, RECORD_FREQ);
 	tapePos = EmuTime::zero;
 	setState(RECORD, filename, time);
 }
@@ -459,7 +487,7 @@ void CassettePlayer::updateTapePosition(
 
 void CassettePlayer::generateRecordOutput(EmuDuration::param duration)
 {
-	if (!recordImage.get() || !isRolling()) return;
+	if (!recordImage || !isRolling()) return;
 
 	double out = lastOutput ? OUTPUT_AMP : -OUTPUT_AMP;
 	double samples = duration.toDouble() * RECORD_FREQ;
@@ -488,7 +516,7 @@ void CassettePlayer::generateRecordOutput(EmuDuration::param duration)
 
 void CassettePlayer::fillBuf(size_t length, double x)
 {
-	assert(recordImage.get());
+	assert(recordImage);
 	static const double A = 252.0 / 256.0;
 
 	double y = lastY + (x - lastX);
@@ -553,7 +581,7 @@ void CassettePlayer::unplugHelper(EmuTime::param time)
 void CassettePlayer::generateChannels(int** buffers, unsigned num)
 {
 	if ((getState() != PLAY) || !isRolling()) {
-		buffers[0] = 0;
+		buffers[0] = nullptr;
 		return;
 	}
 	// Note: fillBuffer() replaces the values in the buffer. It should add
@@ -564,7 +592,7 @@ void CassettePlayer::generateChannels(int** buffers, unsigned num)
 }
 
 
-int CassettePlayer::signalEvent(const shared_ptr<const Event>& event)
+int CassettePlayer::signalEvent(const std::shared_ptr<const Event>& event)
 {
 	if (event->getType() == OPENMSX_BOOT_EVENT) {
 		if (!getImageName().empty()) {
@@ -648,8 +676,7 @@ string TapeCommand::execute(const vector<string>& tokens, EmuTime::param time)
 	} else if (tokens[1] == "insert" && tokens.size() == 3) {
 		try {
 			result << "Changing tape";
-			UserFileContext context;
-			Filename filename(tokens[2], context);
+			Filename filename(tokens[2], UserFileContext());
 			cassettePlayer.playTape(filename, time);
 		} catch (MSXException& e) {
 			throw CommandException(e.getMessage());
@@ -718,8 +745,7 @@ string TapeCommand::execute(const vector<string>& tokens, EmuTime::param time)
 	} else {
 		try {
 			result << "Changing tape";
-			UserFileContext context;
-			Filename filename(tokens[1], context);
+			Filename filename(tokens[1], UserFileContext());
 			cassettePlayer.playTape(filename, time);
 		} catch (MSXException& e) {
 			throw CommandException(e.getMessage());
@@ -812,25 +838,16 @@ string TapeCommand::help(const vector<string>& tokens) const
 void TapeCommand::tabCompletion(vector<string>& tokens) const
 {
 	if (tokens.size() == 2) {
-		set<string> extra;
-		extra.insert("eject");
-		extra.insert("rewind");
-		extra.insert("motorcontrol");
-		extra.insert("insert");
-		extra.insert("new");
-		extra.insert("play");
-	//	extra.insert("record");
-		extra.insert("getpos");
-		extra.insert("getlength");
-		UserFileContext context;
-		completeFileName(tokens, context, extra);
+		static const char* const cmds[] = {
+			"eject", "rewind", "motorcontrol", "insert", "new",
+			"play", "getpos", "getlength",
+			//"record",
+		};
+		completeFileName(tokens, UserFileContext(), cmds);
 	} else if ((tokens.size() == 3) && (tokens[1] == "insert")) {
-		UserFileContext context;
-		completeFileName(tokens, context);
+		completeFileName(tokens, UserFileContext());
 	} else if ((tokens.size() == 3) && (tokens[1] == "motorcontrol")) {
-		set<string> extra;
-		extra.insert("on");
-		extra.insert("off");
+		static const char* const extra[] = { "on", "off" };
 		completeString(tokens, extra);
 	}
 }
@@ -853,7 +870,7 @@ SERIALIZE_ENUM(CassettePlayer::State, stateInfo);
 template<typename Archive>
 void CassettePlayer::serialize(Archive& ar, unsigned version)
 {
-	if (recordImage.get()) {
+	if (recordImage) {
 		// buf, sampcnt
 		flushOutput();
 	}
@@ -861,7 +878,7 @@ void CassettePlayer::serialize(Archive& ar, unsigned version)
 	ar.serialize("casImage", casImage);
 
 	Sha1Sum oldChecksum;
-	if (!ar.isLoader() && playImage.get()) {
+	if (!ar.isLoader() && playImage) {
 		oldChecksum = playImage->getSha1Sum();
 	}
 	if (ar.versionAtLeast(version, 2)) {
@@ -880,15 +897,30 @@ void CassettePlayer::serialize(Archive& ar, unsigned version)
 		casImage.updateAfterLoadState();
 		if (!oldChecksum.empty() &&
 		    !FileOperations::exists(casImage.getResolved())) {
-			std::auto_ptr<File> file = filePool.getFile(
-				FilePool::TAPE, oldChecksum);
-			if (file.get()) {
+			if (auto file = filePool.getFile(
+			                     FilePool::TAPE, oldChecksum)) {
 				casImage.setResolved(file->getURL());
 			}
 		}
-		insertTape(casImage);
+		try {
+			insertTape(casImage);
+		} catch (MSXException&) {
+			if (oldChecksum.empty()) {
+				// It's OK if we cannot reinsert an empty
+				// image. One likely scenario for this case is
+				// the following:
+				//  - cassetteplayer new myfile.wav
+				//  - don't actually start saving to tape yet
+				//  - create a savestate and load that state
+				// Because myfile.wav contains no data yet, it
+				// is deleted from the filesystem. So on a
+				// loadstate it won't be found.
+			} else {
+				throw;
+			}
+		}
 
-		if (playImage.get() && !oldChecksum.empty()) {
+		if (playImage && !oldChecksum.empty()) {
 			Sha1Sum newChecksum = playImage->getSha1Sum();
 			if (oldChecksum != newChecksum) {
 				motherBoard.getMSXCliComm().printWarning(
@@ -906,7 +938,7 @@ void CassettePlayer::serialize(Archive& ar, unsigned version)
 	//double lastY;
 	//double partialOut;
 	//double partialInterval;
-	//std::auto_ptr<WavWriter> recordImage;
+	//std::unique_ptr<WavWriter> recordImage;
 
 	ar.serialize("tapePos", tapePos);
 	ar.serialize("prevSyncTime", prevSyncTime);
@@ -917,8 +949,26 @@ void CassettePlayer::serialize(Archive& ar, unsigned version)
 	ar.serialize("motorControl", motorControl);
 
 	if (ar.isLoader()) {
+		if (playImage && (tapePos > playImage->getEndTime())) {
+			tapePos = playImage->getEndTime();
+			motherBoard.getMSXCliComm().printWarning("Tape position  "
+				"beyond tape end! Setting tape position to end. "
+				"This can happen if you load a replay from an "
+				"older openMSX version with a different CAS-to-WAV "
+				"baud rate or when the tape image has been changed "
+				"compared to when the replay was created.");
+		}
 		if (state == RECORD) {
 			// TODO we don't support savestates in RECORD mode yet
+			motherBoard.getMSXCliComm().printWarning(
+				"Restoring a state where the MSX was saving to "
+				"tape is not yet supported. Emulation will "
+				"continue without actually saving.");
+			setState(STOP, getImageName(), getCurrentTime());
+		}
+		if (!playImage && (state == PLAY)) {
+			// This should only happen for manually edited
+			// savestates, though we shouldn't crash on it.
 			setState(STOP, getImageName(), getCurrentTime());
 		}
 		updateLoadingState(getCurrentTime());
