@@ -1,5 +1,3 @@
-// $Id: CassettePlayerCLI.cc 12628 2012-06-14 20:15:37Z m9710797 $
-
 #include "CassettePlayerCLI.hh"
 #include "CommandLineParser.hh"
 #include "GlobalCommandController.hh"
@@ -11,17 +9,16 @@ using std::string;
 
 namespace openmsx {
 
-CassettePlayerCLI::CassettePlayerCLI(CommandLineParser& commandLineParser)
-	: commandController(commandLineParser.getGlobalCommandController())
+CassettePlayerCLI::CassettePlayerCLI(CommandLineParser& parser_)
+	: parser(parser_)
 {
-	commandLineParser.registerOption("-cassetteplayer", *this);
-	commandLineParser.registerFileClass("cassetteimage", *this);
+	parser.registerOption("-cassetteplayer", *this);
+	parser.registerFileType("cas,wav", *this);
 }
 
-bool CassettePlayerCLI::parseOption(const string& option, deque<string>& cmdLine)
+void CassettePlayerCLI::parseOption(const string& option, deque<string>& cmdLine)
 {
 	parseFileType(getArgument(option, cmdLine), cmdLine);
-	return true;
 }
 
 string_ref CassettePlayerCLI::optionHelp() const
@@ -33,10 +30,10 @@ string_ref CassettePlayerCLI::optionHelp() const
 void CassettePlayerCLI::parseFileType(const string& filename,
                                       deque<string>& /*cmdLine*/)
 {
-	if (!commandController.hasCommand("cassetteplayer")) {
+	if (!parser.getGlobalCommandController().hasCommand("cassetteplayer")) {
 		throw MSXException("No cassetteplayer.");
 	}
-	TclObject command(commandController.getInterpreter());
+	TclObject command(parser.getGlobalCommandController().getInterpreter());
 	command.addListElement("cassetteplayer");
 	command.addListElement(filename);
 	command.executeCommand();
