@@ -1,5 +1,3 @@
-// $Id: CassettePort.cc 12629 2012-06-14 20:16:30Z m9710797 $
-
 #include "CassettePort.hh"
 #include "CassetteDevice.hh"
 #include "CassettePlayer.hh"
@@ -13,9 +11,9 @@
 #include "PluggingController.hh"
 #include "checked_cast.hh"
 #include "serialize.hh"
-#include <memory>
+#include "memory.hh"
 
-using std::auto_ptr;
+using std::unique_ptr;
 using std::string;
 
 namespace openmsx {
@@ -57,16 +55,16 @@ void DummyCassettePort::setLaserdiscPlayer(LaserdiscPlayer* /* laserdisc */)
 
 CassettePort::CassettePort(const HardwareConfig& hwConf)
 	: Connector(hwConf.getMotherBoard().getPluggingController(), "cassetteport",
-	            auto_ptr<Pluggable>(new DummyCassetteDevice()))
+	            make_unique<DummyCassetteDevice>())
 	, motherBoard(hwConf.getMotherBoard())
 #if COMPONENT_LASERDISC
-	, laserdiscPlayer(NULL)
+	, laserdiscPlayer(nullptr)
 #endif
 	, lastOutput(false)
 	, motorControl(false)
 {
-	getPluggingController().registerPluggable(auto_ptr<Pluggable>(
-		new CassettePlayer(hwConf)));
+	getPluggingController().registerPluggable(
+		make_unique<CassettePlayer>(hwConf));
 }
 
 CassettePort::~CassettePort()
@@ -101,7 +99,7 @@ bool CassettePort::cassetteIn(EmuTime::param time)
 	//   we just assume sample has no DC component
 	short sample;
 #if COMPONENT_LASERDISC
-	if (!motorControl && laserdiscPlayer != NULL) {
+	if (!motorControl && laserdiscPlayer) {
 		sample = laserdiscPlayer->readSample(time);
 	} else
 #endif
