@@ -1,5 +1,3 @@
-// $Id: GlobalCommandController.cc 12805 2012-08-13 19:26:18Z m9710797 $
-
 #include "GlobalCommandController.hh"
 #include "Command.hh"
 #include "Setting.hh"
@@ -19,12 +17,14 @@
 #include "TclObject.hh"
 #include "Version.hh"
 #include "ScopedAssign.hh"
+#include "StringOp.hh"
 #include "checked_cast.hh"
 #include "openmsx.hh"
+#include "memory.hh"
+#include "xrange.hh"
 #include <cassert>
 #include <cstdlib>
 
-using std::set;
 using std::string;
 using std::vector;
 
@@ -64,6 +64,15 @@ private:
 	CliConnection& getConnection();
 };
 
+class PlatformInfo : public InfoTopic
+{
+public:
+	explicit PlatformInfo(InfoCommand& openMSXInfoCommand);
+	virtual void execute(const vector<TclObject>& tokens,
+	                     TclObject& result) const;
+	virtual string help(const vector<string>& tokens) const;
+};
+
 class VersionInfo : public InfoTopic
 {
 public:
@@ -78,16 +87,17 @@ GlobalCommandController::GlobalCommandController(
 	EventDistributor& eventDistributor,
 	GlobalCliComm& cliComm_, Reactor& reactor_)
 	: cliComm(cliComm_)
-	, connection(NULL)
+	, connection(nullptr)
 	, reactor(reactor_)
-	, interpreter(new Interpreter(eventDistributor))
-	, openMSXInfoCommand(new InfoCommand(*this, "openmsx_info"))
-	, hotKey(new HotKey(*this, eventDistributor))
-	, helpCmd(new HelpCmd(*this))
-	, tabCompletionCmd(new TabCompletionCmd(*this))
-	, proxyCmd(new ProxyCmd(*this, reactor))
-	, versionInfo(new VersionInfo(getOpenMSXInfoCommand()))
-	, romInfoTopic(new RomInfoTopic(getOpenMSXInfoCommand()))
+	, interpreter(make_unique<Interpreter>(eventDistributor))
+	, openMSXInfoCommand(make_unique<InfoCommand>(*this, "openmsx_info"))
+	, hotKey(make_unique<HotKey>(*this, eventDistributor))
+	, helpCmd(make_unique<HelpCmd>(*this))
+	, tabCompletionCmd(make_unique<TabCompletionCmd>(*this))
+	, proxyCmd(make_unique<ProxyCmd>(*this, reactor))
+	, platformInfo(make_unique<PlatformInfo>(getOpenMSXInfoCommand()))
+	, versionInfo(make_unique<VersionInfo>(getOpenMSXInfoCommand()))
+	, romInfoTopic(make_unique<RomInfoTopic>(getOpenMSXInfoCommand()))
 {
 	// For backwards compatibility:
 	//  In the past we had an openMSX command 'update'. This was a mistake
@@ -114,7 +124,7 @@ GlobalCommandController::GlobalCommandController(
 		"        eval \"openmsx_update $args\"\n"
 		"    }\n"
 		"}\n");
-	updateCmd.reset(new UpdateCmd(*this));
+	updateCmd = make_unique<UpdateCmd>(*this);
 }
 
 GlobalCommandController::~GlobalCommandController()
@@ -123,6 +133,7 @@ GlobalCommandController::~GlobalCommandController()
 	// but we need it slightly earlier to test the assertions.
 	// TODO find a cleaner way to do this
 	romInfoTopic.reset();
+	platformInfo.reset();
 	versionInfo.reset();
 	updateCmd.reset();
 	tabCompletionCmd.reset();
@@ -157,25 +168,20 @@ void GlobalCommandController::unregisterProxyCommand(string_ref name)
 GlobalCommandController::ProxySettings::iterator
 GlobalCommandController::findProxySetting(const std::string& name)
 {
-	for (ProxySettings::iterator it = proxySettings.begin();
-	     it != proxySettings.end(); ++it) {
-		if (it->first->getName() == name) {
-			return it;
-		}
-	}
-	return proxySettings.end();
+	return find_if(proxySettings.begin(), proxySettings.end(),
+		[&](ProxySettings::value_type& v) { return v.first->getName() == name; });
 }
 
 void GlobalCommandController::registerProxySetting(Setting& setting)
 {
-	const string& name = setting.getName();
-	ProxySettings::iterator it = findProxySetting(name);
+	const auto& name = setting.getName();
+	auto it = findProxySetting(name);
 	if (it == proxySettings.end()) {
 		// first occurrence
-		ProxySetting* proxy = new ProxySetting(*this, reactor, name);
-		proxySettings.push_back(std::make_pair(proxy, 1));
+		auto proxy = make_unique<ProxySetting>(reactor, name);
 		getSettingsConfig().getSettingsManager().registerSetting(*proxy, name);
 		getInterpreter().registerSetting(*proxy, name);
+		proxySettings.push_back(std::make_pair(std::move(proxy), 1));
 	} else {
 		// was already registered
 		++(it->second);
@@ -184,17 +190,16 @@ void GlobalCommandController::registerProxySetting(Setting& setting)
 
 void GlobalCommandController::unregisterProxySetting(Setting& setting)
 {
-	const string& name = setting.getName();
-	ProxySettings::iterator it = findProxySetting(name);
+	const auto& name = setting.getName();
+	auto it = findProxySetting(name);
 	assert(it != proxySettings.end());
 	assert(it->second);
 	--(it->second);
 	if (it->second == 0) {
-		ProxySetting* proxy = it->first;
-		getInterpreter().unregisterSetting(*proxy, name);
-		getSettingsConfig().getSettingsManager().unregisterSetting(*proxy, name);
+		auto& proxy = *it->first;
+		getInterpreter().unregisterSetting(proxy, name);
+		getSettingsConfig().getSettingsManager().unregisterSetting(proxy, name);
 		proxySettings.erase(it);
-		delete proxy;
 	}
 }
 
@@ -220,8 +225,8 @@ InfoCommand& GlobalCommandController::getOpenMSXInfoCommand()
 
 SettingsConfig& GlobalCommandController::getSettingsConfig()
 {
-	if (!settingsConfig.get()) {
-		settingsConfig.reset(new SettingsConfig(*this, *hotKey));
+	if (!settingsConfig) {
+		settingsConfig = make_unique<SettingsConfig>(*this, *hotKey);
 	}
 	return *settingsConfig;
 }
@@ -263,19 +268,19 @@ void GlobalCommandController::unregisterCompleter(
 
 void GlobalCommandController::registerSetting(Setting& setting)
 {
-	const string& name = setting.getName();
+	const auto& name = setting.getName();
 	getSettingsConfig().getSettingsManager().registerSetting(setting, name);
 	interpreter->registerSetting(setting, name);
 }
 
 void GlobalCommandController::unregisterSetting(Setting& setting)
 {
-	const string& name = setting.getName();
+	const auto& name = setting.getName();
 	interpreter->unregisterSetting(setting, name);
 	getSettingsConfig().getSettingsManager().unregisterSetting(setting, name);
 }
 
-Setting* GlobalCommandController::findSetting(string_ref name)
+BaseSetting* GlobalCommandController::findSetting(string_ref name)
 {
 	return getSettingsConfig().getSettingsManager().findSetting(name);
 }
@@ -302,8 +307,7 @@ void GlobalCommandController::split(string_ref str, vector<string>& tokens,
 	enum ParseState {Alpha, BackSlash, Quote};
 	ParseState state = Alpha;
 
-	for (unsigned i = 0; i < str.size(); ++i) {
-		char chr = str[i];
+	for (auto chr : str) {
 		switch (state) {
 			case Alpha:
 				if (tokens.empty()) {
@@ -341,8 +345,7 @@ string GlobalCommandController::removeEscaping(const string& str)
 	ParseState state = Alpha;
 
 	string result;
-	for (unsigned i = 0; i < str.length(); ++i) {
-		char chr = str[i];
+	for (auto chr : str) {
 		switch (state) {
 			case Alpha:
 				if (chr == '\\') {
@@ -369,27 +372,25 @@ string GlobalCommandController::removeEscaping(const string& str)
 	return result;
 }
 
-void GlobalCommandController::removeEscaping(
-	const vector<string>& input, vector<string>& result,
-	bool keepLastIfEmpty)
+vector<string> GlobalCommandController::removeEscaping(
+	const vector<string>& input, bool keepLastIfEmpty)
 {
-	for (vector<string>::const_iterator it = input.begin();
-	     it != input.end();
-	     ++it) {
-		if (!it->empty()) {
-			result.push_back(removeEscaping(*it));
+	vector<string> result;
+	for (auto& s : input) {
+		if (!s.empty()) {
+			result.push_back(removeEscaping(s));
 		}
 	}
 	if (keepLastIfEmpty && (input.empty() || input.back().empty())) {
 		result.push_back("");
 	}
+	return result;
 }
 
 static string escapeChars(const string& str, const string& chars)
 {
 	string result;
-	for (unsigned i = 0; i < str.length(); ++i) {
-		char chr = str[i];
+	for (auto chr : str) {
 		if (chars.find(chr) != string::npos) {
 			result += '\\';
 		}
@@ -420,16 +421,14 @@ string GlobalCommandController::addEscaping(const string& str, bool quote,
 string GlobalCommandController::join(
 	const vector<string>& tokens, char delimiter)
 {
-	string result;
+	StringOp::Builder result;
 	bool first = true;
-	for (vector<string>::const_iterator it = tokens.begin();
-	     it != tokens.end();
-	     ++it) {
+	for (auto& t : tokens) {
 		if (!first) {
-			result += delimiter;
+			result << delimiter;
 		}
 		first = false;
-		result += *it;
+		result << t;
 	}
 	return result;
 }
@@ -446,10 +445,9 @@ string GlobalCommandController::executeCommand(
 	return interpreter->execute(cmd);
 }
 
-void GlobalCommandController::splitList(
-	const string& list, vector<string>& result)
+vector<string> GlobalCommandController::splitList(const string& list)
 {
-	interpreter->splitList(list, result);
+	return interpreter->splitList(list);
 }
 
 void GlobalCommandController::source(const string& script)
@@ -483,11 +481,10 @@ string GlobalCommandController::tabCompletion(string_ref command)
 	}
 
 	// complete last token
-	vector<string> tokens;
-	removeEscaping(originalTokens, tokens, true);
-	size_t oldNum = tokens.size();
+	auto tokens = removeEscaping(originalTokens, true);
+	auto oldNum = tokens.size();
 	tabCompletion(tokens);
-	size_t newNum = tokens.size();
+	auto newNum = tokens.size();
 	bool tokenFinished = oldNum != newNum;
 
 	// replace last token
@@ -515,33 +512,29 @@ void GlobalCommandController::tabCompletion(vector<string>& tokens)
 	}
 	if (tokens.size() == 1) {
 		// build a list of all command strings
-		set<string> cmds;
-		interpreter->getCommandNames(cmds);
-		Completer::completeString(tokens, cmds);
+		Completer::completeString(tokens,
+		                          interpreter->getCommandNames());
 	} else {
-		CompleterMap::const_iterator it = commandCompleters.find(tokens.front());
+		auto it = commandCompleters.find(tokens.front());
 		if (it != commandCompleters.end()) {
 			it->second->tabCompletion(tokens);
 		} else {
 			TclObject command(*interpreter);
 			command.addListElement("openmsx::tabcompletion");
-			command.addListElements(tokens.begin(), tokens.end());
+			command.addListElements(tokens);
 			try {
-				string result = command.executeCommand();
-				vector<string> split;
-				splitList(result, split);
+				auto list = splitList(command.executeCommand());
 				bool sensitive = true;
-				if (!split.empty()) {
-					if (split.back() == "false") {
-						split.pop_back();
+				if (!list.empty()) {
+					if (list.back() == "false") {
+						list.pop_back();
 						sensitive = false;
-					} else if (split.back() == "true") {
-						split.pop_back();
+					} else if (list.back() == "true") {
+						list.pop_back();
 						sensitive = true;
 					}
 				}
-				set<string> completions(split.begin(), split.end());
-				Completer::completeString(tokens, completions, sensitive);
+				Completer::completeString(tokens, list, sensitive);
 			} catch (CommandException& e) {
 				cliComm.printWarning(
 					"Error while executing tab-completion "
@@ -567,10 +560,8 @@ void HelpCmd::execute(const vector<TclObject>& tokens, TclObject& result)
 		string text =
 			"Use 'help [command]' to get help for a specific command\n"
 			"The following commands exist:\n";
-		for (GlobalCommandController::CompleterMap::const_iterator it =
-		         controller.commandCompleters.begin();
-		     it != controller.commandCompleters.end(); ++it) {
-			string_ref key = it->first();
+		for (auto& p : controller.commandCompleters) {
+			const auto& key = p.first();
 			text.append(key.data(), key.size());
 			text += '\n';
 		}
@@ -578,11 +569,10 @@ void HelpCmd::execute(const vector<TclObject>& tokens, TclObject& result)
 		break;
 	}
 	default: {
-		 GlobalCommandController::CompleterMap::const_iterator it =
-			controller.commandCompleters.find(tokens[1].getString());
+		auto it = controller.commandCompleters.find(tokens[1].getString());
 		if (it != controller.commandCompleters.end()) {
 			vector<string> tokens2;
-			vector<TclObject>::const_iterator it2 = tokens.begin();
+			auto it2 = tokens.begin();
 			for (++it2; it2 != tokens.end(); ++it2) {
 				tokens2.push_back(it2->getString().str());
 			}
@@ -625,8 +615,7 @@ void TabCompletionCmd::execute(const vector<TclObject>& tokens, TclObject& resul
 	switch (tokens.size()) {
 	case 2: {
 		// TODO this prints list of possible completions in the console
-		string_ref command = tokens[1].getString();
-		result.setString(controller.tabCompletion(command));
+		result.setString(controller.tabCompletion(tokens[1].getString()));
 		break;
 	}
 	default:
@@ -652,8 +641,8 @@ UpdateCmd::UpdateCmd(CommandController& commandController)
 
 static GlobalCliComm::UpdateType getType(const string& name)
 {
-	const char* const* updateStr = CliComm::getUpdateStrings();
-	for (unsigned i = 0; i < CliComm::NUM_UPDATES; ++i) {
+	auto updateStr = CliComm::getUpdateStrings();
+	for (auto i : xrange(updateStr.size())) {
 		if (updateStr[i] == name) {
 			return static_cast<CliComm::UpdateType>(i);
 		}
@@ -663,14 +652,13 @@ static GlobalCliComm::UpdateType getType(const string& name)
 
 CliConnection& UpdateCmd::getConnection()
 {
-	GlobalCommandController* controller =
-		checked_cast<GlobalCommandController*>(&getCommandController());
-	CliConnection* connection = controller->getConnection();
-	if (!connection) {
-		throw CommandException("This command only makes sense when "
-		                       "it's used from an external application.");
+	auto* controller = checked_cast<GlobalCommandController*>(
+		&getCommandController());
+	if (auto* connection = controller->getConnection()) {
+		return *connection;
 	}
-	return *connection;
+	throw CommandException("This command only makes sense when "
+	                       "it's used from an external application.");
 }
 
 string UpdateCmd::execute(const vector<string>& tokens)
@@ -698,21 +686,35 @@ void UpdateCmd::tabCompletion(vector<string>& tokens) const
 {
 	switch (tokens.size()) {
 	case 2: {
-		set<string> ops;
-		ops.insert("enable");
-		ops.insert("disable");
+		static const char* const ops[] = { "enable", "disable" };
 		completeString(tokens, ops);
 		break;
 	}
-	case 3: {
-		const char* const* updateStr = CliComm::getUpdateStrings();
-		set<string> types(updateStr, updateStr + CliComm::NUM_UPDATES);
-		completeString(tokens, types);
-	}
+	case 3:
+		completeString(tokens, CliComm::getUpdateStrings());
+		break;
 	}
 }
 
 
+// Platform info
+
+PlatformInfo::PlatformInfo(InfoCommand& openMSXInfoCommand)
+	: InfoTopic(openMSXInfoCommand, "platform")
+{
+}
+
+void PlatformInfo::execute(const vector<TclObject>& /*tokens*/,
+                          TclObject& result) const
+{
+	result.setString(TARGET_PLATFORM);
+}
+
+string PlatformInfo::help(const vector<string>& /*tokens*/) const
+{
+	return "Prints openMSX platform.";
+}
+
 // Version info
 
 VersionInfo::VersionInfo(InfoCommand& openMSXInfoCommand)
@@ -723,7 +725,7 @@ VersionInfo::VersionInfo(InfoCommand& openMSXInfoCommand)
 void VersionInfo::execute(const vector<TclObject>& /*tokens*/,
                           TclObject& result) const
 {
-	result.setString(Version::FULL_VERSION);
+	result.setString(Version::full());
 }
 
 string VersionInfo::help(const vector<string>& /*tokens*/) const
