@@ -1,5 +1,3 @@
-// $Id: InfoCommand.cc 12805 2012-08-13 19:26:18Z m9710797 $
-
 #include "InfoCommand.hh"
 #include "InfoTopic.hh"
 #include "TclObject.hh"
@@ -7,11 +5,11 @@
 #include "CliComm.hh"
 #include "CommandException.hh"
 #include "MSXMotherBoard.hh"
+#include "KeyRange.hh"
 #include "unreachable.hh"
 #include <iostream>
 #include <cassert>
 
-using std::set;
 using std::string;
 using std::vector;
 
@@ -59,16 +57,15 @@ void InfoCommand::execute(const vector<TclObject>& tokens,
 	switch (tokens.size()) {
 	case 1:
 		// list topics
-		for (InfoTopics::const_iterator it = infoTopics.begin();
-		     it != infoTopics.end(); ++it) {
-			result.addListElement(it->first());
+		for (auto& p : infoTopics) {
+			result.addListElement(p.first());
 		}
 		break;
 	default:
 		// show info about topic
 		assert(tokens.size() >= 2);
-		string_ref topic = tokens[1].getString();
-		InfoTopics::const_iterator it = infoTopics.find(topic);
+		const auto& topic = tokens[1].getString();
+		auto it = infoTopics.find(topic);
 		if (it == infoTopics.end()) {
 			throw CommandException("No info on: " + topic);
 		}
@@ -89,7 +86,7 @@ string InfoCommand::help(const vector<string>& tokens) const
 	default:
 		// show help on a certain topic
 		assert(tokens.size() >= 2);
-		InfoTopics::const_iterator it = infoTopics.find(tokens[1]);
+		auto it = infoTopics.find(tokens[1]);
 		if (it == infoTopics.end()) {
 			throw CommandException("No info on: " + tokens[1]);
 		}
@@ -104,18 +101,13 @@ void InfoCommand::tabCompletion(vector<string>& tokens) const
 	switch (tokens.size()) {
 	case 2: {
 		// complete topic
-		set<string> topics;
-		for (InfoTopics::const_iterator it = infoTopics.begin();
-		     it != infoTopics.end(); ++it) {
-			topics.insert(it->first().str());
-		}
-		completeString(tokens, topics);
+		completeString(tokens, keys(infoTopics));
 		break;
 	}
 	default:
 		// show help on a certain topic
 		assert(tokens.size() >= 3);
-		InfoTopics::const_iterator it = infoTopics.find(tokens[1]);
+		auto it = infoTopics.find(tokens[1]);
 		if (it != infoTopics.end()) {
 			it->second->tabCompletion(tokens);
 		}
