@@ -1,5 +1,3 @@
-// $Id: Interpreter.cc 12805 2012-08-13 19:26:18Z m9710797 $
-
 #include "Interpreter.hh"
 #include "EventDistributor.hh"
 #include "Command.hh"
@@ -10,23 +8,21 @@
 #include "Setting.hh"
 #include "InterpreterOutput.hh"
 #include "MSXCPUInterface.hh"
-#include "openmsx.hh"
 #include "FileOperations.hh"
-#include "StringOp.hh"
 #include "unreachable.hh"
+#include "xrange.hh"
 #include <iostream>
+#include <map>
 //#include <tk.h>
 #include "openmsx.hh"
 
-using std::set;
 using std::string;
 using std::vector;
 
 namespace openmsx {
 
 // See comments in traceProc()
-typedef std::map<long, Setting*> TraceMap;
-static TraceMap traceMap;
+static std::map<long, BaseSetting*> traceMap;
 static long traceCount = 0;
 
 
@@ -49,22 +45,22 @@ static int dummyGetHandle(ClientData /*instanceData*/, int /*direction*/,
 }
 Tcl_ChannelType Interpreter::channelType = {
 	const_cast<char*>("openMSX console"),// Type name
-	NULL,			 // Always non-blocking
+	nullptr,		 // Always non-blocking
 	dummyClose,		 // Close proc
 	dummyInput,		 // Input proc
 	Interpreter::outputProc, // Output proc
-	NULL,			 // Seek proc
-	NULL,			 // Set option proc
-	NULL,			 // Get option proc
+	nullptr,		 // Seek proc
+	nullptr,		 // Set option proc
+	nullptr,		 // Get option proc
 	dummyWatch,		 // Watch for events on console
 	dummyGetHandle,		 // Get a handle from the device
-	NULL,			 // Tcl_DriverClose2Proc
-	NULL,			 // Tcl_DriverBlockModeProc
-	NULL,			 // Tcl_DriverFlushProc
-	NULL,			 // Tcl_DriverHandlerProc
-	NULL,			 // Tcl_DriverWideSeekProc
-	NULL,			 // Tcl_DriverThreadActionProc
-	NULL,			 // Tcl_DriverTruncateProc
+	nullptr,		 // Tcl_DriverClose2Proc
+	nullptr,		 // Tcl_DriverBlockModeProc
+	nullptr,		 // Tcl_DriverFlushProc
+	nullptr,		 // Tcl_DriverHandlerProc
+	nullptr,		 // Tcl_DriverWideSeekProc
+	nullptr,		 // Tcl_DriverThreadActionProc
+	nullptr,		 // Tcl_DriverTruncateProc
 };
 
 void Interpreter::init(const char* programName)
@@ -93,7 +89,7 @@ Interpreter::Interpreter(EventDistributor& eventDistributor_)
 
 	Tcl_Channel channel = Tcl_CreateChannel(&channelType,
 		"openMSX console", this, TCL_WRITABLE);
-	if (channel != NULL) {
+	if (channel) {
 		Tcl_SetChannelOption(interp, channel, "-translation", "binary");
 		Tcl_SetChannelOption(interp, channel, "-buffering", "line");
 		Tcl_SetChannelOption(interp, channel, "-encoding", "utf-8");
@@ -131,8 +127,7 @@ int Interpreter::outputProc(ClientData clientData, const char* buf,
                  int toWrite, int* /*errorCodePtr*/)
 {
 	try {
-		InterpreterOutput* output =
-			static_cast<Interpreter*>(clientData)->output;
+		auto* output = static_cast<Interpreter*>(clientData)->output;
 		string_ref text(buf, toWrite);
 		if (!text.empty() && output) {
 			output->output(text);
@@ -148,12 +143,12 @@ void Interpreter::registerCommand(const string& name, Command& command)
 	assert(commandTokenMap.find(name) == commandTokenMap.end());
 	commandTokenMap[name] = Tcl_CreateObjCommand(
 		interp, name.c_str(), commandProc,
-		static_cast<ClientData>(&command), NULL);
+		static_cast<ClientData>(&command), nullptr);
 }
 
 void Interpreter::unregisterCommand(string_ref name, Command& /*command*/)
 {
-	CommandTokenMap::iterator it = commandTokenMap.find(name);
+	auto it = commandTokenMap.find(name);
 	assert(it != commandTokenMap.end());
 	Tcl_DeleteCommandFromToken(interp, it->second);
 	commandTokenMap.erase(it);
@@ -163,17 +158,17 @@ int Interpreter::commandProc(ClientData clientData, Tcl_Interp* interp,
                            int objc, Tcl_Obj* const objv[])
 {
 	try {
-		Command& command = *static_cast<Command*>(clientData);
+		auto& command = *static_cast<Command*>(clientData);
 		vector<TclObject> tokens;
 		tokens.reserve(objc);
-		for (int i = 0; i < objc; ++i) {
+		for (auto i : xrange(objc)) {
 			tokens.push_back(TclObject(interp, objv[i]));
 		}
 		int res = TCL_OK;
 		TclObject result(interp);
 		try {
 			if (!command.isAllowedInEmptyMachine()) {
-				if (MSXCommandController* controller =
+				if (auto controller =
 					dynamic_cast<MSXCommandController*>(
 						&command.getCommandController())) {
 					if (!controller->getMSXMotherBoard().getMachineConfig()) {
@@ -202,17 +197,9 @@ int Interpreter::commandProc(ClientData clientData, Tcl_Interp* interp,
 //   - build-in Tcl commands
 //   - openmsx commands
 //   - user-defined procs
-void Interpreter::getCommandNames(set<string>& result)
+vector<string> Interpreter::getCommandNames()
 {
-	string list = execute("info commands");
-
-	int argc;
-	const char** argv;
-	if (Tcl_SplitList(interp, list.c_str(), &argc, &argv) != TCL_OK) {
-		return;
-	}
-	result.insert(&argv[0], &argv[argc]);
-	Tcl_Free(reinterpret_cast<char*>(argv));
+	return splitList(execute("openmsx::all_command_names"), interp);
 }
 
 bool Interpreter::isComplete(const string& command) const
@@ -254,7 +241,10 @@ static const char* getVar(Tcl_Interp* interp, const char* name)
 
 void Interpreter::setVariable(const string& name, const string& value)
 {
-	setVar(interp, name.c_str(), value.c_str());
+	if (!Tcl_SetVar(interp, name.c_str(), value.c_str(),
+		        TCL_GLOBAL_ONLY | TCL_LEAVE_ERR_MSG)) {
+		throw CommandException(Tcl_GetStringResult(interp));
+	}
 }
 
 void Interpreter::unsetVariable(const string& name)
@@ -267,20 +257,20 @@ const char* Interpreter::getVariable(const string& name) const
 	return getVar(interp, name.c_str());
 }
 
-static string getSafeValueString(Setting& setting)
+static string getSafeValueString(BaseSetting& setting)
 {
 	try {
-		return setting.getValueString();
+		return setting.getString();
 	} catch (MSXException&) {
 		return "0"; // 'safe' value, see comment in registerSetting()
 	}
 }
-void Interpreter::registerSetting(Setting& variable, const string& name)
+void Interpreter::registerSetting(BaseSetting& variable, const string& name)
 {
 	if (const char* tclVarValue = getVariable(name)) {
 		// Tcl var already existed, use this value
 		try {
-			variable.setValueStringDirect(tclVarValue);
+			variable.setStringDirect(tclVarValue);
 		} catch (MSXException&) {
 			// Ignore: can happen in case of proxy settings when
 			// the current machine doesn't have this setting.
@@ -326,9 +316,9 @@ void Interpreter::registerSetting(Setting& variable, const string& name)
 	             traceProc, reinterpret_cast<ClientData>(traceID));
 }
 
-void Interpreter::unregisterSetting(Setting& variable, const string& name)
+void Interpreter::unregisterSetting(BaseSetting& variable, const string& name)
 {
-	TraceMap::iterator it = traceMap.begin();
+	auto it = traceMap.begin();
 	while (true) {
 		assert(it != traceMap.end());
 		if (it->second == &variable) break;
@@ -343,10 +333,10 @@ void Interpreter::unregisterSetting(Setting& variable, const string& name)
 	unsetVariable(name);
 }
 
-static Setting* getTraceSetting(unsigned traceID)
+static BaseSetting* getTraceSetting(unsigned traceID)
 {
-	TraceMap::const_iterator it = traceMap.find(traceID);
-	return (it != traceMap.end()) ? it->second : NULL;
+	auto it = traceMap.find(traceID);
+	return (it != traceMap.end()) ? it->second : nullptr;
 }
 
 char* Interpreter::traceProc(ClientData clientData, Tcl_Interp* interp,
@@ -383,14 +373,14 @@ char* Interpreter::traceProc(ClientData clientData, Tcl_Interp* interp,
 		// a map. If the Setting was deleted, we won't find it anymore
 		// in the map and return.
 
-		long traceID = reinterpret_cast<long>(clientData);
-		Setting* variable = getTraceSetting(traceID);
-		if (!variable) return NULL;
+		auto traceID = reinterpret_cast<uintptr_t>(clientData);
+		auto* variable = getTraceSetting(traceID);
+		if (!variable) return nullptr;
 
 		static string static_string;
 		if (flags & TCL_TRACE_READS) {
 			try {
-				setVar(interp, part1, variable->getValueString().c_str());
+				setVar(interp, part1, variable->getString().c_str());
 			} catch (MSXException& e) {
 				static_string = e.getMessage();
 				return const_cast<char*>(static_string.c_str());
@@ -400,8 +390,8 @@ char* Interpreter::traceProc(ClientData clientData, Tcl_Interp* interp,
 			try {
 				const char* v = getVar(interp, part1);
 				string newValue = v ? v : "";
-				variable->setValueStringDirect(newValue);
-				string newValue2 = variable->getValueString();
+				variable->setStringDirect(newValue);
+				string newValue2 = variable->getString();
 				if (newValue != newValue2) {
 					setVar(interp, part1, newValue2.c_str());
 				}
@@ -416,8 +406,8 @@ char* Interpreter::traceProc(ClientData clientData, Tcl_Interp* interp,
 				// note we cannot use restoreDefault(), because
 				// that goes via Tcl and the Tcl variable
 				// doesn't exist at this point
-				variable->setValueStringDirect(
-					variable->getRestoreValueString());
+				variable->setStringDirect(
+					variable->getRestoreValue());
 			} catch (MSXException&) {
 				// for some reason default value is not valid ATM,
 				// keep current value (happened for videosource
@@ -433,7 +423,7 @@ char* Interpreter::traceProc(ClientData clientData, Tcl_Interp* interp,
 	} catch (...) {
 		UNREACHABLE; // we cannot let exceptions pass through Tcl
 	}
-	return NULL;
+	return nullptr;
 }
 
 void Interpreter::createNamespace(const std::string& name)
@@ -446,27 +436,26 @@ void Interpreter::deleteNamespace(const std::string& name)
 	execute("namespace delete ::" + name);
 }
 
-void Interpreter::splitList(const std::string& list,
-	                    std::vector<std::string>& result)
+vector<string> Interpreter::splitList(const std::string& list)
 {
-	splitList(list, result, interp);
+	return splitList(list, interp);
 }
 
-void Interpreter::splitList(const string& list, vector<string>& result,
-                            Tcl_Interp* interp)
+vector<string> Interpreter::splitList(const string& list, Tcl_Interp* interp)
 {
 	int argc;
 	const char** argv;
 	if (Tcl_SplitList(interp, list.c_str(), &argc, &argv) == TCL_ERROR) {
-		string message = interp ? Tcl_GetStringResult(interp)
-		                        : "splitList error";
-		throw CommandException(message);
+		throw CommandException(
+			interp ? Tcl_GetStringResult(interp)
+			       : "splitList error");
 	}
-	result.assign(argv, argv + argc);
+	vector<string> result(argv, argv + argc);
 	Tcl_Free(reinterpret_cast<char*>(argv));
+	return result;
 }
 
-int Interpreter::signalEvent(const shared_ptr<const Event>& event)
+int Interpreter::signalEvent(const std::shared_ptr<const Event>& event)
 {
 	(void)event;
 	assert(event->getType() == OPENMSX_POLL_EVENT);
