@@ -1,5 +1,3 @@
-// $Id: MSXCommandController.cc 12721 2012-07-12 19:27:39Z m9710797 $
-
 #include "MSXCommandController.hh"
 #include "GlobalCommandController.hh"
 #include "Reactor.hh"
@@ -11,6 +9,8 @@
 #include "Interpreter.hh"
 #include "Setting.hh"
 #include "Event.hh"
+#include "MSXException.hh"
+#include "memory.hh"
 #include <iostream>
 
 using std::string;
@@ -32,7 +32,7 @@ MSXCommandController::MSXCommandController(
 {
 	globalCommandController.getInterpreter().createNamespace(machineID);
 
-	machineInfoCommand.reset(new InfoCommand(*this, "machine_info"));
+	machineInfoCommand = make_unique<InfoCommand>(*this, "machine_info");
 	machineInfoCommand->setAllowedInEmptyMachine(true);
 
 	msxEventDistributor.registerEventListener(*this);
@@ -45,13 +45,11 @@ MSXCommandController::~MSXCommandController()
 	machineInfoCommand.reset();
 
 	#ifndef NDEBUG
-	for (CommandMap::const_iterator it = commandMap.begin();
-	     it != commandMap.end(); ++it) {
-		std::cout << "Command not unregistered: " << it->first() << std::endl;
+	for (auto& p : commandMap) {
+		std::cout << "Command not unregistered: " << p.first() << std::endl;
 	}
-	for (SettingMap::const_iterator it = settingMap.begin();
-	     it != settingMap.end(); ++it) {
-		std::cout << "Setting not unregistered: " << it->first() << std::endl;
+	for (auto& p : settingMap) {
+		std::cout << "Setting not unregistered: " << p.first() << std::endl;
 	}
 	assert(commandMap.empty());
 	assert(settingMap.empty());
@@ -150,19 +148,24 @@ void MSXCommandController::changeSetting(Setting& setting, const string& value)
 
 Command* MSXCommandController::findCommand(string_ref name) const
 {
-	CommandMap::const_iterator it = commandMap.find(name);
-	return (it != commandMap.end()) ? it->second : NULL;
+	auto it = commandMap.find(name);
+	return (it != commandMap.end()) ? it->second : nullptr;
 }
 
-Setting* MSXCommandController::findSetting(string_ref name)
+BaseSetting* MSXCommandController::findSetting(string_ref name)
 {
-	SettingMap::const_iterator it = settingMap.find(name);
-	return (it != settingMap.end()) ? it->second : NULL;
+	auto it = settingMap.find(name);
+	return (it != settingMap.end()) ? it->second : nullptr;
+}
+
+const BaseSetting* MSXCommandController::findSetting(string_ref setting) const
+{
+	return const_cast<MSXCommandController*>(this)->findSetting(setting);
 }
 
 bool MSXCommandController::hasCommand(string_ref command) const
 {
-	return findCommand(command) != NULL;
+	return findCommand(command) != nullptr;
 }
 
 string MSXCommandController::executeCommand(const string& command,
@@ -171,10 +174,9 @@ string MSXCommandController::executeCommand(const string& command,
 	return globalCommandController.executeCommand(command, connection);
 }
 
-void MSXCommandController::splitList(const string& list,
-	                             vector<string>& result)
+vector<string> MSXCommandController::splitList(const string& list)
 {
-	globalCommandController.splitList(list, result);
+	return globalCommandController.splitList(list);
 }
 
 CliComm& MSXCommandController::getCliComm()
@@ -182,15 +184,23 @@ CliComm& MSXCommandController::getCliComm()
 	return motherboard.getMSXCliComm();
 }
 
+Interpreter& MSXCommandController::getInterpreter()
+{
+	return globalCommandController.getInterpreter();
+}
+
 void MSXCommandController::signalEvent(
-	const shared_ptr<const Event>& event, EmuTime::param /*time*/)
+	const std::shared_ptr<const Event>& event, EmuTime::param /*time*/)
 {
 	if (event->getType() != OPENMSX_MACHINE_ACTIVATED) return;
 
 	// simple way to synchronize proxy settings
-	for (SettingMap::const_iterator it = settingMap.begin();
-	     it != settingMap.end(); ++it) {
-		changeSetting(*it->second, it->second->getValueString());
+	for (auto& p : settingMap) {
+		try {
+			changeSetting(*p.second, p.second->getString());
+		} catch (MSXException&) {
+			// ignore
+		}
 	}
 }
 
@@ -199,4 +209,18 @@ bool MSXCommandController::isActive() const
 	return reactor.getMotherBoard() == &motherboard;
 }
 
+void MSXCommandController::transferSettings(const MSXCommandController& from)
+{
+	for (auto& p : settingMap) {
+		if (auto* fromSetting = from.findSetting(p.first())) {
+			if (!fromSetting->needTransfer()) continue;
+			try {
+				changeSetting(*p.second, fromSetting->getString());
+			} catch (MSXException&) {
+				// ignore
+			}
+		}
+	}
+}
+
 } // namespace openmsx
