@@ -1,5 +1,3 @@
-// $Id: TclObject.cc 12807 2012-08-13 19:26:53Z m9710797 $
-
 #include "TclObject.hh"
 #include "Interpreter.hh"
 #include "CommandException.hh"
@@ -24,8 +22,14 @@ TclObject::TclObject(Tcl_Interp* interp_, string_ref value)
 	init(Tcl_NewStringObj(value.data(), int(value.size())));
 }
 
+TclObject::TclObject(Interpreter& interp_, string_ref value)
+	: interp(interp_.interp)
+{
+	init(Tcl_NewStringObj(value.data(), int(value.size())));
+}
+
 TclObject::TclObject(string_ref value)
-	: interp(0)
+	: interp(nullptr)
 {
 	init(Tcl_NewStringObj(value.data(), int(value.size())));
 }
@@ -49,7 +53,7 @@ TclObject::TclObject(const TclObject& object)
 }
 
 TclObject::TclObject()
-	: interp(0)
+	: interp(nullptr)
 {
 	init(Tcl_NewObj());
 }
@@ -238,6 +242,16 @@ TclObject TclObject::getListIndex(unsigned index) const
 	               : TclObject(interp);
 }
 
+TclObject TclObject::getDictValue(const TclObject& key) const
+{
+	Tcl_Obj* value;
+	if (Tcl_DictObjGet(interp, obj, key.obj, &value) != TCL_OK) {
+		throwException();
+	}
+	return value ? TclObject(interp, value)
+	             : TclObject(interp);
+}
+
 bool TclObject::evalBool() const
 {
 	int result;
@@ -247,60 +261,6 @@ bool TclObject::evalBool() const
 	return result != 0;
 }
 
-void TclObject::checkExpression() const
-{
-	string_ref tmp = getString();
-	parse(tmp.data(), int(tmp.size()), true);
-}
-
-void TclObject::checkCommand() const
-{
-	string_ref tmp = getString();
-	parse(tmp.data(), int(tmp.size()), false);
-}
-
-void TclObject::parse(const char* str, int len, bool expression) const
-{
-	assert(interp);
-	Tcl_Parse info;
-	if (expression ?
-	    Tcl_ParseExpr(interp, str, len, &info) :
-	    Tcl_ParseCommand(interp, str, len, 1, &info) != TCL_OK) {
-		throw CommandException(Tcl_GetStringResult(interp));
-	}
-	struct Cleanup {
-		~Cleanup() { Tcl_FreeParse(p); }
-		Tcl_Parse* p;
-	} cleanup = { &info };
-	(void)cleanup;
-
-	if (!expression && (info.tokenPtr[0].type == TCL_TOKEN_SIMPLE_WORD)) {
-		// simple command name
-		Tcl_CmdInfo cmdinfo;
-		Tcl_Token& token2 = info.tokenPtr[1];
-		string procname(token2.start, token2.size);
-		if (!Tcl_GetCommandInfo(interp, procname.c_str(), &cmdinfo)) {
-			throw CommandException("invalid command name: \"" +
-			                       procname + '\"');
-		}
-	}
-	for (int i = 0; i < info.numTokens; ++i) {
-		Tcl_Token& token = info.tokenPtr[i];
-		if (token.type == TCL_TOKEN_COMMAND) {
-			parse(token.start + 1, token.size - 2, false);
-		} else if ((token.type == TCL_TOKEN_VARIABLE) &&
-				(token.numComponents == 1)) {
-			// simple variable (no array)
-			Tcl_Token& token2 = info.tokenPtr[i + 1];
-			string varname(token2.start, token2.size);
-			if (!Tcl_GetVar2Ex(interp, varname.c_str(), NULL,
-			                   TCL_LEAVE_ERR_MSG)) {
-				throw CommandException(Tcl_GetStringResult(interp));
-			}
-		}
-	}
-}
-
 string TclObject::executeCommand(bool compile)
 {
 	assert(interp);
@@ -313,6 +273,4 @@ string TclObject::executeCommand(bool compile)
 	return result;
 }
 
-
 } // namespace openmsx
-
