@@ -1,8 +1,5 @@
-// $Id: SettingsConfig.cc 12609 2012-06-12 21:07:44Z m9710797 $
-
 #include "SettingsConfig.hh"
 #include "SettingsManager.hh"
-#include "XMLElement.hh"
 #include "XMLLoader.hh"
 #include "LocalFileReference.hh"
 #include "File.hh"
@@ -14,10 +11,9 @@
 #include "GlobalCommandController.hh"
 #include "Command.hh"
 #include "openmsx.hh"
-#include <memory>
+#include "memory.hh"
 #include <cassert>
 
-using std::auto_ptr;
 using std::string;
 using std::vector;
 
@@ -52,10 +48,12 @@ SettingsConfig::SettingsConfig(
 		GlobalCommandController& globalCommandController,
 		HotKey& hotKey_)
 	: commandController(globalCommandController)
-	, saveSettingsCommand(new SaveSettingsCommand(commandController, *this))
-	, loadSettingsCommand(new LoadSettingsCommand(commandController, *this))
-	, settingsManager(new SettingsManager(globalCommandController))
-	, xmlElement(new XMLElement("settings"))
+	, saveSettingsCommand(make_unique<SaveSettingsCommand>(
+		commandController, *this))
+	, loadSettingsCommand(make_unique<LoadSettingsCommand>(
+		commandController, *this))
+	, settingsManager(make_unique<SettingsManager>(
+		globalCommandController))
 	, hotKey(hotKey_)
 	, mustSaveSettings(false)
 {
@@ -63,12 +61,9 @@ SettingsConfig::SettingsConfig(
 
 SettingsConfig::~SettingsConfig()
 {
-	PRT_DEBUG("~SettingsConfig...");
 	if (mustSaveSettings) {
 		try {
-			PRT_DEBUG("Saving Settings...");
 			saveSetting();
-			PRT_DEBUG("Saving Settings... DONE");
 		} catch (FileException& e) {
 			commandController.getCliComm().printWarning(
 				"Auto-saving of settings failed: " + e.getMessage() );
@@ -76,18 +71,18 @@ SettingsConfig::~SettingsConfig()
 	}
 }
 
-void SettingsConfig::loadSetting(FileContext& context, const string& filename)
+void SettingsConfig::loadSetting(const FileContext& context, const string& filename)
 {
 	LocalFileReference file(context.resolve(filename));
 	xmlElement = XMLLoader::load(file.getFilename(), "settings.dtd");
-	getSettingsManager().loadSettings(*xmlElement);
-	hotKey.loadBindings(*xmlElement);
+	getSettingsManager().loadSettings(xmlElement);
+	hotKey.loadBindings(xmlElement);
 
 	// only set saveName after file was successfully parsed
 	setSaveFilename(context, filename);
 }
 
-void SettingsConfig::setSaveFilename(FileContext& context, const string& filename)
+void SettingsConfig::setSaveFilename(const FileContext& context, const string& filename)
 {
 	saveName = context.resolveCreate(filename);
 }
@@ -97,13 +92,19 @@ void SettingsConfig::saveSetting(const string& filename)
 	const string& name = filename.empty() ? saveName : filename;
 	if (name.empty()) return;
 
-	getSettingsManager().saveSettings(*xmlElement);
-	hotKey.saveBindings(*xmlElement);
+	// Normally the following isn't needed. Only when there was no
+	// settings.xml in either the user or the system directory (so an
+	// incomplete openMSX installation!!) the top level element will have
+	// an empty name. And we shouldn't write an invalid xml file.
+	xmlElement.setName("settings");
+
+	// settings are kept up-to-date
+	hotKey.saveBindings(xmlElement);
 
 	File file(name, File::TRUNCATE);
 	string data = "<!DOCTYPE settings SYSTEM 'settings.dtd'>\n" +
-	              xmlElement->dump();
-	file.write(data.data(), unsigned(data.size()));
+	              xmlElement.dump();
+	file.write(data.data(), data.size());
 }
 
 void SettingsConfig::setSaveSettings(bool save)
@@ -116,12 +117,6 @@ SettingsManager& SettingsConfig::getSettingsManager()
 	return *settingsManager; // ***
 }
 
-XMLElement& SettingsConfig::getXMLElement()
-{
-	assert(xmlElement.get());
-	return *xmlElement;
-}
-
 
 // class SaveSettingsCommand
 
@@ -160,8 +155,7 @@ string SaveSettingsCommand::help(const vector<string>& /*tokens*/) const
 void SaveSettingsCommand::tabCompletion(vector<string>& tokens) const
 {
 	if (tokens.size() == 2) {
-		SystemFileContext context;
-		completeFileName(tokens, context);
+		completeFileName(tokens, SystemFileContext());
 	}
 }
 
@@ -181,8 +175,7 @@ string LoadSettingsCommand::execute(const vector<string>& tokens)
 	if (tokens.size() != 2) {
 		throw SyntaxError();
 	}
-	SystemFileContext context;
-	settingsConfig.loadSetting(context, tokens[1]);
+	settingsConfig.loadSetting(SystemFileContext(), tokens[1]);
 	return "";
 }
 
@@ -194,8 +187,7 @@ string LoadSettingsCommand::help(const vector<string>& /*tokens*/) const
 void LoadSettingsCommand::tabCompletion(vector<string>& tokens) const
 {
 	if (tokens.size() == 2) {
-		SystemFileContext context;
-		completeFileName(tokens, context);
+		completeFileName(tokens, SystemFileContext());
 	}
 }
 
