@@ -1,42 +1,32 @@
-// $Id: XMLElement.cc 12622 2012-06-14 20:11:33Z m9710797 $
-
 #include "XMLElement.hh"
 #include "StringOp.hh"
 #include "FileContext.hh" // for bwcompat
 #include "ConfigException.hh"
 #include "serialize.hh"
 #include "serialize_stl.hh"
+#include "memory.hh"
+#include "xrange.hh"
 #include <libxml/uri.h>
 #include <cassert>
 #include <algorithm>
 
-using std::auto_ptr;
+using std::unique_ptr;
 using std::string;
 
 namespace openmsx {
 
 XMLElement::XMLElement(string_ref name_)
-	: name(name_.data(), name_.size())
+	: name(name_.str())
 {
 }
 
 XMLElement::XMLElement(string_ref name_, string_ref data_)
-	: name(name_.data(), name_.size())
-	, data(data_.data(), data_.size())
+	: name(name_.str())
+	, data(data_.str())
 {
 }
 
-XMLElement::XMLElement(const XMLElement& element)
-{
-	*this = element;
-}
-
-XMLElement::~XMLElement()
-{
-	removeAllChildren();
-}
-
-void XMLElement::addChild(auto_ptr<XMLElement> child)
+XMLElement& XMLElement::addChild(XMLElement child)
 {
 	// Mixed-content elements are not supported by this class. In the past
 	// we had a 'assert(data.empty())' here to enforce this, though that
@@ -46,38 +36,29 @@ void XMLElement::addChild(auto_ptr<XMLElement> child)
 	// When you add child nodes to a node with data, that data will be
 	// ignored when this node is later written to disk. In the case of
 	// settings.xml this behaviour is fine.
-	assert(child.get());
-	XMLElement* child2 = child.release();
-	children.push_back(child2);
+	children.push_back(std::move(child));
+	return children.back();
 }
 
-auto_ptr<XMLElement> XMLElement::removeChild(const XMLElement& child)
+void XMLElement::removeChild(const XMLElement& child)
 {
-	assert(std::count(children.begin(), children.end(), &child) == 1);
-	children.erase(std::find(children.begin(), children.end(), &child));
-	XMLElement& child2 = const_cast<XMLElement&>(child);
-	return auto_ptr<XMLElement>(&child2);
+	assert(std::count_if(children.begin(), children.end(),
+		[&](Children::value_type& v) { return &v == &child; }) == 1);
+	auto it = std::find_if(children.begin(), children.end(),
+		[&](Children::value_type& v) { return &v == &child; });
+	assert(it != children.end());
+	children.erase(it);
 }
 
 XMLElement::Attributes::iterator XMLElement::findAttribute(string_ref name)
 {
-	for (Attributes::iterator it = attributes.begin();
-	     it != attributes.end(); ++it) {
-		if (it->first == name) {
-			return it;
-		}
-	}
-	return attributes.end();
+	return find_if(attributes.begin(), attributes.end(),
+	               [&](Attribute& a) { return a.first == name; });
 }
 XMLElement::Attributes::const_iterator XMLElement::findAttribute(string_ref name) const
 {
-	for (Attributes::const_iterator it = attributes.begin();
-	     it != attributes.end(); ++it) {
-		if (it->first == name) {
-			return it;
-		}
-	}
-	return attributes.end();
+	return find_if(attributes.begin(), attributes.end(),
+	               [&](const Attribute& a) { return a.first == name; });
 }
 
 void XMLElement::addAttribute(string_ref name, string_ref value)
@@ -88,9 +69,9 @@ void XMLElement::addAttribute(string_ref name, string_ref value)
 
 void XMLElement::setAttribute(string_ref name, string_ref value)
 {
-	Attributes::iterator it = findAttribute(name);
+	auto it = findAttribute(name);
 	if (it != attributes.end()) {
-		it->second.assign(value.data(), value.size());
+		it->second = value.str();
 	} else {
 		attributes.push_back(make_pair(name.str(), value.str()));
 	}
@@ -98,7 +79,7 @@ void XMLElement::setAttribute(string_ref name, string_ref value)
 
 void XMLElement::removeAttribute(string_ref name)
 {
-	Attributes::iterator it = findAttribute(name);
+	auto it = findAttribute(name);
 	if (it != attributes.end()) {
 		attributes.erase(it);
 	}
@@ -121,7 +102,7 @@ double XMLElement::getDataAsDouble() const
 
 void XMLElement::setName(string_ref name_)
 {
-	name.assign(name_.data(), name_.size());
+	name = name_.str();
 }
 
 void XMLElement::clearName()
@@ -132,28 +113,28 @@ void XMLElement::clearName()
 void XMLElement::setData(string_ref data_)
 {
 	assert(children.empty()); // no mixed-content elements
-	data.assign(data_.data(), data_.size());
+	data = data_.str();
 }
 
-void XMLElement::getChildren(string_ref name, Children& result) const
+std::vector<const XMLElement*> XMLElement::getChildren(string_ref name) const
 {
-	for (Children::const_iterator it = children.begin();
-	     it != children.end(); ++it) {
-		if ((*it)->getName() == name) {
-			result.push_back(*it);
+	std::vector<const XMLElement*> result;
+	for (auto& c : children) {
+		if (c.getName() == name) {
+			result.push_back(&c);
 		}
 	}
+	return result;
 }
 
 XMLElement* XMLElement::findChild(string_ref name)
 {
-	for (Children::const_iterator it = children.begin();
-	     it != children.end(); ++it) {
-		if ((*it)->getName() == name) {
-			return *it;
+	for (auto& c : children) {
+		if (c.getName() == name) {
+			return &c;
 		}
 	}
-	return NULL;
+	return nullptr;
 }
 const XMLElement* XMLElement::findChild(string_ref name) const
 {
@@ -161,36 +142,33 @@ const XMLElement* XMLElement::findChild(string_ref name) const
 }
 
 const XMLElement* XMLElement::findNextChild(string_ref name,
-	                                    unsigned& fromIndex) const
+	                                    size_t& fromIndex) const
 {
-	unsigned numChildren = unsigned(children.size());
-	for (unsigned i = fromIndex; i != numChildren; ++i) {
-		if (children[i]->getName() == name) {
+	for (auto i : xrange(fromIndex, children.size())) {
+		if (children[i].getName() == name) {
 			fromIndex = i + 1;
-			return children[i];
+			return &children[i];
 		}
 	}
-	for (unsigned i = 0; i < fromIndex; ++i) {
-		if (children[i]->getName() == name) {
+	for (auto i : xrange(fromIndex)) {
+		if (children[i].getName() == name) {
 			fromIndex = i + 1;
-			return children[i];
+			return &children[i];
 		}
 	}
-	return NULL;
+	return nullptr;
 }
 
 XMLElement* XMLElement::findChildWithAttribute(string_ref name,
 	string_ref attName, string_ref attValue)
 {
-	Children children;
-	getChildren(name, children);
-	for (Children::const_iterator it = children.begin();
-	     it != children.end(); ++it) {
-		if ((*it)->getAttribute(attName) == attValue) {
-			return *it;
+	for (auto& c : children) {
+		if ((c.getName() == name) &&
+		    (c.getAttribute(attName) == attValue)) {
+			return &c;
 		}
 	}
-	return NULL;
+	return nullptr;
 }
 
 const XMLElement* XMLElement::findChildWithAttribute(string_ref name,
@@ -202,12 +180,11 @@ const XMLElement* XMLElement::findChildWithAttribute(string_ref name,
 
 XMLElement& XMLElement::getChild(string_ref name)
 {
-	XMLElement* elem = findChild(name);
-	if (!elem) {
-		throw ConfigException(StringOp::Builder() <<
-			"Missing tag \"" << name << "\".");
+	if (auto* elem = findChild(name)) {
+		return *elem;
 	}
-	return *elem;
+	throw ConfigException(StringOp::Builder() <<
+		"Missing tag \"" << name << "\".");
 }
 const XMLElement& XMLElement::getChild(string_ref name) const
 {
@@ -217,67 +194,59 @@ const XMLElement& XMLElement::getChild(string_ref name) const
 XMLElement& XMLElement::getCreateChild(string_ref name,
                                        string_ref defaultValue)
 {
-	XMLElement* result = findChild(name);
-	if (!result) {
-		result = new XMLElement(name, defaultValue);
-		addChild(auto_ptr<XMLElement>(result));
+	if (auto* result = findChild(name)) {
+		return *result;
 	}
-	return *result;
+	return addChild(XMLElement(name, defaultValue));
 }
 
 XMLElement& XMLElement::getCreateChildWithAttribute(
 	string_ref name, string_ref attName,
 	string_ref attValue)
 {
-	XMLElement* result = findChildWithAttribute(name, attName, attValue);
-	if (!result) {
-		result = new XMLElement(name);
-		result->addAttribute(attName, attValue);
-		addChild(auto_ptr<XMLElement>(result));
+	if (auto* result = findChildWithAttribute(name, attName, attValue)) {
+		return *result;
 	}
-	return *result;
+	auto& result = addChild(XMLElement(name));
+	result.addAttribute(attName, attValue);
+	return result;
 }
 
 const string& XMLElement::getChildData(string_ref name) const
 {
-	const XMLElement& child = getChild(name);
-	return child.getData();
+	return getChild(name).getData();
 }
 
 string_ref XMLElement::getChildData(string_ref name,
                                     string_ref defaultValue) const
 {
-	const XMLElement* child = findChild(name);
+	auto* child = findChild(name);
 	return child ? child->getData() : defaultValue;
 }
 
 bool XMLElement::getChildDataAsBool(string_ref name, bool defaultValue) const
 {
-	const XMLElement* child = findChild(name);
+	auto* child = findChild(name);
 	return child ? StringOp::stringToBool(child->getData()) : defaultValue;
 }
 
 int XMLElement::getChildDataAsInt(string_ref name, int defaultValue) const
 {
-	const XMLElement* child = findChild(name);
+	auto* child = findChild(name);
 	return child ? StringOp::stringToInt(child->getData()) : defaultValue;
 }
 
 void XMLElement::setChildData(string_ref name, string_ref value)
 {
-	if (XMLElement* child = findChild(name)) {
+	if (auto* child = findChild(name)) {
 		child->setData(value);
 	} else {
-		addChild(auto_ptr<XMLElement>(new XMLElement(name, value)));
+		addChild(XMLElement(name, value));
 	}
 }
 
 void XMLElement::removeAllChildren()
 {
-	for (Children::const_iterator it = children.begin();
-	     it != children.end(); ++it) {
-		delete *it;
-	}
 	children.clear();
 }
 
@@ -288,7 +257,7 @@ bool XMLElement::hasAttribute(string_ref name) const
 
 const string& XMLElement::getAttribute(string_ref attName) const
 {
-	Attributes::const_iterator it = findAttribute(attName);
+	auto it = findAttribute(attName);
 	if (it == attributes.end()) {
 		throw ConfigException("Missing attribute \"" +
 		                      attName + "\".");
@@ -299,14 +268,14 @@ const string& XMLElement::getAttribute(string_ref attName) const
 string_ref XMLElement::getAttribute(string_ref attName,
 	                            string_ref defaultValue) const
 {
-	Attributes::const_iterator it = findAttribute(attName);
+	auto it = findAttribute(attName);
 	return (it == attributes.end()) ? defaultValue : it->second;
 }
 
 bool XMLElement::getAttributeAsBool(string_ref attName,
                                     bool defaultValue) const
 {
-	Attributes::const_iterator it = findAttribute(attName);
+	auto it = findAttribute(attName);
 	return (it == attributes.end()) ? defaultValue
 	                                : StringOp::stringToBool(it->second);
 }
@@ -314,7 +283,7 @@ bool XMLElement::getAttributeAsBool(string_ref attName,
 int XMLElement::getAttributeAsInt(string_ref attName,
                                   int defaultValue) const
 {
-	Attributes::const_iterator it = findAttribute(attName);
+	auto it = findAttribute(attName);
 	return (it == attributes.end()) ? defaultValue
 	                                : StringOp::stringToInt(it->second);
 }
@@ -322,7 +291,7 @@ int XMLElement::getAttributeAsInt(string_ref attName,
 bool XMLElement::findAttributeInt(string_ref attName,
                                   unsigned& result) const
 {
-	Attributes::const_iterator it = findAttribute(attName);
+	auto it = findAttribute(attName);
 	if (it != attributes.end()) {
 		result = StringOp::stringToInt(it->second);
 		return true;
@@ -331,24 +300,6 @@ bool XMLElement::findAttributeInt(string_ref attName,
 	}
 }
 
-XMLElement& XMLElement::operator=(const XMLElement& element)
-{
-	if (&element == this) {
-		// assign to itself
-		return *this;
-	}
-	name = element.name;
-	data = element.data;
-	attributes = element.attributes;
-
-	removeAllChildren();
-	for (Children::const_iterator it = element.children.begin();
-	     it != element.children.end(); ++it) {
-		addChild(auto_ptr<XMLElement>(new XMLElement(**it)));
-	}
-	return *this;
-}
-
 string XMLElement::dump() const
 {
 	StringOp::Builder result;
@@ -360,10 +311,9 @@ void XMLElement::dump(StringOp::Builder& result, unsigned indentNum) const
 {
 	string indent(indentNum, ' ');
 	result << indent << '<' << getName();
-	for (Attributes::const_iterator it = attributes.begin();
-	     it != attributes.end(); ++it) {
-		result << ' ' << it->first
-		       << "=\"" << XMLEscape(it->second) << '"';
+	for (auto& p : attributes) {
+		result << ' ' << p.first
+		       << "=\"" << XMLEscape(p.second) << '"';
 	}
 	if (children.empty()) {
 		if (data.empty()) {
@@ -374,9 +324,8 @@ void XMLElement::dump(StringOp::Builder& result, unsigned indentNum) const
 		}
 	} else {
 		result << ">\n";
-		for (Children::const_iterator it = children.begin();
-		     it != children.end(); ++it) {
-			(*it)->dump(result, indentNum + 2);
+		for (auto& c : children) {
+			c.dump(result, indentNum + 2);
 		}
 		result << indent << "</" << getName() << ">\n";
 	}
@@ -385,60 +334,47 @@ void XMLElement::dump(StringOp::Builder& result, unsigned indentNum) const
 string XMLElement::XMLEscape(const string& str)
 {
 	xmlChar* buffer = xmlEncodeEntitiesReentrant(
-		NULL, reinterpret_cast<const xmlChar*>(str.c_str()));
+		nullptr, reinterpret_cast<const xmlChar*>(str.c_str()));
 	string result = reinterpret_cast<const char*>(buffer);
 	xmlFree(buffer);
 	return result;
 }
 
-static auto_ptr<FileContext> lastSerializedFileContext;
-auto_ptr<FileContext> XMLElement::getLastSerializedFileContext()
+static unique_ptr<FileContext> lastSerializedFileContext;
+unique_ptr<FileContext> XMLElement::getLastSerializedFileContext()
 {
-	return lastSerializedFileContext; // this also sets value to NULL;
+	return std::move(lastSerializedFileContext); // this also sets value to nullptr;
 }
 // version 1: initial version
 // version 2: removed 'context' tag
 //            also removed 'parent', but that was never serialized
+//        2b: (no need to increase version) name and data members are
+//            serialized as normal members instead of constructor parameters
+//        2c: (no need to increase version) attributes were initially stored as
+//            map<string, string>, later this was changed to
+//            vector<pair<string, string>>. To keep bw-compat the serialize()
+//            method converted between these two formats. Though (by luck) in
+//            the XML output both datastructures are serialized to the same
+//            format, so we can drop this conversion step without breaking
+//            bw-compat.
 template<typename Archive>
 void XMLElement::serialize(Archive& ar, unsigned version)
 {
-	// note: In the past attributes were stored in a map instead of a
-	//       vector. To keep backwards compatible with the serialized
-	//       format, we still convert attributes to this format.
-	typedef std::map<string, string> AttributesMap;
+	ar.serialize("name", name);
+	ar.serialize("data", data);
+	ar.serialize("attributes", attributes);
+	ar.serialize("children", children);
 
-	if (!ar.isLoader()) {
-		AttributesMap tmpAtt(attributes.begin(), attributes.end());
-		ar.serialize("attributes", tmpAtt);
-		ar.serialize("children", getChildren());
-	} else {
-		AttributesMap tmpAtt;
-		ar.serialize("attributes", tmpAtt);
-		for (AttributesMap::const_iterator it = tmpAtt.begin();
-		     it != tmpAtt.end(); ++it) {
-			// TODO "string -> char* -> string" conversion can
-			//       be optimized
-			addAttribute(it->first.c_str(), it->second);
-		}
-
-		XMLElement::Children tmp;
-		ar.serialize("children", tmp);
-		for (XMLElement::Children::const_iterator it = tmp.begin();
-		     it != tmp.end(); ++it) {
-			addChild(auto_ptr<XMLElement>(*it));
-		}
-	}
 	if (ar.versionBelow(version, 2)) {
 		assert(ar.isLoader());
-		auto_ptr<FileContext> context;
+		unique_ptr<FileContext> context;
 		ar.serialize("context", context);
-		if (context.get()) {
-			assert(!lastSerializedFileContext.get());
-			lastSerializedFileContext = context;
+		if (context) {
+			assert(!lastSerializedFileContext);
+			lastSerializedFileContext = std::move(context);
 		}
 	}
 }
 INSTANTIATE_SERIALIZE_METHODS(XMLElement);
 
-
 } // namespace openmsx
