@@ -1,16 +1,18 @@
-// $Id: XMLLoader.cc 12736 2012-07-17 16:25:51Z m9710797 $
-
 #include "XMLLoader.hh"
 #include "XMLElement.hh"
 #include "XMLException.hh"
 #include "File.hh"
 #include "FileException.hh"
+#include "memory.hh"
+#include "xrange.hh"
 #include <cassert>
+#include <limits>
 #include <cstring>
 #include <libxml/parser.h>
 #include <libxml/xmlversion.h>
 
-using std::auto_ptr;
+using std::unique_ptr;
+using std::vector;
 using std::string;
 
 namespace openmsx {
@@ -22,10 +24,10 @@ struct XMLLoaderHelper
 	{
 	}
 
-	std::auto_ptr<XMLElement> root;
-	std::vector<XMLElement*> current;
-	std::string data;
-	std::string systemID;
+	XMLElement root;
+	vector<XMLElement*> current;
+	string data;
+	string systemID;
 };
 
 static void cbStartElement(
@@ -34,26 +36,25 @@ static void cbStartElement(
 	int /*nb_namespaces*/, const xmlChar** /*namespaces*/,
 	int nb_attributes, int /*nb_defaulted*/, const xmlChar** attrs)
 {
-	XMLLoaderHelper* helper = static_cast<XMLLoaderHelper*>(helper_);
-	std::auto_ptr<XMLElement> newElem(
-		new XMLElement(reinterpret_cast<const char*>(localname)));
+	auto* helper = static_cast<XMLLoaderHelper*>(helper_);
+	XMLElement newElem(reinterpret_cast<const char*>(localname));
 
-	for (int i = 0; i < nb_attributes; i++) {
-		const char* valueStart =
+	for (auto i : xrange(nb_attributes)) {
+		auto valueStart =
 			reinterpret_cast<const char*>(attrs[i * 5 + 3]);
-		const char* valueEnd =
+		auto valueEnd =
 			reinterpret_cast<const char*>(attrs[i * 5 + 4]);
-		newElem->addAttribute(
+		newElem.addAttribute(
 			reinterpret_cast<const char*>(attrs[i * 5 + 0]),
-			std::string(valueStart, valueEnd - valueStart)
-			);
+			std::string(valueStart, valueEnd - valueStart));
 	}
 
-	XMLElement* newElem2 = newElem.get();
+	XMLElement* newElem2;
 	if (!helper->current.empty()) {
-		helper->current.back()->addChild(newElem);
+		newElem2 = &helper->current.back()->addChild(std::move(newElem));
 	} else {
-		helper->root = newElem;
+		helper->root = std::move(newElem);
+		newElem2 = &helper->root;
 	}
 	helper->current.push_back(newElem2);
 
@@ -64,9 +65,9 @@ static void cbEndElement(
 	void* helper_,
 	const xmlChar* localname, const xmlChar* /*prefix*/, const xmlChar* /*uri*/)
 {
-	XMLLoaderHelper* helper = static_cast<XMLLoaderHelper*>(helper_);
+	auto* helper = static_cast<XMLLoaderHelper*>(helper_);
 	assert(!helper->current.empty());
-	XMLElement& current = *helper->current.back();
+	auto& current = *helper->current.back();
 	assert(reinterpret_cast<const char*>(localname) == current.getName());
 	(void)localname;
 
@@ -78,7 +79,7 @@ static void cbEndElement(
 
 static void cbCharacters(void* helper_, const xmlChar* chars, int len)
 {
-	XMLLoaderHelper* helper = static_cast<XMLLoaderHelper*>(helper_);
+	auto* helper = static_cast<XMLLoaderHelper*>(helper_);
 	assert(!helper->current.empty());
 	if (!helper->current.back()->hasChildren()) {
 		helper->data.append(reinterpret_cast<const char*>(chars), len);
@@ -88,23 +89,27 @@ static void cbCharacters(void* helper_, const xmlChar* chars, int len)
 static void cbInternalSubset(void* helper_, const xmlChar* /*name*/,
                              const xmlChar* /*extID*/, const xmlChar* systemID)
 {
-	XMLLoaderHelper* helper = static_cast<XMLLoaderHelper*>(helper_);
+	auto* helper = static_cast<XMLLoaderHelper*>(helper_);
 	helper->systemID = reinterpret_cast<const char*>(systemID);
 }
 
-auto_ptr<XMLElement> load(const string& filename, const string& systemID)
+XMLElement load(const string& filename, const string& systemID)
 {
 	File file(filename);
 	// TODO: Reading blocks to a fixed-size buffer would require less memory
 	//       when reading (g)zipped XML.
 	// Note: On destruction of "file", munmap() is called automatically.
 	const byte* fileContent;
-	unsigned size;
+	size_t size_;
 	try {
-		fileContent = file.mmap(size);
+		fileContent = file.mmap(size_);
 	} catch (FileException& e) {
 		throw XMLException(filename + ": failed to mmap: " + e.getMessage());
 	}
+	if (size_ > size_t(std::numeric_limits<int>::max())) {
+		throw XMLException(filename + ": file too big");
+	}
+	auto size = int(size_);
 
 	xmlSAXHandler handler;
 	memset(&handler, 0, sizeof(handler));
@@ -117,7 +122,7 @@ auto_ptr<XMLElement> load(const string& filename, const string& systemID)
 	XMLLoaderHelper helper;
 
 	xmlParserCtxtPtr ctxt = xmlCreatePushParserCtxt(
-		&handler, &helper, NULL, 0, filename.c_str()
+		&handler, &helper, nullptr, 0, filename.c_str()
 		);
 	if (!ctxt) {
 		throw XMLException(filename + ": Could not create XML parser context");
@@ -129,7 +134,7 @@ auto_ptr<XMLElement> load(const string& filename, const string& systemID)
 		throw XMLException(filename + ": Document parsing failed");
 	}
 
-	if (!helper.root.get()) {
+	if (helper.root.getName().empty()) {
 		throw XMLException(filename +
 			": Document doesn't contain mandatory root Element");
 	}
@@ -143,7 +148,7 @@ auto_ptr<XMLElement> load(const string& filename, const string& systemID)
 			"You're probably using an old incompatible file format.");
 	}
 
-	return helper.root;
+	return std::move(helper.root);
 }
 
 } // namespace XMLLoader
