@@ -1,5 +1,3 @@
-// $Id: CommandConsole.cc 12700 2012-07-07 20:21:13Z m9710797 $
-
 #include "CommandConsole.hh"
 #include "CommandException.hh"
 #include "GlobalCommandController.hh"
@@ -23,6 +21,8 @@
 #include "utf8_unchecked.hh"
 #include "StringOp.hh"
 #include "ScopedAssign.hh"
+#include "memory.hh"
+#include "xrange.hh"
 #include <algorithm>
 #include <fstream>
 #include <cassert>
@@ -41,20 +41,20 @@ ConsoleLine::ConsoleLine()
 }
 
 ConsoleLine::ConsoleLine(string_ref line_, unsigned rgb)
-	: line(line_.data(), line_.size())
+	: line(line_.str())
 	, chunks(1, make_pair(rgb, 0))
 {
 }
 
 void ConsoleLine::addChunk(string_ref text, unsigned rgb)
 {
-	chunks.push_back(make_pair(rgb, string_ref::size_type(line.size())));
-	line.append(text.data(), string_ref::size_type(text.size()));
+	chunks.push_back(make_pair(rgb, line.size()));
+	line.append(text.data(), text.size());
 }
 
 unsigned ConsoleLine::numChars() const
 {
-	return utf8::unchecked::size(line);
+	return unsigned(utf8::unchecked::size(line));
 }
 
 const string& ConsoleLine::str() const
@@ -76,10 +76,10 @@ unsigned ConsoleLine::chunkColor(unsigned i) const
 string_ref ConsoleLine::chunkText(unsigned i) const
 {
 	assert(i < chunks.size());
-	string_ref::size_type pos = chunks[i].second;
-	string_ref::size_type len = ((i + 1) == chunks.size())
-	                          ? string_ref::npos
-	                          : chunks[i + 1].second - pos;
+	auto pos = chunks[i].second;
+	auto len = ((i + 1) == chunks.size())
+	         ? string_ref::npos
+	         : chunks[i + 1].second - pos;
 	return string_ref(line).substr(pos, len);
 }
 
@@ -92,9 +92,9 @@ ConsoleLine ConsoleLine::substr(unsigned pos, unsigned len) const
 		return result;
 	}
 
-	string::const_iterator begin = line.begin();
+	auto begin = line.begin();
 	utf8::unchecked::advance(begin, pos);
-	string::const_iterator end = begin;
+	auto end = begin;
 	while (len-- && (end != line.end())) {
 		utf8::unchecked::next(end);
 	}
@@ -128,13 +128,16 @@ CommandConsole::CommandConsole(
 	: commandController(commandController_)
 	, eventDistributor(eventDistributor_)
 	, display(display_)
-	, consoleSetting(new BooleanSetting(commandController, "console",
+	, consoleSetting(make_unique<BooleanSetting>(
+		commandController, "console",
 		"turns console display on/off", false, Setting::DONT_SAVE))
-	, historySizeSetting(new IntegerSetting(commandController, "console_history_size",
+	, historySizeSetting(make_unique<IntegerSetting>(
+		commandController, "console_history_size",
 		"amount of commands kept in console history", 100, 0, 10000))
-	, removeDoublesSetting(new BooleanSetting(commandController,
-		"console_remove_doubles", "don't add the command to history if "
-		"it's the same as the previous one", true))
+	, removeDoublesSetting(make_unique<BooleanSetting>(
+		commandController, "console_remove_doubles",
+		"don't add the command to history if it's the same as the previous one",
+		true))
 	, executingCommand(false)
 {
 	resetScrollBack();
@@ -144,8 +147,9 @@ CommandConsole::CommandConsole(
 	loadHistory();
 	Completer::setOutput(this);
 
-	print(Version::FULL_VERSION);
-	print(string(Version::FULL_VERSION.size(), '-'));
+	const auto& fullVersion = Version::full();
+	print(fullVersion);
+	print(string(fullVersion.size(), '-'));
 	print("\n"
 	      "General information about openMSX is available at "
 	      "http://www.openmsx.org.\n"
@@ -167,8 +171,8 @@ CommandConsole::~CommandConsole()
 {
 	eventDistributor.unregisterEventListener(OPENMSX_KEY_DOWN_EVENT, *this);
 	eventDistributor.unregisterEventListener(OPENMSX_KEY_UP_EVENT, *this);
-	commandController.getInterpreter().setOutput(NULL);
-	Completer::setOutput(NULL);
+	commandController.getInterpreter().setOutput(nullptr);
+	Completer::setOutput(nullptr);
 }
 
 void CommandConsole::saveHistory()
@@ -182,9 +186,8 @@ void CommandConsole::saveHistory()
 			throw FileException(
 				"Error while saving the console history.");
 		}
-		for (History::const_iterator it = history.begin();
-		     it != history.end(); ++it) {
-			outputfile << string_ref(*it).substr(string_ref::size_type(prompt.size())) << '\n';
+		for (auto& s : history) {
+			outputfile << string_ref(s).substr(prompt.size()) << '\n';
 		}
 	} catch (FileException& e) {
 		commandController.getCliComm().printWarning(e.getMessage());
@@ -216,7 +219,7 @@ void CommandConsole::loadHistory()
 
 BooleanSetting& CommandConsole::getConsoleSetting()
 {
-	return *consoleSetting.get();
+	return *consoleSetting;
 }
 
 void CommandConsole::getCursorPosition(unsigned& xPosition, unsigned& yPosition) const
@@ -234,7 +237,7 @@ unsigned CommandConsole::getScrollBack() const
 ConsoleLine CommandConsole::getLine(unsigned line) const
 {
 	unsigned count = 0;
-	for (unsigned buf = 0; buf < lines.size(); ++buf) {
+	for (auto buf : xrange(lines.size())) {
 		count += (lines[buf].numChars() / getColumns()) + 1;
 		if (count > line) {
 			return lines[buf].substr(
@@ -245,10 +248,10 @@ ConsoleLine CommandConsole::getLine(unsigned line) const
 	return ConsoleLine();
 }
 
-int CommandConsole::signalEvent(const shared_ptr<const Event>& event)
+int CommandConsole::signalEvent(const std::shared_ptr<const Event>& event)
 {
-	const KeyEvent& keyEvent = checked_cast<const KeyEvent&>(*event);
-	if (!consoleSetting->getValue()) {
+	auto& keyEvent = checked_cast<const KeyEvent&>(*event);
+	if (!consoleSetting->getBoolean()) {
 		return 0;
 	}
 
@@ -262,8 +265,7 @@ int CommandConsole::signalEvent(const shared_ptr<const Event>& event)
 			if (handleEvent(keyEvent)) {
 				// event was used
 				display.repaintDelayed(40000); // 25fps
-				return EventDistributor::HOTKEY |
-				       EventDistributor::MSX;
+				return EventDistributor::HOTKEY; // block HOTKEY and MSX
 			}
 		} else {
 			// For commands that take a long time to execute (e.g.
@@ -275,115 +277,113 @@ int CommandConsole::signalEvent(const shared_ptr<const Event>& event)
 	} else {
 		assert(event->getType() == OPENMSX_KEY_UP_EVENT);
 	}
-	return EventDistributor::MSX;
+	return EventDistributor::MSX; // block MSX
 }
 
 bool CommandConsole::handleEvent(const KeyEvent& keyEvent)
 {
-	Keys::KeyCode keyCode = keyEvent.getKeyCode();
+	auto keyCode = keyEvent.getKeyCode();
 	int key = keyCode &  Keys::K_MASK;
 	int mod = keyCode & ~Keys::K_MASK;
-	word chr = keyEvent.getUnicode();
 
-	bool used = true;
 	switch (mod) {
 	case Keys::KM_CTRL:
 		switch (key) {
 		case Keys::K_H:
 			backspace();
-			break;
+			return true;
 		case Keys::K_A:
 			cursorPosition = unsigned(prompt.size());
-			break;
+			return true;
 		case Keys::K_E:
 			cursorPosition = lines[0].numChars();
-			break;
+			return true;
 		case Keys::K_C:
 			clearCommand();
-			break;
-		default:
-			used = false;
-			break;
+			return true;
 		}
 		break;
 	case Keys::KM_SHIFT:
 		switch (key) {
 		case Keys::K_PAGEUP:
 			scroll(max<int>(getRows() - 1, 1));
-			break;
+			return true;
 		case Keys::K_PAGEDOWN:
 			scroll(-max<int>(getRows() - 1, 1));
-			break;
-		default:
-			if (chr) {
-				normalKey(chr);
-			} else {
-				used = false;
-			}
-			break;
+			return true;
 		}
 		break;
-	case Keys::KM_MODE: // e.g. to type '1-9' on a N900
-	case Keys::KM_ALT:  // e.g. to type | [ ] on a azerty keyboard layout
-	// Don't add the META modifier here because that will break the Cmd+L
-	// hotkey on MacOSX to toggle the console.
-	case 0:
+	case 0: // no modifier
 		switch (key) {
 		case Keys::K_PAGEUP:
 			scroll(1);
-			break;
+			return true;
 		case Keys::K_PAGEDOWN:
 			scroll(-1);
-			break;
+			return true;
 		case Keys::K_UP:
 			prevCommand();
-			break;
+			return true;
 		case Keys::K_DOWN:
 			nextCommand();
-			break;
+			return true;
 		case Keys::K_BACKSPACE:
 			backspace();
-			break;
+			return true;
 		case Keys::K_DELETE:
 			delete_key();
-			break;
+			return true;
 		case Keys::K_TAB:
 			tabCompletion();
-			break;
+			return true;
 		case Keys::K_RETURN:
 		case Keys::K_KP_ENTER:
 			commandExecute();
 			cursorPosition = unsigned(prompt.size());
-			break;
+			return true;
 		case Keys::K_LEFT:
 			if (cursorPosition > prompt.size()) {
 				--cursorPosition;
 			}
-			break;
+			return true;
 		case Keys::K_RIGHT:
 			if (cursorPosition < lines[0].numChars()) {
 				++cursorPosition;
 			}
-			break;
+			return true;
 		case Keys::K_HOME:
 			cursorPosition = unsigned(prompt.size());
-			break;
+			return true;
 		case Keys::K_END:
 			cursorPosition = lines[0].numChars();
-			break;
-		default:
-			if (chr) {
-				normalKey(chr);
-			} else {
-				used = false;
-			}
-			break;
+			return true;
 		}
 		break;
-	default:
-		used = false;
 	}
-	return used;
+
+	uint16_t unicode = keyEvent.getUnicode();
+	if (!unicode || (mod & Keys::KM_META)) {
+		// Disallow META modifer for 'normal' key presses because on
+		// MacOSX Cmd+L is used as a hotkey to toggle the console.
+		// Hopefully there are no systems that require META to type
+		// normal keys. However there _are_ systems that require the
+		// following modifiers, some examples:
+		//  MODE:     to type '1-9' on a N900
+		//  ALT:      to type | [ ] on a azerty keyboard layout
+		//  CTRL+ALT: to type '#' on a spanish keyboard layout (windows)
+		//
+		// Event was not used by the console, allow the other
+		// subsystems to process it. E.g. F10, or Cmd+L to close the
+		// console.
+		return false;
+	}
+
+	if (unicode >= 0x20) {
+		normalKey(unicode);
+	} else {
+		// Skip CTRL-<X> combinations, but still return true.
+	}
+	return true;
 }
 
 void CommandConsole::setColumns(unsigned columns_)
@@ -419,7 +419,7 @@ unsigned CommandConsole::getOutputColumns() const
 void CommandConsole::print(string_ref text, unsigned rgb)
 {
 	while (true) {
-		string_ref::size_type pos = text.find('\n');
+		auto pos = text.find('\n');
 		newLineConsole(ConsoleLine(text.substr(0, pos), rgb));
 		if (pos == string_ref::npos) return;
 		text = text.substr(pos + 1); // skip newline
@@ -448,7 +448,7 @@ void CommandConsole::putCommandHistory(const string& command)
 	if (command == prompt) {
 		return;
 	}
-	if (removeDoublesSetting->getValue() && !history.empty()
+	if (removeDoublesSetting->getBoolean() && !history.empty()
 		&& (history.back() == command)) {
 		return;
 	}
@@ -456,7 +456,7 @@ void CommandConsole::putCommandHistory(const string& command)
 	history.push_back(command);
 
 	// if necessary, shrink history to the desired (smaller) size
-	while (history.size() > unsigned(historySizeSetting->getValue())) {
+	while (history.size() > unsigned(historySizeSetting->getInt())) {
 		history.pop_front();
 	}
 }
@@ -498,7 +498,7 @@ void CommandConsole::commandExecute()
 ConsoleLine CommandConsole::highLight(string_ref line)
 {
 	assert(line.starts_with(prompt));
-	string_ref command = line.substr(string_ref::size_type(prompt.size()));
+	string_ref command = line.substr(prompt.size());
 	ConsoleLine result;
 	result.addChunk(prompt, 0xffffff);
 
@@ -544,15 +544,15 @@ void CommandConsole::tabCompletion()
 	string_ref front = utf8::unchecked::substr(lines[0].str(), pl, cursorPosition - pl);
 	string_ref back  = utf8::unchecked::substr(lines[0].str(), cursorPosition);
 	string newFront = commandController.tabCompletion(front);
-	cursorPosition = pl + utf8::unchecked::size(newFront);
+	cursorPosition = pl + unsigned(utf8::unchecked::size(newFront));
 	currentLine = prompt + newFront + back;
 	lines[0] = highLight(currentLine);
 }
 
 void CommandConsole::scroll(int delta)
 {
-	consoleScrollBack = min<int>(max(consoleScrollBack + delta, 0),
-	                             lines.size());
+	consoleScrollBack = min(max(consoleScrollBack + delta, 0),
+	                        int(lines.size()));
 }
 
 void CommandConsole::prevCommand()
@@ -562,7 +562,7 @@ void CommandConsole::prevCommand()
 		return; // no elements
 	}
 	bool match = false;
-	History::const_iterator tempScrollBack = commandScrollBack;
+	auto tempScrollBack = commandScrollBack;
 	while ((tempScrollBack != history.begin()) && !match) {
 		--tempScrollBack;
 		match = StringOp::startsWith(*tempScrollBack, currentLine);
@@ -581,7 +581,7 @@ void CommandConsole::nextCommand()
 		return; // don't loop !
 	}
 	bool match = false;
-	History::const_iterator tempScrollBack = commandScrollBack;
+	auto tempScrollBack = commandScrollBack;
 	while ((++tempScrollBack != history.end()) && !match) {
 		match = StringOp::startsWith(*tempScrollBack, currentLine);
 	}
@@ -611,9 +611,9 @@ void CommandConsole::backspace()
 	resetScrollBack();
 	if (cursorPosition > prompt.size()) {
 		currentLine = lines[0].str();
-		string::iterator begin = currentLine.begin();
+		auto begin = currentLine.begin();
 		utf8::unchecked::advance(begin, cursorPosition - 1);
-		string::iterator end = begin;
+		auto end = begin;
 		utf8::unchecked::advance(end, 1);
 		currentLine.erase(begin, end);
 		lines[0] = highLight(currentLine);
@@ -626,21 +626,21 @@ void CommandConsole::delete_key()
 	resetScrollBack();
 	if (lines[0].numChars() > cursorPosition) {
 		currentLine = lines[0].str();
-		string::iterator begin = currentLine.begin();
+		auto begin = currentLine.begin();
 		utf8::unchecked::advance(begin, cursorPosition);
-		string::iterator end = begin;
+		auto end = begin;
 		utf8::unchecked::advance(end, 1);
 		currentLine.erase(begin, end);
 		lines[0] = highLight(currentLine);
 	}
 }
 
-void CommandConsole::normalKey(word chr)
+void CommandConsole::normalKey(uint16_t chr)
 {
 	assert(chr);
 	resetScrollBack();
 	currentLine = lines[0].str();
-	string::iterator pos = currentLine.begin();
+	auto pos = currentLine.begin();
 	utf8::unchecked::advance(pos, cursorPosition);
 	utf8::unchecked::append(uint32_t(chr), inserter(currentLine, pos));
 	lines[0] = highLight(currentLine);
