@@ -1,5 +1,3 @@
-// $Id: OSDConsoleRenderer.cc 12804 2012-08-13 17:54:47Z m9710797 $
-
 #include "OSDConsoleRenderer.hh"
 #include "CommandConsole.hh"
 #include "EnumSetting.hh"
@@ -7,7 +5,6 @@
 #include "BooleanSetting.hh"
 #include "FilenameSetting.hh"
 #include "GlobalSettings.hh"
-#include "TTFFont.hh"
 #include "SDLImage.hh"
 #include "Display.hh"
 #include "InputEventGenerator.hh"
@@ -17,7 +14,9 @@
 #include "Reactor.hh"
 #include "MSXException.hh"
 #include "openmsx.hh"
+#include "memory.hh"
 #include "unreachable.hh"
+#include "xrange.hh"
 #include <algorithm>
 #include <cassert>
 
@@ -35,19 +34,24 @@ namespace openmsx {
   * that image's alpha channel is used instead.
   */
 static const int CONSOLE_ALPHA = 180;
-static const unsigned long long BLINK_RATE = 500000; // us
+static const uint64_t BLINK_RATE = 500000; // us
 static const int CHAR_BORDER = 4;
 
 
-class OSDSettingChecker : public SettingChecker<FilenameSetting::Policy>
+// class OSDConsoleRenderer::TextCacheElement
+
+OSDConsoleRenderer::TextCacheElement::TextCacheElement(
+		const std::string& text_, unsigned rgb_,
+		std::unique_ptr<BaseImage> image_, unsigned width_)
+	: text(text_), image(std::move(image_)), rgb(rgb_), width(width_)
 {
-public:
-	OSDSettingChecker(OSDConsoleRenderer& renderer);
-	virtual void check(SettingImpl<FilenameSetting::Policy>& setting,
-	                   std::string& value);
-private:
-	OSDConsoleRenderer& renderer;
-};
+}
+
+OSDConsoleRenderer::TextCacheElement::TextCacheElement(TextCacheElement&& rhs)
+	: text(std::move(rhs.text)), image(std::move(rhs.image))
+	, rgb(std::move(rhs.rgb)), width(std::move(rhs.width))
+{
+}
 
 
 // class OSDConsoleRenderer
@@ -60,14 +64,10 @@ OSDConsoleRenderer::OSDConsoleRenderer(
 	, reactor(reactor_)
 	, console(console_)
 	, consoleSetting(console.getConsoleSetting())
-	, settingChecker(new OSDSettingChecker(*this))
 	, screenW(screenW_)
 	, screenH(screenH_)
 	, openGL(openGL_)
 {
-	// cacheHint must always point to a valid item, so insert a dummy entry
-	textCache.push_back(TextCacheElement("", 0, NULL, 0));
-	cacheHint = textCache.begin();
 #if !COMPONENT_GL
 	assert(!openGL);
 #endif
@@ -82,23 +82,26 @@ OSDConsoleRenderer::OSDConsoleRenderer(
 
 	// font size
 	CommandController& commandController = reactor.getCommandController();
-	fontSizeSetting.reset(new IntegerSetting(commandController,
-		"consolefontsize", "Size of the console font", 12, 8, 32));
+	fontSizeSetting = make_unique<IntegerSetting>(commandController,
+		"consolefontsize", "Size of the console font", 12, 8, 32);
 
 	// font
 	const string& defaultFont = "skins/VeraMono.ttf.gz";
-	fontSetting.reset(new FilenameSetting(commandController,
-		"consolefont", "console font file", defaultFont));
+	fontSetting = make_unique<FilenameSetting>(commandController,
+		"consolefont", "console font file", defaultFont);
+	fontSetting->setChecker([this](TclObject& value) {
+		loadFont(value.getString().str());
+	});
 	try {
-		fontSetting->setChecker(settingChecker.get());
+		loadFont(fontSetting->getString());
 	} catch (MSXException&) {
 		// This will happen when you upgrade from the old .png based
 		// fonts to the new .ttf fonts. So provide a smooth upgrade path.
 		reactor.getCliComm().printWarning(
-			"Loading selected font (" + fontSetting->getValue() +
+			"Loading selected font (" + fontSetting->getString() +
 			") failed. Reverting to default font (" + defaultFont + ").");
-		fontSetting->changeValue(defaultFont);
-		if (!font.get()) {
+		fontSetting->setString(defaultFont);
+		if (font.empty()) {
 			// we can't continue without font
 			throw FatalError("Couldn't load default console font.\n"
 			                 "Please check your installation.");
@@ -106,50 +109,53 @@ OSDConsoleRenderer::OSDConsoleRenderer(
 	}
 
 	// rows / columns
-	int columns = (((screenW - CHAR_BORDER) / font->getWidth()) * 30) / 32;
-	int rows = ((screenH / font->getHeight()) * 6) / 15;
-	consoleColumnsSetting.reset(new IntegerSetting(commandController,
+	int columns = (((screenW - CHAR_BORDER) / font.getWidth()) * 30) / 32;
+	int rows = ((screenH / font.getHeight()) * 6) / 15;
+	consoleColumnsSetting = make_unique<IntegerSetting>(commandController,
 		"consolecolumns", "number of columns in the console", columns,
-		32, 999));
-	consoleRowsSetting.reset(new IntegerSetting(commandController,
-		"consolerows", "number of rows in the console", rows, 1, 99));
+		32, 999);
+	consoleRowsSetting = make_unique<IntegerSetting>(commandController,
+		"consolerows", "number of rows in the console", rows, 1, 99);
 	adjustColRow();
 
 	// placement
-	typedef EnumSetting<Placement>::Map PlaceMap;
-	PlaceMap placeMap;
-	placeMap["topleft"]     = CP_TOPLEFT;
-	placeMap["top"]         = CP_TOP;
-	placeMap["topright"]    = CP_TOPRIGHT;
-	placeMap["left"]        = CP_LEFT;
-	placeMap["center"]      = CP_CENTER;
-	placeMap["right"]       = CP_RIGHT;
-	placeMap["bottomleft"]  = CP_BOTTOMLEFT;
-	placeMap["bottom"]      = CP_BOTTOM;
-	placeMap["bottomright"] = CP_BOTTOMRIGHT;
-	consolePlacementSetting.reset(new EnumSetting<Placement>(commandController,
-		"consoleplacement", "position of the console within the emulator",
-		CP_BOTTOM, placeMap));
+	EnumSetting<Placement>::Map placeMap;
+	placeMap.push_back(std::make_pair("topleft",     CP_TOPLEFT));
+	placeMap.push_back(std::make_pair("top",         CP_TOP));
+	placeMap.push_back(std::make_pair("topright",    CP_TOPRIGHT));
+	placeMap.push_back(std::make_pair("left",        CP_LEFT));
+	placeMap.push_back(std::make_pair("center",      CP_CENTER));
+	placeMap.push_back(std::make_pair("right",       CP_RIGHT));
+	placeMap.push_back(std::make_pair("bottomleft",  CP_BOTTOMLEFT));
+	placeMap.push_back(std::make_pair("bottom",      CP_BOTTOM));
+	placeMap.push_back(std::make_pair("bottomright", CP_BOTTOMRIGHT));
+	// On Android, console must by default be placed on top, in order to prevent
+	// that it overlaps with the virtual Android keyboard, which is always placed
+	// at the bottom of the screen
+	consolePlacementSetting = make_unique<EnumSetting<Placement>>(
+		commandController, "consoleplacement",
+		"position of the console within the emulator",
+		PLATFORM_ANDROID ? CP_TOP : CP_BOTTOM, placeMap);
 
 	// background (only load backgound on first paint())
-	backgroundSetting.reset(new FilenameSetting(commandController,
+	backgroundSetting = make_unique<FilenameSetting>(commandController,
 		"consolebackground", "console background file",
-		"skins/ConsoleBackgroundGrey.png"));
-	backgroundSetting->setChecker(settingChecker.get(), false); // don't load
+		"skins/ConsoleBackgroundGrey.png");
+	backgroundSetting->setChecker([this](TclObject& value) {
+		loadBackground(value.getString().str());
+	});
+	// don't yet load background
 
 	consoleSetting.attach(*this);
+	fontSetting->attach(*this);
 	fontSizeSetting->attach(*this);
-	setActive(consoleSetting.getValue());
+	setActive(consoleSetting.getBoolean());
 }
 
 OSDConsoleRenderer::~OSDConsoleRenderer()
 {
-	for (TextCache::iterator it = textCache.begin();
-	     it != textCache.end(); ++it) {
-		delete it->image;
-	}
-
 	fontSizeSetting->detach(*this);
+	fontSetting->detach(*this);
 	consoleSetting.detach(*this);
 	setActive(false);
 }
@@ -157,11 +163,11 @@ OSDConsoleRenderer::~OSDConsoleRenderer()
 void OSDConsoleRenderer::adjustColRow()
 {
 	unsigned consoleColumns = std::min<unsigned>(
-		consoleColumnsSetting->getValue(),
-		(screenW - CHAR_BORDER) / font->getWidth());
+		consoleColumnsSetting->getInt(),
+		(screenW - CHAR_BORDER) / font.getWidth());
 	unsigned consoleRows = std::min<unsigned>(
-		consoleRowsSetting->getValue(),
-		screenH / font->getHeight());
+		consoleRowsSetting->getInt(),
+		screenH / font.getHeight());
 	console.setColumns(consoleColumns);
 	console.setRows(consoleRows);
 }
@@ -169,9 +175,10 @@ void OSDConsoleRenderer::adjustColRow()
 void OSDConsoleRenderer::update(const Setting& setting)
 {
 	if (&setting == &consoleSetting) {
-		setActive(consoleSetting.getValue());
-	} else if (&setting == fontSizeSetting.get()) {
-		loadFont(fontSetting->getValue());
+		setActive(consoleSetting.getBoolean());
+	} else if ((&setting == fontSetting.get()) ||
+	           (&setting == fontSizeSetting.get())) {
+		loadFont(fontSetting->getString());
 	} else {
 		UNREACHABLE;
 	}
@@ -191,11 +198,11 @@ void OSDConsoleRenderer::setActive(bool active_)
 
 byte OSDConsoleRenderer::getVisibility() const
 {
-	const unsigned long long FADE_IN_DURATION  = 100000;
-	const unsigned long long FADE_OUT_DURATION = 150000;
+	const uint64_t FADE_IN_DURATION  = 100000;
+	const uint64_t FADE_OUT_DURATION = 150000;
 
-	unsigned long long now = Timer::getTime();
-	unsigned long long dur = now - activeTime;
+	auto now = Timer::getTime();
+	auto dur = now - activeTime;
 	if (active) {
 		if (dur > FADE_IN_DURATION) {
 			return 255;
@@ -218,11 +225,11 @@ bool OSDConsoleRenderer::updateConsoleRect()
 	adjustColRow();
 
 	unsigned x, y, w, h;
-	h = font->getHeight() * console.getRows();
-	w = (font->getWidth() * console.getColumns()) + CHAR_BORDER;
+	h = font.getHeight() * console.getRows();
+	w = (font.getWidth() * console.getColumns()) + CHAR_BORDER;
 
 	// TODO use setting listener in the future
-	switch (consolePlacementSetting->getValue()) {
+	switch (consolePlacementSetting->getEnum()) {
 		case CP_TOPLEFT:
 		case CP_LEFT:
 		case CP_BOTTOMLEFT:
@@ -240,7 +247,7 @@ bool OSDConsoleRenderer::updateConsoleRect()
 			x = (screenW - w) / 2;
 			break;
 	}
-	switch (consolePlacementSetting->getValue()) {
+	switch (consolePlacementSetting->getEnum()) {
 		case CP_TOPLEFT:
 		case CP_TOP:
 		case CP_TOPRIGHT:
@@ -267,9 +274,13 @@ bool OSDConsoleRenderer::updateConsoleRect()
 
 void OSDConsoleRenderer::loadFont(const string& value)
 {
-	SystemFileContext context;
-	string filename = context.resolve(value);
-	font.reset(new TTFFont(filename, fontSizeSetting->getValue()));
+	string filename = SystemFileContext().resolve(value);
+	auto newFont = TTFFont(filename, fontSizeSetting->getInt());
+	if (!newFont.isFixedWidth()) {
+		throw MSXException(value + " is not a monospaced font");
+	}
+	font = std::move(newFont);
+	clearCache();
 }
 
 void OSDConsoleRenderer::loadBackground(const string& value)
@@ -278,14 +289,13 @@ void OSDConsoleRenderer::loadBackground(const string& value)
 		backgroundImage.reset();
 		return;
 	}
-	SystemFileContext context;
-	string filename = context.resolve(value);
+	string filename = SystemFileContext().resolve(value);
 	if (!openGL) {
-		backgroundImage.reset(new SDLImage(filename, destW, destH));
+		backgroundImage = make_unique<SDLImage>(filename, destW, destH);
 	}
 #if COMPONENT_GL
 	else {
-		backgroundImage.reset(new GLImage(filename, destW, destH));
+		backgroundImage = make_unique<GLImage>(filename, destW, destH);
 	}
 #endif
 }
@@ -311,11 +321,11 @@ void OSDConsoleRenderer::drawText2(OutputSurface& output, string_ref text,
 		SDLSurfacePtr surf;
 		try {
 			unsigned dummyHeight;
-			font->getSize(textStr, width, dummyHeight);
-			surf = font->render(textStr,
-			                    (rgb >> 16) & 0xff,
-			                    (rgb >>  8) & 0xff,
-			                    (rgb >>  0) & 0xff);
+			font.getSize(textStr, width, dummyHeight);
+			surf = font.render(textStr,
+			                   (rgb >> 16) & 0xff,
+			                   (rgb >>  8) & 0xff,
+			                   (rgb >>  0) & 0xff);
 		} catch (MSXException& e) {
 			static bool alreadyPrinted = false;
 			if (!alreadyPrinted) {
@@ -326,18 +336,19 @@ void OSDConsoleRenderer::drawText2(OutputSurface& output, string_ref text,
 			}
 			return; // don't cache negative results
 		}
-		if (!surf.get()) {
+		std::unique_ptr<BaseImage> image2;
+		if (!surf) {
 			// nothing was rendered, so do nothing
-			image = NULL;
 		} else if (!openGL) {
-			image = new SDLImage(surf);
+			image2 = make_unique<SDLImage>(std::move(surf));
 		}
 #if COMPONENT_GL
 		else {
-			image = new GLImage(surf);
+			image2 = make_unique<GLImage>(std::move(surf));
 		}
 #endif
-		insertInCache(textStr, rgb, image, width);
+		image = image2.get();
+		insertInCache(textStr, rgb, std::move(image2), width);
 	}
 	if (image) image->draw(output, x, y, alpha);
 	x += width; // in case of trailing whitespace width != image->getWidth()
@@ -350,7 +361,7 @@ bool OSDConsoleRenderer::getFromCache(string_ref text, unsigned rgb,
 	// located right in front of the previously found item. (Though
 	// duplicate items (e.g. the command prompt '> ') degrade this
 	// heuristic).
-	TextCache::iterator it = cacheHint;
+	auto it = cacheHint;
 	if ((it->text == text) && (it->rgb  == rgb)) {
 		goto found;
 	}
@@ -361,7 +372,7 @@ bool OSDConsoleRenderer::getFromCache(string_ref text, unsigned rgb,
 	for (it = textCache.begin(); it != textCache.end(); ++it) {
 		if (it->text != text) continue;
 		if (it->rgb  != rgb ) continue;
-found:		image = it->image;
+found:		image = it->image.get();
 		width = it->width;
 		cacheHint = it;
 		if (it != textCache.begin()) {
@@ -374,19 +385,30 @@ found:		image = it->image;
 	return false;
 }
 
-void OSDConsoleRenderer::insertInCache(const string& text, unsigned rgb,
-                                       BaseImage* image, unsigned width)
+void OSDConsoleRenderer::insertInCache(
+	const string& text, unsigned rgb, std::unique_ptr<BaseImage> image,
+	unsigned width)
 {
 	static const unsigned MAX_TEXT_CACHE_SIZE = 250;
 	if (textCache.size() == MAX_TEXT_CACHE_SIZE) {
 		// flush the least recently used entry
-		TextCache::iterator it = textCache.end();
+		auto it = textCache.end();
 		--it;
-		assert(it != cacheHint);
-		delete it->image;
+		if (it == cacheHint) {
+			cacheHint = textCache.begin();
+		}
 		textCache.pop_back();
 	}
-	textCache.push_front(TextCacheElement(text, rgb, image, width));
+	textCache.push_front(TextCacheElement(
+		text, rgb, std::move(image), width));
+}
+
+void OSDConsoleRenderer::clearCache()
+{
+	// cacheHint must always point to a valid item, so insert a dummy entry
+	textCache.clear();
+	textCache.push_back(TextCacheElement("", 0, nullptr, 0));
+	cacheHint = textCache.begin();
 }
 
 void OSDConsoleRenderer::paint(OutputSurface& output)
@@ -396,45 +418,44 @@ void OSDConsoleRenderer::paint(OutputSurface& output)
 
 	if (updateConsoleRect()) {
 		try {
-			loadBackground(backgroundSetting->getValue());
+			loadBackground(backgroundSetting->getString());
 		} catch (MSXException& e) {
 			reactor.getCliComm().printWarning(e.getMessage());
 		}
 	}
 
 	// draw the background image if there is one
-	if (!backgroundImage.get()) {
+	if (!backgroundImage) {
 		// no background image, try to create an empty one
 		try {
 			if (!openGL) {
-				backgroundImage.reset(new SDLImage(
-					destW, destH, CONSOLE_ALPHA));
+				backgroundImage = make_unique<SDLImage>(
+					destW, destH, CONSOLE_ALPHA);
 			}
 #if COMPONENT_GL
 			else {
-				backgroundImage.reset(new GLImage(
-					destW, destH, CONSOLE_ALPHA));
+				backgroundImage = make_unique<GLImage>(
+					destW, destH, CONSOLE_ALPHA);
 			}
 #endif
 		} catch (MSXException&) {
 			// nothing
 		}
 	}
-	if (backgroundImage.get()) {
+	if (backgroundImage) {
 		backgroundImage->draw(output, destX, destY, visibility);
 	}
 
-	int screenlines = destH / font->getHeight();
-	for (int loop = 0; loop < screenlines; ++loop) {
+	for (auto loop : xrange(destH / font.getHeight())) {
 		drawText(output,
 		         console.getLine(loop + console.getScrollBack()),
 		         destX + CHAR_BORDER,
-		         destY + destH - (1 + loop) * font->getHeight() - 1,
+		         destY + destH - (1 + loop) * font.getHeight() - 1,
 		         visibility);
 	}
 
 	// Check if the blink period is over
-	unsigned long long now = Timer::getTime();
+	auto now = Timer::getTime();
 	if (lastBlinkTime < now) {
 		lastBlinkTime = now + BLINK_RATE;
 		blink = !blink;
@@ -450,35 +471,10 @@ void OSDConsoleRenderer::paint(OutputSurface& output)
 	}
 	if (blink && (console.getScrollBack() == 0)) {
 		drawText(output, ConsoleLine("_"),
-		         destX + CHAR_BORDER + cursorX * font->getWidth(),
-		         destY + destH - (font->getHeight() * (cursorY + 1)) - 1,
+		         destX + CHAR_BORDER + cursorX * font.getWidth(),
+		         destY + destH - (font.getHeight() * (cursorY + 1)) - 1,
 		         visibility);
 	}
 }
 
-string_ref OSDConsoleRenderer::getLayerName() const
-{
-	return "console";
-}
-
-
-// class OSDSettingChecker
-
-OSDSettingChecker::OSDSettingChecker(OSDConsoleRenderer& renderer_)
-	: renderer(renderer_)
-{
-}
-
-void OSDSettingChecker::check(SettingImpl<FilenameSetting::Policy>& setting,
-	                      string& value)
-{
-	if (&setting == renderer.backgroundSetting.get()) {
-		renderer.loadBackground(value);
-	} else if (&setting == renderer.fontSetting.get()) {
-		renderer.loadFont(value);
-	} else {
-		UNREACHABLE;
-	}
-}
-
 } // namespace openmsx
