@@ -1,5 +1,3 @@
-// $Id: OSDGUI.cc 12805 2012-08-13 19:26:18Z m9710797 $
-
 #include "OSDGUI.hh"
 #include "OSDWidget.hh"
 #include "OSDTopWidget.hh"
@@ -10,13 +8,13 @@
 #include "CommandException.hh"
 #include "TclObject.hh"
 #include "StringOp.hh"
-#include "shared_ptr.hh"
 #include "openmsx.hh"
+#include "memory.hh"
 #include <algorithm>
 #include <cassert>
 
 using std::string;
-using std::set;
+using std::shared_ptr;
 using std::vector;
 
 namespace openmsx {
@@ -49,8 +47,8 @@ private:
 
 OSDGUI::OSDGUI(CommandController& commandController, Display& display_)
 	: display(display_)
-	, osdCommand(new OSDCommand(*this, commandController))
-	, topWidget(new OSDTopWidget())
+	, osdCommand(make_unique<OSDCommand>(*this, commandController))
+	, topWidget(make_unique<OSDTopWidget>())
 {
 }
 
@@ -141,9 +139,9 @@ shared_ptr<OSDWidget> OSDCommand::create(
 		string_ref type, const string& name) const
 {
 	if (type == "rectangle") {
-		return shared_ptr<OSDWidget>(new OSDRectangle(gui, name));
+		return std::make_shared<OSDRectangle>(gui, name);
 	} else if (type == "text") {
-		return shared_ptr<OSDWidget>(new OSDText(gui, name));
+		return std::make_shared<OSDText>(gui, name);
 	} else {
 		throw CommandException(
 			"Invalid widget type '" + type + "', expected "
@@ -175,17 +173,15 @@ void OSDCommand::info(const vector<TclObject>& tokens, TclObject& result)
 	switch (tokens.size()) {
 	case 2: {
 		// list widget names
-		set<string> names;
+		vector<string> names;
 		gui.getTopWidget().listWidgetNames("", names);
-		result.addListElements(names.begin(), names.end());
+		result.addListElements(names);
 		break;
 	}
 	case 3: {
 		// list properties for given widget
 		const OSDWidget& widget = getWidget(tokens[2].getString());
-		set<string> properties;
-		widget.getProperties(properties);
-		result.addListElements(properties.begin(), properties.end());
+		result.addListElements(widget.getProperties());
 		break;
 	}
 	case 4: {
@@ -205,7 +201,7 @@ void OSDCommand::exists(const vector<TclObject>& tokens, TclObject& result)
 		throw SyntaxError();
 	}
 	OSDWidget* widget = gui.getTopWidget().findSubWidget(tokens[2].getString());
-	result.setBoolean(widget != NULL);
+	result.setBoolean(widget != nullptr);
 }
 
 void OSDCommand::configure(const vector<TclObject>& tokens, TclObject& /*result*/)
@@ -226,8 +222,8 @@ void OSDCommand::configure(OSDWidget& widget, const vector<TclObject>& tokens,
 			"Missing value for '" + tokens.back().getString() + "'.");
 	}
 
-	for (unsigned i = skip; i < tokens.size(); i += 2) {
-		string_ref name  = tokens[i + 0].getString();
+	for (size_t i = skip; i < tokens.size(); i += 2) {
+		const auto& name = tokens[i + 0].getString();
 		widget.setProperty(name, tokens[i + 1]);
 	}
 }
@@ -240,7 +236,7 @@ string OSDCommand::help(const vector<string>& tokens) const
 			  "osd create <type> <widget-path> [<property-name> <property-value>]...\n"
 			  "\n"
 			  "Creates a new OSD widget of given type. Path is a "
-			  "hierarchical name for the wiget (separated by '.')."
+			  "hierarchical name for the widget (separated by '.'). "
 			  "The parent widget for this new widget must already "
 			  "exist.\n"
 			  "Optionally you can set initial values for one or "
@@ -298,33 +294,28 @@ string OSDCommand::help(const vector<string>& tokens) const
 void OSDCommand::tabCompletion(vector<string>& tokens) const
 {
 	if (tokens.size() == 2) {
-		set<string> cmds;
-		cmds.insert("create");
-		cmds.insert("destroy");
-		cmds.insert("info");
-		cmds.insert("exists");
-		cmds.insert("configure");
+		static const char* const cmds[] = {
+			"create", "destroy", "info", "exists", "configure"
+		};
 		completeString(tokens, cmds);
 	} else if ((tokens.size() == 3) && (tokens[1] == "create")) {
-		set<string> types;
-		types.insert("rectangle");
-		types.insert("text");
+		static const char* const types[] = { "rectangle", "text" };
 		completeString(tokens, types);
 	} else if ((tokens.size() == 3) ||
 	           ((tokens.size() == 4) && (tokens[1] == "create"))) {
-		set<string> names;
+		vector<string> names;
 		gui.getTopWidget().listWidgetNames("", names);
 		completeString(tokens, names);
 	} else {
 		try {
-			set<string> properties;
+			vector<string_ref> properties;
 			if (tokens[1] == "create") {
 				shared_ptr<OSDWidget> widget = create(tokens[2], "");
-				widget->getProperties(properties);
+				properties = widget->getProperties();
 			} else if ((tokens[1] == "configure") ||
 			           (tokens[1] == "info")) {
 				const OSDWidget& widget = getWidget(tokens[2]);
-				widget.getProperties(properties);
+				properties = widget.getProperties();
 			}
 			completeString(tokens, properties);
 		} catch (MSXException&) {
