@@ -1,5 +1,3 @@
-// $Id: OSDGUILayer.cc 12658 2012-06-23 19:58:36Z m9710797 $
-
 #include "OSDGUILayer.hh"
 #include "OSDGUI.hh"
 #include "OSDWidget.hh"
@@ -24,11 +22,6 @@ OSDGUI& OSDGUILayer::getGUI()
 	return gui;
 }
 
-string_ref OSDGUILayer::getLayerName() const
-{
-	return "OSDGUI";
-}
-
 
 // class SDLOSDGUILayer
 
