@@ -1,5 +1,3 @@
-// $Id: OSDImageBasedWidget.cc 12620 2012-06-14 20:10:19Z m9710797 $
-
 #include "OSDImageBasedWidget.hh"
 #include "OSDGUI.hh"
 #include "BaseImage.hh"
@@ -9,10 +7,11 @@
 #include "TclObject.hh"
 #include "CommandException.hh"
 #include "Timer.hh"
+#include "xrange.hh"
 #include <cassert>
 
 using std::string;
-using std::set;
+using std::vector;
 
 namespace openmsx {
 
@@ -25,7 +24,7 @@ OSDImageBasedWidget::OSDImageBasedWidget(const OSDGUI& gui_, const string& name)
 	, startFadeValue(1.0)
 	, error(false)
 {
-	for (int i = 0; i < 4; ++i) {
+	for (auto i : xrange(4)) {
 		rgba[i] = 0x000000ff;
 	}
 }
@@ -34,27 +33,27 @@ OSDImageBasedWidget::~OSDImageBasedWidget()
 {
 }
 
-void OSDImageBasedWidget::getProperties(set<string>& result) const
+vector<string_ref> OSDImageBasedWidget::getProperties() const
 {
-	result.insert("-rgba");
-	result.insert("-rgb");
-	result.insert("-alpha");
-	result.insert("-fadePeriod");
-	result.insert("-fadeTarget");
-	result.insert("-fadeCurrent");
-	OSDWidget::getProperties(result);
+	auto result = OSDWidget::getProperties();
+	static const char* const vals[] = {
+		"-rgba", "-rgb", "-alpha", "-fadePeriod", "-fadeTarget",
+		"-fadeCurrent",
+	};
+	result.insert(result.end(), std::begin(vals), std::end(vals));
+	return result;
 }
 
 static void get4(const TclObject& value, unsigned* result)
 {
 	unsigned len = value.getListLength();
 	if (len == 4) {
-		for (unsigned i = 0; i < 4; ++i) {
+		for (auto i : xrange(4)) {
 			result[i] = value.getListIndex(i).getInt();
 		}
 	} else if (len == 1) {
 		unsigned val = value.getInt();
-		for (unsigned i = 0; i < 4; ++i) {
+		for (auto i : xrange(4)) {
 			result[i] = val;
 		}
 	} else {
@@ -71,7 +70,7 @@ void OSDImageBasedWidget::setProperty(string_ref name, const TclObject& value)
 		unsigned newRGB[4];
 		get4(value, newRGB);
 		unsigned newRGBA[4];
-		for (unsigned i = 0; i < 4; ++i) {
+		for (auto i : xrange(4)) {
 			newRGBA[i] = (rgba[i]          & 0x000000ff) |
 			             ((newRGB[i] << 8) & 0xffffff00);
 		}
@@ -80,7 +79,7 @@ void OSDImageBasedWidget::setProperty(string_ref name, const TclObject& value)
 		unsigned newAlpha[4];
 		get4(value, newAlpha);
 		unsigned newRGBA[4];
-		for (unsigned i = 0; i < 4; ++i) {
+		for (auto i : xrange(4)) {
 			newRGBA[i] = (rgba[i]     & 0xffffff00) |
 			             (newAlpha[i] & 0x000000ff);
 		}
@@ -109,7 +108,7 @@ void OSDImageBasedWidget::setRGBA(const unsigned newRGBA[4])
 		return;
 	}
 	invalidateLocal();
-	for (unsigned i = 0; i < 4; ++i) {
+	for (auto i : xrange(4)) {
 		rgba[i] = newRGBA[i];
 	}
 }
@@ -120,7 +119,7 @@ static void set4(const unsigned rgba[4], unsigned mask, unsigned shift, TclObjec
 		result.setInt((rgba[0] & mask) >> shift);
 	} else {
 
-		for (unsigned i = 0; i < 4; ++i) {
+		for (auto i : xrange(4)) {
 			result.addListElement(int((rgba[i] & mask) >> shift));
 		}
 	}
@@ -173,7 +172,7 @@ double OSDImageBasedWidget::getCurrentFadeValue() const
 	return getCurrentFadeValue(Timer::getTime());
 }
 
-double OSDImageBasedWidget::getCurrentFadeValue(unsigned long long now) const
+double OSDImageBasedWidget::getCurrentFadeValue(uint64_t now) const
 {
 	assert(now >= startFadeTime);
 
@@ -199,7 +198,7 @@ double OSDImageBasedWidget::getCurrentFadeValue(unsigned long long now) const
 
 void OSDImageBasedWidget::updateCurrentFadeValue()
 {
-	unsigned long long now = Timer::getTime();
+	auto now = Timer::getTime();
 	if (isFading()) {
 		startFadeValue = getCurrentFadeValue(now);
 	}
@@ -251,12 +250,12 @@ void OSDImageBasedWidget::paintGL(OutputSurface& output)
 
 void OSDImageBasedWidget::createImage(OutputRectangle& output)
 {
-	if (!image.get() && !hasError()) {
+	if (!image && !hasError()) {
 		try {
 			if (gui.isOpenGL()) {
-				image.reset(createGL(output));
+				image = createGL(output);
 			} else {
-				image.reset(createSDL(output));
+				image = createSDL(output);
 			}
 		} catch (MSXException& e) {
 			setError(e.getMessage());
@@ -273,7 +272,7 @@ void OSDImageBasedWidget::paint(OutputSurface& output, bool openGL)
 	createImage(output);
 
 	byte fadedAlpha = getFadedAlpha();
-	if ((fadedAlpha != 0) && image.get()) {
+	if ((fadedAlpha != 0) && image) {
 		double x, y;
 		getTransformedXY(output, x, y);
 		image->draw(output, int(x + 0.5), int(y + 0.5), fadedAlpha);
