@@ -1,5 +1,3 @@
-// $Id: OSDText.cc 12674 2012-06-25 18:13:20Z m9710797 $
-
 #include "OSDText.hh"
 #include "TTFFont.hh"
 #include "SDLImage.hh"
@@ -11,6 +9,7 @@
 #include "StringOp.hh"
 #include "utf8_core.hh"
 #include "unreachable.hh"
+#include "memory.hh"
 #include "components.hh"
 #include <cassert>
 #if COMPONENT_GL
@@ -34,16 +33,15 @@ OSDText::~OSDText()
 {
 }
 
-void OSDText::getProperties(std::set<string>& result) const
+vector<string_ref> OSDText::getProperties() const
 {
-	result.insert("-text");
-	result.insert("-font");
-	result.insert("-size");
-	result.insert("-wrap");
-	result.insert("-wrapw");
-	result.insert("-wraprelw");
-	result.insert("-query-size");
-	OSDImageBasedWidget::getProperties(result);
+	auto result = OSDImageBasedWidget::getProperties();
+	static const char* const vals[] = {
+		"-text", "-font", "-size", "-wrap", "-wrapw", "-wraprelw",
+		"-query-size",
+	};
+	result.insert(result.end(), std::begin(vals), std::end(vals));
+	return result;
 }
 
 void OSDText::setProperty(string_ref name, const TclObject& value)
@@ -59,8 +57,7 @@ void OSDText::setProperty(string_ref name, const TclObject& value)
 	} else if (name == "-font") {
 		string val = value.getString().str();
 		if (fontfile != val) {
-			SystemFileContext context;
-			string file = context.resolve(val);
+			string file = SystemFileContext().resolve(val);
 			if (!FileOperations::isRegularFile(file)) {
 				throw CommandException("Not a valid font file: " + val);
 			}
@@ -143,7 +140,7 @@ void OSDText::getProperty(string_ref name, TclObject& result) const
 
 void OSDText::invalidateLocal()
 {
-	font.reset();
+	font = TTFFont(); // clear font
 	OSDImageBasedWidget::invalidateLocal();
 }
 
@@ -156,7 +153,7 @@ string_ref OSDText::getType() const
 void OSDText::getWidthHeight(const OutputRectangle& /*output*/,
                              double& width, double& height) const
 {
-	if (image.get()) {
+	if (image) {
 		width  = image->getWidth();
 		height = image->getHeight();
 	} else {
@@ -172,18 +169,18 @@ byte OSDText::getFadedAlpha() const
 	return byte((getRGBA(0) & 0xff) * getRecursiveFadeValue());
 }
 
-template <typename IMAGE> BaseImage* OSDText::create(OutputRectangle& output)
+template <typename IMAGE> std::unique_ptr<BaseImage> OSDText::create(
+	OutputRectangle& output)
 {
 	if (text.empty()) {
-		return new IMAGE(0, 0, unsigned(0));
+		return make_unique<IMAGE>(0, 0, 0);
 	}
 	int scale = getScaleFactor(output);
-	if (!font.get()) {
+	if (font.empty()) {
 		try {
-			SystemFileContext context;
-			string file = context.resolve(fontfile);
+			string file = SystemFileContext().resolve(fontfile);
 			int ptSize = size * scale;
-			font.reset(new TTFFont(file, ptSize));
+			font = TTFFont(file, ptSize);
 		} catch (MSXException& e) {
 			throw MSXException("Couldn't open font: " + e.getMessage());
 		}
@@ -211,12 +208,12 @@ template <typename IMAGE> BaseImage* OSDText::create(OutputRectangle& output)
 		// An alternative is to pass vector<string> to TTFFont::render().
 		// That way we can avoid StringOp::join() (in the wrap functions)
 		// followed by // StringOp::split() (in TTFFont::render()).
-		SDLSurfacePtr surface(font->render(wrappedText,
+		SDLSurfacePtr surface(font.render(wrappedText,
 			(rgba >> 24) & 0xff, (rgba >> 16) & 0xff, (rgba >> 8) & 0xff));
-		if (surface.get()) {
-			return new IMAGE(surface);
+		if (surface) {
+			return make_unique<IMAGE>(std::move(surface));
 		} else {
-			return new IMAGE(0, 0, unsigned(0));
+			return make_unique<IMAGE>(0, 0, 0);
 		}
 	} catch (MSXException& e) {
 		throw MSXException("Couldn't render text: " + e.getMessage());
@@ -227,21 +224,21 @@ template <typename IMAGE> BaseImage* OSDText::create(OutputRectangle& output)
 // Search for a position strictly between min and max which also points to the
 // start of a (possibly multi-byte) utf8-character. If no such position exits,
 // this function returns 'min'.
-static unsigned findCharSplitPoint(const string& line, unsigned min, unsigned max)
+static size_t findCharSplitPoint(const string& line, size_t min, size_t max)
 {
-	unsigned pos = (min + max) / 2;
-	const char* beginIt = line.data();
-	const char* posIt = beginIt + pos;
+	auto pos = (min + max) / 2;
+	auto beginIt = line.data();
+	auto posIt = beginIt + pos;
 
-	const char* fwdIt = utf8::sync_forward(posIt);
-	const char* maxIt = beginIt + max;
+	auto fwdIt = utf8::sync_forward(posIt);
+	auto maxIt = beginIt + max;
 	assert(fwdIt <= maxIt);
 	if (fwdIt != maxIt) {
 		return fwdIt - beginIt;
 	}
 
-	const char* bwdIt = utf8::sync_backward(posIt);
-	const char* minIt = beginIt + min;
+	auto bwdIt = utf8::sync_backward(posIt);
+	auto minIt = beginIt + min;
 	assert(minIt <= bwdIt); (void)minIt;
 	return bwdIt - beginIt;
 }
@@ -251,13 +248,13 @@ static unsigned findCharSplitPoint(const string& line, unsigned min, unsigned ma
 // exits, this function returns 'min'.
 // This function works correctly with multi-byte utf8-encoding as long as
 // all delimiter characters are single byte chars.
-static unsigned findWordSplitPoint(string_ref line, unsigned min, unsigned max)
+static size_t findWordSplitPoint(string_ref line, size_t min, size_t max)
 {
 	static const char* const delimiters = " -/";
 
 	// initial guess for a good position
 	assert(min < max);
-	unsigned pos = (min + max) / 2;
+	size_t pos = (min + max) / 2;
 	if (pos == min) {
 		// can't reduce further
 		return min;
@@ -265,38 +262,38 @@ static unsigned findWordSplitPoint(string_ref line, unsigned min, unsigned max)
 
 	// try searching backward (this also checks current position)
 	assert(pos > min);
-	string_ref::size_type pos2 = line.substr(min, pos - min).find_last_of(delimiters);
+	auto pos2 = line.substr(min, pos - min).find_last_of(delimiters);
 	if (pos2 != string_ref::npos) {
 		pos2 += min + 1;
 		assert(min < pos2);
 		assert(pos2 <= pos);
-		return unsigned(pos2);
+		return pos2;
 	}
 
 	// try searching forward
-	string_ref::size_type pos3 = line.substr(pos, max - pos).find_first_of(delimiters);
+	auto pos3 = line.substr(pos, max - pos).find_first_of(delimiters);
 	if (pos3 != string_ref::npos) {
 		pos3 += pos;
 		assert(pos3 < max);
 		pos3 += 1; // char directly after a delimiter;
 		if (pos3 < max) {
-			return unsigned(pos3);
+			return pos3;
 		}
 	}
 
 	return min;
 }
 
-static unsigned takeSingleChar(const string& /*line*/, unsigned /*maxWidth*/)
+static size_t takeSingleChar(const string& /*line*/, unsigned /*maxWidth*/)
 {
 	return 1;
 }
 
 template<typename FindSplitPointFunc, typename CantSplitFunc>
-unsigned OSDText::split(const string& line, unsigned maxWidth,
-                        FindSplitPointFunc findSplitPoint,
-                        CantSplitFunc cantSplit,
-                        bool removeTrailingSpaces) const
+size_t OSDText::split(const string& line, unsigned maxWidth,
+                      FindSplitPointFunc findSplitPoint,
+                      CantSplitFunc cantSplit,
+                      bool removeTrailingSpaces) const
 {
 	if (line.empty()) {
 		// empty line always fits (explicitly handle this because
@@ -305,19 +302,19 @@ unsigned OSDText::split(const string& line, unsigned maxWidth,
 	}
 
 	unsigned width, height;
-	font->getSize(line, width, height);
+	font.getSize(line, width, height);
 	if (width <= maxWidth) {
 		// whole line fits
-		return unsigned(line.size());
+		return line.size();
 	}
 
 	// binary search till we found the largest initial substring that is
 	// not wider than maxWidth
-	unsigned min = 0;
-	unsigned max = unsigned(line.size());
+	size_t min = 0;
+	size_t max = line.size();
 	// invariant: line.substr(0, min) DOES     fit
 	//            line.substr(0, max) DOES NOT fit
-	unsigned cur = findSplitPoint(line, min, max);
+	size_t cur = findSplitPoint(line, min, max);
 	if (cur == 0) {
 		// Could not find a valid split point, then split on char
 		// (this also handles the case of a single too wide char)
@@ -330,10 +327,10 @@ unsigned OSDText::split(const string& line, unsigned maxWidth,
 		if (removeTrailingSpaces) {
 			StringOp::trimRight(curStr, ' ');
 		}
-		font->getSize(curStr, width, height);
+		font.getSize(curStr, width, height);
 		if (width <= maxWidth) {
 			// still fits, try to enlarge
-			unsigned next = findSplitPoint(line, cur, max);
+			size_t next = findSplitPoint(line, cur, max);
 			if (next == cur) {
 				return cur;
 			}
@@ -341,7 +338,7 @@ unsigned OSDText::split(const string& line, unsigned maxWidth,
 			cur = next;
 		} else {
 			// doesn't fit anymore, try to shrink
-			unsigned next = findSplitPoint(line, min, cur);
+			size_t next = findSplitPoint(line, min, cur);
 			if (next == min) {
 				if (min == 0) {
 					// even the first word does not fit,
@@ -356,62 +353,50 @@ unsigned OSDText::split(const string& line, unsigned maxWidth,
 	}
 }
 
-unsigned OSDText::splitAtChar(const std::string& line, unsigned maxWidth) const
+size_t OSDText::splitAtChar(const std::string& line, unsigned maxWidth) const
 {
 	return split(line, maxWidth, findCharSplitPoint, takeSingleChar, false);
 }
 
 struct SplitAtChar {
 	SplitAtChar(const OSDText& osdText_) : osdText(osdText_) {}
-	unsigned operator()(const string& line, unsigned maxWidth) {
+	size_t operator()(const string& line, unsigned maxWidth) {
 		return osdText.splitAtChar(line, maxWidth);
 	}
 	const OSDText& osdText;
 };
-unsigned OSDText::splitAtWord(const std::string& line, unsigned maxWidth) const
+size_t OSDText::splitAtWord(const std::string& line, unsigned maxWidth) const
 {
 	return split(line, maxWidth, findWordSplitPoint, SplitAtChar(*this), true);
 }
 
 string OSDText::getCharWrappedText(const string& text, unsigned maxWidth) const
 {
-	vector<string> lines;
-	StringOp::split(text, "\n", lines);
-
-	vector<string> wrappedLines;
-	for (vector<string>::const_iterator it = lines.begin();
-	     it != lines.end(); ++it) {
-		string line = *it;
+	vector<string_ref> wrappedLines;
+	for (auto& line : StringOp::split(text, '\n')) {
 		do {
-			unsigned pos = splitAtChar(line, maxWidth);
+			auto pos = splitAtChar(line.str(), maxWidth);
 			wrappedLines.push_back(line.substr(0, pos));
 			line = line.substr(pos);
 		} while (!line.empty());
 	}
-
-	return StringOp::join(wrappedLines, "\n");
+	return StringOp::join(wrappedLines, '\n');
 }
 
 string OSDText::getWordWrappedText(const string& text, unsigned maxWidth) const
 {
-	vector<string> lines;
-	StringOp::split(text, "\n", lines);
-
-	vector<string> wrappedLines;
-	for (vector<string>::const_iterator it = lines.begin();
-	     it != lines.end(); ++it) {
-		string line = *it;
+	vector<string_ref> wrappedLines;
+	for (auto& line : StringOp::split(text, '\n')) {
 		do {
-			unsigned pos = splitAtWord(line, maxWidth);
-			string_ref first = string_ref(line).substr(0, pos);
+			auto pos = splitAtWord(line.str(), maxWidth);
+			string_ref first = line.substr(0, pos);
 			StringOp::trimRight(first, ' '); // remove trailing spaces
-			wrappedLines.push_back(first.str());
+			wrappedLines.push_back(first);
 			line = line.substr(pos);
 			StringOp::trimLeft(line, " "); // remove leading spaces
 		} while (!line.empty());
 	}
-
-	return StringOp::join(wrappedLines, "\n");
+	return StringOp::join(wrappedLines, '\n');
 }
 
 void OSDText::getRenderedSize(double& outX, double& outY) const
@@ -427,7 +412,7 @@ void OSDText::getRenderedSize(double& outX, double& outY) const
 
 	unsigned width = 0;
 	unsigned height = 0;
-	if (image.get()) {
+	if (image) {
 		width  = image->getWidth();
 		height = image->getHeight();
 	}
@@ -437,18 +422,18 @@ void OSDText::getRenderedSize(double& outX, double& outY) const
 	outY = height / scale;
 }
 
-BaseImage* OSDText::createSDL(OutputRectangle& output)
+std::unique_ptr<BaseImage> OSDText::createSDL(OutputRectangle& output)
 {
 	return create<SDLImage>(output);
 }
 
-BaseImage* OSDText::createGL(OutputRectangle& output)
+std::unique_ptr<BaseImage> OSDText::createGL(OutputRectangle& output)
 {
 #if COMPONENT_GL
 	return create<GLImage>(output);
 #else
 	(void)&output;
-	return NULL;
+	return nullptr;
 #endif
 }
 
