@@ -1,5 +1,3 @@
-// $Id: OSDWidget.cc 12686 2012-06-29 17:27:04Z m9710797 $
-
 #include "OSDWidget.hh"
 #include "OutputSurface.hh"
 #include "CommandException.hh"
@@ -8,12 +6,14 @@
 #include "GLUtil.hh"
 #include "openmsx.hh"
 #include "unreachable.hh"
+#include "memory.hh"
 #include <SDL.h>
 #include <algorithm>
 #include <limits>
 
 using std::string;
-using std::set;
+using std::vector;
+using std::shared_ptr;
 
 namespace openmsx {
 
@@ -54,7 +54,7 @@ private:
 
 
 SDLScopedClip::SDLScopedClip(OutputSurface& output, int x, int y, int w, int h)
-	: surface(output.getSDLWorkSurface())
+	: surface(output.getSDLSurface())
 {
 	normalize(x, w); normalize(y, h);
 	SDL_GetClipRect(surface, &origClip);
@@ -120,7 +120,7 @@ GLScopedClip::~GLScopedClip()
 ////
 
 OSDWidget::OSDWidget(const string& name_)
-	: parent(NULL)
+	: parent(nullptr)
 	, name(name_)
 	, x(0.0), y(0.0), z(0.0)
 	, relx(0.0), rely(0.0)
@@ -161,8 +161,8 @@ OSDWidget* OSDWidget::findSubWidget(string_ref name)
 	}
 	string_ref first, last;
 	StringOp::splitOnFirst(name, '.', first, last);
-	SubWidgetsMap::const_iterator it = subWidgetsMap.find(first);
-	return it == subWidgetsMap.end() ? NULL : it->second->findSubWidget(last);
+	auto it = subWidgetsMap.find(first);
+	return it == subWidgetsMap.end() ? nullptr : it->second->findSubWidget(last);
 }
 
 const OSDWidget* OSDWidget::findSubWidget(string_ref name) const
@@ -187,7 +187,7 @@ void OSDWidget::addWidget(const shared_ptr<OSDWidget>& widget)
 	if (subWidgets.empty() || (subWidgets.back()->getZ() <= z)) {
 		subWidgets.push_back(widget);
 	} else {
-		SubWidgets::iterator it = subWidgets.begin();
+		auto it = subWidgets.begin();
 		while ((*it)->getZ() <= z) ++it;
 		subWidgets.insert(it, widget);
 
@@ -197,12 +197,10 @@ void OSDWidget::addWidget(const shared_ptr<OSDWidget>& widget)
 void OSDWidget::deleteWidget(OSDWidget& widget)
 {
 	string widgetName = widget.getName();
-	for (SubWidgets::iterator it = subWidgets.begin();
-	     it != subWidgets.end(); ++it) {
+	for (auto it = subWidgets.begin(); it != subWidgets.end(); ++it) {
 		if (it->get() == &widget) {
 			subWidgets.erase(it);
-			SubWidgetsMap::size_type existed =
-				subWidgetsMap.erase(widgetName);
+			auto existed = subWidgetsMap.erase(widgetName);
 			assert(existed); (void)existed;
 			return;
 		}
@@ -211,24 +209,6 @@ void OSDWidget::deleteWidget(OSDWidget& widget)
 }
 
 #ifdef DEBUG
-// Note: this function has the same name and the same functionality as the
-// std::is_sorted function in c++11. Once we switch to a c++11 compiler (or
-// enable c++11 mode) this function can be removed. Vc++ has c++11 mode enabled
-// by default, this currently causes an ambiguous function call, as a temporary
-// fix we have to explicitly qualify all calls to this function.
-template<class ForwardIterator, class StrictWeakOrdering>
-bool is_sorted(ForwardIterator first, ForwardIterator last,
-               StrictWeakOrdering comp)
-{
-	if (first == last) return true;
-	ForwardIterator next = first;
-	++next;
-	while (next != last) {
-		if (comp(*next, *first)) return false;
-		++first; ++next;
-	}
-	return true;
-}
 struct AscendingZ {
 	bool operator()(const shared_ptr<OSDWidget>& lhs,
 	                const shared_ptr<OSDWidget>& rhs) const {
@@ -239,51 +219,46 @@ struct AscendingZ {
 void OSDWidget::resortUp(OSDWidget* elem)
 {
 	// z-coordinate was increased, first search for elements current position
-	SubWidgets::iterator it1 = subWidgets.begin();
+	auto it1 = subWidgets.begin();
 	while (it1->get() != elem) ++it1;
 	// next search for the position were it belongs
 	double z = elem->getZ();
-	SubWidgets::iterator it2 = it1;
+	auto it2 = it1;
 	++it2;
 	while ((it2 != subWidgets.end()) && ((*it2)->getZ() < z)) ++it2;
 	// now move elements to correct position
 	rotate(it1, it1 + 1, it2);
 #ifdef DEBUG
-	assert(openmsx::is_sorted(subWidgets.begin(), subWidgets.end(), AscendingZ()));
+	assert(std::is_sorted(subWidgets.begin(), subWidgets.end(), AscendingZ()));
 #endif
 }
 void OSDWidget::resortDown(OSDWidget* elem)
 {
 	// z-coordinate was decreased, first search for new position
-	SubWidgets::iterator it1 = subWidgets.begin();
+	auto it1 = subWidgets.begin();
 	double z = elem->getZ();
 	while ((*it1)->getZ() <= z) {
 		++it1;
 		if (it1 == subWidgets.end()) return;
 	}
 	// next search for the elements current position
-	SubWidgets::iterator it2 = it1;
+	auto it2 = it1;
 	if ((it2 != subWidgets.begin()) && ((it2 - 1)->get() == elem)) return;
 	while (it2->get() != elem) ++it2;
 	// now move elements to correct position
 	rotate(it1, it2, it2 + 1);
 #ifdef DEBUG
-	assert(openmsx::is_sorted(subWidgets.begin(), subWidgets.end(), AscendingZ()));
+	assert(std::is_sorted(subWidgets.begin(), subWidgets.end(), AscendingZ()));
 #endif
 }
 
-void OSDWidget::getProperties(set<string>& result) const
+vector<string_ref> OSDWidget::getProperties() const
 {
-	result.insert("-type");
-	result.insert("-x");
-	result.insert("-y");
-	result.insert("-z");
-	result.insert("-relx");
-	result.insert("-rely");
-	result.insert("-scaled");
-	result.insert("-clip");
-	result.insert("-mousecoord");
-	result.insert("-suppressErrors");
+	static const char* const vals[] = {
+		"-type", "-x", "-y", "-z", "-relx", "-rely", "-scaled",
+		"-clip", "-mousecoord", "-suppressErrors",
+	};
+	return vector<string_ref>(std::begin(vals), std::end(vals));
 }
 
 void OSDWidget::setProperty(string_ref name, const TclObject& value)
@@ -372,9 +347,8 @@ void OSDWidget::invalidateRecursive()
 
 void OSDWidget::invalidateChildren()
 {
-	for (SubWidgets::const_iterator it = subWidgets.begin();
-	     it != subWidgets.end(); ++it) {
-		(*it)->invalidateRecursive();
+	for (auto& s : subWidgets) {
+		s->invalidateRecursive();
 	}
 }
 
@@ -391,21 +365,20 @@ void OSDWidget::paintSDLRecursive(OutputSurface& output)
 {
 	paintSDL(output);
 
-	std::auto_ptr<SDLScopedClip> scopedClip;
+	std::unique_ptr<SDLScopedClip> scopedClip;
 	if (clip) {
 		int x, y, w, h;
 		getBoundingBox(output, x, y, w, h);
-		scopedClip.reset(new SDLScopedClip(output, x, y, w, h));
+		scopedClip = make_unique<SDLScopedClip>(output, x, y, w, h);
 	}
 
 	// Iterate over a copy because drawing could cause errors (e.g. can't
 	// load font or image), those error are (indirectly via CliComm) passed
 	// to a Tcl callback and that callback can destroy or create extra
 	// widgets.
-	SubWidgets copy(subWidgets);
-	for (SubWidgets::const_iterator it = copy.begin();
-	     it != copy.end(); ++it) {
-		(*it)->paintSDLRecursive(output);
+	auto copy = subWidgets;
+	for (auto& s : copy) {
+		s->paintSDLRecursive(output);
 	}
 }
 
@@ -415,17 +388,16 @@ void OSDWidget::paintGLRecursive (OutputSurface& output)
 #if COMPONENT_GL
 	paintGL(output);
 
-	std::auto_ptr<GLScopedClip> scopedClip;
+	std::unique_ptr<GLScopedClip> scopedClip;
 	if (clip) {
 		int x, y, w, h;
 		getBoundingBox(output, x, y, w, h);
-		scopedClip.reset(new GLScopedClip(output, x, y, w, h));
+		scopedClip = make_unique<GLScopedClip>(output, x, y, w, h);
 	}
 
-	SubWidgets copy(subWidgets);
-	for (SubWidgets::const_iterator it = copy.begin();
-	     it != copy.end(); ++it) {
-		(*it)->paintGLRecursive(output);
+	auto copy = subWidgets;
+	for (auto& s : copy) {
+		s->paintGLRecursive(output);
 	}
 #endif
 }
@@ -531,15 +503,14 @@ void OSDWidget::getBoundingBox(const OutputRectangle& output,
 	h = int(y2 - y1 + 0.5);
 }
 
-void OSDWidget::listWidgetNames(const string& parentName, set<string>& result) const
+void OSDWidget::listWidgetNames(const string& parentName, vector<string>& result) const
 {
 	string pname = parentName;
 	if (!pname.empty()) pname += '.';
-	for (SubWidgets::const_iterator it = subWidgets.begin();
-	     it != subWidgets.end(); ++it) {
-		string name = pname + (*it)->getName();
-		result.insert(name);
-		(*it)->listWidgetNames(name, result);
+	for (auto& s : subWidgets) {
+		string name = pname + s->getName();
+		result.push_back(name);
+		s->listWidgetNames(name, result);
 	}
 }
 
