@@ -1,5 +1,3 @@
-// $Id: MSXCPUInterface.cc 12841 2012-09-06 20:14:09Z m9710797 $
-
 #include "MSXCPUInterface.hh"
 #include "BreakPoint.hh"
 #include "DebugCondition.hh"
@@ -26,11 +24,10 @@
 #include "serialize.hh"
 #include "StringOp.hh"
 #include "checked_cast.hh"
-#include <tcl.h>
 #include "unreachable.hh"
+#include "memory.hh"
+#include <tcl.h>
 #include <cstdio>
-#include <memory> // for auto_ptr
-#include <set>
 #include <sstream>
 #include <iomanip>
 #include <algorithm>
@@ -38,7 +35,6 @@
 #include <cstring>
 #include <iterator>
 
-using std::auto_ptr;
 using std::ostringstream;
 using std::setfill;
 using std::setw;
@@ -46,6 +42,7 @@ using std::uppercase;
 using std::string;
 using std::vector;
 using std::min;
+using std::shared_ptr;
 
 namespace openmsx {
 
@@ -140,7 +137,7 @@ MSXCPUInterface::BreakPoints MSXCPUInterface::breakPoints;
 //TODO watchpoints
 MSXCPUInterface::Conditions  MSXCPUInterface::conditions;
 
-static ReadOnlySetting<BooleanSetting>* breakedSetting = 0;
+static std::unique_ptr<ReadOnlySetting> breakedSetting;
 static unsigned breakedSettingCount = 0;
 
 
@@ -151,21 +148,25 @@ static const byte GLOBAL_WRITE_BIT   = 0x04;
 
 
 MSXCPUInterface::MSXCPUInterface(MSXMotherBoard& motherBoard_)
-	: memoryDebug       (new MemoryDebug       (*this, motherBoard_))
-	, slottedMemoryDebug(new SlottedMemoryDebug(*this, motherBoard_))
-	, ioDebug           (new IODebug           (*this, motherBoard_))
-	, slotInfo        (new SlotInfo(
+	: memoryDebug(make_unique<MemoryDebug>(
+		*this, motherBoard_))
+	, slottedMemoryDebug(make_unique<SlottedMemoryDebug>(
+		*this, motherBoard_))
+	, ioDebug(make_unique<IODebug>(
+		*this, motherBoard_))
+	, slotInfo(make_unique<SlotInfo>(
 		motherBoard_.getMachineInfoCommand(), *this))
-	, subSlottedInfo  (new SubSlottedInfo(
+	, subSlottedInfo(make_unique<SubSlottedInfo>(
 		motherBoard_.getMachineInfoCommand(), *this))
-	, externalSlotInfo(new ExternalSlotInfo(
+	, externalSlotInfo(make_unique<ExternalSlotInfo>(
 		motherBoard_.getMachineInfoCommand(),
 		motherBoard_.getSlotManager()))
-	, inputPortInfo (new IOInfo(
+	, inputPortInfo(make_unique<IOInfo>(
 	        motherBoard_.getMachineInfoCommand(), *this, true))
-	, outputPortInfo(new IOInfo(
+	, outputPortInfo(make_unique<IOInfo>(
 	        motherBoard_.getMachineInfoCommand(), *this, false))
-	, dummyDevice(DeviceFactory::createDummyDevice(*motherBoard_.getMachineConfig()))
+	, dummyDevice(DeviceFactory::createDummyDevice(
+		*motherBoard_.getMachineConfig()))
 	, msxcpu(motherBoard_.getCPU())
 	, cliComm(motherBoard_.getMSXCliComm())
 	, motherBoard(motherBoard_)
@@ -212,9 +213,9 @@ MSXCPUInterface::MSXCPUInterface(MSXMotherBoard& motherBoard_)
 
 	if (breakedSettingCount++ == 0) {
 		assert(!breakedSetting);
-		breakedSetting = new ReadOnlySetting<BooleanSetting>(
+		breakedSetting = make_unique<ReadOnlySetting>(
 			motherBoard.getReactor().getCommandController(),
-			"breaked", "Similar to 'debug breaked'", false);
+			"breaked", "Similar to 'debug breaked'", "false");
 	}
 }
 
@@ -222,13 +223,12 @@ MSXCPUInterface::~MSXCPUInterface()
 {
 	if (--breakedSettingCount == 0) {
 		assert(breakedSetting);
-		delete breakedSetting;
-		breakedSetting = 0;
+		breakedSetting = nullptr;
 	}
 
 	removeAllWatchPoints();
 
-	if (delayDevice.get()) {
+	if (delayDevice) {
 		for (int port = 0x98; port <= 0x9B; ++port) {
 			assert(IO_In [port] == delayDevice.get());
 			assert(IO_Out[port] == delayDevice.get());
@@ -237,7 +237,7 @@ MSXCPUInterface::~MSXCPUInterface()
 		}
 	}
 
-	msxcpu.setInterface(NULL);
+	msxcpu.setInterface(nullptr);
 
 	#ifndef NDEBUG
 	for (int port = 0; port < 256; ++port) {
@@ -298,12 +298,11 @@ void MSXCPUInterface::writeMemSlow(word address, byte value, EmuTime::param time
 	// something special in this region?
 	if (unlikely(disallowWriteCache[address >> CacheLine::BITS])) {
 		// slot-select-ignore writes (Super Lode Runner)
-		for (GlobalWrites::const_iterator it = globalWrites.begin();
-		     it != globalWrites.end(); ++it) {
+		for (auto& g : globalWrites) {
 			// very primitive address selection mechanism,
 			// but more than enough for now
-			if (unlikely(it->addr == address)) {
-				it->device->globalWrite(address, value, time);
+			if (unlikely(g.addr == address)) {
+				g.device->globalWrite(address, value, time);
 			}
 		}
 		// execute write watches after actual write
@@ -325,26 +324,28 @@ void MSXCPUInterface::setExpanded(int ps)
 		}
 	}
 	expanded[ps]++;
+	changeExpanded(isExpanded(primarySlotState[3]));
 }
 
 void MSXCPUInterface::testUnsetExpanded(
-		int ps, vector<MSXDevice*>& alreadyRemoved) const
+		int ps, vector<MSXDevice*> allowed) const
 {
 	// TODO handle multi-devices
-	std::set<MSXDevice*> allowed(alreadyRemoved.begin(), alreadyRemoved.end());
-	allowed.insert(dummyDevice.get());
+	allowed.push_back(dummyDevice.get());
+	sort(allowed.begin(), allowed.end()); // for set_difference()
 	assert(isExpanded(ps));
 	if (expanded[ps] != 1) return; // ok, still expanded after this
 
-	std::set<MSXDevice*> inUse;
+	std::vector<MSXDevice*> inUse;
 	for (int ss = 0; ss < 4; ++ss) {
 		for (int page = 0; page < 4; ++page) {
 			MSXDevice* device = slotLayout[ps][ss][page];
-			std::set<MSXDevice*> devices;
-			if (MSXMultiMemDevice* memDev = dynamic_cast<MSXMultiMemDevice*>(device)) {
-				memDev->getDevices(devices);
+			std::vector<MSXDevice*> devices;
+			if (auto memDev = dynamic_cast<MSXMultiMemDevice*>(device)) {
+				devices = memDev->getDevices();
+				sort(devices.begin(), devices.end()); // for set_difference()
 			} else {
-				devices.insert(device);
+				devices.push_back(device);
 			}
 			std::set_difference(devices.begin(), devices.end(),
 			                    allowed.begin(), allowed.end(),
@@ -357,9 +358,8 @@ void MSXCPUInterface::testUnsetExpanded(
 	StringOp::Builder msg;
 	msg << "Can't remove slot expander from slot " << ps
 	    << " because the following devices are still inserted:";
-	for (std::set<MSXDevice*>::const_iterator it = inUse.begin();
-	     it != inUse.end(); ++it) {
-		msg << " " << (*it)->getName();
+	for (auto& d : inUse) {
+		msg << " " << d->getName();
 	}
 	msg << '.';
 	throw MSXException(msg);
@@ -376,12 +376,25 @@ void MSXCPUInterface::unsetExpanded(int ps)
 	}
 #endif
 	expanded[ps]--;
+	changeExpanded(isExpanded(primarySlotState[3]));
+}
+
+void MSXCPUInterface::changeExpanded(bool isExpanded)
+{
+	if (isExpanded) {
+		disallowReadCache [0xFF] |=  SECUNDARY_SLOT_BIT;
+		disallowWriteCache[0xFF] |=  SECUNDARY_SLOT_BIT;
+	} else {
+		disallowReadCache [0xFF] &= ~SECUNDARY_SLOT_BIT;
+		disallowWriteCache[0xFF] &= ~SECUNDARY_SLOT_BIT;
+	}
+	msxcpu.invalidateMemCache(0xFFFF & CacheLine::HIGH, 0x100);
 }
 
 MSXDevice*& MSXCPUInterface::getDevicePtr(byte port, bool isIn)
 {
 	MSXDevice** devicePtr = isIn ? &IO_In[port] : &IO_Out[port];
-	while (MSXWatchIODevice* watch = dynamic_cast<MSXWatchIODevice*>(*devicePtr)) {
+	while (auto watch = dynamic_cast<MSXWatchIODevice*>(*devicePtr)) {
 		devicePtr = &watch->getDevicePtr();
 	}
 	if (*devicePtr == delayDevice.get()) {
@@ -425,8 +438,7 @@ void MSXCPUInterface::register_IO(int port, bool isIn,
 		// first, replace DummyDevice
 		devicePtr = device;
 	} else {
-		if (MSXMultiIODevice* multi = dynamic_cast<MSXMultiIODevice*>(
-		                                                   devicePtr)) {
+		if (auto multi = dynamic_cast<MSXMultiIODevice*>(devicePtr)) {
 			// third or more, add to existing MultiIO device
 			multi->addDevice(device);
 		} else {
@@ -447,11 +459,10 @@ void MSXCPUInterface::register_IO(int port, bool isIn,
 
 void MSXCPUInterface::unregister_IO(MSXDevice*& devicePtr, MSXDevice* device)
 {
-	if (MSXMultiIODevice* multi = dynamic_cast<MSXMultiIODevice*>(
-	                                                          devicePtr)) {
+	if (auto multi = dynamic_cast<MSXMultiIODevice*>(devicePtr)) {
 		// remove from MultiIO device
 		multi->removeDevice(device);
-		MSXMultiIODevice::Devices& devices = multi->getDevices();
+		auto& devices = multi->getDevices();
 		if (devices.size() == 1) {
 			// only one remaining, remove MultiIO device
 			devicePtr = devices.front();
@@ -487,8 +498,7 @@ void MSXCPUInterface::testRegisterSlot(
 		// partial page
 		if (slot == dummyDevice.get()) {
 			// first, ok
-		} else if (MSXMultiMemDevice* multi =
-		                  dynamic_cast<MSXMultiMemDevice*>(slot)) {
+		} else if (auto multi = dynamic_cast<MSXMultiMemDevice*>(slot)) {
 			// second (or more), check for overlap
 			if (!multi->canAdd(base, size)) {
 				reportMemOverlap(ps, ss, *slot, device);
@@ -520,8 +530,7 @@ void MSXCPUInterface::registerSlot(
 				new MSXMultiMemDevice(device.getHardwareConfig());
 			multi->add(device, base, size);
 			slot = multi;
-		} else if (MSXMultiMemDevice* multi =
-		                  dynamic_cast<MSXMultiMemDevice*>(slot)) {
+		} else if (auto multi = dynamic_cast<MSXMultiMemDevice*>(slot)) {
 			// second or more
 			assert(multi->canAdd(base, size));
 			multi->add(device, base, size);
@@ -538,7 +547,7 @@ void MSXCPUInterface::unregisterSlot(
 {
 	int page = base >> 14;
 	MSXDevice*& slot = slotLayout[ps][ss][page];
-	if (MSXMultiMemDevice* multi = dynamic_cast<MSXMultiMemDevice*>(slot)) {
+	if (auto multi = dynamic_cast<MSXMultiMemDevice*>(slot)) {
 		// partial range
 		multi->remove(device, base, size);
 		if (multi->empty()) {
@@ -607,14 +616,12 @@ void MSXCPUInterface::registerGlobalWrite(MSXDevice& device, word address)
 void MSXCPUInterface::unregisterGlobalWrite(MSXDevice& device, word address)
 {
 	GlobalWriteInfo info = { &device, address };
-	GlobalWrites::iterator it =
-		find(globalWrites.begin(), globalWrites.end(), info);
+	auto it = find(globalWrites.begin(), globalWrites.end(), info);
 	assert(it != globalWrites.end());
 	globalWrites.erase(it);
 
-	for (GlobalWrites::const_iterator it = globalWrites.begin();
-	     it != globalWrites.end(); ++it) {
-		if ((it->addr >> CacheLine::BITS) ==
+	for (auto& g : globalWrites) {
+		if ((g.addr >> CacheLine::BITS) ==
 		    (address  >> CacheLine::BITS)) {
 			// there is still a global write in this region
 			return;
@@ -624,22 +631,17 @@ void MSXCPUInterface::unregisterGlobalWrite(MSXDevice& device, word address)
 	msxcpu.invalidateMemCache(address & CacheLine::HIGH, 0x100);
 }
 
-void MSXCPUInterface::updateVisible(int page)
+ALWAYS_INLINE void MSXCPUInterface::updateVisible(int page, int ps, int ss)
 {
-	MSXDevice* newDevice = slotLayout[primarySlotState[page]]
-	                                 [secondarySlotState[page]]
-	                                 [page];
+	MSXDevice* newDevice = slotLayout[ps][ss][page];
 	if (visibleDevices[page] != newDevice) {
 		visibleDevices[page] = newDevice;
-		msxcpu.updateVisiblePage(page, primarySlotState[page],
-		                               secondarySlotState[page]);
+		msxcpu.updateVisiblePage(page, ps, ss);
 	}
-	/*
-	PRT_DEBUG(" page: " << (int)page <<
-	          " ps: " << (int)primarySlotState[page] <<
-	          " ss: " << (int)secondarySlotState[page] <<
-	          " device: " << newDevice->getName());
-	*/
+}
+void MSXCPUInterface::updateVisible(int page)
+{
+	updateVisible(page, primarySlotState[page], secondarySlotState[page]);
 }
 
 void MSXCPUInterface::reset()
@@ -657,22 +659,51 @@ byte MSXCPUInterface::readIRQVector()
 
 void MSXCPUInterface::setPrimarySlots(byte value)
 {
-	for (int page = 0; page < 4; ++page, value >>= 2) {
-		// Change the slot structure
-		primarySlotState[page] = value & 3;
-		secondarySlotState[page] =
-			(subSlotRegister[value & 3] >> (page * 2)) & 3;
-		// Change the visible devices
-		updateVisible(page);
+	// Change the slot structure.
+	// Originally the code below was a loop over the 4 pages, and the check
+	// for (un)expanded-slot was done unconditionally at the end. I've
+	// completely unrolled the loop and only check for (un)expanded slot
+	// when the slot in page 3 has changed. I've also added checks for slot
+	// changes for the other 3 pages. Usually when this register is written
+	// only one of the 4 pages actually changes, so these extra checks do
+	// pay off. This does make the code a bit more complex (and the
+	// generated code slightly bigger), but it does make a measurable speed
+	// difference.  Changing the slots several hundreds of times per
+	// (EmuTime) is not unusual. So this routine ended up quite high
+	// (top-10) in some profile results.
+	int ps0 = (value >> 0) & 3;
+	if (unlikely(primarySlotState[0] != ps0)) {
+		primarySlotState[0] = ps0;
+		int ss0 = (subSlotRegister[ps0] >> 0) & 3;
+		secondarySlotState[0] = ss0;
+		updateVisible(0, ps0, ss0);
 	}
-	if (isExpanded(primarySlotState[3])) {
-		disallowReadCache [0xFF] |=  SECUNDARY_SLOT_BIT;
-		disallowWriteCache[0xFF] |=  SECUNDARY_SLOT_BIT;
-	} else {
-		disallowReadCache [0xFF] &= ~SECUNDARY_SLOT_BIT;
-		disallowWriteCache[0xFF] &= ~SECUNDARY_SLOT_BIT;
+	int ps1 = (value >> 2) & 3;
+	if (unlikely(primarySlotState[1] != ps1)) {
+		primarySlotState[1] = ps1;
+		int ss1 = (subSlotRegister[ps1] >> 2) & 3;
+		secondarySlotState[1] = ss1;
+		updateVisible(1, ps1, ss1);
+	}
+	int ps2 = (value >> 4) & 3;
+	if (unlikely(primarySlotState[2] != ps2)) {
+		primarySlotState[2] = ps2;
+		int ss2 = (subSlotRegister[ps2] >> 4) & 3;
+		secondarySlotState[2] = ss2;
+		updateVisible(2, ps2, ss2);
+	}
+	int ps3 = (value >> 6) & 3;
+	if (unlikely(primarySlotState[3] != ps3)) {
+		bool oldExpanded = isExpanded(primarySlotState[3]);
+		bool newExpanded = isExpanded(ps3);
+		primarySlotState[3] = ps3;
+		int ss3 = (subSlotRegister[ps3] >> 6) & 3;
+		secondarySlotState[3] = ss3;
+		updateVisible(3, ps3, ss3);
+		if (unlikely(oldExpanded != newExpanded)) {
+			changeExpanded(newExpanded);
+		}
 	}
-	msxcpu.invalidateMemCache(0xFFFF & CacheLine::HIGH, 0x100);
 }
 
 void MSXCPUInterface::setSubSlot(byte primSlot, byte value)
@@ -761,9 +792,8 @@ void MSXCPUInterface::insertBreakPoint(const shared_ptr<BreakPoint>& bp)
 
 void MSXCPUInterface::removeBreakPoint(const BreakPoint& bp)
 {
-	std::pair<BreakPoints::iterator, BreakPoints::iterator> range =
-		breakPoints.equal_range(bp.getAddress());
-	for (BreakPoints::iterator it = range.first; it != range.second; ++it) {
+	auto range = breakPoints.equal_range(bp.getAddress());
+	for (auto it = range.first; it != range.second; ++it) {
 		if (it->second.get() == &bp) {
 			breakPoints.erase(it);
 			break;
@@ -784,14 +814,12 @@ void MSXCPUInterface::checkBreakPoints(
 	//  - keeps object alive by holding a shared_ptr to it
 	//  - avoids iterating over a changing collection
 	BreakPoints bpCopy(range.first, range.second);
-	for (BreakPoints::const_iterator it = bpCopy.begin();
-	     it != bpCopy.end(); ++it) {
-		it->second->checkAndExecute();
+	for (auto& p : bpCopy) {
+		p.second->checkAndExecute();
 	}
-	Conditions condCopy(conditions);
-	for (Conditions::const_iterator it = condCopy.begin();
-	     it != condCopy.end(); ++it) {
-		(*it)->checkAndExecute();
+	auto condCopy = conditions;
+	for (auto& c : condCopy) {
+		c->checkAndExecute();
 	}
 }
 
@@ -821,8 +849,7 @@ void MSXCPUInterface::removeWatchPoint(shared_ptr<WatchPoint> watchPoint)
 	// Pass shared_ptr by value to keep the object alive for the duration
 	// of this function, otherwise it gets deleted as soon as it's removed
 	// from the watchPoints collection.
-	for (WatchPoints::iterator it = watchPoints.begin();
-	     it != watchPoints.end(); ++it) {
+	for (auto it = watchPoints.begin(); it != watchPoints.end(); ++it) {
 		if (*it == watchPoint) {
 			// remove before calling updateMemWatch()
 			watchPoints.erase(it);
@@ -859,8 +886,7 @@ void MSXCPUInterface::setCondition(const shared_ptr<DebugCondition>& cond)
 
 void MSXCPUInterface::removeCondition(const DebugCondition& cond)
 {
-	for (Conditions::iterator it = conditions.begin();
-	     it != conditions.end(); ++it) {
+	for (auto it = conditions.begin(); it != conditions.end(); ++it) {
 		if (it->get() == &cond) {
 			conditions.erase(it);
 			break;
@@ -877,7 +903,7 @@ const MSXCPUInterface::Conditions& MSXCPUInterface::getConditions()
 void MSXCPUInterface::registerIOWatch(WatchPoint& watchPoint, MSXDevice** devices)
 {
 	assert(dynamic_cast<WatchIO*>(&watchPoint));
-	WatchIO& ioWatch = static_cast<WatchIO&>(watchPoint);
+	auto& ioWatch = static_cast<WatchIO&>(watchPoint);
 	unsigned beginPort = ioWatch.getBeginAddress();
 	unsigned endPort   = ioWatch.getEndAddress();
 	assert(beginPort <= endPort);
@@ -891,7 +917,7 @@ void MSXCPUInterface::registerIOWatch(WatchPoint& watchPoint, MSXDevice** device
 void MSXCPUInterface::unregisterIOWatch(WatchPoint& watchPoint, MSXDevice** devices)
 {
 	assert(dynamic_cast<WatchIO*>(&watchPoint));
-	WatchIO& ioWatch = static_cast<WatchIO&>(watchPoint);
+	auto& ioWatch = static_cast<WatchIO&>(watchPoint);
 	unsigned beginPort = ioWatch.getBeginAddress();
 	unsigned endPort   = ioWatch.getEndAddress();
 	assert(beginPort <= endPort);
@@ -915,11 +941,10 @@ void MSXCPUInterface::updateMemWatch(WatchPoint::Type type)
 	for (unsigned i = 0; i < CacheLine::NUM; ++i) {
 		watchSet[i].reset();
 	}
-	for (WatchPoints::const_iterator it = watchPoints.begin();
-	     it != watchPoints.end(); ++it) {
-		if ((*it)->getType() == type) {
-			unsigned beginAddr = (*it)->getBeginAddress();
-			unsigned endAddr   = (*it)->getEndAddress();
+	for (auto& w : watchPoints) {
+		if (w->getType() == type) {
+			unsigned beginAddr = w->getBeginAddress();
+			unsigned endAddr   = w->getEndAddress();
 			assert(beginAddr <= endAddr);
 			assert(endAddr < 0x10000);
 			for (unsigned addr = beginAddr; addr <= endAddr; ++addr) {
@@ -957,13 +982,12 @@ void MSXCPUInterface::executeMemWatch(WatchPoint::Type type,
 			   TCL_GLOBAL_ONLY);
 	}
 
-	WatchPoints wpCopy(watchPoints);
-	for (WatchPoints::const_iterator it = wpCopy.begin();
-	     it != wpCopy.end(); ++it) {
-		if (((*it)->getBeginAddress() <= address) &&
-		    ((*it)->getEndAddress()   >= address) &&
-		    ((*it)->getType()         == type)) {
-			(*it)->checkAndExecute();
+	auto wpCopy = watchPoints;
+	for (auto& w : wpCopy) {
+		if ((w->getBeginAddress() <= address) &&
+		    (w->getEndAddress()   >= address) &&
+		    (w->getType()         == type)) {
+			w->checkAndExecute();
 		}
 	}
 
@@ -981,10 +1005,10 @@ void MSXCPUInterface::doBreak()
 
 	Reactor& reactor = motherBoard.getReactor();
 	reactor.block();
-	breakedSetting->setReadOnlyValue(true);
+	breakedSetting->setReadOnlyValue("true");
 	reactor.getCliComm().update(CliComm::STATUS, "cpu", "suspended");
 	reactor.getEventDistributor().distributeEvent(
-		new SimpleEvent(OPENMSX_BREAK_EVENT));
+		std::make_shared<SimpleEvent>(OPENMSX_BREAK_EVENT));
 }
 
 void MSXCPUInterface::doStep()
@@ -1009,7 +1033,7 @@ void MSXCPUInterface::doContinue2()
 {
 	breaked = false;
 	Reactor& reactor = motherBoard.getReactor();
-	breakedSetting->setReadOnlyValue(false);
+	breakedSetting->setReadOnlyValue("false");
 	reactor.getCliComm().update(CliComm::STATUS, "cpu", "running");
 	reactor.unblock();
 }
@@ -1100,7 +1124,7 @@ void SlotInfo::execute(const vector<TclObject>& tokens,
 	if (!interface.isExpanded(ps)) {
 		ss = 0;
 	}
-	result.setString(interface.slotLayout[ps][ss][page]->getName());
+	interface.slotLayout[ps][ss][page]->getNameList(result);
 }
 
 string SlotInfo::help(const vector<string>& /*tokens*/) const
@@ -1238,7 +1262,7 @@ void MSXCPUInterface::serialize(Archive& ar, unsigned /*version*/)
 		}
 	}
 
-	if (delayDevice.get()) {
+	if (delayDevice) {
 		ar.serialize("vdpDelay", *delayDevice);
 	}
 }
