@@ -1,7 +1,7 @@
-// $Id: MSXMultiIODevice.cc 12528 2012-05-17 17:36:10Z m9710797 $
-
 #include "MSXMultiIODevice.hh"
+#include "TclObject.hh"
 #include "StringOp.hh"
+#include "xrange.hh"
 #include <algorithm>
 #include <cassert>
 
@@ -36,13 +36,18 @@ MSXMultiIODevice::Devices& MSXMultiIODevice::getDevices()
 
 std::string MSXMultiIODevice::getName() const
 {
-	assert(!devices.empty());
-	StringOp::Builder result;
-	result << devices[0]->getName();
-	for (unsigned i = 1; i < devices.size(); ++i) {
-		result << "  " << devices[i]->getName();
+	TclObject list;
+	getNameList(list);
+	return list.getString().str();
+}
+void MSXMultiIODevice::getNameList(TclObject& result) const
+{
+	for (auto* dev : devices) {
+		const auto& name = dev->getName();
+		if (!name.empty()) {
+			result.addListElement(name);
+		}
 	}
-	return result;
 }
 
 byte MSXMultiIODevice::readIO(word port, EmuTime::param time)
@@ -50,7 +55,7 @@ byte MSXMultiIODevice::readIO(word port, EmuTime::param time)
 	// conflict: return the result from the first device, call readIO()
 	//           also on all other devices, but discard result
 	assert(!devices.empty());
-	Devices::iterator it = devices.begin();
+	auto it = devices.begin();
 	byte result = (*it)->readIO(port, time);
 	for (++it; it != devices.end(); ++it) {
 		(*it)->readIO(port, time);
@@ -67,9 +72,8 @@ byte MSXMultiIODevice::peekIO(word port, EmuTime::param time) const
 
 void MSXMultiIODevice::writeIO(word port, byte value, EmuTime::param time)
 {
-	for (Devices::iterator it = devices.begin();
-	     it != devices.end(); ++it) {
-		(*it)->writeIO(port, value, time);
+	for (auto& d : devices) {
+		d->writeIO(port, value, time);
 	}
 }
 
