@@ -1,10 +1,11 @@
-// $Id: MSXMultiMemDevice.cc 12841 2012-09-06 20:14:09Z m9710797 $
-
 #include "MSXMultiMemDevice.hh"
 #include "DummyDevice.hh"
 #include "MSXCPUInterface.hh"
+#include "TclObject.hh"
 #include "StringOp.hh"
 #include "likely.hh"
+#include "unreachable.hh"
+#include "xrange.hh"
 #include <algorithm>
 #include <cassert>
 
@@ -50,7 +51,7 @@ static bool overlap(unsigned start1, unsigned size1,
 
 bool MSXMultiMemDevice::canAdd(int base, int size)
 {
-	for (unsigned i = 0; i < (ranges.size() -1); ++i) {
+	for (auto i : xrange(ranges.size() - 1)) {
 		if (overlap(base, size, ranges[i].base, ranges[i].size)) {
 			return false;
 		}
@@ -66,8 +67,8 @@ void MSXMultiMemDevice::add(MSXDevice& device, int base, int size)
 
 void MSXMultiMemDevice::remove(MSXDevice& device, int base, int size)
 {
-	Ranges::iterator it = find(ranges.begin(), ranges.end(),
-	                           Range(base, size, device));
+	auto it = find(ranges.begin(), ranges.end(),
+	               Range(base, size, device));
 	assert(it != ranges.end());
 	ranges.erase(it);
 }
@@ -77,32 +78,39 @@ bool MSXMultiMemDevice::empty() const
 	return ranges.size() == 1;
 }
 
-void MSXMultiMemDevice::getDevices(std::set<MSXDevice*>& result) const
+std::vector<MSXDevice*> MSXMultiMemDevice::getDevices() const
 {
-	for (unsigned i = 0; i < (ranges.size() - 1); ++i) {
-		result.insert(ranges[i].device);
-	}
-}
-
-std::string MSXMultiMemDevice::getName() const
-{
-	assert(!empty());
-	StringOp::Builder result;
-	result << ranges[0].device->getName();
-	for (unsigned i = 1; i < (ranges.size() - 1); ++i) {
-		result << "  " << ranges[i].device->getName();
+	std::vector<MSXDevice*> result;
+	for (auto i : xrange(ranges.size() - 1)) {
+		result.push_back(ranges[i].device);
 	}
 	return result;
 }
 
+std::string MSXMultiMemDevice::getName() const
+{
+	TclObject list;
+	getNameList(list);
+	return list.getString().str();
+}
+void MSXMultiMemDevice::getNameList(TclObject& result) const
+{
+	for (auto& r : ranges) {
+		const auto& name = r.device->getName();
+		if (!name.empty()) {
+			result.addListElement(name);
+		}
+	}
+}
+
 const MSXMultiMemDevice::Range& MSXMultiMemDevice::searchRange(unsigned address) const
 {
-	for (Ranges::const_iterator it = ranges.begin(); true; ++it) {
-		if (isInside(address, it->base, it->size)) {
-			return *it;
+	for (auto& r : ranges) {
+		if (isInside(address, r.base, r.size)) {
+			return r;
 		}
-		assert(it != ranges.end());
 	}
+	UNREACHABLE;
 }
 
 MSXDevice* MSXMultiMemDevice::searchDevice(unsigned address) const
@@ -131,11 +139,11 @@ const byte* MSXMultiMemDevice::getReadCacheLine(word start) const
 	// Because start is aligned we don't need to wory about the begin
 	// address of the range. But we must make sure the end of the range
 	// doesn't only fill a partial cacheline.
-	const Range& range = searchRange(start);
+	const auto& range = searchRange(start);
 	if (unlikely(((range.base + range.size) & CacheLine::HIGH) == start)) {
 		// The end of this memory device only fills a partial
 		// cacheline. This can't be cached.
-		return NULL;
+		return nullptr;
 	}
 	return searchDevice(start)->getReadCacheLine(start);
 }
@@ -143,9 +151,9 @@ const byte* MSXMultiMemDevice::getReadCacheLine(word start) const
 byte* MSXMultiMemDevice::getWriteCacheLine(word start) const
 {
 	assert((start & CacheLine::HIGH) == start);
-	const Range& range = searchRange(start);
+	const auto& range = searchRange(start);
 	if (unlikely(((range.base + range.size) & CacheLine::HIGH) == start)) {
-		return NULL;
+		return nullptr;
 	}
 	return searchDevice(start)->getWriteCacheLine(start);
 }
