@@ -1,5 +1,3 @@
-// $Id: MSXWatchIODevice.cc 12808 2012-08-13 19:27:30Z m9710797 $
-
 #include "MSXWatchIODevice.hh"
 #include "MSXMotherBoard.hh"
 #include "Reactor.hh"
@@ -23,16 +21,13 @@ WatchIO::WatchIO(MSXMotherBoard& motherboard,
 	, cpuInterface(motherboard.getCPUInterface())
 {
 	for (unsigned i = byte(beginAddr); i <= byte(endAddr); ++i) {
-		ios.push_back(new MSXWatchIODevice(
+		ios.push_back(make_unique<MSXWatchIODevice>(
 			*motherboard.getMachineConfig(), *this));
 	}
 }
 
 WatchIO::~WatchIO()
 {
-	for (IOs::iterator it = ios.begin(); it != ios.end(); ++it) {
-		delete *it;
-	}
 }
 
 MSXWatchIODevice& WatchIO::getDevice(byte port)
@@ -84,7 +79,7 @@ MSXWatchIODevice::MSXWatchIODevice(
 		const HardwareConfig& hwConf, WatchIO& watchIO_)
 	: MSXMultiDevice(hwConf)
 	, watchIO(watchIO_)
-	, device(0)
+	, device(nullptr)
 {
 }
 
