@@ -1,5 +1,3 @@
-// $Id: Debugger.cc 12808 2012-08-13 19:27:30Z m9710797 $
-
 #include "Debugger.hh"
 #include "Debuggable.hh"
 #include "Probe.hh"
@@ -16,16 +14,18 @@
 #include "CommandException.hh"
 #include "MemBuffer.hh"
 #include "StringOp.hh"
-#include "shared_ptr.hh"
+#include "KeyRange.hh"
 #include "unreachable.hh"
+#include "memory.hh"
 #include <cassert>
-#include <memory>
 
 using std::map;
-using std::set;
+using std::shared_ptr;
+using std::make_shared;
 using std::string;
 using std::vector;
-using std::auto_ptr;
+using std::begin;
+using std::end;
 
 namespace openmsx {
 
@@ -64,9 +64,9 @@ private:
 	                      TclObject& result);
 	void listBreakPoints(const vector<TclObject>& tokens,
 	                     TclObject& result);
-	set<string> getBreakPointIdsAsStringSet() const;
-	set<string> getWatchPointIdsAsStringSet() const;
-	set<string> getConditionIdsAsStringSet() const;
+	vector<string> getBreakPointIds() const;
+	vector<string> getWatchPointIds() const;
+	vector<string> getConditionIds() const;
 	void setWatchPoint(const vector<TclObject>& tokens,
 	                   TclObject& result);
 	void removeWatchPoint(const vector<TclObject>& tokens,
@@ -101,21 +101,17 @@ private:
 
 Debugger::Debugger(MSXMotherBoard& motherBoard_)
 	: motherBoard(motherBoard_)
-	, debugCmd(new DebugCmd(motherBoard.getCommandController(),
-	                        motherBoard.getStateChangeDistributor(),
-	                        motherBoard.getScheduler(),
-	                        motherBoard.getReactor().getGlobalCliComm(), *this))
-	, cpu(0)
+	, debugCmd(make_unique<DebugCmd>(
+		motherBoard.getCommandController(),
+		motherBoard.getStateChangeDistributor(),
+		motherBoard.getScheduler(),
+		motherBoard.getReactor().getGlobalCliComm(), *this))
+	, cpu(nullptr)
 {
 }
 
 Debugger::~Debugger()
 {
-	for (ProbeBreakPoints::const_iterator it = probeBreakPoints.begin();
-	     it != probeBreakPoints.end(); ++it) {
-		delete *it;
-	}
-
 	assert(!cpu);
 	assert(debuggables.empty());
 }
@@ -134,15 +130,15 @@ void Debugger::registerDebuggable(string_ref name, Debuggable& debuggable)
 void Debugger::unregisterDebuggable(string_ref name, Debuggable& debuggable)
 {
 	(void)debuggable;
-	Debuggables::iterator it = debuggables.find(name);
+	auto it = debuggables.find(name);
 	assert(it != debuggables.end() && (it->second == &debuggable));
 	debuggables.erase(it);
 }
 
 Debuggable* Debugger::findDebuggable(string_ref name)
 {
-	Debuggables::iterator it = debuggables.find(name);
-	return (it != debuggables.end()) ? it->second : NULL;
+	auto it = debuggables.find(name);
+	return (it != debuggables.end()) ? it->second : nullptr;
 }
 
 Debuggable& Debugger::getDebuggable(string_ref name)
@@ -154,15 +150,6 @@ Debuggable& Debugger::getDebuggable(string_ref name)
 	return *result;
 }
 
-void Debugger::getDebuggables(set<string>& result) const
-{
-	for (Debuggables::const_iterator it = debuggables.begin();
-	     it != debuggables.end(); ++it) {
-		result.insert(it->first().str());
-	}
-}
-
-
 void Debugger::registerProbe(string_ref name, ProbeBase& probe)
 {
 	assert(probes.find(name) == probes.end());
@@ -172,15 +159,15 @@ void Debugger::registerProbe(string_ref name, ProbeBase& probe)
 void Debugger::unregisterProbe(string_ref name, ProbeBase& probe)
 {
 	(void)probe;
-	Probes::iterator it = probes.find(name);
+	auto it = probes.find(name);
 	assert(it != probes.end() && (it->second == &probe));
 	probes.erase(it);
 }
 
 ProbeBase* Debugger::findProbe(string_ref name)
 {
-	Probes::iterator it = probes.find(name);
-	return (it != probes.end()) ? it->second : NULL;
+	auto it = probes.find(name);
+	return (it != probes.end()) ? it->second : nullptr;
 }
 
 ProbeBase& Debugger::getProbe(string_ref name)
@@ -192,24 +179,16 @@ ProbeBase& Debugger::getProbe(string_ref name)
 	return *result;
 }
 
-void Debugger::getProbes(std::set<std::string>& result) const
-{
-	for (Probes::const_iterator it = probes.begin();
-	     it != probes.end(); ++it) {
-		result.insert(it->first().str());
-	}
-}
-
-
 unsigned Debugger::insertProbeBreakPoint(
 	TclObject command, TclObject condition,
 	ProbeBase& probe, unsigned newId /*= -1*/)
 {
-	ProbeBreakPoint* bp = new ProbeBreakPoint(
+	auto bp = make_unique<ProbeBreakPoint>(
 		motherBoard.getReactor().getGlobalCliComm(),
 		command, condition, *this, probe, newId);
-	probeBreakPoints.push_back(bp);
-	return bp->getId();
+	unsigned result = bp->getId();
+	probeBreakPoints.push_back(std::move(bp));
+	return result;
 }
 
 void Debugger::removeProbeBreakPoint(string_ref name)
@@ -217,10 +196,9 @@ void Debugger::removeProbeBreakPoint(string_ref name)
 	if (name.starts_with("pp#")) {
 		// remove by id
 		unsigned id = stoi(name.substr(3));
-		for (ProbeBreakPoints::iterator it = probeBreakPoints.begin();
+		for (auto it = probeBreakPoints.begin();
 		     it != probeBreakPoints.end(); ++it) {
 			if ((*it)->getId() == id) {
-				delete *it;
 				probeBreakPoints.erase(it);
 				return;
 			}
@@ -228,10 +206,9 @@ void Debugger::removeProbeBreakPoint(string_ref name)
 		throw CommandException("No such breakpoint: " + name);
 	} else {
 		// remove by probe, only works for unconditional bp
-		for (ProbeBreakPoints::iterator it = probeBreakPoints.begin();
+		for (auto it = probeBreakPoints.begin();
 		     it != probeBreakPoints.end(); ++it) {
 			if ((*it)->getProbe().getName() == name) {
-				delete *it;
 				probeBreakPoints.erase(it);
 				return;
 			}
@@ -243,10 +220,9 @@ void Debugger::removeProbeBreakPoint(string_ref name)
 
 void Debugger::removeProbeBreakPoint(ProbeBreakPoint& bp)
 {
-	ProbeBreakPoints::iterator it =
-		find(probeBreakPoints.begin(), probeBreakPoints.end(), &bp);
+	auto it = std::find_if(probeBreakPoints.begin(), probeBreakPoints.end(),
+		[&](ProbeBreakPoints::value_type& v) { return v.get() == &bp; });
 	assert(it != probeBreakPoints.end());
-	delete *it;
 	probeBreakPoints.erase(it);
 }
 
@@ -257,10 +233,26 @@ unsigned Debugger::setWatchPoint(TclObject command, TclObject condition,
 {
 	shared_ptr<WatchPoint> wp;
 	if ((type == WatchPoint::READ_IO) || (type == WatchPoint::WRITE_IO)) {
-		wp.reset(new WatchIO(motherBoard, type, beginAddr, endAddr,
-		                     command, condition, newId));
+		// Workaround visual studio 2012 limitation:
+		//   True variadic templates are not yet supported. Instead the
+		//   visual stdio's standard library supports make_shared with
+		//   (only) upto 5 parameters. It is possible to increase this
+		//   limit to 10 (by defining _VARIADIC_MAX) but that also
+		//   slows down compilation. It's easy enough to work around.
+		//   Also the next version of visual studio (still to be
+		//   release in 2012) will properly support this.
+		//wp = make_shared<WatchIO>(
+		//	motherBoard, type, beginAddr, endAddr,
+		//	command, condition, newId);
+		wp.reset(new WatchIO(
+			motherBoard, type, beginAddr, endAddr,
+			command, condition, newId));
 	} else {
-		wp.reset(new WatchPoint(motherBoard.getReactor().getGlobalCliComm(),
+		//wp = make_shared<WatchPoint>(
+		//	motherBoard.getReactor().getGlobalCliComm(),
+		//	command, condition, type, beginAddr, endAddr, newId);
+		wp.reset(new WatchPoint(
+			motherBoard.getReactor().getGlobalCliComm(),
 			command, condition, type, beginAddr, endAddr, newId));
 	}
 	motherBoard.getCPUInterface().setWatchPoint(wp);
@@ -271,25 +263,19 @@ void Debugger::transfer(Debugger& other)
 {
 	// Copy watchpoints to new machine.
 	assert(motherBoard.getCPUInterface().getWatchPoints().empty());
-	const MSXCPUInterface::WatchPoints& watchPoints =
-		other.motherBoard.getCPUInterface().getWatchPoints();
-	for (MSXCPUInterface::WatchPoints::const_iterator it =
-	     watchPoints.begin(); it != watchPoints.end(); ++it) {
-		const WatchPoint& wp = **it;
-		setWatchPoint(wp.getCommandObj(), wp.getConditionObj(),
-		              wp.getType(), wp.getBeginAddress(),
-		              wp.getEndAddress(), wp.getId());
+	for (auto& wp : other.motherBoard.getCPUInterface().getWatchPoints()) {
+		setWatchPoint(wp->getCommandObj(), wp->getConditionObj(),
+		              wp->getType(),       wp->getBeginAddress(),
+		              wp->getEndAddress(), wp->getId());
 	}
 
 	// Copy probes to new machine.
 	assert(probeBreakPoints.empty());
-	for (ProbeBreakPoints::const_iterator it = other.probeBreakPoints.begin();
-	     it != other.probeBreakPoints.end(); ++it) {
-		const ProbeBreakPoint& bp = **it;
-		if (ProbeBase* probe = findProbe(bp.getProbe().getName())) {
-			insertProbeBreakPoint(bp.getCommandObj(),
-			                      bp.getConditionObj(),
-			                      *probe, bp.getId());
+	for (auto& bp : other.probeBreakPoints) {
+		if (ProbeBase* probe = findProbe(bp->getProbe().getName())) {
+			insertProbeBreakPoint(bp->getCommandObj(),
+			                      bp->getConditionObj(),
+			                      *probe, bp->getId());
 		}
 	}
 
@@ -392,9 +378,7 @@ void DebugCmd::execute(const vector<TclObject>& tokens,
 
 void DebugCmd::list(TclObject& result)
 {
-	set<string> debuggables;
-	debugger.getDebuggables(debuggables);
-	result.addListElements(debuggables.begin(), debuggables.end());
+	result.addListElements(keys(debugger.debuggables));
 }
 
 void DebugCmd::desc(const vector<TclObject>& tokens, TclObject& result)
@@ -504,17 +488,13 @@ void DebugCmd::setBreakPoint(const vector<TclObject>& tokens,
 	switch (tokens.size()) {
 	case 5: // command
 		command = tokens[4];
-		command.checkCommand();
 		// fall-through
 	case 4: // condition
 		condition = tokens[3];
-		if (!condition.getString().empty()) {
-			condition.checkExpression();
-		}
 		// fall-through
 	case 3: // address
 		addr = getAddress(tokens);
-		bp.reset(new BreakPoint(cliComm, addr, command, condition));
+		bp = make_shared<BreakPoint>(cliComm, addr, command, condition);
 		break;
 	default:
 		if (tokens.size() < 3) {
@@ -533,16 +513,15 @@ void DebugCmd::removeBreakPoint(const vector<TclObject>& tokens,
 	if (tokens.size() != 3) {
 		throw SyntaxError();
 	}
-	MSXCPUInterface& interface = debugger.motherBoard.getCPUInterface();
-	const BreakPoints& breakPoints = interface.getBreakPoints();
+	auto& interface = debugger.motherBoard.getCPUInterface();
+	auto& breakPoints = interface.getBreakPoints();
 
 	string_ref tmp = tokens[2].getString();
 	if (tmp.starts_with("bp#")) {
 		// remove by id
 		unsigned id = stoi(tmp.substr(3));
-		for (BreakPoints::const_iterator it = breakPoints.begin();
-		     it != breakPoints.end(); ++it) {
-			const BreakPoint& bp = *it->second;
+		for (auto& p : breakPoints) {
+			const BreakPoint& bp = *p.second;
 			if (bp.getId() == id) {
 				interface.removeBreakPoint(bp);
 				return;
@@ -552,11 +531,8 @@ void DebugCmd::removeBreakPoint(const vector<TclObject>& tokens,
 	} else {
 		// remove by addr, only works for unconditional bp
 		word addr = getAddress(tokens);
-		std::pair<BreakPoints::const_iterator,
-			  BreakPoints::const_iterator> range =
-				breakPoints.equal_range(addr);
-		for (BreakPoints::const_iterator it = range.first;
-		     it != range.second; ++it) {
+		auto range = breakPoints.equal_range(addr);
+		for (auto it = range.first; it != range.second; ++it) {
 			const BreakPoint& bp = *it->second;
 			if (bp.getCondition().empty()) {
 				interface.removeBreakPoint(bp);
@@ -571,12 +547,10 @@ void DebugCmd::removeBreakPoint(const vector<TclObject>& tokens,
 void DebugCmd::listBreakPoints(const vector<TclObject>& /*tokens*/,
                                TclObject& result)
 {
-	MSXCPUInterface& interface = debugger.motherBoard.getCPUInterface();
-	const BreakPoints& breakPoints = interface.getBreakPoints();
 	string res;
-	for (BreakPoints::const_iterator it = breakPoints.begin();
-	     it != breakPoints.end(); ++it) {
-		const BreakPoint& bp = *it->second;
+	auto& interface = debugger.motherBoard.getCPUInterface();
+	for (auto& p : interface.getBreakPoints()) {
+		const BreakPoint& bp = *p.second;
 		TclObject line(result.getInterpreter());
 		line.addListElement(StringOp::Builder() << "bp#" << bp.getId());
 		line.addListElement("0x" + StringOp::toHexString(bp.getAddress(), 4));
@@ -599,13 +573,9 @@ void DebugCmd::setWatchPoint(const vector<TclObject>& tokens,
 	switch (tokens.size()) {
 	case 6: // command
 		command = tokens[5];
-		command.checkCommand();
 		// fall-through
 	case 5: // condition
 		condition = tokens[4];
-		if (!condition.getString().empty()) {
-			condition.checkExpression();
-		}
 		// fall-through
 	case 4: { // address + type
 		string_ref typeStr = tokens[2].getString();
@@ -659,16 +629,12 @@ void DebugCmd::removeWatchPoint(const vector<TclObject>& tokens,
 	if (tokens.size() != 3) {
 		throw SyntaxError();
 	}
-	MSXCPUInterface& interface = debugger.motherBoard.getCPUInterface();
-	const MSXCPUInterface::WatchPoints& watchPoints = interface.getWatchPoints();
-
 	string_ref tmp = tokens[2].getString();
 	if (tmp.starts_with("wp#")) {
 		// remove by id
 		unsigned id = stoi(tmp.substr(3));
-		for (MSXCPUInterface::WatchPoints::const_iterator it =
-			watchPoints.begin(); it != watchPoints.end(); ++it) {
-			shared_ptr<WatchPoint> wp = *it;
+		auto& interface = debugger.motherBoard.getCPUInterface();
+		for (auto& wp : interface.getWatchPoints()) {
 			if (wp->getId() == id) {
 				interface.removeWatchPoint(wp);
 				return;
@@ -681,16 +647,13 @@ void DebugCmd::removeWatchPoint(const vector<TclObject>& tokens,
 void DebugCmd::listWatchPoints(const vector<TclObject>& /*tokens*/,
                                TclObject& result)
 {
-	MSXCPUInterface& interface = debugger.motherBoard.getCPUInterface();
-	const MSXCPUInterface::WatchPoints& watchPoints = interface.getWatchPoints();
 	string res;
-	for (MSXCPUInterface::WatchPoints::const_iterator it = watchPoints.begin();
-	     it != watchPoints.end(); ++it) {
-		const WatchPoint& wp = **it;
+	auto& interface = debugger.motherBoard.getCPUInterface();
+	for (auto& wp : interface.getWatchPoints()) {
 		TclObject line(result.getInterpreter());
-		line.addListElement(StringOp::Builder() << "wp#" << wp.getId());
+		line.addListElement(StringOp::Builder() << "wp#" << wp->getId());
 		string type;
-		switch (wp.getType()) {
+		switch (wp->getType()) {
 		case WatchPoint::READ_IO:
 			type = "read_io";
 			break;
@@ -707,8 +670,8 @@ void DebugCmd::listWatchPoints(const vector<TclObject>& /*tokens*/,
 			UNREACHABLE; break;
 		}
 		line.addListElement(type);
-		unsigned beginAddr = wp.getBeginAddress();
-		unsigned endAddr   = wp.getEndAddress();
+		unsigned beginAddr = wp->getBeginAddress();
+		unsigned endAddr   = wp->getEndAddress();
 		if (beginAddr == endAddr) {
 			line.addListElement("0x" + StringOp::toHexString(beginAddr, 4));
 		} else {
@@ -717,8 +680,8 @@ void DebugCmd::listWatchPoints(const vector<TclObject>& /*tokens*/,
 			range.addListElement("0x" + StringOp::toHexString(endAddr,   4));
 			line.addListElement(range);
 		}
-		line.addListElement(wp.getCondition());
-		line.addListElement(wp.getCommand());
+		line.addListElement(wp->getCondition());
+		line.addListElement(wp->getCommand());
 		res += line.getString() + '\n';
 	}
 	result.setString(res);
@@ -735,14 +698,10 @@ void DebugCmd::setCondition(const vector<TclObject>& tokens,
 	switch (tokens.size()) {
 	case 4: // command
 		command = tokens[3];
-		command.checkCommand();
 		// fall-through
 	case 3: // condition
 		condition = tokens[2];
-		if (!condition.getString().empty()) {
-			condition.checkExpression();
-		}
-		dc.reset(new DebugCondition(cliComm, command, condition));
+		dc = make_shared<DebugCondition>(cliComm, command, condition);
 		break;
 	default:
 		if (tokens.size() < 3) {
@@ -761,18 +720,15 @@ void DebugCmd::removeCondition(const vector<TclObject>& tokens,
 	if (tokens.size() != 3) {
 		throw SyntaxError();
 	}
-	MSXCPUInterface& interface = debugger.motherBoard.getCPUInterface();
-	const MSXCPUInterface::Conditions& conditions = interface.getConditions();
 
 	string_ref tmp = tokens[2].getString();
 	if (tmp.starts_with("cond#")) {
 		// remove by id
 		unsigned id = stoi(tmp.substr(5));
-		for (MSXCPUInterface::Conditions::const_iterator it =
-			conditions.begin(); it != conditions.end(); ++it) {
-			DebugCondition& cond = **it;
-			if (cond.getId() == id) {
-				interface.removeCondition(cond);
+		auto& interface = debugger.motherBoard.getCPUInterface();
+		for (auto& c : interface.getConditions()) {
+			if (c->getId() == id) {
+				interface.removeCondition(*c);
 				return;
 			}
 		}
@@ -783,16 +739,13 @@ void DebugCmd::removeCondition(const vector<TclObject>& tokens,
 void DebugCmd::listConditions(const vector<TclObject>& /*tokens*/,
                               TclObject& result)
 {
-	MSXCPUInterface& interface = debugger.motherBoard.getCPUInterface();
-	const MSXCPUInterface::Conditions& conditions = interface.getConditions();
 	string res;
-	for (MSXCPUInterface::Conditions::const_iterator it = conditions.begin();
-	     it != conditions.end(); ++it) {
-		const DebugCondition& cond = **it;
+	auto& interface = debugger.motherBoard.getCPUInterface();
+	for (auto& c : interface.getConditions()) {
 		TclObject line(result.getInterpreter());
-		line.addListElement(StringOp::Builder() << "cond#" << cond.getId());
-		line.addListElement(cond.getCondition());
-		line.addListElement(cond.getCommand());
+		line.addListElement(StringOp::Builder() << "cond#" << c->getId());
+		line.addListElement(c->getCondition());
+		line.addListElement(c->getCommand());
 		res += line.getString() + '\n';
 	}
 	result.setString(res);
@@ -825,9 +778,7 @@ void DebugCmd::probe(const vector<TclObject>& tokens,
 void DebugCmd::probeList(const vector<TclObject>& /*tokens*/,
                          TclObject& result)
 {
-	set<string> probes;
-	debugger.getProbes(probes);
-	result.addListElements(probes.begin(), probes.end());
+	result.addListElements(keys(debugger.probes));
 }
 void DebugCmd::probeDesc(const vector<TclObject>& tokens,
                          TclObject& result)
@@ -857,13 +808,9 @@ void DebugCmd::probeSetBreakPoint(const vector<TclObject>& tokens,
 	switch (tokens.size()) {
 	case 6: // command
 		command = tokens[5];
-		command.checkCommand();
 		// fall-through
 	case 5: // condition
 		condition = tokens[4];
-		if (!condition.getString().empty()) {
-			condition.checkExpression();
-		}
 		// fall-through
 	case 4: { // probe
 		probe = &debugger.getProbe(tokens[3].getString());
@@ -892,15 +839,12 @@ void DebugCmd::probeListBreakPoints(const vector<TclObject>& /*tokens*/,
                                     TclObject& result)
 {
 	string res;
-	for (Debugger::ProbeBreakPoints::const_iterator it =
-	         debugger.probeBreakPoints.begin();
-	     it != debugger.probeBreakPoints.end(); ++it) {
-		const ProbeBreakPoint& bp = **it;
+	for (auto& p : debugger.probeBreakPoints) {
 		TclObject line(result.getInterpreter());
-		line.addListElement(StringOp::Builder() << "pp#" << bp.getId());
-		line.addListElement(bp.getProbe().getName());
-		line.addListElement(bp.getCondition());
-		line.addListElement(bp.getCommand());
+		line.addListElement(StringOp::Builder() << "pp#" << p->getId());
+		line.addListElement(p->getProbe().getName());
+		line.addListElement(p->getCondition());
+		line.addListElement(p->getCommand());
 		res += line.getString() + '\n';
 	}
 	result.setString(res);
@@ -1146,113 +1090,87 @@ string DebugCmd::help(const vector<string>& tokens) const
 	}
 }
 
-set<string> DebugCmd::getBreakPointIdsAsStringSet() const
+vector<string> DebugCmd::getBreakPointIds() const
 {
-	MSXCPUInterface& interface = debugger.motherBoard.getCPUInterface();
-	const BreakPoints& breakPoints = interface.getBreakPoints();
-	set<string> bpids;
-	for (BreakPoints::const_iterator it = breakPoints.begin();
-	     it != breakPoints.end(); ++it) {
-		bpids.insert(StringOp::Builder() << "bp#" << it->second->getId());
+	vector<string> bpids;
+	auto& interface = debugger.motherBoard.getCPUInterface();
+	for (auto& p : interface.getBreakPoints()) {
+		bpids.push_back(StringOp::Builder() << "bp#" << p.second->getId());
 	}
 	return bpids;
 }
-set<string> DebugCmd::getWatchPointIdsAsStringSet() const
+vector<string> DebugCmd::getWatchPointIds() const
 {
-	MSXCPUInterface& interface = debugger.motherBoard.getCPUInterface();
-	const MSXCPUInterface::WatchPoints& watchPoints = interface.getWatchPoints();
-	set<string> wpids;
-	for (MSXCPUInterface::WatchPoints::const_iterator it = watchPoints.begin();
-	     it != watchPoints.end(); ++it) {
-		wpids.insert(StringOp::Builder() << "wp#" << (*it)->getId());
+	vector<string> wpids;
+	auto& interface = debugger.motherBoard.getCPUInterface();
+	for (auto& w : interface.getWatchPoints()) {
+		wpids.push_back(StringOp::Builder() << "wp#" << w->getId());
 	}
 	return wpids;
 }
-set<string> DebugCmd::getConditionIdsAsStringSet() const
+vector<string> DebugCmd::getConditionIds() const
 {
-	MSXCPUInterface& interface = debugger.motherBoard.getCPUInterface();
-	const MSXCPUInterface::Conditions& conditions = interface.getConditions();
-	set<string> condids;
-	for (MSXCPUInterface::Conditions::const_iterator it = conditions.begin();
-	     it != conditions.end(); ++it) {
-		condids.insert(StringOp::Builder() << "cond#" << (*it)->getId());
+	vector<string> condids;
+	auto& interface = debugger.motherBoard.getCPUInterface();
+	for (auto& c : interface.getConditions()) {
+		condids.push_back(StringOp::Builder() << "cond#" << c->getId());
 	}
 	return condids;
 }
 
 void DebugCmd::tabCompletion(vector<string>& tokens) const
 {
-	set<string> singleArgCmds;
-	singleArgCmds.insert("list");
-	singleArgCmds.insert("step");
-	singleArgCmds.insert("cont");
-	singleArgCmds.insert("break");
-	singleArgCmds.insert("breaked");
-	singleArgCmds.insert("list_bp");
-	singleArgCmds.insert("list_watchpoints");
-	singleArgCmds.insert("list_conditions");
-	set<string> debuggableArgCmds;
-	debuggableArgCmds.insert("desc");
-	debuggableArgCmds.insert("size");
-	debuggableArgCmds.insert("read");
-	debuggableArgCmds.insert("read_block");
-	debuggableArgCmds.insert("write");
-	debuggableArgCmds.insert("write_block");
-	set<string> otherCmds;
-	otherCmds.insert("disasm");
-	otherCmds.insert("set_bp");
-	otherCmds.insert("remove_bp");
-	otherCmds.insert("set_watchpoint");
-	otherCmds.insert("remove_watchpoint");
-	otherCmds.insert("set_condition");
-	otherCmds.insert("remove_condition");
-	otherCmds.insert("probe");
+	static const char* const singleArgCmds[] = {
+		"list", "step", "cont", "break", "breaked",
+		"list_bp", "list_watchpoints", "list_conditions",
+	};
+	static const char* const debuggableArgCmds[] = {
+		"desc", "size", "read", "read_block",
+		"write", "write_block",
+	};
+	static const char* const otherCmds[] = {
+		"disasm", "set_bp", "remove_bp", "set_watchpoint",
+		"remove_watchpoint", "set_condition", "remove_condition",
+		"probe",
+	};
 	switch (tokens.size()) {
 	case 2: {
-		set<string> cmds;
-		cmds.insert(singleArgCmds.begin(), singleArgCmds.end());
-		cmds.insert(debuggableArgCmds.begin(), debuggableArgCmds.end());
-		cmds.insert(otherCmds.begin(), otherCmds.end());
+		vector<const char*> cmds;
+		cmds.insert(end(cmds), begin(singleArgCmds),     end(singleArgCmds));
+		cmds.insert(end(cmds), begin(debuggableArgCmds), end(debuggableArgCmds));
+		cmds.insert(end(cmds), begin(otherCmds),         end(otherCmds));
 		completeString(tokens, cmds);
 		break;
 	}
 	case 3:
-		if (singleArgCmds.find(tokens[1]) ==
-				singleArgCmds.end()) {
+		if (find(begin(singleArgCmds), end(singleArgCmds), tokens[1]) ==
+		    end(singleArgCmds)) {
 			// this command takes (an) argument(s)
-			if (debuggableArgCmds.find(tokens[1]) !=
-				debuggableArgCmds.end()) {
+			if (find(begin(debuggableArgCmds), end(debuggableArgCmds),
+			         tokens[1]) !=
+			    end(debuggableArgCmds)) {
 				// it takes a debuggable here
-				set<string> debuggables;
-				debugger.getDebuggables(debuggables);
-				completeString(tokens, debuggables);
+				completeString(tokens, keys(debugger.debuggables));
 			} else if (tokens[1] == "remove_bp") {
 				// this one takes a bp id
-				set<string> bpids = getBreakPointIdsAsStringSet();
-				completeString(tokens, bpids);
+				completeString(tokens, getBreakPointIds());
 			} else if (tokens[1] == "remove_watchpoint") {
 				// this one takes a wp id
-				set<string> wpids = getWatchPointIdsAsStringSet();
-				completeString(tokens, wpids);
+				completeString(tokens, getWatchPointIds());
 			} else if (tokens[1] == "remove_condition") {
 				// this one takes a cond id
-				set<string> condids = getConditionIdsAsStringSet();
-				completeString(tokens, condids);
+				completeString(tokens, getConditionIds());
 			} else if (tokens[1] == "set_watchpoint") {
-				set<string> types;
-				types.insert("write_io");
-				types.insert("write_mem");
-				types.insert("read_io");
-				types.insert("read_mem");
+				static const char* const types[] = {
+					"write_io", "write_mem",
+					"read_io", "read_mem",
+				};
 				completeString(tokens, types);
 			} else if (tokens[1] == "probe") {
-				set<string> subCmds;
-				subCmds.insert("list");
-				subCmds.insert("desc");
-				subCmds.insert("read");
-				subCmds.insert("set_bp");
-				subCmds.insert("remove_bp");
-				subCmds.insert("list_bp");
+				static const char* const subCmds[] = {
+					"list", "desc", "read", "set_bp",
+					"remove_bp", "list_bp",
+				};
 				completeString(tokens, subCmds);
 			}
 		}
@@ -1261,9 +1179,7 @@ void DebugCmd::tabCompletion(vector<string>& tokens) const
 		if ((tokens[1] == "probe") &&
 		    ((tokens[2] == "desc") || (tokens[2] == "read") ||
 		     (tokens[2] == "set_bp"))) {
-			set<string> probes;
-			debugger.getProbes(probes);
-			completeString(tokens, probes);
+			completeString(tokens, keys(debugger.probes));
 		}
 		break;
 	}
