@@ -1,5 +1,3 @@
-// $Id: AfterCommand.cc 12809 2012-08-13 19:27:49Z m9710797 $
-
 #include "AfterCommand.hh"
 #include "CommandController.hh"
 #include "CliComm.hh"
@@ -17,14 +15,17 @@
 #include "StringOp.hh"
 #include "openmsx.hh"
 #include "unreachable.hh"
+#include "memory.hh"
 #include <algorithm>
+#include <iterator>
 #include <cstdlib>
 #include <sstream>
 
 using std::ostringstream;
 using std::string;
 using std::vector;
-using std::set;
+using std::unique_ptr;
+using std::move;
 
 namespace openmsx {
 
@@ -39,8 +40,8 @@ public:
 protected:
 	AfterCmd(AfterCommand& afterCommand,
 		 const TclObject& command);
-	shared_ptr<AfterCmd> removeSelf();
-private:
+	unique_ptr<AfterCmd> removeSelf();
+
 	AfterCommand& afterCommand;
 	TclObject command;
 	string id;
@@ -60,7 +61,8 @@ private:
 	virtual void executeUntil(EmuTime::param time, int userData);
 	virtual void schedulerDeleted();
 
-	double time;
+	double time; // Zero when expired, otherwise the original duration (to
+	             // be able to reschedule for 'after idle').
 };
 
 class AfterTimeCmd : public AfterTimedCmd
@@ -109,7 +111,6 @@ class AfterRealTimeCmd : public AfterCmd, private Alarm
 {
 public:
 	AfterRealTimeCmd(AfterCommand& afterCommand,
-	                 EventDistributor& eventDistributor,
 	                 const TclObject& command, double time);
 	virtual ~AfterRealTimeCmd();
 	virtual string getType() const;
@@ -118,7 +119,6 @@ public:
 private:
 	virtual bool alarm();
 
-	EventDistributor& eventDistributor;
 	bool expired;
 };
 
@@ -160,10 +160,14 @@ AfterCommand::AfterCommand(Reactor& reactor_,
 		OPENMSX_MACHINE_LOADED_EVENT, *this);
 	eventDistributor.registerEventListener(
 		OPENMSX_AFTER_REALTIME_EVENT, *this);
+	eventDistributor.registerEventListener(
+		OPENMSX_AFTER_TIMED_EVENT, *this);
 }
 
 AfterCommand::~AfterCommand()
 {
+	eventDistributor.unregisterEventListener(
+		OPENMSX_AFTER_TIMED_EVENT, *this);
 	eventDistributor.unregisterEventListener(
 		OPENMSX_AFTER_REALTIME_EVENT, *this);
 	eventDistributor.unregisterEventListener(
@@ -252,10 +256,10 @@ void AfterCommand::afterTime(const vector<TclObject>& tokens, TclObject& result)
 	MSXMotherBoard* motherBoard = reactor.getMotherBoard();
 	if (!motherBoard) return;
 	double time = getTime(tokens[2]);
-	shared_ptr<AfterCmd> cmd(new AfterTimeCmd(
-		motherBoard->getScheduler(), *this, tokens[3], time));
-	afterCmds.push_back(cmd);
+	auto cmd = make_unique<AfterTimeCmd>(
+		motherBoard->getScheduler(), *this, tokens[3], time);
 	result.setString(cmd->getId());
+	afterCmds.push_back(move(cmd));
 }
 
 void AfterCommand::afterRealTime(const vector<TclObject>& tokens, TclObject& result)
@@ -264,10 +268,10 @@ void AfterCommand::afterRealTime(const vector<TclObject>& tokens, TclObject& res
 		throw SyntaxError();
 	}
 	double time = getTime(tokens[2]);
-	shared_ptr<AfterCmd> cmd(new AfterRealTimeCmd(
-		*this, eventDistributor, tokens[3], time));
-	afterCmds.push_back(cmd);
+	auto cmd = make_unique<AfterRealTimeCmd>(
+		*this, tokens[3], time);
 	result.setString(cmd->getId());
+	afterCmds.push_back(move(cmd));
 }
 
 void AfterCommand::afterTclTime(
@@ -275,10 +279,10 @@ void AfterCommand::afterTclTime(
 {
 	TclObject command(tokens.front().getInterpreter());
 	command.addListElements(tokens.begin() + 2, tokens.end());
-	shared_ptr<AfterCmd> cmd(new AfterRealTimeCmd(
-		*this, eventDistributor, command, ms / 1000.0));
-	afterCmds.push_back(cmd);
+	auto cmd = make_unique<AfterRealTimeCmd>(
+		*this, command, ms / 1000.0);
 	result.setString(cmd->getId());
+	afterCmds.push_back(move(cmd));
 }
 
 template<EventType T>
@@ -287,9 +291,10 @@ void AfterCommand::afterEvent(const vector<TclObject>& tokens, TclObject& result
 	if (tokens.size() != 3) {
 		throw SyntaxError();
 	}
-	shared_ptr<AfterCmd> cmd(new AfterEventCmd<T>(*this, tokens[1], tokens[2]));
-	afterCmds.push_back(cmd);
+	auto cmd = make_unique<AfterEventCmd<T>>(
+		*this, tokens[1], tokens[2]);
 	result.setString(cmd->getId());
+	afterCmds.push_back(move(cmd));
 }
 
 void AfterCommand::afterInputEvent(
@@ -298,9 +303,10 @@ void AfterCommand::afterInputEvent(
 	if (tokens.size() != 3) {
 		throw SyntaxError();
 	}
-	shared_ptr<AfterCmd> cmd(new AfterInputEventCmd(*this, event, tokens[2]));
-	afterCmds.push_back(cmd);
+	auto cmd = make_unique<AfterInputEventCmd>(
+		*this, event, tokens[2]);
 	result.setString(cmd->getId());
+	afterCmds.push_back(move(cmd));
 }
 
 void AfterCommand::afterIdle(const vector<TclObject>& tokens, TclObject& result)
@@ -311,22 +317,19 @@ void AfterCommand::afterIdle(const vector<TclObject>& tokens, TclObject& result)
 	MSXMotherBoard* motherBoard = reactor.getMotherBoard();
 	if (!motherBoard) return;
 	double time = getTime(tokens[2]);
-	shared_ptr<AfterCmd> cmd(new AfterIdleCmd(
-		motherBoard->getScheduler(), *this, tokens[3], time));
-	afterCmds.push_back(cmd);
+	auto cmd = make_unique<AfterIdleCmd>(
+		motherBoard->getScheduler(), *this, tokens[3], time);
 	result.setString(cmd->getId());
+	afterCmds.push_back(move(cmd));
 }
 
 void AfterCommand::afterInfo(const vector<TclObject>& /*tokens*/, TclObject& result)
 {
 	ostringstream str;
-	for (AfterCmds::const_iterator it = afterCmds.begin();
-	     it != afterCmds.end(); ++it) {
-		const AfterCmd* cmd = it->get();
+	for (auto& cmd : afterCmds) {
 		str << cmd->getId() << ": ";
 		str << cmd->getType() << ' ';
-		if (dynamic_cast<const AfterTimedCmd*>(cmd)) {
-			const AfterTimedCmd* cmd2 = static_cast<const AfterTimedCmd*>(cmd);
+		if (auto cmd2 = dynamic_cast<const AfterTimedCmd*>(cmd.get())) {
 			str.precision(3);
 			str << std::fixed << std::showpoint << cmd2->getTime() << ' ';
 		}
@@ -342,8 +345,7 @@ void AfterCommand::afterCancel(const vector<TclObject>& tokens, TclObject& /*res
 		throw SyntaxError();
 	}
 	if (tokens.size() == 3) {
-		for (AfterCmds::iterator it = afterCmds.begin();
-		     it != afterCmds.end(); ++it) {
+		for (auto it = afterCmds.begin(); it != afterCmds.end(); ++it) {
 			if ((*it)->getId() == tokens[2].getString()) {
 				afterCmds.erase(it);
 				return;
@@ -353,8 +355,7 @@ void AfterCommand::afterCancel(const vector<TclObject>& tokens, TclObject& /*res
 	TclObject command;
 	command.addListElements(tokens.begin() + 2, tokens.end());
 	string_ref cmdStr = command.getString();
-	for (AfterCmds::iterator it = afterCmds.begin();
-	     it != afterCmds.end(); ++it) {
+	for (auto it = afterCmds.begin(); it != afterCmds.end(); ++it) {
 		if ((*it)->getCommand() == cmdStr) {
 			afterCmds.erase(it);
 			// Tcl manual is not clear about this, but it seems
@@ -382,17 +383,11 @@ string AfterCommand::help(const vector<string>& /*tokens*/) const
 
 void AfterCommand::tabCompletion(vector<string>& tokens) const
 {
-	if (tokens.size()==2) {
-		set<string> cmds;
-		cmds.insert("time");
-		cmds.insert("realtime");
-		cmds.insert("idle");
-		cmds.insert("frame");
-		cmds.insert("break");
-		cmds.insert("boot");
-		cmds.insert("machine_switch");
-		cmds.insert("info");
-		cmds.insert("cancel");
+	if (tokens.size() == 2) {
+		static const char* const cmds[] = {
+			"time", "realtime", "idle", "frame", "break", "boot",
+			"machine_switch", "info", "cancel",
+		};
 		completeString(tokens, cmds);
 	}
 	// TODO : make more complete
@@ -401,16 +396,17 @@ void AfterCommand::tabCompletion(vector<string>& tokens) const
 template<typename PRED> void AfterCommand::executeMatches(PRED pred)
 {
 	// predicate should return false on matches
-	AfterCmds::iterator it = partition(afterCmds.begin(), afterCmds.end(), pred);
-	AfterCmds tmp(it, afterCmds.end());
+	auto it = partition(afterCmds.begin(), afterCmds.end(), pred);
+	AfterCmds tmp(std::make_move_iterator(it),
+	              std::make_move_iterator(afterCmds.end()));
 	afterCmds.erase(it, afterCmds.end());
-	for (AfterCmds::iterator it = tmp.begin(); it != tmp.end(); ++it) {
-		(*it)->execute();
+	for (auto& c : tmp) {
+		c->execute();
 	}
 }
 
 template<EventType T> struct AfterEventPred {
-	bool operator()(const shared_ptr<AfterCmd>& x) const {
+	bool operator()(const unique_ptr<AfterCmd>& x) const {
 		return !dynamic_cast<AfterEventCmd<T>*>(x.get());
 	}
 };
@@ -420,35 +416,40 @@ template<EventType T> void AfterCommand::executeEvents()
 }
 
 struct AfterTimePred {
-	bool operator()(const shared_ptr<AfterCmd>& x) const {
-		if (AfterRealTimeCmd* realtimeCmd =
-		              dynamic_cast<AfterRealTimeCmd*>(x.get())) {
-			if (realtimeCmd->hasExpired()) {
+	bool operator()(const unique_ptr<AfterCmd>& x) const {
+		if (auto* cmd = dynamic_cast<AfterRealTimeCmd*>(x.get())) {
+			if (cmd->hasExpired()) {
+				return false;
+			}
+		}
+		return true;
+	}
+};
+
+struct AfterEmuTimePred {
+	bool operator()(const unique_ptr<AfterCmd>& x) const {
+		if (auto* cmd = dynamic_cast<AfterTimedCmd*>(x.get())) {
+			if (cmd->getTime() == 0.0) {
 				return false;
 			}
 		}
 		return true;
 	}
 };
-void AfterCommand::executeRealTime()
-{
-	executeMatches(AfterTimePred());
-}
 
 struct AfterInputEventPred {
 	AfterInputEventPred(const AfterCommand::EventPtr& event_)
 		: event(event_) {}
-	bool operator()(const shared_ptr<AfterCmd>& x) const {
-		if (AfterInputEventCmd* cmd =
-		                 dynamic_cast<AfterInputEventCmd*>(x.get())) {
-			if (*cmd->getEvent() == *event) return false;
+	bool operator()(const unique_ptr<AfterCmd>& x) const {
+		if (auto* cmd = dynamic_cast<AfterInputEventCmd*>(x.get())) {
+			if (cmd->getEvent()->matches(*event)) return false;
 		}
 		return true;
 	}
 	AfterCommand::EventPtr event;
 };
 
-int AfterCommand::signalEvent(const shared_ptr<const Event>& event)
+int AfterCommand::signalEvent(const std::shared_ptr<const Event>& event)
 {
 	if (event->getType() == OPENMSX_FINISH_FRAME_EVENT) {
 		executeEvents<OPENMSX_FINISH_FRAME_EVENT>();
@@ -461,14 +462,14 @@ int AfterCommand::signalEvent(const shared_ptr<const Event>& event)
 	} else if (event->getType() == OPENMSX_MACHINE_LOADED_EVENT) {
 		executeEvents<OPENMSX_MACHINE_LOADED_EVENT>();
 	} else if (event->getType() == OPENMSX_AFTER_REALTIME_EVENT) {
-		executeRealTime();
+		executeMatches(AfterTimePred());
+	} else if (event->getType() == OPENMSX_AFTER_TIMED_EVENT) {
+		executeMatches(AfterEmuTimePred());
 	} else {
 		executeMatches(AfterInputEventPred(event));
-		for (AfterCmds::const_iterator it = afterCmds.begin();
-		     it != afterCmds.end(); ++it) {
-			if (AfterIdleCmd* idleCmd =
-			              dynamic_cast<AfterIdleCmd*>(it->get())) {
-				idleCmd->reschedule();
+		for (auto& c : afterCmds) {
+			if (auto* cmd = dynamic_cast<AfterIdleCmd*>(c.get())) {
+				cmd->reschedule();
 			}
 		}
 	}
@@ -512,17 +513,17 @@ void AfterCmd::execute()
 	}
 }
 
-shared_ptr<AfterCmd> AfterCmd::removeSelf()
+unique_ptr<AfterCmd> AfterCmd::removeSelf()
 {
-	for (AfterCommand::AfterCmds::iterator it = afterCommand.afterCmds.begin();
+	for (auto it = afterCommand.afterCmds.begin();
 	     it != afterCommand.afterCmds.end(); ++it) {
 		if (it->get() == this) {
-			shared_ptr<AfterCmd> result = *it;
+			auto result = move(*it);
 			afterCommand.afterCmds.erase(it);
 			return result;
 		}
 	}
-	UNREACHABLE; return shared_ptr<AfterCmd>(0);
+	UNREACHABLE; return nullptr;
 }
 
 
@@ -554,8 +555,9 @@ void AfterTimedCmd::reschedule()
 void AfterTimedCmd::executeUntil(EmuTime::param /*time*/,
                                  int /*userData*/)
 {
-	shared_ptr<AfterCmd> self = removeSelf();
-	execute();
+	time = 0.0; // execute on next event
+	afterCommand.eventDistributor.distributeEvent(
+		std::make_shared<SimpleEvent>(OPENMSX_AFTER_TIMED_EVENT));
 }
 
 void AfterTimedCmd::schedulerDeleted()
@@ -632,10 +634,9 @@ string AfterInputEventCmd::getType() const
 // class AfterRealTimeCmd
 
 AfterRealTimeCmd::AfterRealTimeCmd(
-		AfterCommand& afterCommand, EventDistributor& eventDistributor_,
+		AfterCommand& afterCommand,
 		const TclObject& command, double time)
 	: AfterCmd(afterCommand, command)
-	, eventDistributor(eventDistributor_)
 	, expired(false)
 {
 	schedule(unsigned(time * 1000000)); // micro seconds
@@ -656,8 +657,8 @@ bool AfterRealTimeCmd::alarm()
 	// this runs in a different thread, so we can't directly execute the
 	// command here
 	expired = true;
-	eventDistributor.distributeEvent(
-			new SimpleEvent(OPENMSX_AFTER_REALTIME_EVENT));
+	afterCommand.eventDistributor.distributeEvent(
+		std::make_shared<SimpleEvent>(OPENMSX_AFTER_REALTIME_EVENT));
 	return false; // don't repeat alarm
 }
 
