@@ -1,15 +1,12 @@
-// $Id: CliServer.cc 12626 2012-06-14 20:14:17Z m9710797 $
-
 #include "CliServer.hh"
 #include "GlobalCliComm.hh"
 #include "CliConnection.hh"
 #include "StringOp.hh"
 #include "FileOperations.hh"
 #include "MSXException.hh"
-#include "build-info.hh"
+#include "memory.hh"
 #include "statp.hh"
 #include <string>
-#include <memory>
 
 #ifdef _WIN32
 #include <fstream>
@@ -26,9 +23,7 @@ namespace openmsx {
 
 static string getUserName()
 {
-#if defined(_WIN32) || PLATFORM_GP2X
-	// GP2X: For some reason getpwuid() cannot be used in statically
-	//       linked applications.
+#if defined(_WIN32)
 	return "default";
 #else
 	struct passwd* pw = getpwuid(getuid());
@@ -106,7 +101,7 @@ static int openPort(SOCKET listenSock)
 	const int BASE = 9938;
 	const int RANGE = 64;
 
-	srand(unsigned(time(0))); // easily predicatble, but doesn't matter
+	srand(unsigned(time(nullptr))); // easily predicatble, but doesn't matter
 	int first = rand();
 
 	for (int n = 0; n < RANGE; ++n) {
@@ -226,7 +221,12 @@ CliServer::CliServer(CommandController& commandController_,
 {
 	exitLoop = false;
 	sock_startup();
-	thread.start();
+	try {
+		createSocket();
+		thread.start();
+	} catch (MSXException& e) {
+		cliComm.printWarning(e.getMessage());
+	}
 }
 
 CliServer::~CliServer()
@@ -247,20 +247,14 @@ CliServer::~CliServer()
 
 void CliServer::run()
 {
-	try {
-		mainLoop();
-	} catch (MSXException& e) {
-		cliComm.printWarning(e.getMessage());
-	}
+	mainLoop();
 }
 
 void CliServer::mainLoop()
 {
-	createSocket();
-
 	while (!exitLoop) {
 		// wait for incomming connection
-		SOCKET sd = accept(listenSock, NULL, NULL);
+		SOCKET sd = accept(listenSock, nullptr, nullptr);
 		if (sd == OPENMSX_INVALID_SOCKET) {
 			// sock_close(listenSock);  // hangs on win32
 			return;
