@@ -1,8 +1,5 @@
-// $Id: Event.cc 12613 2012-06-14 20:05:32Z m9710797 $
-
 #include "Event.hh"
 #include "TclObject.hh"
-#include "unreachable.hh"
 
 namespace openmsx {
 
@@ -45,14 +42,15 @@ bool Event::operator!=(const Event& other) const
 	return !(*this == other);
 }
 
-void Event::toStringImpl(TclObject& /*result*/) const
+void SimpleEvent::toStringImpl(TclObject& result) const
 {
-	UNREACHABLE;
+	result.addListElement("simple");
+	result.addListElement(int(getType()));
 }
 
-bool Event::lessImpl(const Event& /*other*/) const
+bool SimpleEvent::lessImpl(const Event& /*other*/) const
 {
-	UNREACHABLE; return false;
+	return false;
 }
 
 } // namespace openmsx
