@@ -1,10 +1,9 @@
-// $Id: EventDistributor.cc 12508 2012-05-12 18:24:39Z m9710797 $
-
 #include "EventDistributor.hh"
 #include "EventListener.hh"
 #include "Reactor.hh"
 #include "Thread.hh"
-#include "openmsx.hh"
+#include "stl.hh"
+#include <algorithm>
 #include <cassert>
 
 using std::pair;
@@ -14,6 +13,7 @@ namespace openmsx {
 
 EventDistributor::EventDistributor(Reactor& reactor_)
 	: reactor(reactor_)
+	, listeners(NUM_EVENT_TYPES)
 	, sem(1)
 {
 }
@@ -22,44 +22,40 @@ void EventDistributor::registerEventListener(
 		EventType type, EventListener& listener, Priority priority)
 {
 	ScopedLock lock(sem);
-	PriorityMap& priorityMap = listeners[type];
-	for (PriorityMap::const_iterator it = priorityMap.begin();
-	     it != priorityMap.end(); ++it) {
+	auto& priorityMap = listeners[type];
+	for (auto& p : priorityMap) {
 		// a listener may only be registered once for each type
-		assert(it->second != &listener);
+		assert(p.second != &listener); (void)p;
 	}
-	priorityMap.insert(PriorityMap::value_type(priority, &listener));
+	// insert at highest position that keeps listeners sorted on priority
+	auto it = upper_bound(priorityMap.begin(), priorityMap.end(), priority,
+	                      LessTupleElement<0>());
+	priorityMap.insert(it, std::make_pair(priority, &listener));
 }
 
 void EventDistributor::unregisterEventListener(
 		EventType type, EventListener& listener)
 {
 	ScopedLock lock(sem);
-	PriorityMap& priorityMap = listeners[type];
-	for (PriorityMap::iterator it = priorityMap.begin();
-	     it != priorityMap.end(); ++it) {
-		if (it->second == &listener) {
-			priorityMap.erase(it);
-			break;
-		}
-	}
+	auto& priorityMap = listeners[type];
+	auto it = find_if(priorityMap.begin(), priorityMap.end(),
+		[&](PriorityMap::value_type v) { return v.second == &listener; });
+	assert(it != priorityMap.end());
+	priorityMap.erase(it);
 }
 
-void EventDistributor::distributeEvent(Event* event)
+void EventDistributor::distributeEvent(const EventPtr& event)
 {
 	// TODO: Implement a real solution against modifying data structure while
 	//       iterating through it.
-	//       For example, assign NULL first and then iterate again after
-	//       delivering events to remove the NULL values.
+	//       For example, assign nullptr first and then iterate again after
+	//       delivering events to remove the nullptr values.
 	// TODO: Is it useful to test for 0 listeners or should we just always
 	//       queue the event?
 	assert(event);
 	ScopedLock lock(sem);
 	if (!listeners[event->getType()].empty()) {
-		// shared_ptr is not thread safe of its own, so instead of
-		// passing a shared_ptr as parameter to this method we create
-		// the shared_ptr here while we hold the lock
-		scheduledEvents.push_back(EventPtr(event));
+		scheduledEvents.push_back(event);
 		// must release lock, otherwise there's a deadlock:
 		//   thread 1: Reactor::deleteMotherBoard()
 		//             EventDistributor::unregisterEventListener()
@@ -68,20 +64,13 @@ void EventDistributor::distributeEvent(Event* event)
 		cond.signalAll();
 		lock.release();
 		reactor.enterMainLoop();
-	} else {
-		delete event;
 	}
 }
 
 bool EventDistributor::isRegistered(EventType type, EventListener* listener) const
 {
-	TypeMap::const_iterator it = listeners.find(type);
-	if (it == listeners.end()) return false;
-
-	const PriorityMap& priorityMap = it->second;
-	for (PriorityMap::const_iterator it2 = priorityMap.begin();
-	     it2 != priorityMap.end(); ++it2) {
-		if (it2->second == listener) {
+	for (auto& p : listeners[type]) {
+		if (p.second == listener) {
 			return true;
 		}
 	}
@@ -104,26 +93,23 @@ void EventDistributor::deliverEvents()
 		EventQueue eventsCopy;
 		swap(eventsCopy, scheduledEvents);
 
-		for (EventQueue::const_iterator it = eventsCopy.begin();
-		     it != eventsCopy.end(); ++it) {
-			const EventPtr& event = *it;
-			EventType type = event->getType();
-			PriorityMap priorityMapCopy = listeners[type];
+		for (auto& event : eventsCopy) {
+			auto type = event->getType();
+			auto priorityMapCopy = listeners[type];
 			sem.up();
-			unsigned allowPriorities = unsigned(-1); // all priorities
-			for (PriorityMap::const_iterator it = priorityMapCopy.begin();
-			     it != priorityMapCopy.end(); ++it) {
+			unsigned blockPriority = unsigned(-1); // allow all
+			for (auto& p : priorityMapCopy) {
 				// It's possible delivery to one of the previous
 				// Listeners unregistered the current Listener.
-				if (!isRegistered(type, it->second)) continue;
+				if (!isRegistered(type, p.second)) continue;
 
-				unsigned currentPriority = it->first;
-				if (!(currentPriority & allowPriorities)) continue;
+				unsigned currentPriority = p.first;
+				if (currentPriority >= blockPriority) break;
 
-				unsigned maskPriorities = it->second->signalEvent(event);
-
-				assert(maskPriorities < currentPriority);
-				allowPriorities &= ~maskPriorities;
+				if (unsigned block = p.second->signalEvent(event)) {
+					assert(block > currentPriority);
+					blockPriority = block;
+				}
 			}
 			sem.down();
 		}
