@@ -1,8 +1,9 @@
-// $Id: Keys.cc 11258 2010-02-10 20:40:43Z m9710797 $
-
 #include "Keys.hh"
 #include "StringOp.hh"
-#include <map>
+#include "stl.hh"
+#include <algorithm>
+#include <vector>
+#include <utility>
 
 using std::string;
 
@@ -10,8 +11,8 @@ namespace openmsx {
 
 namespace Keys {
 
-typedef std::map<std::string, KeyCode, StringOp::caseless> KeyMap;
-static KeyMap keymap;
+static std::vector<std::pair<string_ref, KeyCode>> keys;
+typedef CmpTupleElement<0, StringOp::caseless> CmpKeys;
 
 static void initialize()
 {
@@ -285,49 +286,49 @@ static void initialize()
 		{ "PRESS",	KD_PRESS },
 		{ "RELEASE",	KD_RELEASE },
 
-		{ NULL,	K_NONE }
+		{ nullptr,	K_NONE }
 	};
 	const Entry* e = entries;
 	while (e->name) {
-		keymap[e->name] = e->code;
+		keys.push_back(std::make_pair(e->name, e->code));
 		++e;
 	}
+	sort(keys.begin(), keys.end(), CmpKeys());
 }
 
-KeyCode getCode(const string& name)
+KeyCode getCode(string_ref name)
 {
 	initialize();
 
-	KeyCode result = static_cast<KeyCode>(0);
-	string::size_type lastPos = 0;
-	while (lastPos != string::npos) {
-		string::size_type pos = name.find_first_of(",+/", lastPos);
-		string part = (pos != string::npos)
-		            ? name.substr(lastPos, pos - lastPos)
-			    : name.substr(lastPos);
-		KeyMap::const_iterator it = keymap.find(part);
-		if (it != keymap.end()) {
-			KeyCode partCode = it->second;
-			if ((partCode & K_MASK) && (result & K_MASK)) {
-				// more than one non-modifier component
-				// is not allowed
-				return K_NONE;
-			}
-			result = static_cast<KeyCode>(result | partCode);
-		} else {
+	auto result = static_cast<KeyCode>(0);
+	string_ref::size_type lastPos = 0;
+	while (lastPos != string_ref::npos) {
+		auto pos = name.substr(lastPos).find_first_of(",+/");
+		auto part = (pos != string_ref::npos)
+		          ? name.substr(lastPos, pos)
+		          : name.substr(lastPos);
+		auto it = lower_bound(keys.begin(), keys.end(), part, CmpKeys());
+		StringOp::casecmp cmp;
+		if ((it == keys.end()) || !cmp(it->first, part)) {
 			return K_NONE;
 		}
-		lastPos = pos;
-		if (lastPos != string::npos) {
-			++lastPos;
+		KeyCode partCode = it->second;
+		if ((partCode & K_MASK) && (result & K_MASK)) {
+			// more than one non-modifier component
+			// is not allowed
+			return K_NONE;
 		}
+		result = static_cast<KeyCode>(result | partCode);
+		lastPos = (pos != string_ref::npos)
+		        ? lastPos + pos + 1
+		        : string_ref::npos;
 	}
 	return result;
 }
 
 KeyCode getCode(SDLKey key, SDLMod mod, Uint8 scancode, bool release)
 {
-	KeyCode result = static_cast<KeyCode>(key);
+	auto result = static_cast<KeyCode>(key);
 	if (result == 0) {
 		// Assume it is a japanese keyboard and check
 		// scancode to recognize a few japanese
@@ -375,10 +376,9 @@ const string getName(KeyCode keyCode)
 	initialize();
 
 	string result;
-	for (KeyMap::const_iterator it = keymap.begin();
-	     it != keymap.end(); ++it) {
-		if (it->second == (keyCode & K_MASK)) {
-			result = it->first;
+	for (auto& p : keys) {
+		if (p.second == (keyCode & K_MASK)) {
+			result = p.first.str();
 			break;
 		}
 	}
