@@ -1,5 +1,3 @@
-// $Id: MSXCliComm.cc 12625 2012-06-14 20:13:15Z m9710797 $
-
 #include "MSXCliComm.hh"
 #include "GlobalCliComm.hh"
 #include "MSXMotherBoard.hh"
@@ -20,14 +18,14 @@ void MSXCliComm::log(LogLevel level, string_ref message)
 void MSXCliComm::update(UpdateType type, string_ref name, string_ref value)
 {
 	assert(type < NUM_UPDATES);
-	PrevValue::iterator it = prevValues[type].find(name);
+	auto it = prevValues[type].find(name);
 	if (it != prevValues[type].end()) {
 		if (it->second == value) {
 			return;
 		}
-		it->second.assign(value.data(), value.size());
+		it->second = value.str();
 	} else {
-		prevValues[type][name].assign(value.data(), value.size());
+		prevValues[type][name] = value.str();
 	}
 	cliComm.updateHelper(type, motherBoard.getMachineID(), name, value);
 }
