@@ -1,5 +1,3 @@
-// $Id: Socket.cc 9425 2009-03-26 21:06:30Z mthuurne $
-
 #include "Socket.hh"
 #include "MSXException.hh"
 #include <map>
@@ -11,9 +9,8 @@ namespace openmsx {
 std::string sock_error()
 {
 #ifdef _WIN32
-	typedef std::map<int, std::string> ErrMap;
+	static std::map<int, std::string> errMap;
 	static std::string UnknownError = "Unknown error";
-	static ErrMap errMap;
 	static bool alreadyInit = false;
 	if (!alreadyInit) {
 		alreadyInit = true;
@@ -68,7 +65,7 @@ std::string sock_error()
 		errMap[WSAHOST_NOT_FOUND]  = "Host not found";
 		errMap[WSANO_DATA]         = "No host data of that type was found";
 	}
-	ErrMap::const_iterator it = errMap.find(WSAGetLastError());
+	auto it = errMap.find(WSAGetLastError());
 	if (it != errMap.end()) {
 		return it->second;
 	} else {
