@@ -1,5 +1,3 @@
-// $Id: StdioMessages.cc 12625 2012-06-14 20:13:15Z m9710797 $
-
 #include "StdioMessages.hh"
 #include <iostream>
 
@@ -9,7 +7,7 @@ namespace openmsx {
 
 void StdioMessages::log(CliComm::LogLevel level, string_ref message)
 {
-	const char* const* levelStr = CliComm::getLevelStrings();
+	auto levelStr = CliComm::getLevelStrings();
 	((level == CliComm::INFO) ? std::cout : std::cerr) <<
 		levelStr[level] << ": " << message << std::endl;
 
