@@ -1,8 +1,7 @@
-// $Id: TclCallbackMessages.cc 12705 2012-07-10 19:06:58Z m9710797 $
-
 #include "TclCallbackMessages.hh"
 #include "GlobalCliComm.hh"
 #include "TclCallback.hh"
+#include "memory.hh"
 
 using std::string;
 
@@ -11,8 +10,8 @@ namespace openmsx {
 TclCallbackMessages::TclCallbackMessages(GlobalCliComm& cliComm_,
                                          CommandController& controller)
 	: cliComm(cliComm_)
-	, messageCallback(new TclCallback(controller,
-		"message_callback",
+	, messageCallback(make_unique<TclCallback>(
+		controller, "message_callback",
 		"Tcl proc called when a new message is available",
 		false, // don't print callback err on cliComm (would cause infinite loop)
 		false)) // don't save setting
@@ -27,7 +26,7 @@ TclCallbackMessages::~TclCallbackMessages()
 
 void TclCallbackMessages::log(CliComm::LogLevel level, string_ref message)
 {
-	const char* const* levelStr = CliComm::getLevelStrings();
+	auto levelStr = CliComm::getLevelStrings();
 	messageCallback->execute(message, levelStr[level]);
 }
 
