@@ -1,5 +1,3 @@
-// $Id: Disk.cc 12810 2012-08-13 19:46:23Z m9710797 $
-
 #include "Disk.hh"
 #include "DiskExceptions.hh"
 
@@ -44,7 +42,7 @@ bool Disk::isDoubleSided()
 //       conversion without relying on the detected geometry parameters.
 //       Otherwise the detectGeometry() method (which itself reads these
 //       two sectors) would get in an infinite loop.
-int Disk::physToLog(byte track, byte side, byte sector)
+size_t Disk::physToLog(byte track, byte side, byte sector)
 {
 	if ((track == 0) && (side == 0)) {
 		return sector - 1;
@@ -54,20 +52,20 @@ int Disk::physToLog(byte track, byte side, byte sector)
 	}
 	return sectorsPerTrack * (side + nbSides * track) + (sector - 1);
 }
-void Disk::logToPhys(int log, byte& track, byte& side, byte& sector)
+void Disk::logToPhys(size_t log, byte& track, byte& side, byte& sector)
 {
 	if (log <= 1) {
 		track = 0;
 		side = 0;
-		sector = log + 1;
+		sector = byte(log + 1);
 		return;
 	}
 	if (!nbSides) {
 		detectGeometry();
 	}
-	track = log / (nbSides * sectorsPerTrack);
-	side = (log / sectorsPerTrack) % nbSides;
-	sector = (log % sectorsPerTrack) + 1;
+	track  = byte(log / (nbSides * sectorsPerTrack)); // TODO check for overflow
+	side   = byte((log / sectorsPerTrack) % nbSides);
+	sector = byte((log % sectorsPerTrack) + 1);
 }
 
 void Disk::setSectorsPerTrack(unsigned num)
@@ -130,12 +128,12 @@ void Disk::detectGeometry()
 	//     ...
 
 	try {
-		byte buf[SECTOR_SIZE];
+		SectorBuffer buf;
 		readSector(0, buf); // bootsector
-		if ((buf[0] == 0xE9) || (buf[0] ==0xEB)) {
+		if ((buf.raw[0] == 0xE9) || (buf.raw[0] == 0xEB)) {
 			// use values from bootsector
-			sectorsPerTrack = buf[0x18] + 256 * buf[0x19];
-			nbSides         = buf[0x1A] + 256 * buf[0x1B];
+			sectorsPerTrack = buf.bootSector.sectorsTrack;
+			nbSides         = buf.bootSector.nrSides;
 			if ((sectorsPerTrack == 0) || (sectorsPerTrack > 255) ||
 			    (nbSides         == 0) || (nbSides         > 255)) {
 				// seems like bogus values, use defaults
@@ -143,7 +141,7 @@ void Disk::detectGeometry()
 			}
 		} else {
 			readSector(1, buf); // 1st fat sector
-			byte mediaDescriptor = buf[0];
+			byte mediaDescriptor = buf.raw[0];
 			if (mediaDescriptor >= 0xF8) {
 				sectorsPerTrack = (mediaDescriptor & 2) ? 8 : 9;
 				nbSides         = (mediaDescriptor & 1) ? 2 : 1;
@@ -160,4 +158,3 @@ void Disk::detectGeometry()
 }
 
 } // namespace openmsx
-
