@@ -1,5 +1,3 @@
-// $Id: DiskChanger.cc 12846 2012-09-08 08:10:36Z m9710797 $
-
 #include "DiskChanger.hh"
 #include "DiskFactory.hh"
 #include "DummyDisk.hh"
@@ -27,10 +25,9 @@
 #include "serialize.hh"
 #include "serialize_stl.hh"
 #include "serialize_constr.hh"
-#include "shared_ptr.hh"
-#include "ref.hh"
+#include "memory.hh"
+#include <functional>
 
-using std::set;
 using std::string;
 using std::vector;
 
@@ -70,9 +67,9 @@ DiskChanger::DiskChanger(const string& driveName_,
                          DiskManipulator& manipulator_,
                          bool createCmd)
 	: controller(controller_)
-	, stateChangeDistributor(NULL)
-	, scheduler(NULL)
-	, filePool(NULL)
+	, stateChangeDistributor(nullptr)
+	, scheduler(nullptr)
+	, filePool(nullptr)
 	, diskFactory(diskFactory_)
 	, manipulator(manipulator_)
 	, driveName(driveName_)
@@ -92,8 +89,8 @@ void DiskChanger::init(const string& prefix, bool createCmd)
 
 void DiskChanger::createCommand()
 {
-	if (diskCommand.get()) return;
-	diskCommand.reset(new DiskCommand(controller, *this));
+	if (diskCommand) return;
+	diskCommand = make_unique<DiskCommand>(controller, *this);
 }
 
 DiskChanger::~DiskChanger()
@@ -139,7 +136,7 @@ Disk& DiskChanger::getDisk()
 SectorAccessibleDisk* DiskChanger::getSectorAccessibleDisk()
 {
 	if (dynamic_cast<DummyDisk*>(disk.get())) {
-		return NULL;
+		return nullptr;
 	}
 	return dynamic_cast<SectorAccessibleDisk*>(disk.get());
 }
@@ -154,19 +151,17 @@ void DiskChanger::sendChangeDiskEvent(const vector<string>& args)
 	// note: might throw MSXException
 	if (stateChangeDistributor) {
 		stateChangeDistributor->distributeNew(
-			StateChangeDistributor::EventPtr(new MSXCommandEvent(
-				args, scheduler->getCurrentTime())));
+			std::make_shared<MSXCommandEvent>(
+				args, scheduler->getCurrentTime()));
 	} else {
-		signalStateChange(
-			StateChangeDistributor::EventPtr(
-				new MSXCommandEvent(args, EmuTime::zero)));
+		signalStateChange(std::make_shared<MSXCommandEvent>(
+			args, EmuTime::zero));
 	}
 }
 
-void DiskChanger::signalStateChange(const shared_ptr<StateChange>& event)
+void DiskChanger::signalStateChange(const std::shared_ptr<StateChange>& event)
 {
-	MSXCommandEvent* commandEvent =
-		dynamic_cast<MSXCommandEvent*>(event.get());
+	auto commandEvent = dynamic_cast<MSXCommandEvent*>(event.get());
 	if (!commandEvent) return;
 
 	const vector<TclObject>& tokens = commandEvent->getTokens();
@@ -201,24 +196,24 @@ void DiskChanger::insertDisk(const vector<TclObject>& args)
 {
 	UserFileContext context;
 	const string& diskImage = FileOperations::getConventionalPath(args[1].getString());
-	std::auto_ptr<Disk> newDisk(diskFactory.createDisk(diskImage, *this));
+	std::unique_ptr<Disk> newDisk(diskFactory.createDisk(diskImage, *this));
 	for (unsigned i = 2; i < args.size(); ++i) {
 		Filename filename(args[i].getString().str(), context);
 		newDisk->applyPatch(filename);
 	}
 
 	// no errors, only now replace original disk
-	changeDisk(newDisk);
+	changeDisk(std::move(newDisk));
 }
 
 void DiskChanger::ejectDisk()
 {
-	changeDisk(std::auto_ptr<Disk>(new DummyDisk()));
+	changeDisk(make_unique<DummyDisk>());
 }
 
-void DiskChanger::changeDisk(std::auto_ptr<Disk> newDisk)
+void DiskChanger::changeDisk(std::unique_ptr<Disk> newDisk)
 {
-	disk = newDisk;
+	disk = std::move(newDisk);
 	diskChangedFlag = true;
 	controller.getCliComm().update(CliComm::MEDIA, getDriveName(),
 	                               getDiskName().getResolved());
@@ -324,12 +319,10 @@ string DiskCommand::help(const vector<string>& /*tokens*/) const
 void DiskCommand::tabCompletion(vector<string>& tokens) const
 {
 	if (tokens.size() >= 2) {
-		set<string> extra;
-		extra.insert("eject");
-		extra.insert("ramdsk");
-		extra.insert("insert");
-		UserFileContext context;
-		completeFileName(tokens, context, extra);
+		static const char* const extra[] = {
+			"eject", "ramdsk", "insert",
+		};
+		completeFileName(tokens, UserFileContext(), extra);
 	}
 }
 
@@ -364,7 +357,7 @@ void DiskChanger::serialize(Archive& ar, unsigned version)
 
 	vector<Filename> patches;
 	if (!ar.isLoader()) {
-		disk->getPatches(patches);
+		patches = disk->getPatches();
 	}
 	ar.serialize("patches", patches);
 
@@ -387,19 +380,17 @@ void DiskChanger::serialize(Archive& ar, unsigned version)
 			if (!FileOperations::exists(name)) {
 				assert(filePool);
 				assert(!oldChecksum.empty());
-				std::auto_ptr<File> file = filePool->getFile(
-					FilePool::DISK, Sha1Sum(oldChecksum));
-				if (file.get()) {
+				if (auto file = filePool->getFile(
+				          FilePool::DISK, Sha1Sum(oldChecksum))) {
 					name = file->getURL();
 				}
 			}
 			vector<TclObject> args;
 			args.push_back(TclObject("dummy"));
 			args.push_back(TclObject(name));
-			for (vector<Filename>::iterator it = patches.begin();
-			     it != patches.end(); ++it) {
-				it->updateAfterLoadState();
-				args.push_back(TclObject(it->getResolved())); // TODO
+			for (auto& p : patches) {
+				p.updateAfterLoadState();
+				args.push_back(TclObject(p.getResolved())); // TODO
 			}
 
 			try {
@@ -434,7 +425,7 @@ void DiskChanger::serialize(Archive& ar, unsigned version)
 // extra (local) constructor arguments for polymorphic de-serialization
 template<> struct SerializeConstructorArgs<DiskChanger>
 {
-	typedef Tuple<std::string> type;
+	typedef std::tuple<std::string> type;
 
 	template<typename Archive>
 	void save(Archive& ar, const DiskChanger& changer)
@@ -452,6 +443,6 @@ template<> struct SerializeConstructorArgs<DiskChanger>
 
 INSTANTIATE_SERIALIZE_METHODS(DiskChanger);
 REGISTER_POLYMORPHIC_CLASS_1(DiskContainer, DiskChanger, "DiskChanger",
-                             reference_wrapper<MSXMotherBoard>);
+                             std::reference_wrapper<MSXMotherBoard>);
 
 } // namespace openmsx
