@@ -1,5 +1,3 @@
-// $Id: DiskContainer.cc 9286 2009-02-28 18:15:49Z mfeingol $
-
 #include "DiskContainer.hh"
 #include "NowindRomDisk.hh"
 
@@ -11,7 +9,7 @@ DiskContainer::~DiskContainer()
 
 bool DiskContainer::isRomdisk() const
 {
-	return dynamic_cast<const NowindRomDisk*>(this) != NULL;
+	return dynamic_cast<const NowindRomDisk*>(this) != nullptr;
 }
 
 } // namespace openmsx
