@@ -1,5 +1,3 @@
-// $Id: DiskImageUtils.cc 12823 2012-08-19 19:01:36Z m9710797 $
-
 #include "DiskImageUtils.hh"
 #include "DiskPartition.hh"
 #include "CommandException.hh"
@@ -14,8 +12,6 @@
 namespace openmsx {
 namespace DiskImageUtils {
 
-static const unsigned SECTOR_SIZE = SectorAccessibleDisk::SECTOR_SIZE;
-
 static const char PARTAB_HEADER[11] = {
 	'\353', '\376', '\220', 'M', 'S', 'X', '_', 'I', 'D', 'E', ' '
 };
@@ -26,14 +22,14 @@ static bool isPartitionTableSector(const PartitionTable& pt)
 
 bool hasPartitionTable(SectorAccessibleDisk& disk)
 {
-	PartitionTable pt;
-	disk.readSector(0, reinterpret_cast<byte*>(&pt));
-	return isPartitionTableSector(pt);
+	SectorBuffer buf;
+	disk.readSector(0, buf);
+	return isPartitionTableSector(buf.pt);
 }
 
 
 static Partition& checkImpl(SectorAccessibleDisk& disk, unsigned partition,
-                            PartitionTable& pt)
+                            SectorBuffer& buf)
 {
 	// check number in range
 	if (partition < 1 || partition > 31) {
@@ -41,13 +37,13 @@ static Partition& checkImpl(SectorAccessibleDisk& disk, unsigned partition,
 			"Invalid partition number specified (must be 1-31).");
 	}
 	// check drive has a partition table
-	disk.readSector(0, reinterpret_cast<byte*>(&pt));
-	if (!isPartitionTableSector(pt)) {
+	disk.readSector(0, buf);
+	if (!isPartitionTableSector(buf.pt)) {
 		throw CommandException(
 			"No (or invalid) partition table.");
 	}
 	// check valid partition number
-	Partition& p = pt.part[31 - partition];
+	auto& p = buf.pt.part[31 - partition];
 	if (p.start == 0) {
 		throw CommandException(StringOp::Builder() <<
 			"No partition number " << partition);
@@ -57,14 +53,14 @@ static Partition& checkImpl(SectorAccessibleDisk& disk, unsigned partition,
 
 void checkValidPartition(SectorAccessibleDisk& disk, unsigned partition)
 {
-	PartitionTable pt;
-	checkImpl(disk, partition, pt);
+	SectorBuffer buf;
+	checkImpl(disk, partition, buf);
 }
 
 void checkFAT12Partition(SectorAccessibleDisk& disk, unsigned partition)
 {
-	PartitionTable pt;
-	Partition& p = checkImpl(disk, partition, pt);
+	SectorBuffer buf;
+	Partition& p = checkImpl(disk, partition, buf);
 
 	// check partition type
 	if (p.sys_ind != 0x01) {
@@ -74,12 +70,12 @@ void checkFAT12Partition(SectorAccessibleDisk& disk, unsigned partition)
 
 
 // Create a correct bootsector depending on the required size of the filesystem
-static void setBootSector(MSXBootSector& boot, unsigned nbSectors,
-                          unsigned& firstDataSector, byte& descriptor)
+static void setBootSector(MSXBootSector& boot, size_t nbSectors,
+                          unsigned& firstDataSector, byte& descriptor, bool dos1)
 {
 	// start from the default bootblock ..
-	const byte* defaultBootBlock = BootBlocks::dos2BootBlock;
-	memcpy(&boot, defaultBootBlock, SECTOR_SIZE);
+	auto& defaultBootBlock = dos1 ? BootBlocks::dos1BootBlock : BootBlocks::dos2BootBlock;
+	memcpy(&boot, &defaultBootBlock, sizeof(boot));
 
 	// .. and fill-in image-size dependent parameters ..
 	// these are the same for most formats
@@ -167,7 +163,7 @@ static void setBootSector(MSXBootSector& boot, unsigned nbSectors,
 		nbSectors = 720;	// force nbSectors to 720, why?
 	}
 
-	boot.nrSectors = nbSectors;
+	boot.nrSectors = uint16_t(nbSectors); // TODO check for overflow
 	boot.nrSides = nbSides;
 	boot.spCluster = nbSectorsPerCluster;
 	boot.nrFats = nbFats;
@@ -177,49 +173,46 @@ static void setBootSector(MSXBootSector& boot, unsigned nbSectors,
 	boot.resvSectors = nbReservedSectors;
 	boot.hiddenSectors = nbHiddenSectors;
 
-	// set random volume id
-	static bool init = false;
-	if (!init) {
-		init = true;
-		srand(unsigned(time(0)));
+	if (!dos1) {
+		// set random volume id
+		static bool init = false;
+		if (!init) {
+			init = true;
+			srand(unsigned(time(nullptr)));
+		}
+		boot.vol_id = rand() & 0x7F7F7F7F; // why are bits masked?
 	}
-	byte* raw = reinterpret_cast<byte*>(&boot);
-	for (int i = 0x27; i < 0x2B; ++i) {
-		raw[i] = rand() & 0x7F;
-	}
-
 	unsigned nbRootDirSectors = nbDirEntry / 16;
 	unsigned rootDirStart = 1 + nbFats * nbSectorsPerFat;
 	firstDataSector = rootDirStart + nbRootDirSectors;
 }
 
-void format(SectorAccessibleDisk& disk)
+void format(SectorAccessibleDisk& disk, bool dos1)
 {
 	// first create a bootsector for given partition size
-	unsigned nbSectors = disk.getNbSectors();
-	MSXBootSector boot;
+	size_t nbSectors = disk.getNbSectors();
+	SectorBuffer buf;
 	unsigned firstDataSector;
 	byte descriptor;
-	setBootSector(boot, nbSectors, firstDataSector, descriptor);
-	disk.writeSector(0, reinterpret_cast<byte*>(&boot));
+	setBootSector(buf.bootSector, nbSectors, firstDataSector, descriptor, dos1);
+	disk.writeSector(0, buf);
 
 	// write empty FAT and directory sectors
-	byte buf[SECTOR_SIZE];
-	memset(buf, 0x00, sizeof(buf));
+	memset(&buf, 0, sizeof(buf));
 	for (unsigned i = 2; i < firstDataSector; ++i) {
 		disk.writeSector(i, buf);
 	}
 	// first FAT sector is special:
 	//  - first byte contains the media descriptor
 	//  - first two clusters must be marked as EOF
-	buf[0] = descriptor;
-	buf[1] = 0xFF;
-	buf[2] = 0xFF;
+	buf.raw[0] = descriptor;
+	buf.raw[1] = 0xFF;
+	buf.raw[2] = 0xFF;
 	disk.writeSector(1, buf);
 
 	// write 'empty' data sectors
-	memset(buf, 0xE5, sizeof(buf));
-	for (unsigned i = firstDataSector; i < nbSectors; ++i) {
+	memset(&buf, 0xE5, sizeof(buf));
+	for (size_t i = firstDataSector; i < nbSectors; ++i) {
 		disk.writeSector(i, buf);
 	}
 }
@@ -241,16 +234,15 @@ void partition(SectorAccessibleDisk& disk, const std::vector<unsigned>& sizes)
 {
 	assert(sizes.size() <= 31);
 
-	PartitionTable pt;
-	memset(&pt, 0, sizeof(pt));
-	memcpy(pt.header, PARTAB_HEADER, sizeof(PARTAB_HEADER));
-	pt.end[0] = 0x55;
-	pt.end[1] = 0xAA;
+	SectorBuffer buf;
+	memset(&buf, 0, sizeof(buf));
+	memcpy(buf.pt.header, PARTAB_HEADER, sizeof(PARTAB_HEADER));
+	buf.pt.end = 0xAA55;
 
 	unsigned partitionOffset = 1;
 	for (unsigned i = 0; i < sizes.size(); ++i) {
 		unsigned partitionNbSectors = sizes[i];
-		Partition& p = pt.part[31 - i];
+		auto& p = buf.pt.part[30 - i];
 		unsigned startCylinder, startHead, startSector;
 		logicalToCHS(partitionOffset,
 		             startCylinder, startHead, startSector);
@@ -271,7 +263,7 @@ void partition(SectorAccessibleDisk& disk, const std::vector<unsigned>& sizes)
 		format(diskPartition);
 		partitionOffset += partitionNbSectors;
 	}
-	disk.writeSector(0, reinterpret_cast<byte*>(&pt));
+	disk.writeSector(0, buf);
 }
 
 } // namespace DiskImageUtils
