@@ -1,5 +1,3 @@
-// $Id: DiskManipulator.cc 12626 2012-06-14 20:14:17Z m9710797 $
-
 #include "DiskManipulator.hh"
 #include "DiskContainer.hh"
 #include "MSXtar.hh"
@@ -17,13 +15,14 @@
 #include "FileOperations.hh"
 #include "SectorBasedDisk.hh"
 #include "StringOp.hh"
+#include "memory.hh"
+#include "xrange.hh"
 #include <cassert>
 #include <ctype.h>
 
-using std::set;
 using std::string;
 using std::vector;
-using std::auto_ptr;
+using std::unique_ptr;
 
 namespace openmsx {
 
@@ -67,7 +66,7 @@ void DiskManipulator::registerDrive(
 
 void DiskManipulator::unregisterDrive(DiskContainer& drive)
 {
-	Drives::iterator it = findDriveSettings(drive);
+	auto it = findDriveSettings(drive);
 	assert(it != drives.end());
 	drives.erase(it);
 }
@@ -75,23 +74,15 @@ void DiskManipulator::unregisterDrive(DiskContainer& drive)
 DiskManipulator::Drives::iterator DiskManipulator::findDriveSettings(
 	DiskContainer& drive)
 {
-	for (Drives::iterator it = drives.begin(); it != drives.end(); ++it) {
-		if (it->drive == &drive) {
-			return it;
-		}
-	}
-	return drives.end();
+	return find_if(drives.begin(), drives.end(),
+	               [&](DriveSettings& ds) { return ds.drive == &drive; });
 }
 
 DiskManipulator::Drives::iterator DiskManipulator::findDriveSettings(
 	string_ref name)
 {
-	for (Drives::iterator it = drives.begin(); it != drives.end(); ++it) {
-		if (it->driveName == name) {
-			return it;
-		}
-	}
-	return drives.end();
+	return find_if(drives.begin(), drives.end(),
+	               [&](DriveSettings& ds) { return ds.driveName == name; });
 }
 
 DiskManipulator::DriveSettings& DiskManipulator::getDriveSettings(
@@ -99,13 +90,13 @@ DiskManipulator::DriveSettings& DiskManipulator::getDriveSettings(
 {
 	// first split-off the end numbers (if present)
 	// these will be used as partition indication
-	string_ref::size_type pos1 = diskname.find("::");
-	string_ref tmp1 = (pos1 == string_ref::npos) ? diskname : diskname.substr(pos1);
-	string_ref::size_type pos2 = tmp1.find_first_of("0123456789");
-	string_ref::size_type pos1b = (pos1 == string_ref::npos) ? 0 : pos1;
-	string_ref tmp2 = diskname.substr(0, pos2 + pos1b);
+	auto pos1 = diskname.find("::");
+	auto tmp1 = (pos1 == string_ref::npos) ? diskname : diskname.substr(pos1);
+	auto pos2 = tmp1.find_first_of("0123456789");
+	auto pos1b = (pos1 == string_ref::npos) ? 0 : pos1;
+	auto tmp2 = diskname.substr(0, pos2 + pos1b);
 
-	Drives::iterator it = findDriveSettings(tmp2);
+	auto it = findDriveSettings(tmp2);
 	if (it == drives.end()) {
 		it = findDriveSettings(getMachinePrefix() + tmp2);
 		if (it == drives.end()) {
@@ -113,7 +104,7 @@ DiskManipulator::DriveSettings& DiskManipulator::getDriveSettings(
 		}
 	}
 
-	SectorAccessibleDisk* disk = it->drive->getSectorAccessibleDisk();
+	auto* disk = it->drive->getSectorAccessibleDisk();
 	if (!disk) {
 		// not a SectorBasedDisk
 		throw CommandException("Unsupported disk type.");
@@ -123,23 +114,20 @@ DiskManipulator::DriveSettings& DiskManipulator::getDriveSettings(
 		// whole disk
 		it->partition = 0;
 	} else {
-		string_ref num = diskname.substr(pos2);
-		SectorAccessibleDisk* disk =
-			it->drive->getSectorAccessibleDisk();
-		int partition = stoi(num, NULL, 10);
+		const auto& num = diskname.substr(pos2);
+		int partition = stoi(num, nullptr, 10);
 		DiskImageUtils::checkFAT12Partition(*disk, partition);
 		it->partition = partition;
 	}
 	return *it;
 }
 
-auto_ptr<DiskPartition> DiskManipulator::getPartition(
+unique_ptr<DiskPartition> DiskManipulator::getPartition(
 	const DriveSettings& driveData)
 {
-	SectorAccessibleDisk* disk = driveData.drive->getSectorAccessibleDisk();
+	auto* disk = driveData.drive->getSectorAccessibleDisk();
 	assert(disk);
-	return auto_ptr<DiskPartition>(
-		new DiskPartition(*disk, driveData.partition));
+	return make_unique<DiskPartition>(*disk, driveData.partition);
 }
 
 
@@ -151,8 +139,9 @@ string DiskManipulator::execute(const vector<string>& tokens)
 
 	} else if ((tokens.size() != 4 && ( tokens[1] == "savedsk"
 	                                || tokens[1] == "mkdir"))
-	        || (tokens.size() != 3 && (tokens[1] == "dir"
-	                                || tokens[1] == "format"))
+	        || (tokens.size() != 3 && tokens[1] == "dir")
+	        || ((tokens.size() < 3 || tokens.size() > 4) &&
+	                                tokens[1] == "format")
 	        || ((tokens.size() < 3 || tokens.size() > 4) &&
 	                                  (tokens[1] == "chdir"))
 	        || (tokens.size() < 4 && ( tokens[1] == "export"
@@ -160,46 +149,53 @@ string DiskManipulator::execute(const vector<string>& tokens)
 	        || (tokens.size() <= 3 && (tokens[1] == "create"))) {
 		throw CommandException("Incorrect number of parameters");
 
-	} else if ( tokens[1] == "export" ) {
+	} else if (tokens[1] == "export") {
 		if (!FileOperations::isDirectory(tokens[3])) {
 			throw CommandException(tokens[3] + " is not a directory");
 		}
-		DriveSettings& settings = getDriveSettings(tokens[2]);
+		auto& settings = getDriveSettings(tokens[2]);
 		vector<string> lists(tokens.begin() + 4, tokens.end());
 		exprt(settings, tokens[3], lists);
 
-	} else if (tokens[1] == "import" ) {
-		DriveSettings& settings = getDriveSettings(tokens[2]);
+	} else if (tokens[1] == "import") {
+		auto& settings = getDriveSettings(tokens[2]);
 		vector<string> lists(tokens.begin() + 3, tokens.end());
 		result = import(settings, lists);
 
 	} else if (tokens[1] == "savedsk") {
-		DriveSettings& settings = getDriveSettings(tokens[2]);
+		auto& settings = getDriveSettings(tokens[2]);
 		savedsk(settings, tokens[3]);
 
 	} else if (tokens[1] == "chdir") {
-		DriveSettings& settings = getDriveSettings(tokens[2]);
+		auto& settings = getDriveSettings(tokens[2]);
 		if (tokens.size() == 3) {
 			result += "Current directory: " +
 			          settings.workingDir[settings.partition];
 		} else {
-			chdir(settings, tokens[3], result);
+			result += chdir(settings, tokens[3]);
 		}
 
 	} else if (tokens[1] == "mkdir") {
-		DriveSettings& settings = getDriveSettings(tokens[2]);
+		auto& settings = getDriveSettings(tokens[2]);
 		mkdir(settings, tokens[3]);
 
 	} else if (tokens[1] == "create") {
 		create(tokens);
 
 	} else if (tokens[1] == "format") {
-		DriveSettings& settings = getDriveSettings(tokens[2]);
-		format(settings);
+		bool dos1 = false;
+		vector<string> myTokens(tokens);
+		const auto it = find(myTokens.begin(), myTokens.end(), "-dos1");
+		if (it != myTokens.end()) {
+			myTokens.erase(it);
+			dos1 = true;
+		}
+		auto& settings = getDriveSettings(myTokens[2]);
+		format(settings, dos1);
 
 	} else if (tokens[1] == "dir") {
-		DriveSettings& settings = getDriveSettings(tokens[2]);
-		dir(settings, result);
+		auto& settings = getDriveSettings(tokens[2]);
+		result += dir(settings);
 
 	} else {
 		throw CommandException("Unknown subcommand: " + tokens[1]);
@@ -283,38 +279,33 @@ string DiskManipulator::help(const vector<string>& tokens) const
 void DiskManipulator::tabCompletion(vector<string>& tokens) const
 {
 	if (tokens.size() == 2) {
-		set<string> cmds;
-		cmds.insert("import");
-		cmds.insert("export");
-		cmds.insert("savedsk");
-		cmds.insert("dir");
-		cmds.insert("create");
-		cmds.insert("format");
-		cmds.insert("chdir");
-		cmds.insert("mkdir");
+		static const char* const cmds[] = {
+			"import", "export", "savedsk", "dir", "create",
+			"format", "chdir", "mkdir",
+		};
 		completeString(tokens, cmds);
 
 	} else if ((tokens.size() == 3) && (tokens[1] == "create")) {
-		UserFileContext context;
-		completeFileName(tokens, context);
+		completeFileName(tokens, UserFileContext());
 
 	} else if (tokens.size() == 3) {
-		set<string> names;
-		for (Drives::const_iterator it = drives.begin();
-		     it != drives.end(); ++it) {
-			string name1 = it->driveName; // with prexix
-			string name2 = it->drive->getContainerName(); // without prefix
-			names.insert(name1);
-			names.insert(name2);
+		vector<string> names;
+		if ((tokens[1] == "format") || (tokens[1] == "create")) {
+			names.push_back("-dos1");
+		}
+		for (auto& d : drives) {
+			const auto& name1 = d.driveName; // with prexix
+			const auto& name2 = d.drive->getContainerName(); // without prefix
+			names.push_back(name1);
+			names.push_back(name2);
 			// if it has partitions then we also add the partition
 			// numbers to the autocompletion
-			if (SectorAccessibleDisk* disk =
-			    it->drive->getSectorAccessibleDisk()) {
+			if (auto* disk = d.drive->getSectorAccessibleDisk()) {
 				for (unsigned i = 1; i <= MAX_PARTITIONS; ++i) {
 					try {
 						DiskImageUtils::checkFAT12Partition(*disk, i);
-						names.insert(name1 + StringOp::toString(i));
-						names.insert(name2 + StringOp::toString(i));
+						names.push_back(name1 + StringOp::toString(i));
+						names.push_back(name2 + StringOp::toString(i));
 					} catch (MSXException&) {
 						// skip invalid partition
 					}
@@ -327,13 +318,16 @@ void DiskManipulator::tabCompletion(vector<string>& tokens) const
 		if ((tokens[1] == "savedsk") ||
 		    (tokens[1] == "import")  ||
 		    (tokens[1] == "export")) {
-			UserFileContext context;
-			completeFileName(tokens, context);
+			completeFileName(tokens, UserFileContext());
 		} else if (tokens[1] == "create") {
-			set<string> cmds;
-			cmds.insert("360");
-			cmds.insert("720");
-			cmds.insert("32M");
+			static const char* const cmds[] = {
+				"360", "720", "32M", "-dos1"
+			};
+			completeString(tokens, cmds);
+		} else if (tokens[1] == "format") {
+			static const char* const cmds[] = {
+				"-dos1"
+			};
 			completeString(tokens, cmds);
 		}
 	}
@@ -342,13 +336,12 @@ void DiskManipulator::tabCompletion(vector<string>& tokens) const
 void DiskManipulator::savedsk(const DriveSettings& driveData,
                               const string& filename)
 {
-	auto_ptr<DiskPartition> partition = getPartition(driveData);
-	unsigned nrsectors = partition->getNbSectors();
-	byte buf[SectorBasedDisk::SECTOR_SIZE];
+	auto partition = getPartition(driveData);
+	SectorBuffer buf;
 	File file(filename, File::CREATE);
-	for (unsigned i = 0; i < nrsectors; ++i) {
+	for (auto i : xrange(partition->getNbSectors())) {
 		partition->readSector(i, buf);
-		file.write(buf, SectorBasedDisk::SECTOR_SIZE);
+		file.write(&buf, sizeof(buf));
 	}
 }
 
@@ -356,18 +349,25 @@ void DiskManipulator::create(const vector<string>& tokens)
 {
 	vector<unsigned> sizes;
 	unsigned totalSectors = 0;
-	for (unsigned i = 3; i < tokens.size(); ++i) {
+	bool dos1 = false;
+	vector<string> myTokens(tokens);
+	const auto it = find(myTokens.begin(), myTokens.end(), "-dos1");
+	if (it != myTokens.end()) {
+		myTokens.erase(it);
+		dos1 = true;
+	}
+	for (unsigned i = 3; i < myTokens.size(); ++i) {
 		if (sizes.size() >= MAX_PARTITIONS) {
 			throw CommandException(StringOp::Builder() <<
 				"Maximum number of partitions is " << MAX_PARTITIONS);
 		}
 		char* q;
-		int sectors = strtol(tokens[i].c_str(), &q, 0);
+		int sectors = strtol(myTokens[i].c_str(), &q, 0);
 		int scale = 1024; // default is kilobytes
 		if (*q) {
-			if ((q == tokens[i].c_str()) || *(q + 1)) {
+			if ((q == myTokens[i].c_str()) || *(q + 1)) {
 				throw CommandException(
-					"Invalid size: " + tokens[i]);
+					"Invalid size: " + myTokens[i]);
 			}
 			switch (tolower(*q)) {
 				case 'b':
@@ -403,13 +403,16 @@ void DiskManipulator::create(const vector<string>& tokens)
 		sizes.push_back(sectors);
 		totalSectors += sectors;
 	}
+	if (sizes.empty()) {
+		throw CommandException("No size(s) given.");
+	}
 	if (sizes.size() > 1) {
 		// extra sector for partition table
 		++totalSectors;
 	}
 
 	// create file with correct size
-	Filename filename(tokens[2]);
+	Filename filename(myTokens[2]);
 	try {
 		File file(filename, File::CREATE);
 		file.truncate(totalSectors * SectorBasedDisk::SECTOR_SIZE);
@@ -423,18 +426,18 @@ void DiskManipulator::create(const vector<string>& tokens)
 		DiskImageUtils::partition(image, sizes);
 	} else {
 		// only one partition specified, don't create partition table
-		DiskImageUtils::format(image);
+		DiskImageUtils::format(image, dos1);
 	}
 }
 
-void DiskManipulator::format(DriveSettings& driveData)
+void DiskManipulator::format(DriveSettings& driveData, bool dos1)
 {
-	auto_ptr<DiskPartition> partition = getPartition(driveData);
-	DiskImageUtils::format(*partition);
+	auto partition = getPartition(driveData);
+	DiskImageUtils::format(*partition, dos1);
 	driveData.workingDir[driveData.partition] = '/';
 }
 
-auto_ptr<MSXtar> DiskManipulator::getMSXtar(
+unique_ptr<MSXtar> DiskManipulator::getMSXtar(
 	SectorAccessibleDisk& disk, DriveSettings& driveData)
 {
 	if (DiskImageUtils::hasPartitionTable(disk)) {
@@ -442,7 +445,7 @@ auto_ptr<MSXtar> DiskManipulator::getMSXtar(
 			"Please select partition number.");
 	}
 
-	auto_ptr<MSXtar> result(new MSXtar(disk));
+	auto result = make_unique<MSXtar>(disk);
 	try {
 		result->chdir(driveData.workingDir[driveData.partition]);
 	} catch (MSXException&) {
@@ -455,18 +458,17 @@ auto_ptr<MSXtar> DiskManipulator::getMSXtar(
 	return result;
 }
 
-void DiskManipulator::dir(DriveSettings& driveData, string& result)
+string DiskManipulator::dir(DriveSettings& driveData)
 {
-	auto_ptr<DiskPartition> partition = getPartition(driveData);
-	auto_ptr<MSXtar> workhorse = getMSXtar(*partition, driveData);
-	result += workhorse->dir();
+	auto partition = getPartition(driveData);
+	unique_ptr<MSXtar> workhorse = getMSXtar(*partition, driveData);
+	return workhorse->dir();
 }
 
-void DiskManipulator::chdir(DriveSettings& driveData,
-                            const string& filename, string& result)
+string DiskManipulator::chdir(DriveSettings& driveData, const string& filename)
 {
-	auto_ptr<DiskPartition> partition = getPartition(driveData);
-	auto_ptr<MSXtar> workhorse = getMSXtar(*partition, driveData);
+	auto partition = getPartition(driveData);
+	auto workhorse = getMSXtar(*partition, driveData);
 	try {
 		workhorse->chdir(filename);
 	} catch (MSXException& e) {
@@ -480,13 +482,13 @@ void DiskManipulator::chdir(DriveSettings& driveData,
 		if (!StringOp::endsWith(cwd, '/')) cwd += '/';
 		cwd += filename;
 	}
-	result += "New working directory: " + cwd;
+	return "New working directory: " + cwd;
 }
 
 void DiskManipulator::mkdir(DriveSettings& driveData, const string& filename)
 {
-	auto_ptr<DiskPartition> partition = getPartition(driveData);
-	auto_ptr<MSXtar> workhorse = getMSXtar(*partition, driveData);
+	auto partition = getPartition(driveData);
+	auto workhorse = getMSXtar(*partition, driveData);
 	try {
 		workhorse->mkdir(filename);
 	} catch (MSXException& e) {
@@ -495,32 +497,27 @@ void DiskManipulator::mkdir(DriveSettings& driveData, const string& filename)
 }
 
 string DiskManipulator::import(DriveSettings& driveData,
-                             const vector<string>& lists)
+                               const vector<string>& lists)
 {
-	auto_ptr<DiskPartition> partition = getPartition(driveData);
-	auto_ptr<MSXtar> workhorse = getMSXtar(*partition, driveData);
+	auto partition = getPartition(driveData);
+	auto workhorse = getMSXtar(*partition, driveData);
 
 	string messages;
-	for (vector<string>::const_iterator it = lists.begin();
-	     it != lists.end(); ++it) {
-		vector<string> list;
-		getCommandController().splitList(*it, list);
-
-		for (vector<string>::const_iterator it = list.begin();
-		     it != list.end(); ++it) {
+	for (auto& l : lists) {
+		for (auto& i : getCommandController().splitList(l)) {
 			try {
 				FileOperations::Stat st;
-				if (!FileOperations::getStat(*it, st)) {
+				if (!FileOperations::getStat(i, st)) {
 					throw CommandException(
-						"Non-existing file " + *it);
+						"Non-existing file " + i);
 				}
 				if (FileOperations::isDirectory(st)) {
-					messages += workhorse->addDir(*it);
+					messages += workhorse->addDir(i);
 				} else if (FileOperations::isRegularFile(st)) {
-					messages += workhorse->addFile(*it);
+					messages += workhorse->addFile(i);
 				} else {
 					// ignore other stuff (sockets, device nodes, ..)
-					messages += "Ignoring " + *it + '\n';
+					messages += "Ignoring " + i + '\n';
 				}
 			} catch (MSXException& e) {
 				throw CommandException(e.getMessage());
@@ -533,16 +530,15 @@ string DiskManipulator::import(DriveSettings& driveData,
 void DiskManipulator::exprt(DriveSettings& driveData, const string& dirname,
                             const vector<string>& lists)
 {
-	auto_ptr<DiskPartition> partition = getPartition(driveData);
-	auto_ptr<MSXtar> workhorse = getMSXtar(*partition, driveData);
+	auto partition = getPartition(driveData);
+	auto workhorse = getMSXtar(*partition, driveData);
 	try {
 		if (lists.empty()) {
 			// export all
 			workhorse->getDir(dirname);
 		} else {
-			for (vector<string>::const_iterator it = lists.begin();
-			     it != lists.end(); ++it) {
-				workhorse->getItemFromDir(dirname, *it);
+			for (auto& l : lists) {
+				workhorse->getItemFromDir(dirname, l);
 			}
 		}
 	} catch (MSXException& e) {
