@@ -1,11 +1,11 @@
-// $Id: DriveMultiplexer.cc 12732 2012-07-16 16:51:40Z m9710797 $
-
 #include "DriveMultiplexer.hh"
 #include "serialize.hh"
+#include "memory.hh"
 
 namespace openmsx {
 
 DriveMultiplexer::DriveMultiplexer(DiskDrive* drv[4])
+	: dummyDrive(make_unique<DummyDrive>())
 {
 	motor = false;
 	side = false;
@@ -14,12 +14,11 @@ DriveMultiplexer::DriveMultiplexer(DiskDrive* drv[4])
 	drive[DRIVE_B]  = drv[1];
 	drive[DRIVE_C]  = drv[2];
 	drive[DRIVE_D]  = drv[3];
-	drive[NO_DRIVE] = new DummyDrive();
+	drive[NO_DRIVE] = dummyDrive.get();
 }
 
 DriveMultiplexer::~DriveMultiplexer()
 {
-	delete drive[NO_DRIVE];
 }
 
 void DriveMultiplexer::selectDrive(DriveNum num, EmuTime::param time)
