@@ -1,5 +1,3 @@
-// $Id: DummyDisk.cc 9383 2009-03-17 19:26:19Z m9710797 $
-
 #include "DummyDisk.hh"
 #include "DiskExceptions.hh"
 
@@ -21,12 +19,12 @@ bool DummyDisk::isWriteProtectedImpl() const
 	return true; // TODO check
 }
 
-void DummyDisk::readSectorImpl(unsigned /*sector*/, byte* /*buf*/)
+void DummyDisk::readSectorImpl(size_t /*sector*/, SectorBuffer& /*buf*/)
 {
 	throw DriveEmptyException("No disk in drive");
 }
 
-void DummyDisk::writeSectorImpl(unsigned /*sector*/, const byte* /*buf*/)
+void DummyDisk::writeSectorImpl(size_t /*sector*/, const SectorBuffer& /*buf*/)
 {
 	throw DriveEmptyException("No disk in drive");
 }
