@@ -1,5 +1,3 @@
-// $Id: MSXFDC.cc 12583 2012-06-05 17:51:52Z m9710797 $
-
 #include "MSXFDC.hh"
 #include "Rom.hh"
 #include "RealDrive.hh"
@@ -7,14 +5,15 @@
 #include "StringOp.hh"
 #include "MSXException.hh"
 #include "serialize.hh"
+#include "memory.hh"
 
 namespace openmsx {
 
 MSXFDC::MSXFDC(const DeviceConfig& config)
 	: MSXDevice(config)
-	, rom(new Rom(getName() + " ROM", "rom", config))
+	, rom(make_unique<Rom>(getName() + " ROM", "rom", config))
 {
-	bool singleSided = config.findChild("singlesided") != NULL;
+	bool singleSided = config.findChild("singlesided") != nullptr;
 	int numDrives = config.getChildDataAsInt("drives", 1);
 	if ((0 > numDrives) || (numDrives >= 4)) {
 		throw MSXException(StringOp::Builder() <<
@@ -26,12 +25,12 @@ MSXFDC::MSXFDC(const DeviceConfig& config)
 	EmuDuration motorTimeout = EmuDuration::msec(timeout);
 	int i = 0;
 	for ( ; i < numDrives; ++i) {
-		drives[i].reset(new RealDrive(
+		drives[i] = make_unique<RealDrive>(
 			getMotherBoard(), motorTimeout, signalsNeedMotorOn,
-			!singleSided));
+			!singleSided);
 	}
 	for ( ; i < 4; ++i) {
-		drives[i].reset(new DummyDrive());
+		drives[i] = make_unique<DummyDrive>();
 	}
 }
 
@@ -73,7 +72,7 @@ void MSXFDC::serialize(Archive& ar, unsigned /*version*/)
 	// DriveMultiplexer already has pointers to the drives.
 	char tag[7] = { 'd', 'r', 'i', 'v', 'e', 'X', 0 };
 	for (int i = 0; i < 4; ++i) {
-		if (RealDrive* drive = dynamic_cast<RealDrive*>(drives[i].get())) {
+		if (auto drive = dynamic_cast<RealDrive*>(drives[i].get())) {
 			tag[5] = char('a' + i);
 			ar.serialize(tag, *drive);
 		}
