@@ -1,5 +1,3 @@
-// $Id: NowindCommand.cc 12674 2012-06-25 18:13:20Z m9710797 $
-
 #include "NowindCommand.hh"
 #include "NowindRomDisk.hh"
 #include "NowindHost.hh"
@@ -10,11 +8,12 @@
 #include "StringOp.hh"
 #include "FileOperations.hh"
 #include "CommandException.hh"
+#include "memory.hh"
 #include "unreachable.hh"
 #include <cassert>
 #include <deque>
 
-using std::auto_ptr;
+using std::unique_ptr;
 using std::deque;
 using std::set;
 using std::string;
@@ -30,12 +29,11 @@ NowindCommand::NowindCommand(const string& basename,
 {
 }
 
-DiskChanger* NowindCommand::createDiskChanger(
+unique_ptr<DiskChanger> NowindCommand::createDiskChanger(
 	const string& basename, unsigned n, MSXMotherBoard& motherBoard) const
 {
 	string name = StringOp::Builder() << basename << n + 1;
-	DiskChanger* drive = new DiskChanger(motherBoard, name, false);
-	return drive;
+	return make_unique<DiskChanger>(motherBoard, name, false);
 }
 
 unsigned NowindCommand::searchRomdisk(const NowindInterface::Drives& drives) const
@@ -59,14 +57,13 @@ void NowindCommand::processHdimage(
 	// disambiguity we will always interpret the string as <filename> if
 	// it is an existing filename.
 	set<unsigned> partitions;
-	string::size_type pos = hdimage.find_last_of(':');
+	auto pos = hdimage.find_last_of(':');
 	if ((pos != string::npos) && !FileOperations::exists(hdimage)) {
-		StringOp::parseRange(string_ref(hdimage).substr(string_ref::size_type(pos) + 1),
-		                     partitions, 1, 31);
+		partitions = StringOp::parseRange(
+			string_ref(hdimage).substr(pos + 1), 1, 31);
 	}
 
-	shared_ptr<SectorAccessibleDisk> wholeDisk(
-		new DSKDiskImage(Filename(hdimage)));
+	auto wholeDisk = std::make_shared<DSKDiskImage>(Filename(hdimage));
 	bool failOnError = true;
 	if (partitions.empty()) {
 		// insert all partitions
@@ -76,16 +73,19 @@ void NowindCommand::processHdimage(
 		}
 	}
 
-	for (set<unsigned>::const_iterator it = partitions.begin();
-	     it != partitions.end(); ++it) {
+	for (auto& p : partitions) {
 		try {
-			auto_ptr<DiskPartition> partition(
-				new DiskPartition(*wholeDisk, *it, wholeDisk));
-			DiskChanger* drive = createDiskChanger(
+			// Explicit conversion to shared_ptr<SectorAccessibleDisk> is
+			// for some reason needed in 32-bit vs2013 build (not in 64-bit
+			// and not in vs2012, nor gcc/clang). Compiler bug???
+			auto partition = make_unique<DiskPartition>(
+				*wholeDisk, p,
+				std::shared_ptr<SectorAccessibleDisk>(wholeDisk));
+			auto drive = createDiskChanger(
 				interface.basename, unsigned(drives.size()),
 				motherboard);
-			drive->changeDisk(auto_ptr<Disk>(partition));
-			drives.push_back(drive);
+			drive->changeDisk(unique_ptr<Disk>(std::move(partition)));
+			drives.push_back(std::move(drive));
 		} catch (MSXException&) {
 			if (failOnError) throw;
 		}
@@ -104,9 +104,10 @@ string NowindCommand::execute(const vector<string>& tokens)
 		StringOp::Builder result;
 		for (unsigned i = 0; i < drives.size(); ++i) {
 			result << "nowind" << i + 1 << ": ";
-			if (dynamic_cast<NowindRomDisk*>(drives[i])) {
+			if (dynamic_cast<NowindRomDisk*>(drives[i].get())) {
 				result << "romdisk\n";
-			} else if (DiskChanger* changer = dynamic_cast<DiskChanger*>(drives[i])) {
+			} else if (auto changer = dynamic_cast<DiskChanger*>(
+						drives[i].get())) {
 				string filename = changer->getDiskName().getOriginal();
 				result << (filename.empty() ? "--empty--" : filename)
 				       << '\n';
@@ -139,7 +140,7 @@ string NowindCommand::execute(const vector<string>& tokens)
 		bool createDrive = false;
 		string image;
 
-		string arg = args.front();
+		string arg = std::move(args.front());
 		args.pop_front();
 		if        ((arg == "--ctrl")    || (arg == "-c")) {
 			enablePhantom  = false;
@@ -159,7 +160,7 @@ string NowindCommand::execute(const vector<string>& tokens)
 				error = "Can only have one romdisk";
 			} else {
 				romdisk = unsigned(tmpDrives.size());
-				tmpDrives.push_back(new NowindRomDisk());
+				tmpDrives.push_back(make_unique<NowindRomDisk>());
 				changeDrives = true;
 			}
 
@@ -167,7 +168,7 @@ string NowindCommand::execute(const vector<string>& tokens)
 			if (args.empty()) {
 				error = "Missing argument for option: " + arg;
 			} else {
-				image = args.front();
+				image = std::move(args.front());
 				args.pop_front();
 				createDrive = true;
 			}
@@ -177,7 +178,7 @@ string NowindCommand::execute(const vector<string>& tokens)
 				error = "Missing argument for option: " + arg;
 			} else {
 				try {
-					string hdimage = args.front();
+					string hdimage = std::move(args.front());
 					args.pop_front();
 					processHdimage(hdimage, tmpDrives);
 					changeDrives = true;
@@ -193,16 +194,16 @@ string NowindCommand::execute(const vector<string>& tokens)
 		}
 
 		if (createDrive) {
-			DiskChanger* drive = createDiskChanger(
+			auto drive = createDiskChanger(
 				interface.basename, unsigned(tmpDrives.size()),
 				interface.getMotherBoard());
-			tmpDrives.push_back(drive);
 			changeDrives = true;
 			if (!image.empty()) {
 				if (drive->insertDisk(image)) {
 					error = "Invalid disk image: " + image;
 				}
 			}
+			tmpDrives.push_back(std::move(drive));
 		}
 	}
 	if (tmpDrives.size() > 8) {
@@ -236,13 +237,10 @@ string NowindCommand::execute(const vector<string>& tokens)
 	// cleanup tmpDrives, this contains either
 	//   - the old drives (when command was successful)
 	//   - the new drives (when there was an error)
-	for (NowindInterface::Drives::const_iterator it = tmpDrives.begin();
-	     it != tmpDrives.end(); ++it) {
-		delete *it;
-	}
-	for (NowindInterface::Drives::const_iterator it = drives.begin();
-	     it != drives.end(); ++it) {
-		if (DiskChanger* disk = dynamic_cast<DiskChanger*>(*it)) {
+	auto prevSize = tmpDrives.size();
+	tmpDrives.clear();
+	for (auto& d : drives) {
+		if (auto disk = dynamic_cast<DiskChanger*>(d.get())) {
 			disk->createCommand();
 		}
 	}
@@ -253,7 +251,7 @@ string NowindCommand::execute(const vector<string>& tokens)
 
 	// calculate result string
 	string result;
-	if (changeDrives && (tmpDrives.size() != drives.size())) {
+	if (changeDrives && (prevSize != drives.size())) {
 		result += "Number of drives changed. ";
 	}
 	if (changeDrives && (romdisk != oldRomdisk)) {
@@ -328,16 +326,16 @@ string NowindCommand::help(const vector<string>& /*tokens*/) const
 
 void NowindCommand::tabCompletion(vector<string>& tokens) const
 {
-	set<string> extra;
-	extra.insert("--ctrl");     extra.insert("-c");
-	extra.insert("--no-ctrl");  extra.insert("-C");
-	extra.insert("--allow");    extra.insert("-a");
-	extra.insert("--no-allow"); extra.insert("-A");
-	extra.insert("--romdisk");  extra.insert("-j");
-	extra.insert("--image");    extra.insert("-i");
-	extra.insert("--hdimage");  extra.insert("-m");
-	UserFileContext context;
-	completeFileName(tokens, context, extra);
+	static const char* const extra[] = {
+		"-c", "--ctrl",
+		"-C", "--no-ctrl",
+		"-a", "--allow",
+		"-A", "--no-allow",
+		"-j", "--romdisk",
+		"-i", "--image",
+		"-m", "--hdimage",
+	};
+	completeFileName(tokens, UserFileContext(), extra);
 }
 
 } // namespace openmsx
