@@ -1,11 +1,10 @@
-// $Id: NowindHost.cc 12018 2011-03-13 10:05:58Z mthuurne $
-
 #include "NowindHost.hh"
 #include "DiskContainer.hh"
 #include "SectorAccessibleDisk.hh"
 #include "serialize.hh"
 #include "serialize_stl.hh"
 #include "unreachable.hh"
+#include "memory.hh"
 #include <fstream>
 #include <algorithm>
 #include <cassert>
@@ -21,6 +20,8 @@ using std::ios;
 
 namespace openmsx {
 
+static const unsigned SECTOR_SIZE = sizeof(SectorBuffer);
+
 static void DBERR(const char* message, ...)
 {
 	(void)message;
@@ -34,7 +35,7 @@ static void DBERR(const char* message, ...)
 }
 
 
-NowindHost::NowindHost(const vector<DiskContainer*>& drives_)
+NowindHost::NowindHost(const NowindInterface::Drives& drives_)
 	: drives(drives_)
 	, lastTime(0)
 	, state(STATE_SYNC1)
@@ -146,7 +147,7 @@ void NowindHost::write(byte data, unsigned time)
 		extraData[recvCount] = data;
 		if ((data == 0) || (data == ':') ||
 		    (++recvCount == 40)) {
-			char* data = reinterpret_cast<char*>(extraData);
+			auto data = reinterpret_cast<char*>(extraData);
 			callImage(string(data, recvCount));
 			state = STATE_SYNC1;
 		}
@@ -177,7 +178,7 @@ SectorAccessibleDisk* NowindHost::getDisk() const
 {
 	byte num = cmdData[7]; // reg_a
 	if (num >= drives.size()) {
-		return 0;
+		return nullptr;
 	}
 	return drives[num]->getSectorAccessibleDisk();
 }
@@ -213,7 +214,7 @@ void NowindHost::executeCommand()
 	//case 0x30: BDOS_30H_WriteLogicalSector();
 
 	case 0x80: { // DSKIO
-		SectorAccessibleDisk* disk = getDisk();
+		auto* disk = getDisk();
 		if (!disk) {
 			// no such drive or no disk inserted
 			// (causes a timeout on the MSX side)
@@ -279,7 +280,7 @@ void NowindHost::sendHeader()
 
 void NowindHost::DSKCHG()
 {
-	SectorAccessibleDisk* disk = getDisk();
+	auto* disk = getDisk();
 	if (!disk) {
 		// no such drive or no disk inserted
 		return;
@@ -291,12 +292,12 @@ void NowindHost::DSKCHG()
 	if (drives[num]->diskChanged()) {
 		send(255); // changed
 		// read first FAT sector (contains media descriptor)
-		byte sectorBuffer[512];
-		if (disk->readSectors(sectorBuffer, 1, 1)) {
+		SectorBuffer sectorBuffer;
+		if (disk->readSectors(&sectorBuffer, 1, 1)) {
 			// TODO read error
-			sectorBuffer[0] = 0;
+			sectorBuffer.raw[0] = 0;
 		}
-		send(sectorBuffer[0]); // new mediadescriptor
+		send(sectorBuffer.raw[0]); // new mediadescriptor
 	} else {
 		send(0); // not changed
 		// TODO shouldn't we send some (dummy) byte here?
@@ -332,8 +333,8 @@ void NowindHost::INIENV()
 
 void NowindHost::setDateMSX()
 {
-	time_t td = time(NULL);
-	struct tm* tm = localtime(&td);
+	auto td = time(nullptr);
+	auto* tm = localtime(&td);
 
 	sendHeader();
 	send(tm->tm_mday);          // day
@@ -375,9 +376,9 @@ unsigned NowindHost::getCurrentAddress() const
 void NowindHost::diskReadInit(SectorAccessibleDisk& disk)
 {
 	unsigned sectorAmount = getSectorAmount();
-	buffer.resize(sectorAmount * 512);
+	buffer.resize(sectorAmount);
 	unsigned startSector = getStartSector();
-	if (disk.readSectors(&buffer[0], startSector, sectorAmount)) {
+	if (disk.readSectors(buffer.data(), startSector, sectorAmount)) {
 		// read error
 		state = STATE_SYNC1;
 		return;
@@ -390,7 +391,7 @@ void NowindHost::diskReadInit(SectorAccessibleDisk& disk)
 
 void NowindHost::doDiskRead1()
 {
-	unsigned bytesLeft = unsigned(buffer.size()) - transfered;
+	unsigned bytesLeft = unsigned(buffer.size() * SECTOR_SIZE) - transfered;
 	if (bytesLeft == 0) {
 		sendHeader();
 		send(0x01); // end of receive-loop
@@ -437,7 +438,7 @@ void NowindHost::doDiskRead2()
 		retryCount = 0;
 
 		unsigned address = getCurrentAddress();
-		size_t bytesLeft = buffer.size() - transfered;
+		size_t bytesLeft = (buffer.size() * SECTOR_SIZE) - transfered;
 		if ((address == 0x8000) && (bytesLeft > 0)) {
 			sendHeader();
 			send(0x01); // end of receive-loop
@@ -469,7 +470,7 @@ void NowindHost::transferSectors(unsigned transferAddress, unsigned amount)
 	send16(transferAddress);
 	send16(amount);
 
-	const byte* bufferPointer = &buffer[transfered];
+	auto* bufferPointer = buffer[0].raw + transfered;
 	for (unsigned i = 0; i < amount; ++i) {
 		send(bufferPointer[i]);
 	}
@@ -485,7 +486,7 @@ void NowindHost::transferSectorsBackwards(unsigned transferAddress, unsigned amo
 	send16(transferAddress + amount);
 	send(amount / 64);
 
-	const byte* bufferPointer = &buffer[transfered];
+	auto* bufferPointer = buffer[0].raw + transfered;
 	for (int i = amount - 1; i >= 0; --i) {
 		send(bufferPointer[i]);
 	}
@@ -505,19 +506,19 @@ void NowindHost::diskWriteInit(SectorAccessibleDisk& disk)
 	}
 
 	unsigned sectorAmount = std::min(128u, getSectorAmount());
-	buffer.resize(sectorAmount * 512);
+	buffer.resize(sectorAmount);
 	transfered = 0;
 	doDiskWrite1();
 }
 
 void NowindHost::doDiskWrite1()
 {
-	unsigned bytesLeft = unsigned(buffer.size()) - transfered;
+	unsigned bytesLeft = unsigned(buffer.size() * SECTOR_SIZE) - transfered;
 	if (bytesLeft == 0) {
 		// All data transferred!
-		unsigned sectorAmount = unsigned(buffer.size()) / 512;
+		unsigned sectorAmount = unsigned(buffer.size());
 		unsigned startSector = getStartSector();
-		if (SectorAccessibleDisk* disk = getDisk()) {
+		if (auto* disk = getDisk()) {
 			if (disk->writeSectors(&buffer[0], startSector, sectorAmount)) {
 				// TODO write error
 			}
@@ -552,8 +553,9 @@ void NowindHost::doDiskWrite1()
 void NowindHost::doDiskWrite2()
 {
 	assert(recvCount == (transferSize + 2));
+	auto* buf = buffer[0].raw + transfered;
 	for (unsigned i = 0; i < transferSize; ++i) {
-		buffer[i + transfered] = extraData[i + 1];
+		buf[i] = extraData[i + 1];
 	}
 
 	byte seq1 = extraData[0];
@@ -563,7 +565,7 @@ void NowindHost::doDiskWrite2()
 		transfered += transferSize;
 
 		unsigned address = getCurrentAddress();
-		size_t bytesLeft = buffer.size() - transfered;
+		size_t bytesLeft = (buffer.size() * SECTOR_SIZE) - transfered;
 		if ((address == 0x8000) && (bytesLeft > 0)) {
 			sendHeader();
 			send(254); // more data for page 2/3
@@ -602,8 +604,7 @@ int NowindHost::getDeviceNum() const
 {
 	unsigned fcb = getFCB();
 	for (unsigned i = 0; i < MAX_DEVICES; ++i) {
-		if (devices[i].fs.get() &&
-		    devices[i].fcb == fcb) {
+		if (devices[i].fs && devices[i].fcb == fcb) {
 			return i;
 		}
 	}
@@ -620,7 +621,7 @@ int NowindHost::getFreeDeviceNum()
 	}
 	// Search for free device.
 	for (unsigned i = 0; i < MAX_DEVICES; ++i) {
-		if (!devices[i].fs.get()) {
+		if (!devices[i].fs) {
 			return i;
 		}
 	}
@@ -645,7 +646,7 @@ void NowindHost::deviceOpen()
 
 	unsigned fcb = getFCB();
 	unsigned dev = getFreeDeviceNum();
-	devices[dev].fs.reset(new fstream()); // takes care of deleting old fs
+	devices[dev].fs = make_unique<fstream>(); // takes care of deleting old fs
 	devices[dev].fcb = fcb;
 
 	sendHeader();
@@ -764,12 +765,12 @@ void NowindHost::readHelper2(unsigned len, const char* buffer)
 // ie: 'pre("foo")bar' will result in 'foo'
 static string stripquotes(const string& str)
 {
-	string::size_type first = str.find_first_of('\"');
+	auto first = str.find_first_of('\"');
 	if (first == string::npos) {
 		// There are no quotes, return the whole string.
 		return str;
 	}
-	string::size_type last  = str.find_last_of ('\"');
+	auto last = str.find_last_of ('\"');
 	if (first == last) {
 		// Error, there's only a single double-quote char.
 		return "";
@@ -813,7 +814,14 @@ void NowindHost::serialize(Archive& ar, unsigned /*version*/)
 	ar.serialize("recvCount", recvCount);
 	ar.serialize("cmdData", cmdData);
 	ar.serialize("extraData", extraData);
-	ar.serialize("buffer", buffer);
+
+	// for backwards compatibility, serialize buffer as a vector<byte>
+	size_t bufSize = buffer.size() * sizeof(SectorBuffer);
+	byte* bufRaw = buffer.data()->raw;
+	vector<byte> tmp(bufRaw, bufRaw + bufSize);
+	ar.serialize("buffer", tmp);
+	memcpy(bufRaw, tmp.data(), bufSize);
+
 	ar.serialize("transfered", transfered);
 	ar.serialize("retryCount", retryCount);
 	ar.serialize("transferSize", transferSize);
