@@ -1,5 +1,3 @@
-// $Id: NowindInterface.cc 12528 2012-05-17 17:36:10Z m9710797 $
-
 #include "NowindInterface.hh"
 #include "NowindHost.hh"
 #include "NowindCommand.hh"
@@ -11,9 +9,10 @@
 #include "MSXException.hh"
 #include "serialize.hh"
 #include "serialize_stl.hh"
-#include "ref.hh"
+#include "memory.hh"
 #include <bitset>
 #include <cassert>
+#include <functional>
 
 using std::string;
 
@@ -24,21 +23,20 @@ typedef std::bitset<MAX_NOWINDS> NowindsInUse;
 
 NowindInterface::NowindInterface(const DeviceConfig& config)
 	: MSXDevice(config)
-	, rom(new Rom(getName() + " ROM", "rom", config))
-	, flash(new AmdFlash(*rom,
-	                     std::vector<unsigned>(rom->getSize() / 0x10000, 0x10000),
-	                     0, 0x01A4, config))
-	, host(new NowindHost(drives))
+	, rom(make_unique<Rom>(getName() + " ROM", "rom", config))
+	, flash(make_unique<AmdFlash>(
+		*rom, std::vector<unsigned>(rom->getSize() / 0x10000, 0x10000),
+		0, 0x01A4, config))
+	, host(make_unique<NowindHost>(drives))
 	, basename("nowindX")
 {
-	MSXMotherBoard::SharedStuff& info =
-		getMotherBoard().getSharedStuff("nowindsInUse");
+	auto& info = getMotherBoard().getSharedStuff("nowindsInUse");
 	if (info.counter == 0) {
-		assert(info.stuff == NULL);
+		assert(!info.stuff);
 		info.stuff = new NowindsInUse();
 	}
 	++info.counter;
-	NowindsInUse& nowindsInUse = *reinterpret_cast<NowindsInUse*>(info.stuff);
+	auto& nowindsInUse = *reinterpret_cast<NowindsInUse*>(info.stuff);
 
 	unsigned i = 0;
 	while (nowindsInUse[i]) {
@@ -49,25 +47,23 @@ NowindInterface::NowindInterface(const DeviceConfig& config)
 	nowindsInUse[i] = true;
 	basename[6] = char('a' + i);
 
-	command.reset(new NowindCommand(basename, getCommandController(), *this));
+	command = make_unique<NowindCommand>(
+		basename, getCommandController(), *this);
 
 	// start with one (empty) drive
-	DiskChanger* drive = command->createDiskChanger(basename, 0, getMotherBoard());
+	auto drive = command->createDiskChanger(basename, 0, getMotherBoard());
 	drive->createCommand();
-	drives.push_back(drive);
+	drives.push_back(std::move(drive));
 
 	reset(EmuTime::dummy());
 }
 
 NowindInterface::~NowindInterface()
 {
-	deleteDrives();
-
-	MSXMotherBoard::SharedStuff& info =
-		getMotherBoard().getSharedStuff("nowindsInUse");
+	auto& info = getMotherBoard().getSharedStuff("nowindsInUse");
 	assert(info.counter);
 	assert(info.stuff);
-	NowindsInUse& nowindsInUse = *reinterpret_cast<NowindsInUse*>(info.stuff);
+	auto& nowindsInUse = *reinterpret_cast<NowindsInUse*>(info.stuff);
 
 	unsigned i = basename[6] - 'a';
 	assert(nowindsInUse[i]);
@@ -77,15 +73,7 @@ NowindInterface::~NowindInterface()
 	if (info.counter == 0) {
 		assert(nowindsInUse.none());
 		delete &nowindsInUse;
-		info.stuff = NULL;
-	}
-}
-
-void NowindInterface::deleteDrives()
-{
-	for (Drives::const_iterator it = drives.begin();
-	     it != drives.end(); ++it) {
-		delete *it;
+		info.stuff = nullptr;
 	}
 }
 
@@ -130,7 +118,7 @@ const byte* NowindInterface::getReadCacheLine(word address) const
 	if (((0x2000 <= address) && (address < 0x4000)) ||
 	    ((0x8000 <= address) && (address < 0xA000))) {
 		// nowind region, not cachable
-		return NULL;
+		return nullptr;
 	} else if ((0x4000 <= address) && (address < 0xC000)) {
 		// note: range 0x8000-0xA000 is already handled above
 		return flash->getReadCacheLine(bank * 0x4000 + (address & 0x3FFF));
@@ -160,7 +148,7 @@ byte* NowindInterface::getWriteCacheLine(word address) const
 {
 	if (address < 0xC000) {
 		// not cachable
-		return NULL;
+		return nullptr;
 	} else {
 		return unmappedWrite;
 	}
@@ -170,13 +158,9 @@ byte* NowindInterface::getWriteCacheLine(word address) const
 template<typename Archive>
 void NowindInterface::serialize(Archive& ar, unsigned /*version*/)
 {
-	if (ar.isLoader()) {
-		deleteDrives();
-	}
-
 	ar.template serializeBase<MSXDevice>(*this);
 	ar.serialize("flash", *flash);
-	ar.serializeWithID("drives", drives, ref(getMotherBoard()));
+	ar.serializeWithID("drives", drives, std::ref(getMotherBoard()));
 	ar.serialize("nowindhost", *host);
 	ar.serialize("bank", bank);
 
