@@ -1,5 +1,3 @@
-// $Id: NowindRomDisk.cc 10532 2009-09-06 23:03:48Z mthuurne $
-
 #include "NowindRomDisk.hh"
 #include "serialize.hh"
 #include "serialize_meta.hh"
@@ -8,7 +6,7 @@ namespace openmsx {
 
 SectorAccessibleDisk* NowindRomDisk::getSectorAccessibleDisk()
 {
-	return NULL;
+	return nullptr;
 }
 
 const std::string& NowindRomDisk::getContainerName() const
