@@ -1,5 +1,3 @@
-// $Id: RealDrive.cc 12732 2012-07-16 16:51:40Z m9710797 $
-
 #include "RealDrive.hh"
 #include "Disk.hh"
 #include "DiskChanger.hh"
@@ -12,6 +10,7 @@
 #include "GlobalSettings.hh"
 #include "MSXException.hh"
 #include "serialize.hh"
+#include "memory.hh"
 #include <bitset>
 
 using std::string;
@@ -34,7 +33,7 @@ RealDrive::RealDrive(MSXMotherBoard& motherBoard_, EmuDuration::param motorTimeo
                      bool signalsNeedMotorOn_, bool doubleSided)
 	: Schedulable(motherBoard_.getScheduler())
 	, motherBoard(motherBoard_)
-	, loadingIndicator(new LoadingIndicator(
+	, loadingIndicator(make_unique<LoadingIndicator>(
 		motherBoard.getReactor().getGlobalSettings().getThrottleManager()))
 	, motorTimeout(motorTimeout_)
 	, motorTimer(getCurrentTime())
@@ -44,14 +43,13 @@ RealDrive::RealDrive(MSXMotherBoard& motherBoard_, EmuDuration::param motorTimeo
 	, doubleSizedDrive(doubleSided)
 	, signalsNeedMotorOn(signalsNeedMotorOn_)
 {
-	MSXMotherBoard::SharedStuff& info =
-		motherBoard.getSharedStuff("drivesInUse");
+	auto& info = motherBoard.getSharedStuff("drivesInUse");
 	if (info.counter == 0) {
-		assert(info.stuff == NULL);
+		assert(!info.stuff);
 		info.stuff = new DrivesInUse();
 	}
 	++info.counter;
-	DrivesInUse& drivesInUse = *reinterpret_cast<DrivesInUse*>(info.stuff);
+	auto& drivesInUse = *reinterpret_cast<DrivesInUse*>(info.stuff);
 
 	unsigned i = 0;
 	while (drivesInUse[i]) {
@@ -66,20 +64,19 @@ RealDrive::RealDrive(MSXMotherBoard& motherBoard_, EmuDuration::param motorTimeo
 		throw MSXException("Duplicated drive name: " + driveName);
 	}
 	motherBoard.getMSXCliComm().update(CliComm::HARDWARE, driveName, "add");
-	changer.reset(new DiskChanger(motherBoard, driveName, true));
+	changer = make_unique<DiskChanger>(motherBoard, driveName, true);
 }
 
 RealDrive::~RealDrive()
 {
 	doSetMotor(false, getCurrentTime()); // to send LED event
 
-	MSXMotherBoard::SharedStuff& info =
-		motherBoard.getSharedStuff("drivesInUse");
+	auto& info = motherBoard.getSharedStuff("drivesInUse");
 	assert(info.counter);
 	assert(info.stuff);
-	DrivesInUse& drivesInUse = *reinterpret_cast<DrivesInUse*>(info.stuff);
+	auto& drivesInUse = *reinterpret_cast<DrivesInUse*>(info.stuff);
 
-	const string& driveName = changer->getDriveName();
+	const auto& driveName = changer->getDriveName();
 	motherBoard.getMSXCliComm().update(CliComm::HARDWARE, driveName, "remove");
 	unsigned driveNum = driveName[4] - 'a';
 	assert(drivesInUse[driveNum]);
@@ -89,7 +86,7 @@ RealDrive::~RealDrive()
 	if (info.counter == 0) {
 		assert(drivesInUse.none());
 		delete &drivesInUse;
-		info.stuff = NULL;
+		info.stuff = nullptr;
 	}
 }
 
@@ -220,7 +217,7 @@ unsigned RealDrive::getCurrentAngle(EmuTime::param time) const
 {
 	if (motorStatus) {
 		// rotating, take passed time into account
-		unsigned long long deltaAngle = motorTimer.getTicksTillUp(time);
+		auto deltaAngle = motorTimer.getTicksTillUp(time);
 		return (startAngle + deltaAngle) % TICKS_PER_ROTATION;
 	} else {
 		// not rotating, angle didn't change
@@ -279,8 +276,8 @@ EmuTime RealDrive::getTimeTillIndexPulse(EmuTime::param time, int count)
 		return EmuTime::infinity;
 	}
 	unsigned delta = TICKS_PER_ROTATION - getCurrentAngle(time);
-	EmuDuration dur1 = MotorClock::duration(delta);
-	EmuDuration dur2 = MotorClock::duration(TICKS_PER_ROTATION) * (count - 1);
+	auto dur1 = MotorClock::duration(delta);
+	auto dur2 = MotorClock::duration(TICKS_PER_ROTATION) * (count - 1);
 	return time + dur1 + dur2;
 }
 
