@@ -1,10 +1,10 @@
-// $Id: SectorAccessibleDisk.cc 12860 2012-09-09 18:05:31Z m9710797 $
-
 #include "SectorAccessibleDisk.hh"
 #include "EmptyDiskPatch.hh"
 #include "IPSPatch.hh"
 #include "DiskExceptions.hh"
 #include "sha1.hh"
+#include "xrange.hh"
+#include "memory.hh"
 
 namespace openmsx {
 
@@ -12,11 +12,11 @@ namespace openmsx {
 // This line is required according to the c++ standard, but because of a vc++
 // extension, we get a link error in vc++ when we add this line. See also:
 //   http://blogs.msdn.com/b/xiangfan/archive/2010/03/03/vc-s-evil-extension-implicit-definition-of-static-constant-member.aspx
-const unsigned SectorAccessibleDisk::SECTOR_SIZE;
+const size_t SectorAccessibleDisk::SECTOR_SIZE;
 #endif
 
 SectorAccessibleDisk::SectorAccessibleDisk()
-	: patch(new EmptyDiskPatch(*this))
+	: patch(make_unique<EmptyDiskPatch>(*this))
 	, forcedWriteProtect(false)
 	, peekMode(false)
 {
@@ -26,7 +26,7 @@ SectorAccessibleDisk::~SectorAccessibleDisk()
 {
 }
 
-void SectorAccessibleDisk::readSector(unsigned sector, byte* buf)
+void SectorAccessibleDisk::readSector(size_t sector, SectorBuffer& buf)
 {
 	if (!isDummyDisk() && // in that case we want DriveEmptyException
 	    (sector > 1) && // allow reading sector 0 and 1 without calling
@@ -38,13 +38,13 @@ void SectorAccessibleDisk::readSector(unsigned sector, byte* buf)
 	}
 	try {
 		// in the end this calls readSectorImpl()
-		patch->copyBlock(sector * SECTOR_SIZE, buf, SECTOR_SIZE);
+		patch->copyBlock(sector * sizeof(buf), buf.raw, sizeof(buf));
 	} catch (MSXException& e) {
 		throw DiskIOErrorException("Disk I/O error: " + e.getMessage());
 	}
 }
 
-void SectorAccessibleDisk::writeSector(unsigned sector, const byte* buf)
+void SectorAccessibleDisk::writeSector(size_t sector, const SectorBuffer& buf)
 {
 	if (isWriteProtected()) {
 		throw WriteProtectedException("");
@@ -60,19 +60,19 @@ void SectorAccessibleDisk::writeSector(unsigned sector, const byte* buf)
 	flushCaches();
 }
 
-unsigned SectorAccessibleDisk::getNbSectors() const
+size_t SectorAccessibleDisk::getNbSectors() const
 {
 	return getNbSectorsImpl();
 }
 
 void SectorAccessibleDisk::applyPatch(const Filename& patchFile)
 {
-	patch.reset(new IPSPatch(patchFile, patch));
+	patch = make_unique<IPSPatch>(patchFile, std::move(patch));
 }
 
-void SectorAccessibleDisk::getPatches(std::vector<Filename>& result) const
+std::vector<Filename> SectorAccessibleDisk::getPatches() const
 {
-	patch->getFilenames(result);
+	return patch->getFilenames();
 }
 
 bool SectorAccessibleDisk::hasPatches() const
@@ -87,11 +87,10 @@ Sha1Sum SectorAccessibleDisk::getSha1Sum()
 		try {
 			setPeekMode(true);
 			SHA1 sha1;
-			unsigned nbSectors = getNbSectors();
-			for (unsigned i = 0; i < nbSectors; ++i) {
-				byte buf[SECTOR_SIZE];
+			for (auto i : xrange(getNbSectors())) {
+				SectorBuffer buf;
 				readSector(i, buf);
-				sha1.update(buf, SECTOR_SIZE);
+				sha1.update(buf.raw, sizeof(buf));
 			}
 			sha1cache = sha1.digest();
 			setPeekMode(false);
@@ -104,11 +103,11 @@ Sha1Sum SectorAccessibleDisk::getSha1Sum()
 }
 
 int SectorAccessibleDisk::readSectors (
-	byte* buffer, unsigned startSector, unsigned nbSectors)
+	SectorBuffer* buffers, size_t startSector, size_t nbSectors)
 {
 	try {
-		for (unsigned i = 0; i < nbSectors; ++i) {
-			readSector(startSector + i, &buffer[i * SECTOR_SIZE]);
+		for (auto i : xrange(nbSectors)) {
+			readSector(startSector + i, buffers[i]);
 		}
 		return 0;
 	} catch (MSXException&) {
@@ -117,11 +116,11 @@ int SectorAccessibleDisk::readSectors (
 }
 
 int SectorAccessibleDisk::writeSectors(
-	const byte* buffer, unsigned startSector, unsigned nbSectors)
+	const SectorBuffer* buffers, size_t startSector, size_t nbSectors)
 {
 	try {
-		for (unsigned i = 0; i < nbSectors; ++i) {
-			writeSector(startSector + i, &buffer[i * SECTOR_SIZE]);
+		for (auto i : xrange(nbSectors)) {
+			writeSector(startSector + i, buffers[i]);
 		}
 		return 0;
 	} catch (MSXException&) {
