@@ -1,5 +1,3 @@
-// $Id: SectorBasedDisk.cc 12846 2012-09-08 08:10:36Z m9710797 $
-
 #include "SectorBasedDisk.hh"
 #include "DiskExceptions.hh"
 #include "EmptyDiskPatch.hh"
@@ -10,7 +8,7 @@ namespace openmsx {
 
 SectorBasedDisk::SectorBasedDisk(const DiskName& name)
 	: Disk(name)
-	, nbSectors(unsigned(-1)) // to detect misuse
+	, nbSectors(size_t(-1)) // to detect misuse
 	, cachedTrackNum(-1)
 {
 }
@@ -21,18 +19,16 @@ SectorBasedDisk::~SectorBasedDisk()
 
 void SectorBasedDisk::writeTrackImpl(byte track, byte side, const RawTrack& input)
 {
-	std::vector<RawTrack::Sector> sectors = input.decodeAll();
-	for (std::vector<RawTrack::Sector>::const_iterator it = sectors.begin();
-	     it != sectors.end(); ++it) {
+	for (auto& s : input.decodeAll()) {
 		// Ignore 'track' and 'head' information
 		// Always assume sectorsize = 512 (so also ignore sizeCode).
 		// Ignore CRC value/errors of both address and data.
 		// Ignore sector type (deleted or not)
 		// Ignore sectors that are outside the range 1..sectorsPerTrack
-		if ((it->sector < 1) || (it->sector > getSectorsPerTrack())) continue;
-		byte buf[512];
-		input.readBlock(it->dataIdx, 512, buf);
-		unsigned logicalSector = physToLog(track, side, it->sector);
+		if ((s.sector < 1) || (s.sector > getSectorsPerTrack())) continue;
+		SectorBuffer buf;
+		input.readBlock(s.dataIdx, 512, buf.raw);
+		auto logicalSector = physToLog(track, side, s.sector);
 		writeSector(logicalSector, buf);
 		// it's important to use writeSector() and not writeSectorImpl()
 		// because only the former flushes SHA1 cache
@@ -118,10 +114,10 @@ void SectorBasedDisk::readTrack(byte track, byte side, RawTrack& output)
 			for (int i = 0; i <  3; ++i) output.write(idx++, 0xA1); // data mark (1)
 			for (int i = 0; i <  1; ++i) output.write(idx++, 0xFB); //           (2)
 
-			unsigned logicalSector = physToLog(track, side, j + 1);
-			byte sectorBuf[512];
-			readSector(logicalSector, sectorBuf);
-			for (int i = 0; i < 512; ++i) output.write(idx++, sectorBuf[i]);
+			auto logicalSector = physToLog(track, side, j + 1);
+			SectorBuffer buf;
+			readSector(logicalSector, buf);
+			for (int i = 0; i < 512; ++i) output.write(idx++, buf.raw[i]);
 
 			word dataCrc = output.calcCrc(idx - (512 + 4), 512 + 4);
 			output.write(idx++, dataCrc >> 8);   // CRC (high byte)
@@ -150,14 +146,14 @@ void SectorBasedDisk::flushCaches()
 	cachedTrackNum = -1;
 }
 
-unsigned SectorBasedDisk::getNbSectorsImpl() const
+size_t SectorBasedDisk::getNbSectorsImpl() const
 {
-	assert(nbSectors != unsigned(-1)); // must have been initialized
+	assert(nbSectors != size_t(-1)); // must have been initialized
 	return nbSectors;
 }
-void SectorBasedDisk::setNbSectors(unsigned num)
+void SectorBasedDisk::setNbSectors(size_t num)
 {
-	assert(nbSectors == unsigned(-1)); // can only set this once
+	assert(nbSectors == size_t(-1)); // can only set this once
 	nbSectors = num;
 }
 
