@@ -1,19 +1,19 @@
-// $Id: WD2793BasedFDC.cc 12732 2012-07-16 16:51:40Z m9710797 $
-
 #include "WD2793BasedFDC.hh"
 #include "DriveMultiplexer.hh"
 #include "WD2793.hh"
 #include "XMLElement.hh"
 #include "serialize.hh"
+#include "memory.hh"
 
 namespace openmsx {
 
 WD2793BasedFDC::WD2793BasedFDC(const DeviceConfig& config)
 	: MSXFDC(config)
-	, multiplexer(new DriveMultiplexer(reinterpret_cast<DiskDrive**>(drives)))
-	, controller(new WD2793(getScheduler(), *multiplexer,
-	                        getCliComm(), getCurrentTime(),
-	                        config.getXML()->getName() == "WD1770"))
+	, multiplexer(make_unique<DriveMultiplexer>(
+		reinterpret_cast<DiskDrive**>(drives)))
+	, controller(make_unique<WD2793>(
+		getScheduler(), *multiplexer, getCliComm(), getCurrentTime(),
+		config.getXML()->getName() == "WD1770"))
 {
 }
 
