@@ -1,5 +1,3 @@
-// $Id: FileOperations.cc 12867 2012-09-18 20:09:09Z m9710797 $
-
 #ifdef	_WIN32
 #ifndef _WIN32_IE
 #define _WIN32_IE 0x0500	// For SHGetSpecialFolderPathW with MinGW
@@ -8,16 +6,24 @@
 #include "vla.hh"
 #include <windows.h>
 #include <shlobj.h>
+#include <shellapi.h>
 #include <io.h>
 #include <direct.h>
 #include <ctype.h>
 #include <cstdlib>
 #include <cstring>
 #include <algorithm>
-#else
+#else // ifdef _WIN32_ ...
 #include <sys/types.h>
 #include <pwd.h>
 #include <climits>
+#include <unistd.h>
+#endif // ifdef _WIN32_ ... else ...
+
+#include "systemfuncs.hh"
+
+#if HAVE_NFTW
+#include <ftw.h>
 #endif
 
 #if defined(PATH_MAX)
@@ -33,6 +39,7 @@
 #include <Carbon/Carbon.h>
 #endif
 
+#include "openmsx.hh"
 #include "ReadDir.hh"
 #include "FileOperations.hh"
 #include "FileException.hh"
@@ -41,6 +48,7 @@
 #include "unistdp.hh"
 #include "countof.hh"
 #include "build-info.hh"
+#include "AndroidApiWrapper.hh"
 #include <sstream>
 #include <cerrno>
 #include <cstdlib>
@@ -80,7 +88,7 @@ static std::string findShareDir()
 	FSRef parentRef;
 	if (FSGetCatalogInfo(
 		&location, kFSCatInfoVolume | kFSCatInfoNodeFlags,
-		&catalogInfo, NULL, NULL, &parentRef
+		&catalogInfo, nullptr, nullptr, &parentRef
 		) != noErr) {
 		throw FatalError("Failed to get info about bundle path");
 	}
@@ -88,7 +96,7 @@ static std::string findShareDir()
 	// We will need this later to know when to give up.
 	FSRef root;
 	if (FSGetVolumeInfo(
-		catalogInfo.volume, 0, NULL, kFSVolInfoNone, NULL, NULL, &root
+		catalogInfo.volume, 0, nullptr, kFSVolInfoNone, nullptr, nullptr, &root
 		) != noErr) {
 		throw FatalError("Failed to get reference to root directory");
 	}
@@ -115,11 +123,11 @@ static std::string findShareDir()
 				iterator,
 				MAX_SCANNED_FILES,
 				&actualObjects,
-				NULL /*containerChanged*/,
+				nullptr /*containerChanged*/,
 				kFSCatInfoNodeFlags,
 				catalogInfos,
 				refs,
-				NULL /*specs*/,
+				nullptr /*specs*/,
 				names
 				);
 			if (err == errFSNoMoreItems) {
@@ -165,7 +173,7 @@ static std::string findShareDir()
 		}
 		// Go up one level.
 		if (FSGetCatalogInfo(
-			&location, kFSCatInfoNone, NULL, NULL, NULL, &parentRef
+			&location, kFSCatInfoNone, nullptr, nullptr, nullptr, &parentRef
 			) != noErr
 		) {
 			throw FatalError("Failed to get parent directory");
@@ -181,7 +189,7 @@ string expandTilde(string_ref path)
 	if (path.empty() || path[0] != '~') {
 		return path.str();
 	}
-	string_ref::size_type pos = path.find_first_of('/');
+	auto pos = path.find_first_of('/');
 	string_ref user = ((path.size() == 1) || (pos == 1))
 	                ? ""
 	                : path.substr(1, (pos == string_ref::npos) ? pos : pos - 1);
@@ -193,7 +201,7 @@ string expandTilde(string_ref path)
 	if (pos == string_ref::npos) {
 		return result;
 	}
-	if (*result.rbegin() != '/') {
+	if (result.back() != '/') {
 		result += '/';
 	}
 	string_ref last = path.substr(pos + 1);
@@ -255,6 +263,59 @@ int rmdir(const std::string& path)
 #endif
 }
 
+#ifdef _WIN32
+int deleteRecursive(const std::string& path)
+{
+	std::wstring pathW = utf8to16(path);
+
+	SHFILEOPSTRUCTW rmdirFileop;
+	rmdirFileop.hwnd = nullptr;
+	rmdirFileop.wFunc = FO_DELETE;
+	rmdirFileop.pFrom = pathW.c_str();
+	rmdirFileop.pTo = nullptr;
+	rmdirFileop.fFlags = FOF_SILENT | FOF_NOCONFIRMATION | FOF_NOERRORUI;
+	rmdirFileop.fAnyOperationsAborted = FALSE;
+	rmdirFileop.hNameMappings = nullptr;
+	rmdirFileop.lpszProgressTitle = nullptr;
+
+	return SHFileOperationW(&rmdirFileop);
+}
+#elif HAVE_NFTW
+int deleteRecursive_cb(const char* fpath, const struct stat* /*sb*/,
+                       int /*typeflag*/, struct FTW* /*ftwbuf*/)
+{
+	return remove(fpath);
+}
+int deleteRecursive(const std::string& path)
+{
+	return nftw(path.c_str(), deleteRecursive_cb, 64, FTW_DEPTH | FTW_PHYS);
+}
+#else
+// This is a platform independent version of deleteRecursive() (it builds on
+// top of helper routines that _are_ platform specific). Though I still prefer
+// the two platform specific deleteRecursive() routines above because they are
+// likely more optimized and likely contain less bugs than this version (e.g.
+// we're walking over the entries in a directory while simultaneously deleting
+// entries in that same directory. Although this seems to work fine, I'm not
+// 100% sure our ReadDir 'emulation code' for windows covers all corner cases.
+// While the windows version above very likely does handle everything).
+int deleteRecursive(const std::string& path)
+{
+	if (isDirectory(path)) {
+		{
+			ReadDir dir(path);
+			while (dirent* d = dir.getEntry()) {
+				int err = deleteRecursive(d->d_name);
+				if (err) return err;
+			}
+		}
+		return rmdir(path);
+	} else {
+		return unlink(path);
+	}
+}
+#endif
+
 FILE* openFile(const std::string& filename, const std::string& mode)
 {
 	// Mode must contain a 'b' character. On unix this doesn't make any
@@ -295,7 +356,7 @@ void openofstream(std::ofstream& stream, const std::string& filename,
 
 string_ref getFilename(string_ref path)
 {
-	string_ref::size_type pos = path.rfind('/');
+	auto pos = path.rfind('/');
 	if (pos == string_ref::npos) {
 		return path;
 	} else {
@@ -305,7 +366,7 @@ string_ref getFilename(string_ref path)
 
 string_ref getBaseName(string_ref path)
 {
-	string_ref::size_type pos = path.rfind('/');
+	auto pos = path.rfind('/');
 	if (pos == string_ref::npos) {
 		return "";
 	} else {
@@ -316,7 +377,7 @@ string_ref getBaseName(string_ref path)
 string_ref getExtension(string_ref path)
 {
 	string_ref filename = getFilename(path);
-	string_ref::size_type pos = filename.rfind('.');
+	auto pos = filename.rfind('.');
 	if (pos == string_ref::npos) {
 		return "";
 	} else {
@@ -326,7 +387,7 @@ string_ref getExtension(string_ref path)
 
 string_ref stripExtension(string_ref path)
 {
-	string_ref::size_type pos = path.rfind('.');
+	auto pos = path.rfind('.');
 	if (pos == string_ref::npos) {
 		return path;
 	} else {
@@ -422,26 +483,15 @@ string getUserHomeDir(string_ref username)
 	(void)(&username); // ignore parameter, avoid warning
 
 	wchar_t bufW[MAXPATHLEN + 1];
-	if (!SHGetSpecialFolderPathW(NULL, bufW, CSIDL_PERSONAL, TRUE)) {
+	if (!SHGetSpecialFolderPathW(nullptr, bufW, CSIDL_PERSONAL, TRUE)) {
 		throw FatalError(StringOp::Builder() <<
 			"SHGetSpecialFolderPathW failed: " << GetLastError());
 	}
 
 	return getConventionalPath(utf16to8(bufW));
-
-#elif PLATFORM_GP2X
-	return ""; // TODO figure out why stuff below doesn't work
-	// We cannot use generic implementation below, because for some
-	// reason getpwuid() and getpwnam() cannot be used in statically
-	// linked applications.
-	const char* dir = getenv("HOME");
-	if (!dir) {
-		dir = "/root";
-	}
-	return dir;
 #else
-	const char* dir = NULL;
-	struct passwd* pw = NULL;
+	const char* dir = nullptr;
+	struct passwd* pw = nullptr;
 	if (username.empty()) {
 		dir = getenv("HOME");
 		if (!dir) {
@@ -461,6 +511,8 @@ const string& getUserOpenMSXDir()
 {
 #ifdef _WIN32
 	static const string OPENMSX_DIR = expandTilde("~/openMSX");
+#elif PLATFORM_ANDROID
+	static const string OPENMSX_DIR = AndroidApiWrapper::getStorageDirectory() + "/openMSX";
 #else
 	static const string OPENMSX_DIR = expandTilde("~/.openMSX");
 #endif
@@ -484,7 +536,7 @@ string getSystemDataDir()
 	string newValue;
 #ifdef _WIN32
 	wchar_t bufW[MAXPATHLEN + 1];
-	int res = GetModuleFileNameW(NULL, bufW, countof(bufW));
+	int res = GetModuleFileNameW(nullptr, bufW, countof(bufW));
 	if (!res) {
 		throw FatalError(StringOp::Builder() <<
 			"Cannot detect openMSX directory. GetModuleFileNameW failed: " <<
@@ -492,13 +544,16 @@ string getSystemDataDir()
 	}
 
 	string filename = utf16to8(bufW);
-	string::size_type pos = filename.find_last_of('\\');
+	auto pos = filename.find_last_of('\\');
 	if (pos == string::npos) {
 		throw FatalError("openMSX is not in directory!?");
 	}
 	newValue = getConventionalPath(filename.substr(0, pos)) + "/share";
 #elif defined(__APPLE__)
 	newValue = findShareDir();
+#elif PLATFORM_ANDROID
+	newValue = getAbsolutePath("openmsx_system");
+	ad_printf("System data dir: %s", newValue.c_str());
 #else
 	// defined in build-info.hh (default /opt/openMSX/share)
 	newValue = DATADIR;
@@ -525,9 +580,9 @@ string expandCurrentDirFromDrive(string_ref path)
 		if (('a' <= drive) && (drive <= 'z')) {
 			wchar_t bufW[MAXPATHLEN + 1];
 			if (driveExists(drive) &&
-				_wgetdcwd(drive - 'a' + 1, bufW, MAXPATHLEN) != NULL) {
+				_wgetdcwd(drive - 'a' + 1, bufW, MAXPATHLEN)) {
 				result = getConventionalPath(utf16to8(bufW));
-				if (*result.rbegin() != '/') {
+				if (result.back() != '/') {
 					result += '/';
 				}
 				if (path.size() > 2) {
@@ -546,7 +601,7 @@ bool getStat(const string& filename_, Stat& st)
 	string filename = expandTilde(filename_);
 	// workaround for VC++: strip trailing slashes (but keep it if it's the
 	// only character in the path)
-	string::size_type pos = filename.find_last_not_of('/');
+	auto pos = filename.find_last_not_of('/');
 	if (pos == string::npos) {
 		// string was either empty or a (sequence of) '/' character(s)
 		filename = filename.empty() ? "" : "/";
@@ -595,8 +650,8 @@ time_t getModificationDate(const Stat& st)
 static int getNextNum(dirent* d, string_ref prefix, string_ref extension,
                       unsigned nofdigits)
 {
-	string_ref::size_type extensionLen = extension.size();
-	string_ref::size_type prefixLen = prefix.size();
+	auto extensionLen = extension.size();
+	auto prefixLen = prefix.size();
 	string_ref name(d->d_name);
 
 	if ((name.size() != (prefixLen + nofdigits + extensionLen)) ||
@@ -670,7 +725,7 @@ string parseCommandFileArgument(
 string getTempDir()
 {
 #ifdef _WIN32
-	DWORD len = GetTempPathW(0, NULL);
+	DWORD len = GetTempPathW(0, nullptr);
 	if (len) {
 		VLA(wchar_t, bufW, (len+1));
 		len = GetTempPathW(len, bufW);
@@ -684,8 +739,11 @@ string getTempDir()
 	}
 	throw FatalError(StringOp::Builder() <<
 		"GetTempPathW failed: " << GetLastError());
+#elif PLATFORM_ANDROID
+	string result = getSystemDataDir() + "/tmp";
+	return result;
 #else
-	const char* result = NULL;
+	const char* result = nullptr;
 	if (!result) result = getenv("TMPDIR");
 	if (!result) result = getenv("TMP");
 	if (!result) result = getenv("TEMP");
